@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalsimplewarp.cpp,v 1.9 2006/03/21 20:56:00 fwarmerdam Exp $
+ * $Id: gdalsimplewarp.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Mapinfo Image Warper
  * Purpose:  Simple (source in memory) warp algorithm.
@@ -25,43 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gdalsimplewarp.cpp,v $
- * Revision 1.9  2006/03/21 20:56:00  fwarmerdam
- * cleanup headers
- *
- * Revision 1.8  2005/04/04 15:24:16  fwarmerdam
- * added CPL_STDCALL to some functions
- *
- * Revision 1.7  2003/07/08 15:33:04  warmerda
- * avoid warnings
- *
- * Revision 1.6  2003/02/24 17:30:58  warmerda
- * added REMAP support
- *
- * Revision 1.5  2003/02/06 04:56:35  warmerda
- * added documentation
- *
- * Revision 1.4  2002/12/07 22:58:42  warmerda
- * added initialization support for simple warper
- *
- * Revision 1.3  2002/12/06 21:43:28  warmerda
- * removed luts
- *
- * Revision 1.2  2002/12/05 21:44:19  warmerda
- * initial implementation
- *
- * Revision 1.1  2002/12/05 05:43:23  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal_priv.h"
 #include "gdal_alg.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: gdalsimplewarp.cpp,v 1.9 2006/03/21 20:56:00 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdalsimplewarp.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static void 
 GDALSimpleWarpRemapping( int nBandCount, GByte **papabySrcData, 
