@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaltransformer.cpp,v 1.24 2005/10/28 17:47:28 fwarmerdam Exp $
+ * $Id: gdaltransformer.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Mapinfo Image Warper
  * Purpose:  Implementation of one or more GDALTrasformerFunc types, including
@@ -27,98 +27,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gdaltransformer.cpp,v $
- * Revision 1.24  2005/10/28 17:47:28  fwarmerdam
- * GenImgTransformer now supports NULL hSrcDS.  Also now actually pulls
- * coordinate systems from hSrcDS or hDstDS if not provided.
- *
- * Revision 1.23  2005/08/02 22:21:06  fwarmerdam
- * Approx transformer can now own and cleanup it's subtransformer.
- * GenImg transformer now deserialized with property GTI signature.
- *
- * Revision 1.22  2005/04/11 17:36:53  fwarmerdam
- * added GDALDestroyTransformer
- *
- * Revision 1.21  2005/04/04 15:24:16  fwarmerdam
- * added CPL_STDCALL to some functions
- *
- * Revision 1.20  2004/12/26 16:12:21  fwarmerdam
- * thin plate spline support now implemented
- *
- * Revision 1.19  2004/11/05 04:55:48  fwarmerdam
- * We can't compute the diagonal based on the first and last corner points
- * if either didn't transform successfully.  Just fallback to using the
- * diagonal of the min/max window even though it is slightly less precise.
- *
- * Revision 1.18  2004/08/13 14:50:06  warmerda
- * use SetFromUserInput() for SourceSRS and TargetSRS
- *
- * Revision 1.17  2004/08/11 20:42:05  warmerda
- * dont declare static funcs in CPL_C area
- *
- * Revision 1.16  2004/08/11 19:00:37  warmerda
- * added GDALSetGenImgProjTransformerDstGeoTransform
- *
- * Revision 1.15  2004/08/09 14:38:27  warmerda
- * added serialize/deserialize support for warpoptions and transformers
- *
- * Revision 1.14  2004/03/28 16:02:04  warmerda
- * added GDALApplyGeoTransform()
- *
- * Revision 1.13  2004/03/28 15:50:01  warmerda
- * Added docs for GDALInvGeoTransform().
- *
- * Revision 1.12  2004/01/24 09:33:33  warmerda
- * Added support for internal grid in GDALSuggestedWarpOutput() for cases
- * where alot of points around the edge fail to transform.
- * Added use of OGRCoordinateTransformation::TransformEx() to capture per-point
- * reprojection errors.
- * Ensure that approximate transformer reverts to exact transformer if any of
- * the 3 points fail to transform.
- *
- * Revision 1.11  2003/07/08 15:29:14  warmerda
- * avoid warnings
- *
- * Revision 1.10  2003/06/03 19:42:54  warmerda
- * added partial support for RPC in GenImgProjTransformer
- *
- * Revision 1.9  2003/02/06 04:56:35  warmerda
- * added documentation
- *
- * Revision 1.8  2002/12/13 15:55:27  warmerda
- * fix suggested output to preserve orig size exact on null transform
- *
- * Revision 1.7  2002/12/13 04:57:49  warmerda
- * remove approx transformer debug output
- *
- * Revision 1.6  2002/12/09 16:08:32  warmerda
- * added approximating transformer
- *
- * Revision 1.5  2002/12/07 17:09:38  warmerda
- * added order flag to GenImgProjTransformer
- *
- * Revision 1.4  2002/12/06 21:43:56  warmerda
- * added GCP support to general transformer
- *
- * Revision 1.3  2002/12/05 21:45:01  warmerda
- * fixed a few GenImgProj bugs
- *
- * Revision 1.2  2002/12/05 16:37:46  warmerda
- * fixed method name
- *
- * Revision 1.1  2002/12/05 05:43:23  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal_priv.h"
 #include "gdal_alg.h"
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: gdaltransformer.cpp,v 1.24 2005/10/28 17:47:28 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdaltransformer.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 CPL_C_START
 CPLXMLNode *GDALSerializeGCPTransformer( void *pTransformArg );
 void *GDALDeserializeGCPTransformer( CPLXMLNode *psTree );
@@ -233,6 +149,67 @@ GDALSuggestedWarpOutput( GDALDatasetH hSrcDS,
                          double *padfGeoTransformOut, 
                          int *pnPixels, int *pnLines )
 
+{
+    double adfExtent[4];
+
+    return GDALSuggestedWarpOutput2( hSrcDS, pfnTransformer, pTransformArg, 
+                                     padfGeoTransformOut, pnPixels, pnLines, 
+                                     adfExtent, 0 );
+}
+
+/************************************************************************/
+/*                      GDALSuggestedWarpOutput2()                      */
+/************************************************************************/
+
+/**
+ * Suggest output file size.
+ *
+ * This function is used to suggest the size, and georeferenced extents
+ * appropriate given the indicated transformation and input file.  It walks
+ * the edges of the input file (approximately 20 sample points along each
+ * edge) transforming into output coordinates in order to get an extents box.
+ *
+ * Then a resolution is computed with the intent that the length of the
+ * distance from the top left corner of the output imagery to the bottom right
+ * corner would represent the same number of pixels as in the source image. 
+ * Note that if the image is somewhat rotated the diagonal taken isnt of the
+ * whole output bounding rectangle, but instead of the locations where the
+ * top/left and bottom/right corners transform.  The output pixel size is 
+ * always square.  This is intended to approximately preserve the resolution
+ * of the input data in the output file. 
+ * 
+ * The values returned in padfGeoTransformOut, pnPixels and pnLines are
+ * the suggested number of pixels and lines for the output file, and the
+ * geotransform relating those pixels to the output georeferenced coordinates.
+ *
+ * The trickiest part of using the function is ensuring that the 
+ * transformer created is from source file pixel/line coordinates to 
+ * output file georeferenced coordinates.  This can be accomplished with 
+ * GDALCreateGenImProjTransformer() by passing a NULL for the hDstDS.  
+ *
+ * @param hSrcDS the input image (it is assumed the whole input images is
+ * being transformed). 
+ * @param pfnTransformer the transformer function.
+ * @param pTransformArg the callback data for the transformer function.
+ * @param padfGeoTransformOut the array of six doubles in which the suggested
+ * geotransform is returned. 
+ * @param pnPixels int in which the suggest pixel width of output is returned.
+ * @param pnLines int in which the suggest pixel height of output is returned.
+ * @param padfExtent Four entry array to return extents as (xmin, ymin, xmax, ymax). 
+ * @param nOptions Options, currently always zero.
+ *
+ * @return CE_None if successful or CE_Failure otherwise. 
+ */
+
+
+CPLErr CPL_STDCALL
+GDALSuggestedWarpOutput2( GDALDatasetH hSrcDS, 
+                          GDALTransformerFunc pfnTransformer, 
+                          void *pTransformArg, 
+                          double *padfGeoTransformOut, 
+                          int *pnPixels, int *pnLines,
+                          double *padfExtent, int nOptions )
+
 {
 /* -------------------------------------------------------------------- */
 /*      Setup sample points all around the edge of the input raster.    */
@@ -273,6 +250,8 @@ GDALSuggestedWarpOutput( GDALDatasetH hSrcDS,
 
     CPLAssert( nSamplePoints == 84 );
 
+    memset( abSuccess, 1, sizeof(abSuccess) );
+
 /* -------------------------------------------------------------------- */
 /*      Transform them to the output coordinate system.                 */
 /* -------------------------------------------------------------------- */
@@ -400,6 +379,14 @@ GDALSuggestedWarpOutput( GDALDatasetH hSrcDS,
     }
 
     dfDiagonalDist = sqrt( dfDeltaX * dfDeltaX + dfDeltaY * dfDeltaY );
+
+/* -------------------------------------------------------------------- */
+/*      Return raw extents.                                             */
+/* -------------------------------------------------------------------- */
+    padfExtent[0] = dfMinXOut;
+    padfExtent[1] = dfMinYOut;
+    padfExtent[2] = dfMaxXOut;
+    padfExtent[3] = dfMaxYOut;
     
 /* -------------------------------------------------------------------- */
 /*      Compute a pixel size from this.                                 */
@@ -441,6 +428,7 @@ typedef struct {
     void     *pSrcGCPTransformArg;
     void     *pSrcRPCTransformArg;
     void     *pSrcTPSTransformArg;
+    void     *pSrcGeoLocTransformArg;
 
     void     *pReprojectArg;
 
@@ -593,6 +581,18 @@ GDALCreateGenImgProjTransformer( GDALDatasetH hSrcDS, const char *pszSrcWKT,
         }
     }
 
+    else if( (papszMD = GDALGetMetadata( hSrcDS, "GEOLOCATION" )) != NULL )
+    {
+        psInfo->pSrcGeoLocTransformArg = 
+            GDALCreateGeoLocTransformer( hSrcDS, papszMD, FALSE );
+
+        if( psInfo->pSrcGeoLocTransformArg == NULL )
+        {
+            GDALDestroyGenImgProjTransformer( psInfo );
+            return NULL;
+        }
+    }
+
     else
     {
         CPLError( CE_Failure, CPLE_AppDefined, 
@@ -704,6 +704,9 @@ void GDALDestroyGenImgProjTransformer( void *hTransformArg )
     if( psInfo->pSrcTPSTransformArg != NULL )
         GDALDestroyTPSTransformer( psInfo->pSrcTPSTransformArg );
 
+    if( psInfo->pSrcGeoLocTransformArg != NULL )
+        GDALDestroyGeoLocTransformer( psInfo->pSrcGeoLocTransformArg );
+
     if( psInfo->pDstGCPTransformArg != NULL )
         GDALDestroyGCPTransformer( psInfo->pDstGCPTransformArg );
 
@@ -738,6 +741,7 @@ int GDALGenImgProjTransform( void *pTransformArg, int bDstToSrc,
     void *pGCPTransformArg;
     void *pRPCTransformArg;
     void *pTPSTransformArg;
+    void *pGeoLocTransformArg;
 
 /* -------------------------------------------------------------------- */
 /*      Convert from src (dst) pixel/line to src (dst)                  */
@@ -749,6 +753,7 @@ int GDALGenImgProjTransform( void *pTransformArg, int bDstToSrc,
         pGCPTransformArg = psInfo->pDstGCPTransformArg;
         pRPCTransformArg = NULL;
         pTPSTransformArg = NULL;
+        pGeoLocTransformArg = NULL;
     }
     else
     {
@@ -756,6 +761,7 @@ int GDALGenImgProjTransform( void *pTransformArg, int bDstToSrc,
         pGCPTransformArg = psInfo->pSrcGCPTransformArg;
         pRPCTransformArg = psInfo->pSrcRPCTransformArg;
         pTPSTransformArg = psInfo->pSrcTPSTransformArg;
+        pGeoLocTransformArg = psInfo->pSrcGeoLocTransformArg;
     }
 
     if( pGCPTransformArg != NULL )
@@ -779,12 +785,25 @@ int GDALGenImgProjTransform( void *pTransformArg, int bDstToSrc,
                                panSuccess ) )
             return FALSE;
     }
+    else if( pGeoLocTransformArg != NULL )
+    {
+        if( !GDALGeoLocTransform( pGeoLocTransformArg, FALSE, 
+                                  nPointCount, padfX, padfY, padfZ,
+                                  panSuccess ) )
+            return FALSE;
+    }
     else 
     {
         for( i = 0; i < nPointCount; i++ )
         {
             double dfNewX, dfNewY;
             
+            if( padfX[i] == HUGE_VAL || padfY[i] == HUGE_VAL )
+            {
+                panSuccess[i] = FALSE;
+                continue;
+            }
+
             dfNewX = padfGeoTransform[0]
                 + padfX[i] * padfGeoTransform[1]
                 + padfY[i] * padfGeoTransform[2];
@@ -822,6 +841,7 @@ int GDALGenImgProjTransform( void *pTransformArg, int bDstToSrc,
         pGCPTransformArg = psInfo->pSrcGCPTransformArg;
         pRPCTransformArg = psInfo->pSrcRPCTransformArg;
         pTPSTransformArg = psInfo->pSrcTPSTransformArg;
+        pGeoLocTransformArg = psInfo->pSrcGeoLocTransformArg;
     }
     else
     {
@@ -829,6 +849,7 @@ int GDALGenImgProjTransform( void *pTransformArg, int bDstToSrc,
         pGCPTransformArg = psInfo->pDstGCPTransformArg;
         pRPCTransformArg = NULL;
         pTPSTransformArg = NULL;
+        pGeoLocTransformArg = NULL;
     }
         
     if( pGCPTransformArg != NULL )
@@ -852,11 +873,21 @@ int GDALGenImgProjTransform( void *pTransformArg, int bDstToSrc,
                                panSuccess ) )
             return FALSE;
     }
+    else if( pGeoLocTransformArg != NULL )
+    {
+        if( !GDALGeoLocTransform( pGeoLocTransformArg, TRUE,
+                                  nPointCount, padfX, padfY, padfZ,
+                                  panSuccess ) )
+            return FALSE;
+    }
     else
     {
         for( i = 0; i < nPointCount; i++ )
         {
             double dfNewX, dfNewY;
+
+            if( !panSuccess[i] )
+                continue;
             
             dfNewX = padfGeoTransform[0]
                 + padfX[i] * padfGeoTransform[1]
