@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalwarpoperation.cpp,v 1.25 2005/12/23 18:16:04 fwarmerdam Exp $
+ * $Id: gdalwarpoperation.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  High Performance Image Reprojector
  * Purpose:  Implementation of the GDALWarpOperation class.
@@ -25,94 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gdalwarpoperation.cpp,v $
- * Revision 1.25  2005/12/23 18:16:04  fwarmerdam
- * added SKIP_NOSOURCE option
- *
- * Revision 1.24  2005/09/13 01:20:16  fwarmerdam
- * added UNIFIED_SRC_NODATA mechanism
- *
- * Revision 1.23  2004/11/14 04:16:30  fwarmerdam
- * fixup src alpha support
- *
- * Revision 1.22  2004/11/05 18:24:26  fwarmerdam
- * In ComputeSourceWindow() fallback to using grid if any points along edges
- * fail.  Also add a buffer around the imagery read if any points fail.
- *
- * Revision 1.21  2004/10/07 15:50:18  fwarmerdam
- * added preliminary alpha band support
- *
- * Revision 1.20  2004/08/09 14:38:27  warmerda
- * added serialize/deserialize support for warpoptions and transformers
- *
- * Revision 1.19  2004/04/29 13:44:55  warmerda
- * fixed some memory leaks
- *
- * Revision 1.18  2004/03/28 21:22:46  warmerda
- * Substantial improvement in default selection of eWorkingDataType.  In some
- * cases we need to use a more refined data type to preserve nodata values.
- *
- * Revision 1.17  2003/11/22 19:13:31  dron
- * Added C bindings for GDALWarpOperation functions.
- *
- * Revision 1.16  2003/07/26 17:43:15  warmerda
- * Added various warp options in ComputeSourceWindow().
- *
- * Revision 1.15  2003/07/24 16:48:40  warmerda
- * fixed bug with window size, and origin in ComputeSourceWindow
- *
- * Revision 1.14  2003/07/04 11:52:22  dron
- * GRA_CubicSpline added to the list of validated algorithms.
- *
- * Revision 1.13  2003/06/05 17:10:13  warmerda
- * changed to report approximately correct completion amount in multi case
- *
- * Revision 1.12  2003/05/27 20:49:26  warmerda
- * added REPORT_TIMINGS support
- *
- * Revision 1.11  2003/05/16 19:43:28  warmerda
- * ensure that chunks dont get negative sizes
- *
- * Revision 1.10  2003/05/07 19:57:32  warmerda
- * INIT_DEST now supports a list of band values
- *
- * Revision 1.9  2003/05/07 19:13:06  warmerda
- * added pre and post warp chunk processor
- *
- * Revision 1.8  2003/05/06 18:32:48  warmerda
- * Added WRITE_FLUSH option support.
- * Fixed multi-threaded support so that mutexes are released after creation.
- *
- * Revision 1.7  2003/04/28 20:47:59  warmerda
- * use dataset level RasterIO call
- *
- * Revision 1.6  2003/04/23 05:18:57  warmerda
- * added multithread support
- *
- * Revision 1.5  2003/03/02 05:25:59  warmerda
- * added some source nodata support
- *
- * Revision 1.4  2003/02/22 02:05:20  warmerda
- * added defaulting of band mapping, added warp options
- *
- * Revision 1.3  2003/02/21 15:41:19  warmerda
- * working minimally
- *
- * Revision 1.2  2003/02/20 21:53:06  warmerda
- * partial implementation
- *
- * Revision 1.1  2003/02/18 17:25:50  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdalwarper.h"
 #include "cpl_string.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gdalwarpoperation.cpp,v 1.25 2005/12/23 18:16:04 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdalwarpoperation.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -311,14 +230,6 @@ int GDALWarpOperation::ValidateOptions()
         return FALSE;
     }
 
-    if( psOptions->hDstDS == NULL )
-    {
-        CPLError( CE_Failure, CPLE_IllegalArg, 
-                  "GDALWarpOptions.Validate()\n"
-                  "  hDstDS is not set." );
-        return FALSE;
-    }
-
     if( psOptions->nBandCount == 0 )
     {
         CPLError( CE_Failure, CPLE_IllegalArg, 
@@ -327,11 +238,19 @@ int GDALWarpOperation::ValidateOptions()
         return FALSE;
     }
 
-    if( psOptions->panSrcBands == NULL || psOptions->panDstBands == NULL )
+    if( psOptions->panSrcBands == NULL )
     {
         CPLError( CE_Failure, CPLE_IllegalArg, 
                   "GDALWarpOptions.Validate()\n"
-                  "  Either panSrcBands or panDstBands is NULL." );
+                  "  panSrcBands is NULL." );
+        return FALSE;
+    }
+
+    if( psOptions->hDstDS != NULL && psOptions->panDstBands == NULL )
+    {
+        CPLError( CE_Failure, CPLE_IllegalArg, 
+                  "GDALWarpOptions.Validate()\n"
+                  "  panDstBands is NULL." );
         return FALSE;
     }
 
@@ -346,9 +265,10 @@ int GDALWarpOperation::ValidateOptions()
                       iBand, psOptions->panSrcBands[iBand] );
             return FALSE;
         }
-        if( psOptions->panDstBands[iBand] < 1 
-            || psOptions->panDstBands[iBand]
-            > GDALGetRasterCount( psOptions->hDstDS ) )
+        if( psOptions->hDstDS != NULL
+            && (psOptions->panDstBands[iBand] < 1 
+                || psOptions->panDstBands[iBand]
+                > GDALGetRasterCount( psOptions->hDstDS ) ) )
         {
             CPLError( CE_Failure, CPLE_IllegalArg,
                       "panDstBands[%d] = %d ... out of range for dataset.",
@@ -356,9 +276,10 @@ int GDALWarpOperation::ValidateOptions()
             return FALSE;
         }
 
-        if( GDALGetRasterAccess( 
+        if( psOptions->hDstDS != NULL
+            && GDALGetRasterAccess( 
                 GDALGetRasterBand(psOptions->hDstDS,
-                                  psOptions->panDstBands[iBand]) )
+                                  psOptions->panDstBands[iBand]))
             == GA_ReadOnly )
         {
             CPLError( CE_Failure, CPLE_IllegalArg,
@@ -402,7 +323,7 @@ int GDALWarpOperation::ValidateOptions()
     }
 
     if( CSLFetchNameValue( psOptions->papszWarpOptions, 
-                         "SAMPLE_STEPS" ) != NULL )
+                           "SAMPLE_STEPS" ) != NULL )
     {
         if( atoi(CSLFetchNameValue( psOptions->papszWarpOptions, 
                                     "SAMPLE_STEPS" )) < 2 )
