@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalinfo.c,v 1.43 2006/03/21 21:34:43 fwarmerdam Exp $
+ * $Id: gdalinfo.c 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GDAL Utilities
  * Purpose:  Commandline application to list info about a file.
@@ -25,137 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- * ****************************************************************************
- *
- * $Log: gdalinfo.c,v $
- * Revision 1.43  2006/03/21 21:34:43  fwarmerdam
- * cleanup headers
- *
- * Revision 1.42  2005/12/21 05:31:56  fwarmerdam
- * added reporting of band image_structure metadata
- *
- * Revision 1.41  2005/12/21 00:38:05  fwarmerdam
- * Report image_structure metadata.
- *
- * Revision 1.40  2005/10/11 11:29:09  dron
- * Include cpl_multiproc.h for CPLCleanupTLS() function.
- *
- * Revision 1.39  2005/09/11 17:18:33  fwarmerdam
- * added CPLCleanupTLS()
- *
- * Revision 1.38  2005/07/29 04:27:42  fwarmerdam
- * Don't crash if a "paletted" band has no color table object.
- *
- * Revision 1.37  2005/05/11 14:39:43  fwarmerdam
- * added option to report stats
- *
- * Revision 1.36  2004/08/24 20:13:28  warmerda
- * ensure -nomd works for bands too
- *
- * Revision 1.35  2004/08/09 14:39:07  warmerda
- * added shared list dump
- *
- * Revision 1.34  2004/04/29 13:44:25  warmerda
- * added raster category and scale/offset support
- *
- * Revision 1.33  2004/04/13 15:09:41  warmerda
- * clean up arguments
- *
- * Revision 1.32  2004/04/02 17:33:22  warmerda
- * added GDALGeneralCmdLineProcessor()
- *
- * Revision 1.31  2004/03/19 06:30:06  warmerda
- * Changed to compute exact min/max for -mm instead of approximate.
- *
- * Revision 1.30  2003/11/24 17:46:58  warmerda
- * Report full geotransform when image is not northup.
- *
- * Revision 1.29  2003/05/01 13:16:37  warmerda
- * dont generate error reports if instantiating coordinate transform fails
- *
- * Revision 1.28  2003/04/22 16:02:08  dron
- * New switches: -nogcp and -nomd to suppress printing out GCPs list and metadata
- * strings respectively.
- *
- * Revision 1.27  2002/09/20 14:32:15  warmerda
- * don't build gdalinfo statically any more
- *
- * Revision 1.26  2002/09/04 06:53:42  warmerda
- * added GDALDestroyDriverManager() for cleanup
- *
- * Revision 1.25  2002/05/29 15:53:14  warmerda
- * added GDALDumpOpenDatasets
- *
- * Revision 1.24  2002/04/16 14:00:25  warmerda
- * added GDALVersionInfo
- *
- * Revision 1.23  2002/03/25 13:50:07  warmerda
- * only report min/max values if fetch successfully
- *
- * Revision 1.22  2002/01/13 01:42:37  warmerda
- * add -sample test
- *
- * Revision 1.21  2001/11/17 21:40:58  warmerda
- * converted to use OGR projection services
- *
- * Revision 1.20  2001/11/02 22:21:36  warmerda
- * fixed memory leak
- *
- * Revision 1.19  2001/07/18 05:05:12  warmerda
- * added CPL_CSVID
- *
- * Revision 1.18  2001/07/05 13:12:40  warmerda
- * added UnitType support
- *
- * Revision 1.17  2001/06/28 19:40:12  warmerda
- * added subdatset reporting
- *
- * Revision 1.16  2000/11/29 20:52:53  warmerda
- * Add pretty printing of projection.
- *
- * Revision 1.15  2000/08/25 14:26:02  warmerda
- * added nodata, and arbitrary overview reporting
- *
- * Revision 1.14  2000/06/12 14:21:43  warmerda
- * Fixed min/max printf.
- *
- * Revision 1.13  2000/05/15 14:06:26  warmerda
- * Added -mm for computing min/max, and fixed ovewriting of i.
- *
- * Revision 1.12  2000/04/21 21:57:33  warmerda
- * updated the way metadata is handled
- *
- * Revision 1.11  2000/03/31 13:43:21  warmerda
- * added code to report all corners and gcps
- *
- * Revision 1.10  2000/03/29 15:33:32  warmerda
- * added block size
- *
- * Revision 1.9  2000/03/06 21:50:37  warmerda
- * added min/max support
- *
- * Revision 1.8  2000/03/06 02:18:13  warmerda
- * added overviews, and colour table
- *
- * Revision 1.6  1999/12/30 02:40:17  warmerda
- * Report driver used.
- *
- * Revision 1.5  1999/10/21 13:22:59  warmerda
- * Print band type symbolically rather than numerically.
- *
- * Revision 1.4  1999/10/01 14:45:14  warmerda
- * prettied up
- *
- * Revision 1.3  1999/03/02 21:12:01  warmerda
- * add DMS reporting of lat/long
- *
- * Revision 1.2  1999/01/11 15:27:59  warmerda
- * Add projection support
- *
- * Revision 1.1  1998/12/03 18:35:06  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal.h"
 #include "ogr_srs_api.h"
@@ -163,7 +33,7 @@
 #include "cpl_conv.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gdalinfo.c,v 1.43 2006/03/21 21:34:43 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdalinfo.c 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static int 
 GDALInfoReportCorner( GDALDatasetH hDataset, 
@@ -177,8 +47,8 @@ GDALInfoReportCorner( GDALDatasetH hDataset,
 void Usage()
 
 {
-    printf( "Usage: gdalinfo [--help-general] [-mm] [-nogcp] [-nomd] "
-            "datasetname\n");
+    printf( "Usage: gdalinfo [--help-general] [-mm] [-stats] [-nogcp] [-nomd]\n"
+            "                [-mdd domain]* datasetname\n" );
     exit( 1 );
 }
 
@@ -197,8 +67,9 @@ int main( int argc, char ** argv )
     char		**papszMetadata;
     int                 bComputeMinMax = FALSE, bSample = FALSE;
     int                 bShowGCPs = TRUE, bShowMetadata = TRUE ;
-    int                 bStats = FALSE;
+    int                 bStats = FALSE, iMDD;
     const char          *pszFilename = NULL;
+    char              **papszExtraMDDomains = NULL;
 
     GDALAllRegister();
 
@@ -221,6 +92,9 @@ int main( int argc, char ** argv )
             bShowGCPs = FALSE;
         else if( EQUAL(argv[i], "-nomd") )
             bShowMetadata = FALSE;
+        else if( EQUAL(argv[i], "-mdd") && i < argc-1 )
+            papszExtraMDDomains = CSLAddString( papszExtraMDDomains,
+                                                argv[++i] );
         else if( argv[i][0] == '-' )
             Usage();
         else if( pszFilename == NULL )
@@ -299,10 +173,10 @@ int main( int argc, char ** argv )
     {
         if( adfGeoTransform[2] == 0.0 && adfGeoTransform[4] == 0.0 )
         {
-            printf( "Origin = (%.6f,%.6f)\n",
+            printf( "Origin = (%.15f,%.15f)\n",
                     adfGeoTransform[0], adfGeoTransform[3] );
 
-            printf( "Pixel Size = (%.8f,%.8f)\n",
+            printf( "Pixel Size = (%.15f,%.15f)\n",
                     adfGeoTransform[1], adfGeoTransform[5] );
         }
         else
@@ -330,7 +204,7 @@ int main( int argc, char ** argv )
             psGCP = GDALGetGCPs( hDataset ) + i;
 
             printf( "GCP[%3d]: Id=%s, Info=%s\n"
-                    "          (%g,%g) -> (%g,%g,%g)\n", 
+                    "          (%.15g,%.15g) -> (%.15g,%.15g,%.15g)\n", 
                     i, psGCP->pszId, psGCP->pszInfo, 
                     psGCP->dfGCPPixel, psGCP->dfGCPLine, 
                     psGCP->dfGCPX, psGCP->dfGCPY, psGCP->dfGCPZ );
@@ -350,6 +224,19 @@ int main( int argc, char ** argv )
         }
     }
 
+    for( iMDD = 0; iMDD < CSLCount(papszExtraMDDomains); iMDD++ )
+    {
+        papszMetadata = GDALGetMetadata( hDataset, papszExtraMDDomains[iMDD] );
+        if( bShowMetadata && CSLCount(papszMetadata) > 0 )
+        {
+            printf( "Metadata (%s):\n", papszExtraMDDomains[iMDD]);
+            for( i = 0; papszMetadata[i] != NULL; i++ )
+            {
+                printf( "  %s\n", papszMetadata[i] );
+            }
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Report "IMAGE_STRUCTURE" metadata.                              */
 /* -------------------------------------------------------------------- */
@@ -376,6 +263,19 @@ int main( int argc, char ** argv )
         }
     }
 
+/* -------------------------------------------------------------------- */
+/*      Report geolocation.                                             */
+/* -------------------------------------------------------------------- */
+    papszMetadata = GDALGetMetadata( hDataset, "GEOLOCATION" );
+    if( CSLCount(papszMetadata) > 0 )
+    {
+        printf( "Geolocation:\n" );
+        for( i = 0; papszMetadata[i] != NULL; i++ )
+        {
+            printf( "  %s\n", papszMetadata[i] );
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Report corners.                                                 */
 /* -------------------------------------------------------------------- */
@@ -459,7 +359,7 @@ int main( int argc, char ** argv )
         dfNoData = GDALGetRasterNoDataValue( hBand, &bGotNodata );
         if( bGotNodata )
         {
-            printf( "  NoData Value=%g\n", dfNoData );
+            printf( "  NoData Value=%.15g\n", dfNoData );
         }
 
         if( GDALGetOverviewCount(hBand) > 0 )
@@ -506,7 +406,7 @@ int main( int argc, char ** argv )
 
         if( GDALGetRasterScale( hBand, &bSuccess ) != 1.0 
             || GDALGetRasterOffset( hBand, &bSuccess ) != 0.0 )
-            printf( "  Offset: %g,   Scale:%g\n",
+            printf( "  Offset: %.15g,   Scale:%.15g\n",
                     GDALGetRasterOffset( hBand, &bSuccess ),
                     GDALGetRasterScale( hBand, &bSuccess ) );
 
@@ -563,7 +463,8 @@ int main( int argc, char ** argv )
     }
 
     GDALClose( hDataset );
-
+    
+    CSLDestroy( papszExtraMDDomains );
     CSLDestroy( argv );
     
     GDALDumpOpenDatasets( stderr );
