@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalwarp.cpp,v 1.22 2006/04/25 14:28:33 fwarmerdam Exp $
+ * $Id: gdalwarp.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  High Performance Image Reprojector
  * Purpose:  Test program for high performance warper API.
@@ -26,54 +26,15 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gdalwarp.cpp,v $
- * Revision 1.22  2006/04/25 14:28:33  fwarmerdam
- * Check for no usable sources, avoid warning.
- *
- * Revision 1.21  2006/03/21 21:34:43  fwarmerdam
- * cleanup headers
- *
- * Revision 1.20  2006/01/05 19:51:10  fwarmerdam
- * fixed checking of targetsrs
- *
- * Revision 1.19  2005/12/19 20:20:00  fwarmerdam
- * preliminary support for multiple input files
- *
- * Revision 1.18  2005/09/13 01:57:31  fwarmerdam
- * Fixed usage message.
- *
- * Revision 1.17  2005/09/13 01:24:45  fwarmerdam
- * Set UNIFIED_SRC_NODATA by default for -srcnodata switch.
- *
- * Revision 1.16  2005/09/12 18:06:25  fwarmerdam
- * Added quotes around -srcnodata and -dstnodata in usage message.
- *
- * Revision 1.15  2004/12/26 16:14:53  fwarmerdam
- * added -tps flag
- *
- * Revision 1.14  2004/11/29 15:01:55  fwarmerdam
- * Fixed typo in printf as per Bug 691.
- *
- * Revision 1.13  2004/11/14 04:57:04  fwarmerdam
- * added -srcalpha switch, and automatic alpha detection
- *
- * Revision 1.12  2004/11/05 06:15:08  fwarmerdam
- * Don't double free the warpoptions array.
- *
- * Revision 1.11  2004/11/05 05:53:43  fwarmerdam
- * Avoid various memory leaks.
- *
- * Revision 1.10  2004/10/07 15:53:42  fwarmerdam
- * added preliminary alpha band support
- */
+ ****************************************************************************/
 
 #include "gdalwarper.h"
 #include "cpl_string.h"
-#include "ogr_srs_api.h"
+#include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: gdalwarp.cpp,v 1.22 2006/04/25 14:28:33 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdalwarp.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
+
+static CPLString InsertCenterLong( GDALDatasetH hDS, CPLString osWKT );
 
 static GDALDatasetH 
 GDALWarpCreateOutput( char **papszSrcFiles, const char *pszFilename, 
@@ -146,7 +107,7 @@ int main( int argc, char ** argv )
     char               *pszTargetSRS = NULL;
     char               *pszSourceSRS = NULL;
     char              **papszSrcFiles = NULL;
-    const char         *pszDstFilename = NULL;
+    char               *pszDstFilename = NULL;
     int                 bCreateOutput = FALSE, i, nOrder = 0;
     void               *hTransformArg, *hGenImgProjArg=NULL, *hApproxArg=NULL;
     char               **papszWarpOptions = NULL;
@@ -395,8 +356,11 @@ int main( int argc, char ** argv )
 
     for( iSrc = 0; papszSrcFiles[iSrc] != NULL; iSrc++ )
     {
-        const char *pszThisSourceSRS = pszSourceSRS;
+        CPLString osThisSourceSRS;
         GDALDatasetH hSrcDS;
+       
+        if( pszSourceSRS != NULL )
+            osThisSourceSRS = pszSourceSRS;
 
 /* -------------------------------------------------------------------- */
 /*      Open this file.                                                 */
@@ -408,21 +372,21 @@ int main( int argc, char ** argv )
 
         printf( "Processing input file %s.\n", papszSrcFiles[iSrc] );
 
-        if( pszThisSourceSRS == NULL )
+        if( strlen(osThisSourceSRS) == 0 )
         {
             if( GDALGetProjectionRef( hSrcDS ) != NULL 
                 && strlen(GDALGetProjectionRef( hSrcDS )) > 0 )
-                pszThisSourceSRS = GDALGetProjectionRef( hSrcDS );
+                osThisSourceSRS = GDALGetProjectionRef( hSrcDS );
             
             else if( GDALGetGCPProjection( hSrcDS ) != NULL
                      && strlen(GDALGetGCPProjection(hSrcDS)) > 0 
                      && GDALGetGCPCount( hSrcDS ) > 1 )
-                pszThisSourceSRS = GDALGetGCPProjection( hSrcDS );
+                osThisSourceSRS = GDALGetGCPProjection( hSrcDS );
             else
-                pszThisSourceSRS = "";
+                osThisSourceSRS = "";
 
             if( pszTargetSRS != NULL && strlen(pszTargetSRS) > 0 
-                && strlen(pszThisSourceSRS) == 0 )
+                && strlen(osThisSourceSRS) == 0 )
             {
                 fprintf( stderr, "A target coordinate system was specified, but there is no source coordinate\nsystem.  Consider using -s_srs option to provide a source coordinate system.\nOperation terminated.\n" );
                 exit( 1 );
@@ -430,7 +394,7 @@ int main( int argc, char ** argv )
         }
 
 /* -------------------------------------------------------------------- */
-/*      Open the first source dataset for some info.                    */
+/*      Do we have a source alpha band?                                 */
 /* -------------------------------------------------------------------- */
         if( GDALGetRasterColorInterpretation( 
                 GDALGetRasterBand(hSrcDS,GDALGetRasterCount(hSrcDS)) ) 
@@ -442,12 +406,23 @@ int main( int argc, char ** argv )
                     GDALGetRasterCount(hSrcDS) );
         }
 
+/* -------------------------------------------------------------------- */
+/*      If the source coordinate system is geographic, try and          */
+/*      insert a CENTER_LONG extension parameter on the GEOGCS to       */
+/*      handle wrapping better.                                         */
+/* -------------------------------------------------------------------- */
+        if( EQUALN(osThisSourceSRS.c_str(),"GEOGCS[",7) )
+        {
+            osThisSourceSRS = 
+                InsertCenterLong( hSrcDS, osThisSourceSRS );
+        }
+
 /* -------------------------------------------------------------------- */
 /*      Create a transformation object from the source to               */
 /*      destination coordinate system.                                  */
 /* -------------------------------------------------------------------- */
         hTransformArg = hGenImgProjArg = 
-            GDALCreateGenImgProjTransformer( hSrcDS, pszThisSourceSRS, 
+            GDALCreateGenImgProjTransformer( hSrcDS, osThisSourceSRS, 
                                              hDstDS, pszTargetSRS, 
                                              TRUE, 1000.0, nOrder );
         
@@ -664,8 +639,12 @@ int main( int argc, char ** argv )
 /*      Final Cleanup.                                                  */
 /* -------------------------------------------------------------------- */
     GDALClose( hDstDS );
-
+    
+    CPLFree( pszTargetSRS );
+    CPLFree( pszDstFilename );
     CSLDestroy( argv );
+    CSLDestroy( papszSrcFiles );
+    CSLDestroy( papszWarpOptions );
 
     GDALDumpOpenDatasets( stderr );
 
@@ -786,7 +765,7 @@ GDALWarpCreateOutput( char **papszSrcFiles, const char *pszFilename,
         }
 
         if( pszTargetSRS == NULL )
-            pszTargetSRS = CPLStrdup(pszThisSourceSRS);
+            pszTargetSRS = pszThisSourceSRS;
         
 /* -------------------------------------------------------------------- */
 /*      Create a transformation object from the source to               */
@@ -804,13 +783,14 @@ GDALWarpCreateOutput( char **papszSrcFiles, const char *pszFilename,
 /*      Get approximate output definition.                              */
 /* -------------------------------------------------------------------- */
         double adfThisGeoTransform[6];
+        double adfExtent[4];
         int    nThisPixels, nThisLines;
 
-        if( GDALSuggestedWarpOutput( hSrcDS, 
-                                     GDALGenImgProjTransform, hTransformArg, 
-                                     adfThisGeoTransform, 
-                                     &nThisPixels, &nThisLines )
-            != CE_None )
+        if( GDALSuggestedWarpOutput2( hSrcDS, 
+                                      GDALGenImgProjTransform, hTransformArg, 
+                                      adfThisGeoTransform, 
+                                      &nThisPixels, &nThisLines, 
+                                      adfExtent, 0 ) != CE_None )
             return NULL;
 
 /* -------------------------------------------------------------------- */
@@ -819,25 +799,19 @@ GDALWarpCreateOutput( char **papszSrcFiles, const char *pszFilename,
 /* -------------------------------------------------------------------- */
         if( dfWrkMaxX == 0.0 && dfWrkMinX == 0.0 )
         {
-            dfWrkMinX = adfThisGeoTransform[0];
-            dfWrkMaxX = adfThisGeoTransform[0] 
-                + adfThisGeoTransform[1] * nThisPixels;
-            dfWrkMaxY = adfThisGeoTransform[3];
-            dfWrkMinY = adfThisGeoTransform[3] 
-                + adfThisGeoTransform[5] * nThisLines;
+            dfWrkMinX = adfExtent[0];
+            dfWrkMaxX = adfExtent[2];
+            dfWrkMaxY = adfExtent[3];
+            dfWrkMinY = adfExtent[1];
             dfWrkResX = adfThisGeoTransform[1];
             dfWrkResY = ABS(adfThisGeoTransform[5]);
         }
         else
         {
-            dfWrkMinX = MIN(dfWrkMinX,adfThisGeoTransform[0]);
-            dfWrkMaxX = MAX(dfWrkMaxX,
-                         adfThisGeoTransform[0] 
-                         + adfThisGeoTransform[1] * nThisPixels);
-            dfWrkMaxY = MAX(dfWrkMaxY,adfThisGeoTransform[3]);
-            dfWrkMinY = MIN(dfWrkMinY,
-                         adfThisGeoTransform[3] 
-                         + adfThisGeoTransform[5] * nThisLines);
+            dfWrkMinX = MIN(dfWrkMinX,adfExtent[0]);
+            dfWrkMaxX = MAX(dfWrkMaxX,adfExtent[2]);
+            dfWrkMaxY = MAX(dfWrkMaxY,adfExtent[3]);
+            dfWrkMinY = MIN(dfWrkMinY,adfExtent[1]);
             dfWrkResX = MIN(dfWrkResX,adfThisGeoTransform[1]);
             dfWrkResY = MIN(dfWrkResY,ABS(adfThisGeoTransform[5]));
         }
@@ -900,10 +874,10 @@ GDALWarpCreateOutput( char **papszSrcFiles, const char *pszFilename,
     {
         if( dfMinX == 0.0 && dfMinY == 0.0 && dfMaxX == 0.0 && dfMaxY == 0.0 )
         {
-            dfMinX = adfDstGeoTransform[0];
-            dfMaxX = adfDstGeoTransform[0] + adfDstGeoTransform[1] * nPixels;
-            dfMaxY = adfDstGeoTransform[3];
-            dfMinY = adfDstGeoTransform[3] + adfDstGeoTransform[5] * nLines;
+            dfMinX = dfWrkMinX;
+            dfMaxX = dfWrkMaxX;
+            dfMaxY = dfWrkMaxY;
+            dfMinY = dfWrkMinY;
         }
 
         dfXRes = (dfMaxX - dfMinX) / nForcePixels;
@@ -918,6 +892,50 @@ GDALWarpCreateOutput( char **papszSrcFiles, const char *pszFilename,
         nLines = nForceLines;
     }
 
+    else if( nForcePixels != 0 )
+    {
+        if( dfMinX == 0.0 && dfMinY == 0.0 && dfMaxX == 0.0 && dfMaxY == 0.0 )
+        {
+            dfMinX = dfWrkMinX;
+            dfMaxX = dfWrkMaxX;
+            dfMaxY = dfWrkMaxY;
+            dfMinY = dfWrkMinY;
+        }
+
+        dfXRes = (dfMaxX - dfMinX) / nForcePixels;
+        dfYRes = dfXRes;
+
+        adfDstGeoTransform[0] = dfMinX;
+        adfDstGeoTransform[3] = dfMaxY;
+        adfDstGeoTransform[1] = dfXRes;
+        adfDstGeoTransform[5] = -dfYRes;
+
+        nPixels = nForcePixels;
+        nLines = (int) ((dfMaxY - dfMinY + (dfYRes/2.0)) / dfYRes);
+    }
+
+    else if( nForceLines != 0 )
+    {
+        if( dfMinX == 0.0 && dfMinY == 0.0 && dfMaxX == 0.0 && dfMaxY == 0.0 )
+        {
+            dfMinX = dfWrkMinX;
+            dfMaxX = dfWrkMaxX;
+            dfMaxY = dfWrkMaxY;
+            dfMinY = dfWrkMinY;
+        }
+
+        dfYRes = (dfMaxY - dfMinY) / nForceLines;
+        dfXRes = dfYRes;
+
+        adfDstGeoTransform[0] = dfMinX;
+        adfDstGeoTransform[3] = dfMaxY;
+        adfDstGeoTransform[1] = dfXRes;
+        adfDstGeoTransform[5] = -dfYRes;
+
+        nPixels = (int) ((dfMaxX - dfMinX + (dfXRes/2.0)) / dfXRes);
+        nLines = nForceLines;
+    }
+
     else if( dfMinX != 0.0 || dfMinY != 0.0 || dfMaxX != 0.0 || dfMaxY != 0.0 )
     {
         dfXRes = adfDstGeoTransform[1];
@@ -980,3 +998,81 @@ GDALWarpCreateOutput( char **papszSrcFiles, const char *pszFilename,
     return hDstDS;
 }
     
+/************************************************************************/
+/*                          InsertCenterLong()                          */
+/*                                                                      */
+/*      Insert a CENTER_LONG Extension entry on a GEOGCS to indicate    */
+/*      the center longitude of the dataset for wrapping purposes.      */
+/************************************************************************/
+
+static CPLString InsertCenterLong( GDALDatasetH hDS, CPLString osWKT )
+
+{								        
+    if( !EQUALN(osWKT.c_str(), "GEOGCS[", 7) )
+        return osWKT;
+    
+    if( strstr(osWKT,"EXTENSION[\"CENTER_LONG") != NULL )
+        return osWKT;
+
+/* -------------------------------------------------------------------- */
+/*      For now we only do this if we have a geotransform since         */
+/*      other forms require a bunch of extra work.                      */
+/* -------------------------------------------------------------------- */
+    double   adfGeoTransform[6];
+
+    if( GDALGetGeoTransform( hDS, adfGeoTransform ) != CE_None )
+        return osWKT;
+
+/* -------------------------------------------------------------------- */
+/*      Compute min/max longitude based on testing the four corners.    */
+/* -------------------------------------------------------------------- */
+    double dfMinLong, dfMaxLong;
+    int nXSize = GDALGetRasterXSize( hDS );
+    int nYSize = GDALGetRasterYSize( hDS );
+
+    dfMinLong = 
+        MIN(MIN(adfGeoTransform[0] + 0 * adfGeoTransform[1]
+                + 0 * adfGeoTransform[2],
+                adfGeoTransform[0] + nXSize * adfGeoTransform[1]
+                + 0 * adfGeoTransform[2]),
+            MIN(adfGeoTransform[0] + 0 * adfGeoTransform[1]
+                + nYSize * adfGeoTransform[2],
+                adfGeoTransform[0] + nXSize * adfGeoTransform[1]
+                + nYSize * adfGeoTransform[2]));
+    dfMaxLong = 
+        MAX(MAX(adfGeoTransform[0] + 0 * adfGeoTransform[1]
+                + 0 * adfGeoTransform[2],
+                adfGeoTransform[0] + nXSize * adfGeoTransform[1]
+                + 0 * adfGeoTransform[2]),
+            MAX(adfGeoTransform[0] + 0 * adfGeoTransform[1]
+                + nYSize * adfGeoTransform[2],
+                adfGeoTransform[0] + nXSize * adfGeoTransform[1]
+                + nYSize * adfGeoTransform[2]));
+
+    if( dfMaxLong - dfMinLong > 360.0 )
+        return osWKT;
+
+/* -------------------------------------------------------------------- */
+/*      Insert center long.                                             */
+/* -------------------------------------------------------------------- */
+    OGRSpatialReference oSRS( osWKT );
+    double dfCenterLong = (dfMaxLong + dfMinLong) / 2.0;
+    OGR_SRSNode *poExt;
+
+    poExt  = new OGR_SRSNode( "EXTENSION" );
+    poExt->AddChild( new OGR_SRSNode( "CENTER_LONG" ) );
+    poExt->AddChild( new OGR_SRSNode( CPLString().Printf("%g",dfCenterLong) ));
+    
+    oSRS.GetRoot()->AddChild( poExt );
+
+/* -------------------------------------------------------------------- */
+/*      Convert back to wkt.                                            */
+/* -------------------------------------------------------------------- */
+    char *pszWKT = NULL;
+    oSRS.exportToWkt( &pszWKT );
+    
+    osWKT = pszWKT;
+    CPLFree( pszWKT );
+
+    return osWKT;
+}
