@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalwarpsimple.c,v 1.1 2003/09/19 17:40:00 warmerda Exp $
+ * $Id: gdalwarpsimple.c 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Mapinfo Image Warper
  * Purpose:  Commandline program for doing a variety of image warps, including
@@ -27,43 +27,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gdalwarpsimple.c,v $
- * Revision 1.1  2003/09/19 17:40:00  warmerda
- * New
- *
- * Revision 1.8  2003/05/20 18:34:56  warmerda
- * added error reporting if SRS import fails
- *
- * Revision 1.7  2003/03/18 17:33:00  warmerda
- * Copy colortable if there is one on the source file.
- *
- * Revision 1.6  2002/12/17 18:23:19  warmerda
- * copy GCP projection if the main projection isn't set meaningfully
- *
- * Revision 1.5  2002/12/13 04:57:15  warmerda
- * implementing remaining commandline options
- *
- * Revision 1.4  2002/12/09 16:08:57  warmerda
- * added approximating transformer
- *
- * Revision 1.3  2002/12/07 22:58:09  warmerda
- * pass initialization warp option
- *
- * Revision 1.2  2002/12/07 17:09:13  warmerda
- * added -order flag
- *
- * Revision 1.1  2002/12/07 16:13:38  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal_alg.h"
 #include "cpl_string.h"
 #include "ogr_srs_api.h"
 
-CPL_CVSID("$Id: gdalwarpsimple.c,v 1.1 2003/09/19 17:40:00 warmerda Exp $");
+CPL_CVSID("$Id: gdalwarpsimple.c 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static GDALDatasetH 
 GDALWarpCreateOutput( GDALDatasetH hSrcDS, const char *pszFilename, 
