@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: aaigriddataset.cpp,v 1.34 2006/03/03 04:01:52 fwarmerdam Exp $
+ * $Id: aaigriddataset.cpp 11224 2007-04-08 14:41:07Z mloskot $
  *
  * Project:  GDAL
  * Purpose:  Implements Arc/Info ASCII Grid Format.
@@ -25,128 +25,22 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- *****************************************************************************
- *
- * $Log: aaigriddataset.cpp,v $
- * Revision 1.34  2006/03/03 04:01:52  fwarmerdam
- * Added support for DX and DY instead of CELLSIZE as apparently is produced
- * by Golden Surfer ascii grids.
- * http://bugs.mapwindow.org/show_bug.cgi?id=88
- *
- * Revision 1.33  2005/12/01 04:50:12  fwarmerdam
- * Fixed dependency on poOpenInfo->fp.
- *
- * Revision 1.32  2005/12/01 04:26:15  fwarmerdam
- * Use large file API.
- *
- * Revision 1.31  2005/08/23 16:32:23  fwarmerdam
- * Fixed to support tabs for white space too.
- *
- * Revision 1.30  2005/05/05 14:01:36  fwarmerdam
- * PAM Enable
- *
- * Revision 1.29  2005/04/15 18:34:35  fwarmerdam
- * Added AREA_OR_POINT metadata.
- *
- * Revision 1.28  2005/02/08 18:15:34  fwarmerdam
- * Removed tail recursion in IReadBlock() to avoid stack overflows.
- *
- * Revision 1.27  2005/02/08 17:06:27  fwarmerdam
- * Fixed up Delete() method to avoid error if there is no .prj.
- *
- * Revision 1.26  2003/12/02 18:01:09  warmerda
- * Rewrote line reading function to avoid calls to CPLReadLine() since
- * some files have all the data for the whole file in one long line.
- *
- * Revision 1.25  2003/12/02 17:06:11  warmerda
- * Write out integers as integers.
- *
- * Revision 1.24  2003/07/08 21:12:07  warmerda
- * avoid warnings
- *
- * Revision 1.23  2003/05/27 17:34:22  warmerda
- * fixed problem with scanlines split over multiple text lines
- *
- * Revision 1.22  2003/05/02 16:06:36  dron
- * Memory leak fixed.
- *
- * Revision 1.21  2003/04/02 19:05:03  dron
- * Fixes for large file support in Windows environment.
- *
- * Revision 1.20  2003/03/27 19:52:58  dron
- * Implemented Delete() method and large file support.
- *
- * Revision 1.19  2003/02/06 04:55:35  warmerda
- * use default georef info if bounds missing
- *
- * Revision 1.18  2002/11/23 18:54:17  warmerda
- * added CREATIONDATATYPES metadata for drivers
- *
- * Revision 1.17  2002/10/02 13:10:16  warmerda
- * Fixed bug in setting of Y offset derived from yllcenter,  was off 1 pixel.
- * As per GRASS RT bug https://intevation.de/rt/webrt?serial_num=1332.
- *
- * Revision 1.16  2002/09/04 06:50:36  warmerda
- * avoid static driver pointers
- *
- * Revision 1.15  2002/07/09 21:06:54  warmerda
- * fixed free of SrcDS pszProjection in CreateCopy
- *
- * Revision 1.14  2002/06/15 00:07:23  aubin
- * mods to enable 64bit file i/o
- *
- * Revision 1.13  2002/06/14 12:28:41  dron
- * No data handling at writing.
- *
- * Revision 1.12  2002/06/13 13:48:28  dron
- * Added writing of .prj files for Arc/Info 8
- *
- * Revision 1.11  2002/06/12 21:12:24  warmerda
- * update to metadata based driver info
- *
- * Revision 1.10  2002/06/11 13:13:02  dron
- * Write support implemented with CreateCopy() function
- *
- * Revision 1.9  2002/06/04 17:37:23  dron
- * Header records may follow in any order now.
- *
- * Revision 1.8  2001/11/20 14:52:26  warmerda
- * Fix from Markus for center of pixel positioning.
- *
- * Revision 1.7  2001/11/11 23:50:59  warmerda
- * added required class keyword to friend declarations
- *
- * Revision 1.6  2001/11/06 14:34:22  warmerda
- * Fixed bug in YLLCENTER handling.  Added case for alternate line ordering.
- *
- * Revision 1.5  2001/07/18 04:51:56  warmerda
- * added CPL_CVSID
- *
- * Revision 1.4  2001/03/24 20:58:30  warmerda
- * Fixed typo.
- *
- * Revision 1.3  2001/03/13 19:39:03  warmerda
- * Added help link
- *
- * Revision 1.2  2001/03/12 15:35:05  warmerda
- * Fixed prj file naming.
- *
- * Revision 1.1  2001/03/12 15:15:30  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal_pam.h"
 #include <ctype.h>
 #include "cpl_string.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: aaigriddataset.cpp,v 1.34 2006/03/03 04:01:52 fwarmerdam Exp $");
+CPL_CVSID("$Id: aaigriddataset.cpp 11224 2007-04-08 14:41:07Z mloskot $");
 
 CPL_C_START
 void    GDALRegister_AAIGrid(void);
 CPL_C_END
 
+static const char*OSR_GDS( char **papszNV, const char * pszField, 
+                           const char *pszDefaultValue );
+
 /************************************************************************/
 /* ==================================================================== */
 /*                              AAIGDataset                             */
@@ -565,6 +459,7 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Open file with large file API.                                  */
 /* -------------------------------------------------------------------- */
+
     poDS->fp = VSIFOpenL( poOpenInfo->pszFilename, "r" );
     if( poDS->fp == NULL )
     {
@@ -572,7 +467,7 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
                   "VSIFOpenL(%s) failed unexpectedly.", 
                   poOpenInfo->pszFilename );
         return NULL;
-    }
+    } 
 
 /* -------------------------------------------------------------------- */
 /*      Find the start of real data.                                    */
@@ -587,23 +482,52 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
                       "Couldn't find data values in ASCII Grid file.\n" );
             return NULL;                        
         }
-
+        
         if( poOpenInfo->pabyHeader[i-1] == '\n' 
             || poOpenInfo->pabyHeader[i-2] == '\n' )
         {
             if( !isalpha(poOpenInfo->pabyHeader[i]) )
             {
                 nStartOfData = i;
-                if( strstr((const char *)poOpenInfo->pabyHeader+i,".") != NULL)
-                    eDataType = GDT_Float32;
-                else
-                    eDataType = GDT_Int16;
 
+				/* Beginning of real data found. */
                 break;
             }
         }
     }
 
+/* -------------------------------------------------------------------- */
+/*      Recognize the type of data.										*/
+/* -------------------------------------------------------------------- */
+
+    CPLAssert( NULL != poDS->fp );
+
+    /* Default value type. */
+    eDataType = GDT_Int16; 
+
+    /* Allocate 100K chunk + 1 extra byte for NULL character. */
+    const size_t nChunkSize = 1024 * 100;
+    GByte* pabyChunk = (GByte *) CPLCalloc( nChunkSize + 1, sizeof(GByte) );
+    pabyChunk[nChunkSize] = '\0';
+
+    VSIFSeekL( poDS->fp, nStartOfData, SEEK_SET );
+	
+    /* Scan for dot in subsequent chunks of data. */
+    while( !VSIFEofL( poDS->fp) )
+    {
+        VSIFReadL( pabyChunk, sizeof(GByte), nChunkSize, poDS->fp );
+        CPLAssert( pabyChunk[nChunkSize] == '\0' );
+        
+        if( strchr( (const char *)pabyChunk, '.' ) != NULL )
+        {
+            eDataType = GDT_Float32;
+            break;
+        }
+    }
+    
+    /* Deallocate chunk. */
+    VSIFree( pabyChunk );
+
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
@@ -620,14 +544,39 @@ GDALDataset *AAIGDataset::Open( GDALOpenInfo * poOpenInfo )
     pszBasename = CPLStrdup(CPLGetBasename(poOpenInfo->pszFilename));
 
     pszPrjFilename = CPLFormFilename( pszDirname, pszBasename, "prj" );
-    if( VSIStatL( pszPrjFilename, &sStatBuf ) == 0 )
+    int nRet = VSIStatL( pszPrjFilename, &sStatBuf );
+
+#ifndef WIN32
+    if( nRet != 0 )
+    {
+        pszPrjFilename = CPLFormFilename( pszDirname, pszBasename, "PRJ" );
+        nRet = VSIStatL( pszPrjFilename, &sStatBuf );
+    }
+#endif
+
+    if( nRet == 0 )
     {
         OGRSpatialReference     oSRS;
 
         poDS->papszPrj = CSLLoad( pszPrjFilename );
 
+        CPLDebug( "AAIGrid", "Loaded SRS from %s", pszPrjFilename );
+
         if( oSRS.importFromESRI( poDS->papszPrj ) == OGRERR_NONE )
         {
+            // If geographic values are in seconds, we must transform. 
+            // Is there a code for minutes too? 
+            if( oSRS.IsGeographic() 
+                && EQUAL(OSR_GDS( poDS->papszPrj, "Units", ""), "DS") )
+            {
+                poDS->adfGeoTransform[0] /= 3600.0;
+                poDS->adfGeoTransform[1] /= 3600.0;
+                poDS->adfGeoTransform[2] /= 3600.0;
+                poDS->adfGeoTransform[3] /= 3600.0;
+                poDS->adfGeoTransform[4] /= 3600.0;
+                poDS->adfGeoTransform[5] /= 3600.0;
+            }
+
             CPLFree( poDS->pszProjection );
             oSRS.exportToWkt( &(poDS->pszProjection) );
         }
@@ -714,10 +663,14 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
     double      adfGeoTransform[6];
     char        szHeader[2000];
+    const char *pszForceCellsize = 
+        CSLFetchNameValue( papszOptions, "FORCE_CELLSIZE" );
 
     poSrcDS->GetGeoTransform( adfGeoTransform );
 
-    if( ABS(adfGeoTransform[1]+adfGeoTransform[5]) < 0.0000001 )
+    if( ABS(adfGeoTransform[1]+adfGeoTransform[5]) < 0.0000001 
+        || ABS(adfGeoTransform[1]-adfGeoTransform[5]) < 0.0000001 
+        || (pszForceCellsize && CSLTestBoolean(pszForceCellsize)) )
         sprintf( szHeader, 
                  "ncols        %d\n" 
                  "nrows        %d\n"
@@ -729,6 +682,14 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                  adfGeoTransform[3]- nYSize * adfGeoTransform[1],
                  adfGeoTransform[1] );
     else
+    {
+        if( pszForceCellsize == NULL )
+            CPLError( CE_Warning, CPLE_AppDefined, 
+                      "Producing a Golden Surfer style file with DX and DY instead\n"
+                      "of CELLSIZE since the input pixels are non-square.  Use the\n"
+                      "FORCE_CELLSIZE=TRUE creation option to force use of DX for\n"
+                      "even though this will be distorted.  Most ASCII Grid readers\n"
+                      "(ArcGIS included) do not support the DX and DY parameters.\n" );
         sprintf( szHeader, 
                  "ncols        %d\n" 
                  "nrows        %d\n"
@@ -741,6 +702,7 @@ AAIGCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                  adfGeoTransform[3]- nYSize * adfGeoTransform[1],
                  adfGeoTransform[1],
                  fabs(adfGeoTransform[5]) );
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Handle nodata (optionally).                                     */
@@ -904,6 +866,43 @@ CPLErr AAIGDataset::Delete( const char *pszFilename )
     return Remove( pszFilename, TRUE );
 }
 
+/************************************************************************/
+/*                              OSR_GDS()                               */
+/************************************************************************/
+
+static const char*OSR_GDS( char **papszNV, const char * pszField, 
+                           const char *pszDefaultValue )
+
+{
+    int         iLine;
+
+    if( papszNV == NULL || papszNV[0] == NULL )
+        return pszDefaultValue;
+
+    for( iLine = 0; 
+         papszNV[iLine] != NULL && 
+             !EQUALN(papszNV[iLine],pszField,strlen(pszField));
+         iLine++ ) {}
+
+    if( papszNV[iLine] == NULL )
+        return pszDefaultValue;
+    else
+    {
+        static char     szResult[80];
+        char    **papszTokens;
+        
+        papszTokens = CSLTokenizeString(papszNV[iLine]);
+
+        if( CSLCount(papszTokens) > 1 )
+            strncpy( szResult, papszTokens[1], sizeof(szResult));
+        else
+            strncpy( szResult, pszDefaultValue, sizeof(szResult));
+        
+        CSLDestroy( papszTokens );
+        return szResult;
+    }
+}
+
 /************************************************************************/
 /*                        GDALRegister_AAIGrid()                        */
 /************************************************************************/
@@ -926,6 +925,11 @@ void GDALRegister_AAIGrid()
         poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
                                    "Byte UInt16 Int16 Float32" );
 
+        poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST, 
+"<CreationOptionList>\n"
+"   <Option name='FORCE_CELLSIZE' type='boolean' description='Force use of CELLSIZE, default is FALSE.'/>\n"
+"</CreationOptionList>\n" );
+
         poDriver->pfnOpen = AAIGDataset::Open;
         poDriver->pfnCreateCopy = AAIGCreateCopy;
         poDriver->pfnDelete = AAIGDataset::Delete;
