@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: aigopen.c,v 1.18 2005/10/31 04:51:55 fwarmerdam Exp $
+ * $Id: aigopen.c 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Arc/Info Binary Grid Translator
  * Purpose:  Grid file access cover API for non-GDAL use.
@@ -25,68 +25,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: aigopen.c,v $
- * Revision 1.18  2005/10/31 04:51:55  fwarmerdam
- * upgraded to use large file API and GUInt32 for block offsets
- *
- * Revision 1.17  2004/01/29 20:48:10  warmerda
- * Make sure that if a bare "w001001.adf" is given to AIGOpen() it will
- * work.  The name is changed to ".".
- *
- * Revision 1.16  2003/07/08 15:36:10  warmerda
- * avoid warnings
- *
- * Revision 1.15  2002/11/11 18:29:03  warmerda
- * added AIGLLOpen() to support upper case names too
- *
- * Revision 1.14  2002/11/05 03:19:08  warmerda
- * avoid nodata remapping in gridlib, use GInt32 not GUInt for image data
- *
- * Revision 1.13  2002/10/31 03:08:47  warmerda
- * fixed email
- *
- * Revision 1.12  2002/05/14 21:38:57  warmerda
- * fixed up nPixel and nLines calculation to avoid roundoff error
- *
- * Revision 1.11  2001/07/18 04:51:56  warmerda
- * added CPL_CVSID
- *
- * Revision 1.10  2000/11/09 06:22:40  warmerda
- * save cover name
- *
- * Revision 1.9  2000/09/13 14:42:38  warmerda
- * Improved casting thanks for Ian Macleod (Geosoft).
- *
- * Revision 1.8  2000/02/18 14:47:12  warmerda
- * Avoid warnings.
- *
- * Revision 1.7  2000/02/18 04:55:02  warmerda
- * Set bHasWarned flag.
- *
- * Revision 1.6  1999/07/23 14:04:34  warmerda
- * Removed extra printf.
- *
- * Revision 1.5  1999/06/28 01:21:39  warmerda
- * added support for selecting .adf files as well as coverage dir
- *
- * Revision 1.4  1999/05/17 01:52:35  warmerda
- * Fixed argument type problem.
- *
- * Revision 1.3  1999/04/21 16:51:30  warmerda
- * fixed up floating point support
- *
- * Revision 1.2  1999/02/04 22:15:33  warmerda
- * fleshed out implementation
- *
- * Revision 1.1  1999/02/03 14:12:56  warmerda
- * New
- */
+ ****************************************************************************/
 
 #include "aigrid.h"
 
-CPL_CVSID("$Id: aigopen.c,v 1.18 2005/10/31 04:51:55 fwarmerdam Exp $");
+CPL_CVSID("$Id: aigopen.c 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                              AIGOpen()                               */
@@ -221,19 +164,41 @@ CPLErr AIGReadTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
     int		nBlockID;
     CPLErr	eErr;
 
+/* -------------------------------------------------------------------- */
+/*      validate block id.                                              */
+/* -------------------------------------------------------------------- */
     nBlockID = nBlockXOff + nBlockYOff * psInfo->nBlocksPerRow;
-    if( nBlockID < 0 || nBlockID >= psInfo->nBlocks )
+    if( nBlockID < 0 
+        || nBlockID >= psInfo->nBlocksPerRow * psInfo->nBlocksPerColumn )
     {
-        CPLAssert( FALSE );
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "Illegal block requested." );
         return CE_Failure;
     }
+
+    if( nBlockID >= psInfo->nBlocks )
+    {
+        int i;
+        CPLDebug( "AIG", 
+                  "Request legal block, but from beyond end of block map.\n"
+                  "Assuming all nodata." );
+        for( i = psInfo->nBlockXSize * psInfo->nBlockYSize - 1; i >= 0; i-- )
+            panData[i] = ESRI_GRID_NO_DATA;
+        return CE_None;
+    }
     
+/* -------------------------------------------------------------------- */
+/*      Read block.                                                     */
+/* -------------------------------------------------------------------- */
     eErr = AIGReadBlock( psInfo->fpGrid,
                          psInfo->panBlockOffset[nBlockID],
                          psInfo->panBlockSize[nBlockID],
                          psInfo->nBlockXSize, psInfo->nBlockYSize,
                          panData, psInfo->nCellType );
 
+/* -------------------------------------------------------------------- */
+/*      Apply floating point post-processing.                           */
+/* -------------------------------------------------------------------- */
     if( eErr == CE_None && psInfo->nCellType == AIG_CELLTYPE_FLOAT )
     {
         float	*pafData = (float *) panData;
@@ -259,19 +224,41 @@ CPLErr AIGReadFloatTile( AIGInfo_t * psInfo, int nBlockXOff, int nBlockYOff,
     int		nBlockID;
     CPLErr	eErr;
 
+/* -------------------------------------------------------------------- */
+/*      validate block id.                                              */
+/* -------------------------------------------------------------------- */
     nBlockID = nBlockXOff + nBlockYOff * psInfo->nBlocksPerRow;
-    if( nBlockID < 0 || nBlockID >= psInfo->nBlocks )
+    if( nBlockID < 0 
+        || nBlockID >= psInfo->nBlocksPerRow * psInfo->nBlocksPerColumn )
     {
-        CPLAssert( FALSE );
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "Illegal block requested." );
         return CE_Failure;
     }
+
+    if( nBlockID >= psInfo->nBlocks )
+    {
+        int i;
+        CPLDebug( "AIG", 
+                  "Request legal block, but from beyond end of block map.\n"
+                  "Assuming all nodata." );
+        for( i = psInfo->nBlockXSize * psInfo->nBlockYSize - 1; i >= 0; i-- )
+            pafData[i] = ESRI_GRID_FLOAT_NO_DATA;
+        return CE_None;
+    }
     
+/* -------------------------------------------------------------------- */
+/*      Read block.                                                     */
+/* -------------------------------------------------------------------- */
     eErr = AIGReadBlock( psInfo->fpGrid,
                          psInfo->panBlockOffset[nBlockID],
                          psInfo->panBlockSize[nBlockID],
                          psInfo->nBlockXSize, psInfo->nBlockYSize,
                          (GInt32 *) pafData, psInfo->nCellType );
 
+/* -------------------------------------------------------------------- */
+/*      Perform integer post processing.                                */
+/* -------------------------------------------------------------------- */
     if( eErr == CE_None && psInfo->nCellType == AIG_CELLTYPE_INT )
     {
         GUInt32	*panData = (GUInt32 *) pafData;
