@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: aitest.c,v 1.18 2006/04/03 23:29:43 fwarmerdam Exp $
+ * $Id: aitest.c 10475 2006-12-08 22:00:00Z fwarmerdam $
  *
  * Project:  Arc/Info Binary Grid Translator
  * Purpose:  Test mainline for examining AIGrid files.
@@ -29,7 +29,7 @@
 
 #include "aigrid.h"
 
-CPL_CVSID("$Id: aitest.c,v 1.18 2006/04/03 23:29:43 fwarmerdam Exp $");
+CPL_CVSID("$Id: aitest.c 10475 2006-12-08 22:00:00Z fwarmerdam $");
 
 /************************************************************************/
 /*                             DumpMagic()                              */
@@ -52,12 +52,12 @@ static void DumpMagic( AIGInfo_t * psInfo, int bVerbose )
         if( psInfo->panBlockSize[i] == 0 )
             continue;
 
-        VSIFSeek( psInfo->fpGrid, psInfo->panBlockOffset[i], SEEK_SET );
-        VSIFRead( abyBlockSize, 2, 1, psInfo->fpGrid );
+        VSIFSeekL( psInfo->fpGrid, psInfo->panBlockOffset[i], SEEK_SET );
+        VSIFReadL( abyBlockSize, 2, 1, psInfo->fpGrid );
 
         if( psInfo->nCellType == AIG_CELLTYPE_INT )
         {
-            VSIFRead( &byMagic, 1, 1, psInfo->fpGrid );
+            VSIFReadL( &byMagic, 1, 1, psInfo->fpGrid );
 
             if( byMagic != 0 && byMagic != 0x43 && byMagic != 0x04
                 && byMagic != 0x08 && byMagic != 0x10 && byMagic != 0xd7 
