@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gridlib.c,v 1.25 2005/11/24 20:25:32 fwarmerdam Exp $
+ * $Id: gridlib.c 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Arc/Info Binary Grid Translator
  * Purpose:  Grid file reading code.
@@ -25,92 +25,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gridlib.c,v $
- * Revision 1.25  2005/11/24 20:25:32  fwarmerdam
- * Test for hdf.adf and w001001x.adf in ::Open() method, so we don't
- * have to suppress error messages for AIGOpen().  Added error checking
- * in the AIGReadBlockIndex() to recognise corrupted files.
- *
- * Revision 1.24  2005/10/31 04:51:55  fwarmerdam
- * upgraded to use large file API and GUInt32 for block offsets
- *
- * Revision 1.23  2004/04/05 19:59:05  warmerda
- * Fixed 0x01 support, wasn't including nMin.
- *
- * Revision 1.22  2004/03/26 20:38:58  warmerda
- * Added support for 0x20 blocks as per bug 533
- *
- * Revision 1.21  2002/11/11 18:29:03  warmerda
- * added AIGLLOpen() to support upper case names too
- *
- * Revision 1.20  2002/11/05 03:19:08  warmerda
- * avoid nodata remapping in gridlib, use GInt32 not GUInt for image data
- *
- * Revision 1.19  2002/10/31 03:50:21  warmerda
- * fixed check in AIGProcessBlock
- *
- * Revision 1.18  2002/10/31 03:09:35  warmerda
- * added support for FF blocks
- *
- * Revision 1.17  2002/10/28 21:34:55  warmerda
- * made handling of corrupt files much more bulletproof
- *
- * Revision 1.16  2001/12/14 20:36:08  warmerda
- * fixed interpretation of the sign of short RMin values
- *
- * Revision 1.15  2001/07/18 04:51:56  warmerda
- * added CPL_CVSID
- *
- * Revision 1.14  2000/07/18 13:58:58  warmerda
- * no RTileType for float tiles
- *
- * Revision 1.13  2000/04/20 14:05:03  warmerda
- * added more raw float magic codes
- *
- * Revision 1.12  2000/03/15 02:00:52  warmerda
- * Added type 0x01 (untested).
- *
- * Revision 1.11  2000/02/18 14:47:25  warmerda
- * Avoid unused variable warnings.
- *
- * Revision 1.10  2000/02/18 14:41:00  warmerda
- * added support for 0xcf
- *
- * Revision 1.9  2000/02/18 05:05:16  warmerda
- * Do one time warning for unsupported tile types - don't blow assert
- *
- * Revision 1.8  2000/02/18 04:54:40  warmerda
- * Added support for 0xf0 (16bit run length encoded).
- * Fixed up handling of min value and no data values to be more generic.
- *
- * Revision 1.7  1999/08/13 03:28:12  warmerda
- * treat 0x3f to 0x43 as raw 32bit data
- *
- * Revision 1.6  1999/08/12 19:11:17  warmerda
- * corrected negative min handling, and no data values
- *
- * Revision 1.5  1999/08/09 16:30:44  warmerda
- * Modified 0x00 integer blocks to be treated as a constant min valued block.
- *
- * Revision 1.4  1999/04/21 16:51:30  warmerda
- * fixed up floating point support
- *
- * Revision 1.3  1999/03/02 21:10:28  warmerda
- * added some floating point support
- *
- * Revision 1.2  1999/02/04 22:15:33  warmerda
- * fleshed out implementation
- *
- * Revision 1.1  1999/02/03 14:12:56  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "aigrid.h"
 
-CPL_CVSID("$Id: gridlib.c,v 1.25 2005/11/24 20:25:32 fwarmerdam Exp $");
+CPL_CVSID("$Id: gridlib.c 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                    AIGProcessRaw32bitFloatBlock()                    */
