@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ecwcreatecopy.cpp,v 1.25 2006/04/28 04:18:18 fwarmerdam Exp $
+ * $Id: ecwcreatecopy.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GDAL ECW Driver
  * Purpose:  ECW CreateCopy method implementation.
@@ -25,86 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- *****************************************************************************
- *
- * $Log: ecwcreatecopy.cpp,v $
- * Revision 1.25  2006/04/28 04:18:18  fwarmerdam
- * Added support for writing GCPs in GeoTIFF box.
- *
- * Revision 1.24  2006/04/19 14:21:47  fwarmerdam
- * defer initialization till we need it, since shutdown is expensive
- *
- * Revision 1.23  2006/04/02 15:27:16  fwarmerdam
- * Fixed char* / const char * problem for VS8.
- *
- * Revision 1.22  2005/12/09 20:43:43  fwarmerdam
- * create the GeoTIFF box ourselves, we think we do it better
- *
- * Revision 1.21  2005/07/30 03:06:33  fwarmerdam
- * added LARGE_OK create option, and much improved error reporting
- *
- * Revision 1.20  2005/05/17 20:13:28  fwarmerdam
- * use gmljp2://xml format for uri
- *
- * Revision 1.19  2005/05/11 14:38:42  fwarmerdam
- * added dictionary support, updated to latest urn and gml.root-instance plans
- *
- * Revision 1.18  2005/05/03 21:11:47  fwarmerdam
- * upgraded to copy PAM information
- *
- * Revision 1.17  2005/04/12 03:58:34  fwarmerdam
- * turn ownership of fpVSIL over to vsiiostream
- *
- * Revision 1.16  2005/04/02 22:05:21  fwarmerdam
- * ifdef out some stuff on ECW_FW
- *
- * Revision 1.15  2005/04/02 21:26:34  fwarmerdam
- * preliminary support for creating GML coverage boxes
- *
- * Revision 1.14  2005/03/22 04:37:47  fwarmerdam
- * Integrated ECW_LARGE_OK option.
- *
- * Revision 1.13  2005/03/02 14:40:33  fwarmerdam
- * Fixed up TARGET validation.
- *
- * Revision 1.12  2005/02/25 17:01:47  fwarmerdam
- * initialize adfGeoTransform in case GetGeoTransform fails
- *
- * Revision 1.11  2005/02/25 16:44:39  fwarmerdam
- * modified WriteReadLine() to do a dataset level read
- *
- * Revision 1.10  2005/02/22 08:22:03  fwarmerdam
- * added support for capturing color interp on writable bands
- *
- * Revision 1.9  2005/02/17 15:32:27  fwarmerdam
- * Ensure that datatype is preserved on writable rasterbands. Otherwise
- * non-8bit support is broken.
- *
- * Revision 1.8  2005/02/08 04:50:37  fwarmerdam
- * added FlushCache method on writeable dataset
- *
- * Revision 1.7  2005/02/07 22:53:54  fwarmerdam
- * added preliminary Create support for JP2ECW driver
- *
- * Revision 1.6  2005/01/26 20:30:07  fwarmerdam
- * Change GetGDTProjDat to GetProjectionAndDatum().
- *
- * Revision 1.5  2005/01/15 07:48:41  fwarmerdam
- * dos to unix linefeeds
- *
- * Revision 1.4  2004/12/20 22:16:35  fwarmerdam
- * implement seperate ECW/JPEG2000 create entry points
- *
- * Revision 1.3  2004/12/20 05:03:00  fwarmerdam
- * implemented preliminary coordinate system export
- *
- * Revision 1.2  2004/12/10 22:13:58  fwarmerdam
- * convert to unix format
- *
- * Revision 1.1  2004/12/10 19:15:52  fwarmerdam
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal_pam.h"
 #include "ogr_spatialref.h"
@@ -112,8 +33,9 @@
 #include "cpl_conv.h"
 #include "vsiiostream.h"
 #include "jp2userbox.h"
+#include "gdaljp2metadata.h"
 
-CPL_CVSID("$Id: ecwcreatecopy.cpp,v 1.25 2006/04/28 04:18:18 fwarmerdam Exp $");
+CPL_CVSID("$Id: ecwcreatecopy.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 CPL_C_START
 CPLErr CPL_DLL GTIFMemBufFromWkt( const char *pszWKT, 
@@ -145,8 +67,7 @@ public:
     CPLErr  CloseDown();
 
     CPLErr  PrepareCoverageBox( const char *pszWKT, double *padfGeoTransform );
-    CPLErr  PrepareGeoTIFFBox( const char *pszWKT, double *padfGeoTransform,
-                               int nGCPCount, const GDAL_GCP *pasGCPList );
+    CPLErr  WriteJP2Box( GDALJP2Box * );
 
 #ifdef ECW_FW
     CNCSJP2File::CNCSJPXAssocBox  m_oGMLAssoc;
@@ -467,54 +388,26 @@ CPLErr  GDALECWCompressor::PrepareCoverageBox( const char *pszWKT,
 }
 
 /************************************************************************/
-/*                         PrepareGeoTIFFBox()                          */
+/*                            WriteJP2Box()                             */
 /************************************************************************/
 
-CPLErr GDALECWCompressor::PrepareGeoTIFFBox( const char *pszWKT,
-                                             double *padfGeoTransform,
-                                             int nGCPCount, 
-                                             const GDAL_GCP *pasGCPList )
+CPLErr GDALECWCompressor::WriteJP2Box( GDALJP2Box * poBox )
 
 {
-/* -------------------------------------------------------------------- */
-/*      Prepare the memory buffer containing the degenerate GeoTIFF     */
-/*      file.                                                           */
-/* -------------------------------------------------------------------- */
-    int         nGTBufSize = 0;
-    unsigned char *pabyGTBuf = NULL;
+    JP2UserBox  *poECWBox;
 
-    if( GTIFMemBufFromWkt( pszWKT, padfGeoTransform, nGCPCount, pasGCPList,
-                           &nGTBufSize, &pabyGTBuf ) != CE_None )
-        return CE_Failure;
-
-    if( nGTBufSize == 0 )
+    if( poBox == NULL )
         return CE_None;
 
-/* -------------------------------------------------------------------- */
-/*      Write to a box on the JP2 file.                                 */
-/* -------------------------------------------------------------------- */
-    JP2UserBox  *poGTData;
-    GByte *pabyUUIDBoxData;
-    int   nUUIDBoxDataLength;
+    poECWBox = new JP2UserBox();
+    memcpy( &(poECWBox->m_nTBox), poBox->GetType(), 4 );
 
-    static unsigned char msi_uuid2[16] =
-    {0xb1,0x4b,0xf8,0xbd,0x08,0x3d,0x4b,0x43,
-     0xa5,0xae,0x8c,0xd7,0xd5,0xa6,0xce,0x03}; 
+    poECWBox->SetData( poBox->GetDataLength(), 
+                       poBox->GetWritableData() );
 
-    poGTData = new JP2UserBox();
-    poGTData->m_nTBox = 'uuid';
+    AddBox( poECWBox ); // never freed?
 
-    nUUIDBoxDataLength = nGTBufSize + 16;
-    pabyUUIDBoxData = (GByte *) CPLMalloc(nUUIDBoxDataLength);
-    memcpy( pabyUUIDBoxData, msi_uuid2, 16 );
-    memcpy( pabyUUIDBoxData + 16, pabyGTBuf, nGTBufSize );
-
-    poGTData->SetData( nUUIDBoxDataLength, pabyUUIDBoxData );
-
-    CPLFree( pabyUUIDBoxData );
-    CPLFree( pabyGTBuf );
-
-    AddBox( poGTData ); // never freed?
+    delete poBox;
 
     return CE_None;
 }
@@ -923,8 +816,8 @@ CPLErr GDALECWCompressor::Initialize(
 
         pszOption = CSLFetchNameValue(papszOptions, "GEODATA_USAGE");
         if( pszOption == NULL )
-            // Default to supressing ECW SDK geodata, just use our geotiff.
-            SetGeodataUsage( JP2_GEODATA_USE_GML_ONLY );
+            // Default to supressing ECW SDK geodata, just use our own stuff.
+            SetGeodataUsage( JP2_GEODATA_USE_NONE );
         else if( EQUAL(pszOption,"NONE") )
             SetGeodataUsage( JP2_GEODATA_USE_NONE );
         else if( EQUAL(pszOption,"PCS_ONLY") )
@@ -1006,12 +899,19 @@ CPLErr GDALECWCompressor::Initialize(
               szProjection, szDatum );
 
 /* -------------------------------------------------------------------- */
-/*      Setup GML information.                                          */
+/*      Setup GML and GeoTIFF information.                              */
 /* -------------------------------------------------------------------- */
-    PrepareCoverageBox( pszWKT, padfGeoTransform );
-    PrepareGeoTIFFBox( pszWKT, padfGeoTransform,
-                       nGCPCount, pasGCPList );
-    
+    GDALJP2Metadata oJP2MD;
+
+    oJP2MD.SetProjection( pszWKT );
+    oJP2MD.SetGeoTransform( padfGeoTransform );
+    oJP2MD.SetGCPs( nGCPCount, pasGCPList );
+
+    if( CSLFetchBoolean( papszOptions, "GMLJP2", TRUE ) )
+        WriteJP2Box( oJP2MD.CreateGMLJP2(nXSize,nYSize) );
+    if( CSLFetchBoolean( papszOptions, "GeoJP2", TRUE ) )
+        WriteJP2Box( oJP2MD.CreateJP2GeoTIFF() );
+
 /* -------------------------------------------------------------------- */
 /*      Handle special case of a JPEG2000 data stream in another file.  */
 /* -------------------------------------------------------------------- */
