@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ecwdataset.cpp,v 1.62 2006/04/28 04:22:32 fwarmerdam Exp $
+ * $Id: ecwdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GDAL 
  * Purpose:  ECW (ERMapper Wavelet Compression Format) Driver
@@ -25,153 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- *****************************************************************************
- *
- * $Log: ecwdataset.cpp,v $
- * Revision 1.62  2006/04/28 04:22:32  fwarmerdam
- * make PAM fallbacks work properly
- *
- * Revision 1.61  2006/04/28 02:40:18  fwarmerdam
- * Fix some problems with GCP handling.
- *
- * Revision 1.60  2006/04/19 14:21:47  fwarmerdam
- * defer initialization till we need it, since shutdown is expensive
- *
- * Revision 1.59  2006/04/18 19:32:58  fwarmerdam
- * Don't predicate all cleanup in Deregister() on gpaspzCSLookup.
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=1156
- *
- * Revision 1.58  2006/04/13 15:20:27  fwarmerdam
- * Fixed bug in non-direct case with buffer/request different sizes
- * and more than one band.
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=1148
- *
- * Revision 1.57  2006/04/07 05:50:07  fwarmerdam
- * Modified to use GDALJP2Metadata to read all the various kinds of jpeg2000
- * georeferencing information, including world files.
- *
- * Revision 1.56  2006/02/13 23:00:55  fwarmerdam
- * cleanup up mutex when the driver is unloaded.
- *
- * Revision 1.55  2006/01/10 01:18:09  fwarmerdam
- * Avoid const casting warning.
- *
- * Revision 1.54  2005/12/21 15:07:56  fwarmerdam
- * better error
- *
- * Revision 1.53  2005/09/26 21:09:31  fwarmerdam
- * Avoid problems in deletion of the iostream.
- *
- * Revision 1.52  2005/07/30 03:05:56  fwarmerdam
- * Added LARGE_OK, frmt_jp2ecw.html link
- *
- * Revision 1.51  2005/07/05 22:09:23  fwarmerdam
- * added ParseMSIG() call
- *
- * Revision 1.50  2005/06/28 14:17:06  fwarmerdam
- * Added worldfile support (on read).
- *
- * Revision 1.49  2005/06/24 20:54:00  fwarmerdam
- * added explicit init/shutdown calls
- *
- * Revision 1.48  2005/05/23 06:56:10  fwarmerdam
- * disable pam support for subfile streams
- *
- * Revision 1.47  2005/05/03 21:12:14  fwarmerdam
- * use GDALJP2Metadata facilities for GML and GeoTIFF
- *
- * Revision 1.46  2005/04/25 18:40:16  fwarmerdam
- * It seems dynamic cast is bad karma on VC6 with /Ox
- *
- * Revision 1.45  2005/04/25 04:32:23  fwarmerdam
- * dynamic cast seems to throw exception sometimes
- *
- * Revision 1.44  2005/04/12 03:58:34  fwarmerdam
- * turn ownership of fpVSIL over to vsiiostream
- *
- * Revision 1.43  2005/04/11 13:56:27  fwarmerdam
- * Added a bit of reformatting.
- *
- * Revision 1.42  2005/04/11 13:52:50  fwarmerdam
- * added proper handling of iostream cleanup from David Carter
- *
- * Revision 1.41  2005/04/02 22:02:12  fwarmerdam
- * initialize variables
- *
- * Revision 1.40  2005/04/02 21:22:21  fwarmerdam
- * added GML and GeoTIFF box direct reading
- *
- * Revision 1.39  2005/03/03 17:05:40  fwarmerdam
- * fixed up serious problems with non-8bit data in ECWDataset::IRasterIO
- *
- * Revision 1.38  2005/02/25 15:17:00  fwarmerdam
- * Try to ensure that we do a multi-band adviseread in IRasterIO()
- * for line by line access.  This should dramatically improve pixel
- * interleaved line by line access speed.
- *
- * Revision 1.37  2005/02/25 14:54:37  fwarmerdam
- * Removed extra newline in debug statement.
- *
- * Revision 1.36  2005/02/24 15:12:04  fwarmerdam
- * Ensure IOStream and fpVSIL are properly cleaned up
- *
- * Revision 1.35  2005/02/10 04:49:53  fwarmerdam
- * added SetColorInterpretation
- *
- * Revision 1.34  2005/02/10 04:31:55  fwarmerdam
- * added support to preserve YCbCr images, added GetColorInterpretation()
- *
- * Revision 1.33  2005/02/08 04:51:21  fwarmerdam
- * fixed J2K_SUBFILE parsing error, add ECWCreateJPEG2000
- *
- * Revision 1.32  2005/01/26 20:04:44  fwarmerdam
- * Fixed more non8bit issues.
- *
- * Revision 1.31  2005/01/26 18:29:02  fwarmerdam
- * fixed broken non-8bit cases
- *
- * Revision 1.30  2004/12/20 22:17:15  fwarmerdam
- * last change also include split into ECW and JP2ECW drivers
- *
- * Revision 1.29  2004/12/20 22:15:41  fwarmerdam
- * avoid ECWRasterBand::AdviseRead() now calls on poGDS, limit debug msgs
- *
- * Revision 1.28  2004/12/20 06:40:32  fwarmerdam
- * fixed up for reading non-byte data
- *
- * Revision 1.27  2004/12/20 05:44:46  fwarmerdam
- * stripped out old full res mode - use TryWin and AdviseRead
- *
- * Revision 1.26  2004/12/20 05:02:43  fwarmerdam
- * moved list reader into ECWGetCSList()
- *
- * Revision 1.25  2004/12/17 14:50:30  fwarmerdam
- * implement AdviseRead API
- *
- * Revision 1.24  2004/12/10 19:18:05  fwarmerdam
- * Overhauled to use C++ ECW SDK classes.
- * Moved creation code into ecwcreatecopy.cpp
- * Support J2K_SUBFILE for reading "virtually".
- *
- * Revision 1.23  2004/06/24 04:53:40  warmerda
- * remove assert in IRasterIO(), seems to be unneeded
- *
- * Revision 1.22  2004/04/15 13:45:03  warmerda
- * Don't require the file to be open in ::Open() for .ecw files.  Large
- * files (>2GB) will generally not be open.
- *
- * Revision 1.21  2004/04/05 21:29:40  warmerda
- * improved logic to recognise jp2/jpc
- *
- * Revision 1.20  2004/04/05 20:49:34  warmerda
- * allow jpeg2000 files to be handled
- *
- * Revision 1.19  2004/03/30 17:33:51  warmerda
- * fixed last fix for ecwp:
- *
- * Revision 1.18  2004/03/30 17:00:07  warmerda
- * Allow "ecwp:" urls to be opened.
- */
+ ****************************************************************************/
 
 #include "gdal_pam.h"
 #include "gdaljp2metadata.h"
@@ -184,7 +38,7 @@
 #include "ogr_api.h"
 #include "ogr_geometry.h"
 
-CPL_CVSID("$Id: ecwdataset.cpp,v 1.62 2006/04/28 04:22:32 fwarmerdam Exp $");
+CPL_CVSID("$Id: ecwdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 #ifdef FRMT_ecw
 
@@ -1644,6 +1498,12 @@ void ECWInitialize()
 
     NCSecwInit();
     bNCSInitialized = TRUE;
+
+    const char *pszEcwCacheSize = 
+        CPLGetConfigOption("GDAL_ECW_CACHE_MAXMEM",NULL);
+
+    if( pszEcwCacheSize != NULL )
+        NCSecwSetConfig(NCSCFG_CACHE_MAXMEM, atoi(pszEcwCacheSize) );
 }
 
 /************************************************************************/
@@ -1749,6 +1609,8 @@ void GDALRegister_JP2ECW()
 "   <Option name='PROJ' type='string' description='ERMapper Projection Name'/>"
 "   <Option name='DATUM' type='string' description='ERMapper Datum Name' />"
 "   <Option name='LARGE_OK' type='boolean' description='Enable compressing 500+MB files'/>"
+"   <Option name='GeoJP2' type='boolean' description='defaults to ON'/>"
+"   <Option name='GMLJP2' type='boolean' description='defaults to ON'/>"
 "   <Option name='PROFILE' type='string-select'>"
 "       <Value>BASELINE_0</Value>"
 "       <Value>BASELINE_1</Value>"
@@ -1782,3 +1644,4 @@ void GDALRegister_JP2ECW()
 
 
 
+
