@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: grass57dataset.cpp,v 1.5 2005/09/30 21:07:57 fwarmerdam Exp $
+ * $Id: grass57dataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GRASS Driver
  * Purpose:  Implement GRASS raster read/write support
@@ -28,26 +28,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: grass57dataset.cpp,v $
- * Revision 1.5  2005/09/30 21:07:57  fwarmerdam
- * applied patches from bug 822, add georef support
- *
- * Revision 1.4  2005/05/06 18:21:14  fwarmerdam
- * Applied patch from Radim.  Better logic to reset the region.
- *
- * Revision 1.3  2004/09/24 14:28:49  fwarmerdam
- * fixed some typos with floating point (from Radim)
- *
- * Revision 1.2  2004/09/22 20:39:15  fwarmerdam
- * some updates to reduce spurious error reports
- *
- * Revision 1.1  2004/09/13 18:18:26  fwarmerdam
- * Variant code from Radim that works with GRASS 5.7 libraries directly.
- * Currently coordinate system support is broken with this version.
- *
- */
+ ****************************************************************************/
 
 #include <stdlib.h>
 
@@ -55,13 +36,13 @@ extern "C" {
 #ifdef __cplusplus
 #define class _class
 #endif
-#include <imagery.h>
+#include <grass/imagery.h>
 #ifdef __cplusplus
 #undef class
 #endif
     
-#include <gprojects.h>
-#include <gis.h>
+#include <grass/gprojects.h>
+#include <grass/gis.h>
 }
 
 #include "gdal_priv.h"
@@ -70,7 +51,7 @@ extern "C" {
 
 #define GRASS_MAX_COLORS 100000  // what is the right value
 
-CPL_CVSID("$Id: grass57dataset.cpp,v 1.5 2005/09/30 21:07:57 fwarmerdam Exp $");
+CPL_CVSID("$Id: grass57dataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 CPL_C_START
 void	GDALRegister_GRASS(void);
@@ -406,7 +387,10 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 	 sNewWindow->ew_res != sOpenWindow.ew_res || sNewWindow->ns_res != sOpenWindow.ns_res ||
 	 sNewWindow->rows   != sOpenWindow.rows   || sNewWindow->cols   != sOpenWindow.cols )
     {
-        G_close_cell( hCell );
+	if( hCell >= 0 ) {
+            G_close_cell( hCell );
+	    hCell = -1;
+	}
 
 	/* Set window */
 	G_set_window( sNewWindow );
@@ -420,6 +404,7 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 	
 	if ( (hCell = G_open_cell_old( pszCellName, pszMapset)) < 0 ) {
 	    CPLError( CE_Warning, CPLE_AppDefined, "GRASS: Cannot open raster '%s'", pszCellName );
+            this->valid = false;
 	    return CE_Failure;
 	}
 
@@ -456,6 +441,8 @@ CPLErr GRASSRasterBand::ResetReading ( struct Cell_head *sNewWindow )
 CPLErr GRASSRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff, void * pImage )
 
 {
+    if ( ! this->valid ) return CE_Failure;
+
     // Reset window because IRasterIO could be previosly called
     if ( ResetReading ( &(((GRASSDataset *)poDS)->sCellInfo) ) != CE_None ) {
        return CE_Failure;
@@ -507,6 +494,8 @@ CPLErr GRASSRasterBand::IRasterIO ( GDALRWFlag eRWFlag,
     /* Calculate the region */
     struct Cell_head sWindow;
     struct Cell_head *psDsWindow;
+    
+    if ( ! this->valid ) return CE_Failure;
 
     psDsWindow = &(((GRASSDataset *)poDS)->sCellInfo);
     
@@ -523,7 +512,10 @@ CPLErr GRASSRasterBand::IRasterIO ( GDALRWFlag eRWFlag,
     /* Reset resolution */
     G_adjust_Cell_head ( &sWindow, 1, 1);
 
-    ResetReading ( &sWindow );
+    if ( ResetReading ( &sWindow ) != CE_None )
+    {
+        return CE_Failure;
+    }
     
     /* Read Data */
     CELL  *cbuf = NULL;
