@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gt_wkt_srs.cpp,v 1.63 2006/04/28 04:21:57 fwarmerdam Exp $
+ * $Id: gt_wkt_srs.cpp 10850 2007-02-26 18:35:59Z warmerdam $
  *
  * Project:  GeoTIFF Driver
  * Purpose:  Implements translation between GeoTIFF normalized projection
@@ -28,71 +28,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gt_wkt_srs.cpp,v $
- * Revision 1.63  2006/04/28 04:21:57  fwarmerdam
- * make sure pszInfo is dynamically allocated in gcps
- *
- * Revision 1.62  2006/04/21 12:31:29  fwarmerdam
- * Ensure all of adfParms[] is initialized per:
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=1164
- *
- * Revision 1.61  2006/04/05 00:06:14  fwarmerdam
- * fix contact info
- *
- * Revision 1.60  2005/12/22 04:14:59  fwarmerdam
- * Added special method to massage citations coming from Imagine.
- *
- * Revision 1.59  2005/09/13 23:55:00  fwarmerdam
- * fixed VSIUnlink prototype
- *
- * Revision 1.58  2005/09/12 17:06:28  fwarmerdam
- * avoid dependence on tif_vsi.h
- *
- * Revision 1.57  2005/09/12 16:56:44  fwarmerdam
- * fixup to avoid depending on cpl_vsi.h
- *
- * Revision 1.56  2005/09/12 00:28:28  fwarmerdam
- * use vsi memory io instead of memio module
- *
- * Revision 1.55  2005/06/08 18:07:35  fwarmerdam
- * Fixed for memory allocation mixup in GTIFGetOGISDefn() return value.
- *
- * Revision 1.54  2005/05/22 21:00:15  fwarmerdam
- * ensure false easting/northing written out in meters
- *
- * Revision 1.53  2005/03/15 16:04:47  fwarmerdam
- * Emit semimajor+semiminor if inv flattening is zero (sphere) instead of
- * semimajor + 0 inv flattening.
- *
- * Revision 1.52  2005/03/09 17:04:44  fwarmerdam
- * added CEA support
- *
- * Revision 1.51  2005/02/17 21:58:08  fwarmerdam
- * Avoid leak of pszGeogName.
- *
- * Revision 1.50  2005/02/07 13:30:30  dron
- * Memory leak removed.
- *
- * Revision 1.49  2004/10/18 21:12:44  fwarmerdam
- * Always emit angular units as degrees.
- *
- * Revision 1.48  2004/07/10 05:02:57  warmerda
- * Fixed improper projection parameters for false easting/northing for LCC.
- *
- * Revision 1.47  2004/04/29 19:58:43  warmerda
- * export GTIFGetOGISDefn, and GTIFSetFromOGISDefn
- *
- * Revision 1.46  2004/04/29 18:10:35  warmerda
- * try not to crash if GTIF is NULL
- *
- * Revision 1.45  2004/04/21 13:59:15  warmerda
- * try to preserve PROJCS and GEOGCS names in citations
- *
- * Revision 1.44  2004/03/18 09:58:07  dron
- * Use auxiliary functions from the libgeotiff instead of the ones from CPL.
- */
+ ****************************************************************************/
 
 #include "cpl_serv.h"
 #include "geo_tiffp.h"
@@ -103,10 +39,12 @@
 #include "ogr_spatialref.h"
 #include "gdal.h"
 #include "xtiffio.h"
+#include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gt_wkt_srs.cpp,v 1.63 2006/04/28 04:21:57 fwarmerdam Exp $");
+CPL_CVSID("$Id: gt_wkt_srs.cpp 10850 2007-02-26 18:35:59Z warmerdam $");
 
 CPL_C_START
+void GTiffOneTimeInit();
 int CPL_DLL VSIFCloseL( FILE * );
 int CPL_DLL VSIUnlink( const char * );
 FILE CPL_DLL *VSIFileFromMemBuffer( const char *pszFilename, 
@@ -226,11 +164,11 @@ static void WKTMassageDatum( char ** ppszDatum )
 /************************************************************************/
 
 /* For example:
-   GTCitationGeoKey (Ascii,215): "IMAGINE GeoTIFF Support\nCopyright 1991 - 2001 by ERDAS, Inc. All Rights Reserved\n@(#)$RCSfile: gt_wkt_srs.cpp,v $ $Revision: 1.63 $ $Date: 2006/04/28 04:21:57 $\nProjection Name = UTM\nUnits = meters\nGeoTIFF Units = meters"
+   GTCitationGeoKey (Ascii,215): "IMAGINE GeoTIFF Support\nCopyright 1991 - 2001 by ERDAS, Inc. All Rights Reserved\n@(#)$RCSfile$ $Revision: 10850 $ $Date: 2007-02-26 10:35:59 -0800 (Mon, 26 Feb 2007) $\nProjection Name = UTM\nUnits = meters\nGeoTIFF Units = meters"
 
-   GeogCitationGeoKey (Ascii,267): "IMAGINE GeoTIFF Support\nCopyright 1991 - 2001 by ERDAS, Inc. All Rights Reserved\n@(#)$RCSfile: gt_wkt_srs.cpp,v $ $Revision: 1.63 $ $Date: 2006/04/28 04:21:57 $\nUnable to match Ellipsoid (Datum) to a GeographicTypeGeoKey value\nEllipsoid = Clarke 1866\nDatum = NAD27 (CONUS)"
+   GeogCitationGeoKey (Ascii,267): "IMAGINE GeoTIFF Support\nCopyright 1991 - 2001 by ERDAS, Inc. All Rights Reserved\n@(#)$RCSfile$ $Revision: 10850 $ $Date: 2007-02-26 10:35:59 -0800 (Mon, 26 Feb 2007) $\nUnable to match Ellipsoid (Datum) to a GeographicTypeGeoKey value\nEllipsoid = Clarke 1866\nDatum = NAD27 (CONUS)"
 
-   PCSCitationGeoKey (Ascii,214): "IMAGINE GeoTIFF Support\nCopyright 1991 - 2001 by ERDAS, Inc. All Rights Reserved\n@(#)$RCSfile: gt_wkt_srs.cpp,v $ $Revision: 1.63 $ $Date: 2006/04/28 04:21:57 $\nUTM Zone 10N\nEllipsoid = Clarke 1866\nDatum = NAD27 (CONUS)"
+   PCSCitationGeoKey (Ascii,214): "IMAGINE GeoTIFF Support\nCopyright 1991 - 2001 by ERDAS, Inc. All Rights Reserved\n@(#)$RCSfile$ $Revision: 10850 $ $Date: 2007-02-26 10:35:59 -0800 (Mon, 26 Feb 2007) $\nUTM Zone 10N\nEllipsoid = Clarke 1866\nDatum = NAD27 (CONUS)"
  
 */
 
@@ -1629,12 +1567,14 @@ CPLErr GTIFWktFromMemBuf( int nSize, unsigned char *pabyBuffer,
     TIFF        *hTIFF;
     GTIF 	*hGTIF;
     GTIFDefn	sGTIFDefn;
-    const static char *pszFilename = "/vsimem/wkt_from_mem_buf.tif";
+    char        szFilename[100];
+
+    sprintf( szFilename, "/vsimem/wkt_from_mem_buf_%d.tif", CPLGetPID() );
 
 /* -------------------------------------------------------------------- */
 /*      Create a memory file from the buffer.                           */
 /* -------------------------------------------------------------------- */
-    FILE *fp = VSIFileFromMemBuffer( pszFilename, pabyBuffer, nSize, FALSE );
+    FILE *fp = VSIFileFromMemBuffer( szFilename, pabyBuffer, nSize, FALSE );
     if( fp == NULL )
         return CE_Failure;
     VSIFCloseL( fp );
@@ -1642,7 +1582,7 @@ CPLErr GTIFWktFromMemBuf( int nSize, unsigned char *pabyBuffer,
 /* -------------------------------------------------------------------- */
 /*      Initialize access to the memory geotiff structure.              */
 /* -------------------------------------------------------------------- */
-    hTIFF = VSI_TIFFOpen( pszFilename, "r" );
+    hTIFF = VSI_TIFFOpen( szFilename, "r" );
 
     if( hTIFF == NULL )
     {
@@ -1734,7 +1674,7 @@ CPLErr GTIFWktFromMemBuf( int nSize, unsigned char *pabyBuffer,
 /* -------------------------------------------------------------------- */
     XTIFFClose( hTIFF );
 
-    VSIUnlink( pszFilename );
+    VSIUnlink( szFilename );
 
     if( *ppszWKT == NULL )
         return CE_Failure;
@@ -1753,12 +1693,14 @@ CPLErr GTIFMemBufFromWkt( const char *pszWKT, const double *padfGeoTransform,
 {
     TIFF        *hTIFF;
     GTIF 	*hGTIF;
-    const static char *pszFilename = "/vsimem/wkt_from_mem_buf.tif";
+    char        szFilename[100];
+
+    sprintf( szFilename, "/vsimem/wkt_from_mem_buf_%d.tif", CPLGetPID() );
 
 /* -------------------------------------------------------------------- */
 /*      Initialize access to the memory geotiff structure.              */
 /* -------------------------------------------------------------------- */
-    hTIFF = VSI_TIFFOpen( pszFilename, "w" );
+    hTIFF = VSI_TIFFOpen( szFilename, "w" );
 
     if( hTIFF == NULL )
     {
@@ -1877,7 +1819,7 @@ CPLErr GTIFMemBufFromWkt( const char *pszWKT, const double *padfGeoTransform,
 /* -------------------------------------------------------------------- */
     GUIntBig nBigLength;
 
-    *ppabyBuffer = VSIGetMemFileBuffer( pszFilename, &nBigLength, TRUE );
+    *ppabyBuffer = VSIGetMemFileBuffer( szFilename, &nBigLength, TRUE );
     *pnSize = (int) nBigLength;
 
     return CE_None;
