@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hdf4dataset.cpp,v 1.32 2005/04/25 20:05:27 dron Exp $
+ * $Id: hdf4dataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Hierarchical Data Format Release 4 (HDF4)
  * Purpose:  HDF4 Datasets. Open HDF4 file, fetch metadata and list of
@@ -27,107 +27,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- * 
- * $Log: hdf4dataset.cpp,v $
- * Revision 1.32  2005/04/25 20:05:27  dron
- * Memory leak fixed.
- *
- * Revision 1.31  2005/04/19 12:21:03  dron
- * Meory leak fixed.
- *
- * Revision 1.30  2005/03/30 11:21:15  dron
- * Remove inline keyword from function definitions.
- *
- * Revision 1.29  2005/03/29 12:42:53  dron
- * Added AnyTypeToDouble() method.
- *
- * Revision 1.28  2004/09/08 17:55:13  dron
- * Few problems fixed.
- *
- * Revision 1.27  2004/06/19 21:37:31  dron
- * Use HDF-EOS library for appropriate datasets; major cpde rewrite.
- *
- * Revision 1.26  2003/11/07 15:48:01  dron
- * TranslateHDF4Attributes() improved, added GetDataTypeSize().
- *
- * Revision 1.25  2003/11/05 17:05:02  warmerda
- * Fixed type casting problem (bug 431).
- *
- * Revision 1.24  2003/06/26 20:42:31  dron
- * Support for Hyperion Level 1 data product.
- *
- * Revision 1.23  2003/06/25 08:26:18  dron
- * Support for Aster Level 1A/1B/2 products.
- *
- * Revision 1.22  2003/06/12 15:07:04  dron
- * Added support for SeaWiFS Level 3 Standard Mapped Image Products.
- *
- * Revision 1.21  2003/06/10 09:32:31  dron
- * Added support for MODIS Level 3 products.
- *
- * Revision 1.20  2003/05/21 14:11:43  dron
- * MODIS Level 1B earth-view (EV) product now supported.
- *
- * Revision 1.19  2003/03/01 15:54:43  dron
- * Significant improvements in HDF EOS metadata parsing.
- *
- * Revision 1.18  2003/02/27 14:28:55  dron
- * Fixes in HDF-EOS metadata parsing algorithm.
- *
- * Revision 1.17  2002/12/19 17:42:57  dron
- * Size of string buffer in TranslateHDF4EOSAttributes() fixed.
- *
- * Revision 1.16  2002/11/29 18:25:22  dron
- * MODIS determination improved.
- *
- * Revision 1.15  2002/11/13 06:43:15  warmerda
- * quote filename in case it contains a drive indicator
- *
- * Revision 1.14  2002/11/13 06:00:04  warmerda
- * avoid use of long long type as it doesn't exist perse on windows
- *
- * Revision 1.13  2002/11/08 18:29:04  dron
- * Added Create() method.
- *
- * Revision 1.12  2002/11/07 13:23:44  dron
- * Support for projection information writing.
- *
- * Revision 1.11  2002/11/06 15:47:14  dron
- * Added support for 3D datasets creation
- *
- * Revision 1.10  2002/10/25 14:28:54  dron
- * Initial support for HDF4 creation.
- *
- * Revision 1.9  2002/09/25 14:44:40  warmerda
- * Fixed iSubdatasetType initialization.
- *
- * Revision 1.8  2002/09/06 10:42:23  dron
- * Georeferencing for ASTER Level 1b datasets and ASTER DEMs.
- *
- * Revision 1.7  2002/09/04 06:50:37  warmerda
- * avoid static driver pointers
- *
- * Revision 1.6  2002/07/23 12:27:58  dron
- * General Raster Interface support added.
- *
- * Revision 1.5  2002/07/19 13:39:17  dron
- * Lists supported in HDF-EOS attributes.
- *
- * Revision 1.4  2002/07/17 16:24:31  dron
- * MODIS support improved a bit.
- *
- * Revision 1.3  2002/07/17 13:36:18  dron
- * <hdf.h> and <mfhdf.h> changed to "hdf.h" and "mfhdf.h".
- *
- * Revision 1.2  2002/07/16 17:51:10  warmerda
- * removed hdf/ from include statements
- *
- * Revision 1.1  2002/07/16 11:04:11  dron
- * New driver: HDF4 datasets. Initial version.
- *
- *
- */
+ ****************************************************************************/
 
 #include "hdf.h"
 #include "mfhdf.h"
@@ -138,7 +38,7 @@
 #include "cpl_string.h"
 #include "hdf4dataset.h"
 
-CPL_CVSID("$Id: hdf4dataset.cpp,v 1.32 2005/04/25 20:05:27 dron Exp $");
+CPL_CVSID("$Id: hdf4dataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 CPL_C_START
 void	GDALRegister_HDF4(void);
@@ -165,6 +65,7 @@ HDF4Dataset::HDF4Dataset()
     hGR = 0;
     papszGlobalMetadata = NULL;
     papszSubDatasets = NULL;
+    bIsHDFEOS = 0;
 }
 
 /************************************************************************/
@@ -680,12 +581,13 @@ CPLErr HDF4Dataset::ReadGlobalAttributes( int32 iHandler )
     if ( SDfileinfo( hSD, &nDatasets, &nAttributes ) != 0 )
 	return CE_Failure;
 
-    // Loop trough the all attributes
+    // Loop through the all attributes
     for ( iAttribute = 0; iAttribute < nAttributes; iAttribute++ )
     {
         // Get information about the attribute. Note that the first
         // parameter is an SD interface identifier.
         SDattrinfo( iHandler, iAttribute, szAttrName, &iNumType, &nValues );
+
         if ( EQUALN( szAttrName, "coremetadata.", 13 )    ||
 	     EQUALN( szAttrName, "archivemetadata.", 16 ) ||
 	     EQUALN( szAttrName, "productmetadata.", 16 ) ||
@@ -700,17 +602,26 @@ CPLErr HDF4Dataset::ReadGlobalAttributes( int32 iHandler )
 	     EQUALN( szAttrName, "etst_specific", 13 ) ||
 	     EQUALN( szAttrName, "level_1_carryover", 17 ) )
         {
+            bIsHDFEOS = 1;
             papszGlobalMetadata = TranslateHDF4EOSAttributes( iHandler,
 		iAttribute, nValues, papszGlobalMetadata );
         }
+
+        // Skip "StructMetadata.N" records. We will fetch information
+        // from them using HDF-EOS API
 	else if ( EQUALN( szAttrName, "structmetadata.", 15 ) )
+        {
+            bIsHDFEOS = 1;
             continue;
+        }
+
         else
         {
 	    papszGlobalMetadata = TranslateHDF4Attributes( iHandler,
 		iAttribute, szAttrName,	iNumType, nValues, papszGlobalMetadata );
         }
     }
+
     return CE_None;
 }
 
@@ -728,7 +639,11 @@ GDALDataset *HDF4Dataset::Open( GDALOpenInfo * poOpenInfo )
 
     // We have special routine in the HDF library for format checking!
     if ( !Hishdf(poOpenInfo->pszFilename) )
+    {
+        HEPclear();
+        HEshutdown();
 	return NULL;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Try opening the dataset.                                        */
@@ -831,7 +746,11 @@ GDALDataset *HDF4Dataset::Open( GDALOpenInfo * poOpenInfo )
     int32	aiDimSizes[MAX_VAR_DIMS];
     int32	iRank, iNumType, nAttrs;
 
-    if ( CSLFetchNameValue(poDS->papszGlobalMetadata, "HDFEOSVersion") )
+    // Sometimes "HDFEOSVersion" attribute is not defined and we will
+    // determine HDF-EOS datasets using other records
+    // (see ReadGlobalAttributes() method).
+    if ( poDS->bIsHDFEOS
+         || CSLFetchNameValue(poDS->papszGlobalMetadata, "HDFEOSVersion") )
     {
         int32   nSubDatasets, nStrBufSize;
 
@@ -920,10 +839,10 @@ GDALDataset *HDF4Dataset::Open( GDALOpenInfo * poOpenInfo )
                     sprintf( szTemp, "SUBDATASET_%d_NAME", nCount + 1 );
 	            // We will use the field index as an identificator.
                     poDS->papszSubDatasets =
-                        CSLSetNameValue(poDS->papszSubDatasets, szTemp,
-                                CPLSPrintf( "HDF4_EOS:EOS_SWATH:\"%s\":%s:%s",
-                                            poOpenInfo->pszFilename,
-                                            papszSwaths[i], papszFields[j]));
+                        CSLSetNameValue( poDS->papszSubDatasets, szTemp,
+                                CPLSPrintf("HDF4_EOS:EOS_SWATH:\"%s\":%s:%s",
+                                           poOpenInfo->pszFilename,
+                                           papszSwaths[i], papszFields[j]) );
 
                     sprintf( szTemp, "SUBDATASET_%d_DESC", nCount + 1 );
                     pszString = SPrintArray( GDT_UInt32, aiDimSizes,
@@ -931,8 +850,8 @@ GDALDataset *HDF4Dataset::Open( GDALOpenInfo * poOpenInfo )
                     poDS->papszSubDatasets =
                         CSLSetNameValue( poDS->papszSubDatasets, szTemp,
                                          CPLSPrintf( "[%s] %s (%s)", pszString,
-                                            papszFields[j],
-                                            poDS->GetDataTypeName(iNumType) ) );
+                                         papszFields[j],
+                                         poDS->GetDataTypeName(iNumType) ) );
                     CPLFree( pszString );
                 }
 
