@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hfaentry.cpp,v 1.13 2005/08/19 02:14:11 fwarmerdam Exp $
+ * $Id: hfaentry.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Erdas Imagine (.img) Translator
  * Purpose:  Implementation of the HFAEntry class for reading and relating
@@ -32,56 +32,12 @@
  *
  * Implementation of the HFAEntry class.
  *
- * $Log: hfaentry.cpp,v $
- * Revision 1.13  2005/08/19 02:14:11  fwarmerdam
- * bug 857: add ability to set layer names
- *
- * Revision 1.12  2005/05/10 00:55:46  fwarmerdam
- * Added GetFieldCount method
- *
- * Revision 1.11  2005/02/22 21:34:47  fwarmerdam
- * added GetBigIntField method
- *
- * Revision 1.10  2004/10/26 17:49:31  fwarmerdam
- * Changed GetNamedChild() so that if there are multiple children with a
- * search for a given child node will be applied to each till a match is found.
- * This resolves the problem reading the histogram values for a file that
- * also has a color table preceeding it and they are both called
- * Descriptor_Table.
- *
- * Revision 1.9  2003/07/29 10:08:24  dron
- * Fixed definition for GetIntField() method.
- *
- * Revision 1.8  2003/04/22 19:40:36  warmerda
- * fixed email address
- *
- * Revision 1.7  2001/07/18 04:51:57  warmerda
- * added CPL_CVSID
- *
- * Revision 1.6  2001/01/03 16:20:10  warmerda
- * Converted to large file API
- *
- * Revision 1.5  2000/10/12 19:30:32  warmerda
- * substantially improved write support
- *
- * Revision 1.4  2000/09/29 21:42:38  warmerda
- * preliminary write support implemented
- *
- * Revision 1.3  1999/01/22 17:37:30  warmerda
- * fixed types in GetFieldValue() calls
- *
- * Revision 1.2  1999/01/04 22:52:47  warmerda
- * field access working
- *
- * Revision 1.1  1999/01/04 05:28:12  warmerda
- * New
- *
  */
 
 #include "hfa_p.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: hfaentry.cpp,v 1.13 2005/08/19 02:14:11 fwarmerdam Exp $");
+CPL_CVSID("$Id: hfaentry.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                              HFAEntry()                              */
@@ -420,8 +376,8 @@ HFAEntry *HFAEntry::GetNamedChild( const char * pszName )
 /*                           GetFieldValue()                            */
 /************************************************************************/
         
-void *HFAEntry::GetFieldValue( const char * pszFieldPath,
-                               char chReqType )
+int HFAEntry::GetFieldValue( const char * pszFieldPath,
+                             char chReqType, void *pReqReturn )
 
 {
     HFAEntry	*poEntry = this;
@@ -433,7 +389,7 @@ void *HFAEntry::GetFieldValue( const char * pszFieldPath,
     {
         poEntry = GetNamedChild( pszFieldPath );
         if( poEntry == NULL )
-            return NULL;
+            return FALSE;
         
         pszFieldPath = strchr(pszFieldPath,':') + 1;
     }
@@ -444,18 +400,19 @@ void *HFAEntry::GetFieldValue( const char * pszFieldPath,
     LoadData();
 
     if( pabyData == NULL )
-        return NULL;
+        return FALSE;
     
     if( poType == NULL )
-        return NULL;
+        return FALSE;
 
 /* -------------------------------------------------------------------- */
 /*      Extract the instance information.                               */
 /* -------------------------------------------------------------------- */
 
+
     return( poType->ExtractInstValue( pszFieldPath,
                                       pabyData, nDataPos, nDataSize,
-                                      chReqType ) );
+                                      chReqType, pReqReturn ) );
 }
 
 /************************************************************************/
@@ -505,10 +462,9 @@ int HFAEntry::GetFieldCount( const char * pszFieldPath, CPLErr *peErr )
 GInt32 HFAEntry::GetIntField( const char * pszFieldPath, CPLErr *peErr )
 
 {
-    void	*pRetData;
+    GInt32	nIntValue;
 
-    pRetData = GetFieldValue( pszFieldPath, 'i' );
-    if( pRetData == NULL )
+    if( !GetFieldValue( pszFieldPath, 'i', &nIntValue ) )
     {
         if( peErr != NULL )
             *peErr = CE_Failure;
@@ -520,7 +476,7 @@ GInt32 HFAEntry::GetIntField( const char * pszFieldPath, CPLErr *peErr )
         if( peErr != NULL )
             *peErr = CE_None;
 
-        return *((GInt32 *) pRetData);
+        return nIntValue;
     }
 }
 
@@ -559,10 +515,9 @@ GIntBig HFAEntry::GetBigIntField( const char *pszFieldPath, CPLErr *peErr )
 double HFAEntry::GetDoubleField( const char * pszFieldPath, CPLErr *peErr )
 
 {
-    void	*pRetData;
+    double dfDoubleValue;
 
-    pRetData = GetFieldValue( pszFieldPath, 'd' );
-    if( pRetData == NULL )
+    if( !GetFieldValue( pszFieldPath, 'd', &dfDoubleValue ) )
     {
         if( peErr != NULL )
             *peErr = CE_Failure;
@@ -574,7 +529,7 @@ double HFAEntry::GetDoubleField( const char * pszFieldPath, CPLErr *peErr )
         if( peErr != NULL )
             *peErr = CE_None;
 
-        return *((double *) pRetData);
+        return dfDoubleValue;
     }
 }
 
@@ -585,10 +540,9 @@ double HFAEntry::GetDoubleField( const char * pszFieldPath, CPLErr *peErr )
 const char *HFAEntry::GetStringField( const char * pszFieldPath, CPLErr *peErr)
 
 {
-    void	*pRetData;
+    char *pszResult = NULL;
 
-    pRetData = GetFieldValue( pszFieldPath, 's' );
-    if( pRetData == NULL )
+    if( !GetFieldValue( pszFieldPath, 's', &pszResult ) )
     {
         if( peErr != NULL )
             *peErr = CE_Failure;
@@ -600,7 +554,7 @@ const char *HFAEntry::GetStringField( const char * pszFieldPath, CPLErr *peErr)
         if( peErr != NULL )
             *peErr = CE_None;
 
-        return (char *) pRetData;
+        return pszResult;
     }
 }
 
