@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: hfafield.cpp,v 1.20 2006/04/03 04:33:16 fwarmerdam Exp $
+ * $Id: hfafield.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Erdas Imagine (.img) Translator
  * Purpose:  Implementation of the HFAField class for managing information
@@ -26,76 +26,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: hfafield.cpp,v $
- * Revision 1.20  2006/04/03 04:33:16  fwarmerdam
- * Report basedata type.  Support reading basedata as a 1D array.  Fix
- * bug in basedata reading ... wasn't skippig 2 byte code before data.
- *
- * Revision 1.19  2006/03/29 14:24:04  fwarmerdam
- * added preliminary nodata support (readonly)
- *
- * Revision 1.18  2005/10/02 15:14:48  fwarmerdam
- * fixed size for <8bit basedata items
- *
- * Revision 1.17  2005/09/28 19:38:07  fwarmerdam
- * Added partial support for inline defined types.
- *
- * Revision 1.16  2005/05/10 00:56:17  fwarmerdam
- * fixed bug with setting entries in an array (with count setting)
- *
- * Revision 1.15  2004/02/13 15:58:11  warmerda
- * Fixed serious bug with GetInstBytes() for BASEDATA * fields with
- * a count of zero.  Such as the Excluded field of most stats nodes!
- *
- * Revision 1.14  2003/12/08 19:09:34  warmerda
- * implemented DumpInstValue and GetInstBytes for basedata
- *
- * Revision 1.13  2003/05/21 15:35:05  warmerda
- * cleanup type conversion warnings
- *
- * Revision 1.12  2003/04/22 19:40:36  warmerda
- * fixed email address
- *
- * Revision 1.11  2001/07/18 04:51:57  warmerda
- * added CPL_CVSID
- *
- * Revision 1.10  2000/12/29 16:37:32  warmerda
- * Use GUInt32 for all file offsets
- *
- * Revision 1.9  2000/10/12 19:30:32  warmerda
- * substantially improved write support
- *
- * Revision 1.8  2000/09/29 21:42:38  warmerda
- * preliminary write support implemented
- *
- * Revision 1.7  1999/06/01 13:07:59  warmerda
- * added speed up for indexing into fixes size object arrays
- *
- * Revision 1.6  1999/02/15 19:06:18  warmerda
- * Disable warning on field offsets for Intergraph delivery
- *
- * Revision 1.5  1999/01/28 18:28:28  warmerda
- * minor simplification of code
- *
- * Revision 1.4  1999/01/28 18:03:07  warmerda
- * Fixed some byte swapping problems, and problems with accessing data from
- * the file that isn't on a word boundary.
- *
- * Revision 1.3  1999/01/22 19:23:11  warmerda
- * Fixed bug with offset into arrays of structures.
- *
- * Revision 1.2  1999/01/22 17:37:59  warmerda
- * Fixed up support for variable sizes, and arrays of variable sized objects
- *
- * Revision 1.1  1999/01/04 22:52:10  warmerda
- * New
- */
+ ****************************************************************************/
 
 #include "hfa_p.h"
 
-CPL_CVSID("$Id: hfafield.cpp,v 1.20 2006/04/03 04:33:16 fwarmerdam Exp $");
+CPL_CVSID("$Id: hfafield.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 #define MAX_ENTRY_REPORT   16
                            
@@ -457,6 +392,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
         else
             nCount = nIndexValue+1;
 
+        if( (int) nCount + 8 > nDataSize )
+        {
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Attempt to extend field %s in node past end of data,\n"
+                      "not currently supported.",
+                      pszField );
+            return CE_Failure;
+        }
+
         nOffset = nCount;
         HFAStandard( 4, &nOffset );
         memcpy( pabyData, &nOffset, 4 );
@@ -492,6 +436,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
         else
             nBytesToCopy = nBytes;
 
+        if( nBytesToCopy > nDataSize )
+        {
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Attempt to extend field %s in node past end of data,\n"
+                      "not currently supported.",
+                      pszField );
+            return CE_Failure;
+        }
+
         memset( pabyData, 0, nBytesToCopy );
 
         if( pValue != NULL )
@@ -541,6 +494,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
     {
       case 'c':
       case 'C':
+        if( nIndexValue + 1 > nDataSize )
+        {
+            CPLError( CE_Failure, CPLE_AppDefined,
+                      "Attempt to extend field %s in node past end of data,\n"
+                      "not currently supported.",
+                      pszField );
+              return CE_Failure;
+        }
+
         if( chReqType == 's' )
             pabyData[nIndexValue] = ((char *) pValue)[0];
         else
@@ -565,6 +527,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
 
           unsigned short nNumber = (unsigned short) nIntValue;
 
+          if( nIndexValue*2 + 2 > nDataSize )
+          {
+              CPLError( CE_Failure, CPLE_AppDefined,
+                        "Attempt to extend field %s in node past end of data,\n"
+                        "not currently supported.",
+                        pszField );
+              return CE_Failure;
+          }
+
           HFAStandard( 2, &nNumber );
           memcpy( pabyData + nIndexValue*2, &nNumber, 2 );
       }
@@ -574,6 +545,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
       {
           short nNumber;
 
+          if( nIndexValue*2 + 2 > nDataSize )
+          {
+              CPLError( CE_Failure, CPLE_AppDefined,
+                        "Attempt to extend field %s in node past end of data,\n"
+                        "not currently supported.",
+                        pszField );
+              return CE_Failure;
+          }
+
           nNumber = (short) nIntValue;
           HFAStandard( 2, &nNumber );
           memcpy( pabyData + nIndexValue*2, &nNumber, 2 );
@@ -585,6 +565,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
       {
           GUInt32	nNumber = nIntValue;
 
+          if( nIndexValue*4 + 4 > nDataSize )
+          {
+              CPLError( CE_Failure, CPLE_AppDefined,
+                        "Attempt to extend field %s in node past end of data,\n"
+                        "not currently supported.",
+                        pszField );
+              return CE_Failure;
+          }
+
           HFAStandard( 4, &nNumber );
           memcpy( pabyData + nIndexValue*4, &nNumber, 4 );
       }
@@ -594,6 +583,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
       {
           GInt32	nNumber = nIntValue;
           
+          if( nIndexValue*4 + 4 > nDataSize )
+          {
+              CPLError( CE_Failure, CPLE_AppDefined,
+                        "Attempt to extend field %s in node past end of data,\n"
+                        "not currently supported.",
+                        pszField );
+              return CE_Failure;
+          }
+
           HFAStandard( 4, &nNumber );
           memcpy( pabyData + nIndexValue*4, &nNumber, 4 );
       }
@@ -603,6 +601,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
       {
           float		fNumber = (float) dfDoubleValue;
           
+          if( nIndexValue*4 + 4 > nDataSize )
+          {
+              CPLError( CE_Failure, CPLE_AppDefined,
+                        "Attempt to extend field %s in node past end of data,\n"
+                        "not currently supported.",
+                        pszField );
+              return CE_Failure;
+          }
+
           HFAStandard( 4, &fNumber );
           memcpy( pabyData + nIndexValue*4, &fNumber, 4 );
       }
@@ -612,6 +619,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
       {
           double	dfNumber = dfDoubleValue;
           
+          if( nIndexValue*8 + 8 > nDataSize )
+          {
+              CPLError( CE_Failure, CPLE_AppDefined,
+                        "Attempt to extend field %s in node past end of data,\n"
+                        "not currently supported.",
+                        pszField );
+              return CE_Failure;
+          }
+
           HFAStandard( 8, &dfNumber );
           memcpy( pabyData + nIndexValue*8, &dfNumber, 8 );
       }
@@ -667,15 +683,15 @@ HFAField::SetInstValue( const char * pszField, int nIndexValue,
 /*      substructure.                                                   */
 /************************************************************************/
 
-void *
+int
 HFAField::ExtractInstValue( const char * pszField, int nIndexValue,
                            GByte *pabyData, GUInt32 nDataOffset, int nDataSize,
-                           char chReqType )
+                           char chReqType, void *pReqReturn )
 
 {
     char		*pszStringRet = NULL;
-    static int		nIntRet = 0;
-    static double	dfDoubleRet = 0.0;
+    int			nIntRet = 0;
+    double		dfDoubleRet = 0.0;
     int			nInstItemCount = GetInstCount( pabyData );
     GByte		*pabyRawData = NULL;
 
@@ -685,7 +701,7 @@ HFAField::ExtractInstValue( const char * pszField, int nIndexValue,
 /*      Eventually this will have to account for variable fields.       */
 /* -------------------------------------------------------------------- */
     if( nIndexValue < 0 || nIndexValue >= nInstItemCount )
-        return NULL;
+        return FALSE;
 
 /* -------------------------------------------------------------------- */
 /*	If this field contains a pointer, then we will adjust the	*/
@@ -719,7 +735,10 @@ HFAField::ExtractInstValue( const char * pszField, int nIndexValue,
 /*      handled as a special case.                                      */
 /* -------------------------------------------------------------------- */
     if( (chItemType == 'c' || chItemType == 'C') && chReqType == 's' )
-        return( pabyData );
+    {
+        *((GByte **)pReqReturn) = pabyData;
+        return( pabyData != NULL );
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Handle by type.                                                 */
@@ -820,7 +839,7 @@ HFAField::ExtractInstValue( const char * pszField, int nIndexValue,
           // We ignore the 2 byte objecttype value. 
 
           if( nIndexValue < 0 || nIndexValue >= nRows * nColumns )
-              return NULL;
+              return FALSE;
 
           pabyData += 12;
 
@@ -874,7 +893,7 @@ HFAField::ExtractInstValue( const char * pszField, int nIndexValue,
           else
           {
               CPLAssert( FALSE );
-              return NULL;
+              return FALSE;
           }
       }
       break;
@@ -908,13 +927,13 @@ HFAField::ExtractInstValue( const char * pszField, int nIndexValue,
                         ExtractInstValue( pszField, pabyRawData,
                                           nDataOffset + nExtraOffset,
                                           nDataSize - nExtraOffset,
-                                          chReqType ) );
+                                          chReqType, pReqReturn ) );
             }
         }
         break;
 
       default:
-        return NULL;
+        return FALSE;
         break;
     }
 
@@ -925,24 +944,34 @@ HFAField::ExtractInstValue( const char * pszField, int nIndexValue,
     {
         if( pszStringRet == NULL )
         {
-            static char	szNumber[28];
+            static char	szNumber[28]; // This is NOT threadsafe.
 
             sprintf( szNumber, "%d", nIntRet );
             pszStringRet = szNumber;
         }
         
-        return( pszStringRet );
+        *((char **) pReqReturn) = pszStringRet;
+        return( TRUE );
     }
     else if( chReqType == 'd' )
-        return( &dfDoubleRet );
+    {
+        *((double *)pReqReturn) = dfDoubleRet;
+        return( TRUE );
+    }
     else if( chReqType == 'i' )
-        return( &nIntRet );
+    {
+        *((int *) pReqReturn) = nIntRet;
+        return( TRUE );
+    }
     else if( chReqType == 'p' )
-        return( pabyRawData );
+    {
+        *((GByte **) pReqReturn) = pabyRawData;
+        return( TRUE );
+    }
     else
     {
         CPLAssert( FALSE );
-        return NULL;
+        return FALSE;
     }
 }
 
@@ -1068,10 +1097,9 @@ void HFAField::DumpInstValue( FILE *fpOut,
 /* -------------------------------------------------------------------- */
     if( (chItemType == 'c' || chItemType == 'C') && nEntries > 0 )
     {
-        pReturn = ExtractInstValue( NULL, 0,
-                                    pabyData, nDataOffset, nDataSize,
-                                    's' );
-        if( pReturn != NULL )
+        if( ExtractInstValue( NULL, 0,
+                              pabyData, nDataOffset, nDataSize,
+                              's', &pReturn ) )
             VSIFPrintf( fpOut, "%s%s = `%s'\n",
                         pszPrefix, pszFieldName,
                         (char *) pReturn );
@@ -1097,15 +1125,16 @@ void HFAField::DumpInstValue( FILE *fpOut,
         {
           case 'f':
           case 'd':
-            pReturn = ExtractInstValue( NULL, iEntry,
-                                        pabyData, nDataOffset, nDataSize,
-                                        'd' );
-            if( pReturn != NULL )
-                VSIFPrintf( fpOut, "%f\n",
-                            *((double *) pReturn) );
-            else
-                VSIFPrintf( fpOut, "(access failed)\n" );
-            break;
+          {
+              double  dfValue;
+              if( ExtractInstValue( NULL, iEntry,
+                                    pabyData, nDataOffset, nDataSize,
+                                    'd', &dfValue ) )
+                  VSIFPrintf( fpOut, "%f\n", dfValue );
+              else
+                  VSIFPrintf( fpOut, "(access failed)\n" );
+          }
+          break;
 
           case 'b':
           {
@@ -1126,22 +1155,18 @@ void HFAField::DumpInstValue( FILE *fpOut,
           break;
 
           case 'e':
-            pReturn = ExtractInstValue( NULL, iEntry,
-                                        pabyData, nDataOffset, nDataSize,
-                                        's' );
-            if( pReturn != NULL )
-                VSIFPrintf( fpOut, "%s\n",
-                            (char *) pReturn );
+            if( ExtractInstValue( NULL, iEntry,
+                                  pabyData, nDataOffset, nDataSize,
+                                  's', &pReturn ) )
+                VSIFPrintf( fpOut, "%s\n", (char *) pReturn );
             else
                 VSIFPrintf( fpOut, "(access failed)\n" );
             break;
 
           case 'o':
-            pReturn = ExtractInstValue( NULL, iEntry,
-                                        pabyData, nDataOffset, nDataSize,
-                                        'p' );
-
-            if( pReturn == NULL )
+            if( !ExtractInstValue( NULL, iEntry,
+                                   pabyData, nDataOffset, nDataSize,
+                                   'p', &pReturn ) )
             {
                 VSIFPrintf( fpOut, "(access failed)\n" );
             }
@@ -1164,15 +1189,17 @@ void HFAField::DumpInstValue( FILE *fpOut,
             break;
 
           default:
-            pReturn = ExtractInstValue( NULL, iEntry,
-                                        pabyData, nDataOffset, nDataSize,
-                                        'i' );
-            if( pReturn != NULL )
-                VSIFPrintf( fpOut, "%d\n",
-                            *((int *) pReturn) );
-            else
-                VSIFPrintf( fpOut, "(access failed)\n" );
-            break;
+          {
+              GInt32 nIntValue;
+
+              if( ExtractInstValue( NULL, iEntry,
+                                    pabyData, nDataOffset, nDataSize,
+                                    'i', &nIntValue ) )
+                  VSIFPrintf( fpOut, "%d\n", nIntValue );
+              else
+                  VSIFPrintf( fpOut, "(access failed)\n" );
+          }
+          break;
         }
     }
 
