@@ -23,19 +23,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ilwiscoordinatesystem.cpp,v $
- * Revision 1.6  2006/04/04 03:34:52  fwarmerdam
- * fixed up header after review of log history
- *
- * Revision 1.5  2005/09/14 13:37:18  dron
- * Avoid warnings.
- *
- * Revision 1.4  2005/08/04 15:26:53  fwarmerdam
- * added log headers
- *
- */
+ ****************************************************************************/
 #include "cpl_conv.h"
 #include "ilwisdataset.h"
 
@@ -895,6 +883,21 @@ void WriteLambertConformalConic(string csFileName, OGRSpatialReference oSRS)
 				WriteElement("Projection", ILW_Scale_Factor, csFileName, "1.0000000000"); 
 }
 
+void WriteLambertConformalConic2SP(string csFileName, OGRSpatialReference oSRS)
+{
+	      WriteProjectionName(csFileName, "Lambert Conformal Conic");
+				WriteFalseEastNorth(csFileName, oSRS);
+				WriteElement("Projection", ILW_Central_Meridian, csFileName, 
+											oSRS.GetNormProjParm(SRS_PP_CENTRAL_MERIDIAN, 0.0));
+				WriteElement("Projection", ILW_Central_Parallel, csFileName, 
+											oSRS.GetNormProjParm(SRS_PP_LATITUDE_OF_ORIGIN, 0.0));
+				WriteElement("Projection", ILW_Scale_Factor, csFileName, "1.0000000000"); 
+				WriteElement("Projection", ILW_Standard_Parallel_1, csFileName, 
+											oSRS.GetNormProjParm(SRS_PP_STANDARD_PARALLEL_1, 0.0)); 
+				WriteElement("Projection", ILW_Standard_Parallel_2, csFileName, 
+											oSRS.GetNormProjParm(SRS_PP_STANDARD_PARALLEL_2, 0.0));
+}
+
 void WriteLambertAzimuthalEqualArea(string csFileName, OGRSpatialReference oSRS)
 {
 	      WriteProjectionName(csFileName, "Lambert Azimuthal EqualArea");
@@ -1017,8 +1020,8 @@ CPLErr ILWISDataset::WriteProjection()
 		char		*pszP = pszProjection;
 		
 		string csFileName = CPLResetExtension(pszFileName, "csy" );
-		string pszBaseName = string(CPLStrdup( CPLGetBasename( pszFileName )));
-		string pszPath = string(CPLStrdup( CPLGetPath( pszFileName )));
+		string pszBaseName = string(CPLGetBasename( pszFileName ));
+		string pszPath = string(CPLGetPath( pszFileName ));
 		bool fProjection = ((strlen(pszProjection)>0) && (pszProjection != NULL));
 		if( fProjection && (oSRS.importFromWkt( &pszP ) == OGRERR_NONE) )
 		{
@@ -1136,6 +1139,14 @@ CPLErr ILWISDataset::WriteProjection()
     {
 				WriteLambertConformalConic(csFileName, oSRS);
 		}
+		else if( EQUAL(pszProjName,SRS_PT_LAMBERT_CONFORMAL_CONIC_1SP) )
+    {
+				WriteLambertConformalConic(csFileName, oSRS);
+		}
+		else if( EQUAL(pszProjName,SRS_PT_LAMBERT_CONFORMAL_CONIC_2SP) )
+    {
+				WriteLambertConformalConic2SP(csFileName, oSRS);
+		}
 		else if( EQUAL(pszProjName,SRS_PT_LAMBERT_AZIMUTHAL_EQUAL_AREA) )
     {
 				WriteLambertAzimuthalEqualArea(csFileName, oSRS);
