@@ -24,16 +24,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ilwisdataset.cpp,v $
- * Revision 1.15  2005/09/14 13:37:18  dron
- * Avoid warnings.
- *
- * Revision 1.14  2005/08/04 15:26:53  fwarmerdam
- * added log headers
- *
- */
+ ****************************************************************************/
 
 
 #include "ilwisdataset.h"
@@ -55,8 +46,8 @@ string TrimSpaces(const string& input)
     if ( input.empty()) 
         return string();
 
-    unsigned int iFirstNonSpace = input.find_first_not_of(' ');
-    unsigned int iFindLastSpace = input.find_last_not_of(' ');
+    int iFirstNonSpace = input.find_first_not_of(' ');
+    int iFindLastSpace = input.find_last_not_of(' ');
     if (iFirstNonSpace == string::npos || iFindLastSpace == string::npos)
         return string();
 
@@ -187,7 +178,7 @@ void IniFile::Load()
 
             if (s[0] == '[')
             {
-                unsigned int iLast = s.find_first_of(']');
+                int iLast = s.find_first_of(']');
                 if (iLast != string::npos)
                 {
                     section = s.substr(1, iLast - 1);
@@ -201,7 +192,7 @@ void IniFile::Load()
             s = GetLine(filIni); // fall through (no break)
           case FindKey:
           {
-              unsigned int iEqu = s.find_first_of('=');
+              int iEqu = s.find_first_of('=');
               if (iEqu != string::npos)
               {
                   key = s.substr(0, iEqu);
@@ -319,7 +310,7 @@ bool WriteElement(string sSection, string sEntry,
 static CPLErr GetRowCol(string str,int &Row, int &Col)
 {
     string delimStr = " ,;";
-    unsigned int iPos = str.find_first_of(delimStr);
+    int iPos = str.find_first_of(delimStr);
     if (iPos != string::npos)
     {
         Row = atoi(str.substr(0, iPos).c_str());
@@ -497,8 +488,8 @@ void ILWISDataset::CollectTransformCoef(string &pszRefName)
     if( (georef.length() != 0) && !EQUAL(georef.c_str(),"none"))
     {
         //Form the geo-referencing name
-        string pszBaseName = string(CPLStrdup( CPLGetBasename(georef.c_str()) ));
-        string pszPath = string(CPLStrdup(CPLGetPath( pszFileName )));
+        string pszBaseName = string(CPLGetBasename(georef.c_str()) );
+        string pszPath = string(CPLGetPath( pszFileName ));
         pszRefName = string(CPLFormFilename(pszPath.c_str(),
                                             pszBaseName.c_str(),"grf" ));
 
@@ -585,8 +576,8 @@ CPLErr ILWISDataset::WriteGeoReference()
 
             //Re-write the GeoRef property to raster ODF
             //Form band file name  
-            string sBaseName = string(CPLStrdup( CPLGetBasename(pszFileName) ));
-            string sPath = string(CPLStrdup(CPLGetPath(pszFileName)));
+            string sBaseName = string(CPLGetBasename(pszFileName) );
+            string sPath = string(CPLGetPath(pszFileName));
             if (nBands == 1) 
             {
                 WriteElement("Map", "GeoRef", pszFileName, sBaseName + ".grf");
@@ -693,8 +684,8 @@ GDALDataset *ILWISDataset::Open( GDALOpenInfo * poOpenInfo )
     int    iBandCount;
     string mapsize;
     string maptype = ReadElement("BaseMap", "Type", poOpenInfo->pszFilename);
-    string sBaseName = string(CPLStrdup( CPLGetBasename(poOpenInfo->pszFilename) ));
-    string sPath = string(CPLStrdup(CPLGetPath( poOpenInfo->pszFilename)));
+    string sBaseName = string(CPLGetBasename(poOpenInfo->pszFilename) );
+    string sPath = string(CPLGetPath( poOpenInfo->pszFilename));
 							
     //Verify whether it is a map list or a map
     if( EQUAL(ilwistype.c_str(),"MapList") )
@@ -709,8 +700,8 @@ GDALDataset *ILWISDataset::Open( GDALOpenInfo * poOpenInfo )
             char cBandName[45];
             sprintf( cBandName, "Map%d", iBand);
             string sBandName = ReadElement("MapList", string(cBandName), poOpenInfo->pszFilename);
-            string pszBandBaseName = string(CPLStrdup( CPLGetBasename(sBandName.c_str()) ));
-            string pszBandPath = string(CPLStrdup(CPLGetPath( sBandName.c_str())));
+            string pszBandBaseName = string(CPLGetBasename(sBandName.c_str()) );
+            string pszBandPath = string(CPLGetPath( sBandName.c_str()));
             if ( 0 == pszBandPath.length() )
             { 
                 sBandName = string(CPLFormFilename(sPath.c_str(),
@@ -806,8 +797,8 @@ GDALDataset *ILWISDataset::Open( GDALOpenInfo * poOpenInfo )
             if( !(EQUALN( csy.c_str(), "latlon.csy", 10 )) && 
                 !(EQUALN( csy.c_str(), "LatlonWGS84.csy", 15 )))			
             {
-                string pszBaseName = string(CPLStrdup( CPLGetBasename(csy.c_str()) ));
-                string pszPath = string(CPLStrdup(CPLGetPath( poDS->pszFileName )));
+                string pszBaseName = string(CPLGetBasename(csy.c_str()) );
+                string pszPath = string(CPLGetPath( poDS->pszFileName ));
                 csy = string(CPLFormFilename(pszPath.c_str(),
                                              pszBaseName.c_str(),"csy" ));
                 pszProj = ReadElement("CoordSystem", "Type", csy);
@@ -891,8 +882,8 @@ GDALDataset *ILWISDataset::Create(const char* pszFilename,
 		else if( EQUAL(sStoreType.c_str(),"Real") || EQUAL(sStoreType.c_str(),"float"))
 			stepsize = 0;
 
-    string pszBaseName = string(CPLStrdup( CPLGetBasename( pszFilename )));
-    string pszPath = string(CPLStrdup( CPLGetPath( pszFilename )));
+    string pszBaseName = string(CPLGetBasename( pszFilename ));
+    string pszPath = string(CPLGetPath( pszFilename ));
 		
 /* -------------------------------------------------------------------- */
 /*      Write out object definition file for each band                    */
@@ -1041,8 +1032,8 @@ ILWISDataset::CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 
     if( poDS == NULL )
         return NULL;
-    string pszBaseName = string(CPLStrdup( CPLGetBasename( pszFilename )));
-    string pszPath = string(CPLStrdup( CPLGetPath( pszFilename )));
+    string pszBaseName = string(CPLGetBasename( pszFilename ));
+    string pszPath = string(CPLGetPath( pszFilename ));
 				
 /* -------------------------------------------------------------------- */
 /*  Copy and geo-transform and projection information.                                    */
@@ -1244,9 +1235,9 @@ ILWISRasterBand::ILWISRasterBand( ILWISDataset *poDS, int nBand )
         char cBandName[45];
         sprintf( cBandName, "Map%d", nBand-1);
         sBandName = ReadElement("MapList", string(cBandName), string(poDS->pszFileName));
-        string sInputPath = string(CPLStrdup(CPLGetPath( poDS->pszFileName)));	
-        string sBandPath = string(CPLStrdup(CPLGetPath( sBandName.c_str())));
-        string sBandBaseName = string(CPLStrdup(CPLGetBasename( sBandName.c_str())));
+        string sInputPath = string(CPLGetPath( poDS->pszFileName));	
+        string sBandPath = string(CPLGetPath( sBandName.c_str()));
+        string sBandBaseName = string(CPLGetBasename( sBandName.c_str()));
         if ( 0==sBandPath.length() )
             sBandName = string(CPLFormFilename(sInputPath.c_str(),sBandBaseName.c_str(),"mpr" ));		
         else
@@ -1310,9 +1301,9 @@ void ILWISRasterBand::ILWISOpen(  string pszFileName)
 CPLErr ILWISRasterBand::GetILWISInfo(string pszFileName)
 {
     string domName = ReadElement("BaseMap", "Domain", pszFileName.c_str());
-    string pszBaseName = string(CPLStrdup( CPLGetBasename( domName.c_str()) ));
+    string pszBaseName = string(CPLGetBasename( domName.c_str() ));
     transform(pszBaseName.begin(), pszBaseName.end(), pszBaseName.begin(), tolower);
-    string pszPath = string(CPLStrdup( CPLGetPath( pszFileName.c_str()) ));
+    string pszPath = string(CPLGetPath( pszFileName.c_str() ));
 		
     if (GetStoreType(pszFileName, psInfo.stStoreType) != CE_None)
     {
@@ -1487,6 +1478,9 @@ CPLErr ILWISRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
             ((double *) pImage)[i] = ((double *) pBuffer)[i];
         break;
     }
+
+    CPLFree( pBuffer );
+
     return CE_None;
 }
 
@@ -1823,7 +1817,7 @@ void ValueRange::init(double rRaw0)
         else
             iRawUndef = 0;
     }
-    catch (exception*) {
+    catch (std::exception*) {
         st = stReal;
         _r0 = 0;
         _rStep = 0.0001;
