@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ddfmodule.cpp,v 1.16 2005/10/13 22:02:39 fwarmerdam Exp $
+ * $Id: ddfmodule.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  ISO 8211 Access
  * Purpose:  Implements the DDFModule class.
@@ -25,63 +25,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ddfmodule.cpp,v $
- * Revision 1.16  2005/10/13 22:02:39  fwarmerdam
- * dont add fields that fail to initialize
- *
- * Revision 1.15  2004/02/18 14:09:42  warmerda
- * doc fixups
- *
- * Revision 1.14  2003/07/03 15:38:46  warmerda
- * some write capabilities added
- *
- * Revision 1.13  2001/07/18 04:51:57  warmerda
- * added CPL_CVSID
- *
- * Revision 1.12  2000/09/19 14:08:51  warmerda
- * keep and report _extendedCharSet
- *
- * Revision 1.11  1999/11/18 19:03:04  warmerda
- * expanded tabs
- *
- * Revision 1.10  1999/11/18 19:02:38  warmerda
- * added failquietly to open
- *
- * Revision 1.9  1999/09/03 14:14:39  warmerda
- * fix cloning
- *
- * Revision 1.8  1999/09/02 03:10:01  warmerda
- * fixed subtle problem with rewinding modules with reusable headers
- *
- * Revision 1.7  1999/08/16 15:44:29  warmerda
- * Fixed bug in FindFieldDefn().
- *
- * Revision 1.6  1999/08/13 03:26:14  warmerda
- * added Rewind()
- *
- * Revision 1.5  1999/05/08 20:15:59  warmerda
- * added validity checking, and better cleanup on error
- *
- * Revision 1.4  1999/05/07 14:11:49  warmerda
- * added support for tracking record clones
- *
- * Revision 1.3  1999/05/06 15:39:45  warmerda
- * avoid redeclaring variable i
- *
- * Revision 1.2  1999/05/06 14:23:10  warmerda
- * added Close(), and minor optimizations
- *
- * Revision 1.1  1999/04/27 18:45:05  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "iso8211.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ddfmodule.cpp,v 1.16 2005/10/13 22:02:39 fwarmerdam Exp $");
+CPL_CVSID("$Id: ddfmodule.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                             DDFModule()                              */
@@ -152,7 +101,7 @@ void DDFModule::Close()
 /* -------------------------------------------------------------------- */
     if( fpDDF != NULL )
     {
-        VSIFClose( fpDDF );
+        VSIFCloseL( fpDDF );
         fpDDF = NULL;
     }
 
@@ -223,7 +172,7 @@ int DDFModule::Open( const char * pszFilename, int bFailQuietly )
 /* -------------------------------------------------------------------- */
 /*      Open the file.                                                  */
 /* -------------------------------------------------------------------- */
-    fpDDF = VSIFOpen( pszFilename, "rb" );
+    fpDDF = VSIFOpenL( pszFilename, "rb" );
 
     if( fpDDF == NULL )
     {
@@ -239,9 +188,9 @@ int DDFModule::Open( const char * pszFilename, int bFailQuietly )
 /* -------------------------------------------------------------------- */
     char        achLeader[nLeaderSize];
     
-    if( VSIFRead( achLeader, 1, nLeaderSize, fpDDF ) != nLeaderSize )
+    if( VSIFReadL( achLeader, 1, nLeaderSize, fpDDF ) != nLeaderSize )
     {
-        VSIFClose( fpDDF );
+        VSIFCloseL( fpDDF );
         fpDDF = NULL;
 
         if( !bFailQuietly )
@@ -307,7 +256,7 @@ int DDFModule::Open( const char * pszFilename, int bFailQuietly )
 /* -------------------------------------------------------------------- */
     if( !bValid )
     {
-        VSIFClose( fpDDF );
+        VSIFCloseL( fpDDF );
         fpDDF = NULL;
 
         if( !bFailQuietly )
@@ -326,7 +275,7 @@ int DDFModule::Open( const char * pszFilename, int bFailQuietly )
     pachRecord = (char *) CPLMalloc(_recLength);
     memcpy( pachRecord, achLeader, nLeaderSize );
 
-    if( VSIFRead( pachRecord+nLeaderSize, 1, _recLength-nLeaderSize, fpDDF )
+    if( VSIFReadL( pachRecord+nLeaderSize, 1, _recLength-nLeaderSize, fpDDF )
         != _recLength - nLeaderSize )
     {
         if( !bFailQuietly )
@@ -385,7 +334,7 @@ int DDFModule::Open( const char * pszFilename, int bFailQuietly )
 /*      Record the current file offset, the beginning of the first      */
 /*      data record.                                                    */
 /* -------------------------------------------------------------------- */
-    nFirstRecordOffset = VSIFTell( fpDDF );
+    nFirstRecordOffset = VSIFTellL( fpDDF );
     
     return TRUE;
 }
@@ -430,7 +379,7 @@ int DDFModule::Create( const char *pszFilename )
 /* -------------------------------------------------------------------- */
 /*      Create the file on disk.                                        */
 /* -------------------------------------------------------------------- */
-    fpDDF = VSIFOpen( pszFilename, "wb+" );
+    fpDDF = VSIFOpenL( pszFilename, "wb+" );
     if( fpDDF == NULL )
     {
         CPLError( CE_Failure, CPLE_OpenFailed, 
@@ -479,7 +428,7 @@ int DDFModule::Create( const char *pszFilename )
     sprintf( achLeader+21, "%1d", (int) _sizeFieldPos );
     achLeader[22] = '0';
     sprintf( achLeader+23, "%1d", (int) _sizeFieldTag );
-    VSIFWrite( achLeader, 24, 1, fpDDF );
+    VSIFWriteL( achLeader, 24, 1, fpDDF );
 
 /* -------------------------------------------------------------------- */
 /*      Write out directory entries.                                    */
@@ -498,11 +447,11 @@ int DDFModule::Create( const char *pszFilename )
                  "%04d", nOffset );
         nOffset += nLength;
 
-        VSIFWrite( achDirEntry, 11, 1, fpDDF );
+        VSIFWriteL( achDirEntry, 11, 1, fpDDF );
     }
 
     char chUT = DDF_FIELD_TERMINATOR;
-    VSIFWrite( &chUT, 1, 1, fpDDF );
+    VSIFWriteL( &chUT, 1, 1, fpDDF );
 
 /* -------------------------------------------------------------------- */
 /*      Write out the field descriptions themselves.                    */
@@ -513,7 +462,7 @@ int DDFModule::Create( const char *pszFilename )
         int nLength;
 
         papoFieldDefns[iField]->GenerateDDREntry( &pachData, &nLength );
-        VSIFWrite( pachData, nLength, 1, fpDDF );
+        VSIFWriteL( pachData, nLength, 1, fpDDF );
         CPLFree( pachData );
     }
     
@@ -751,7 +700,7 @@ void DDFModule::Rewind( long nOffset )
     if( fpDDF == NULL )
         return;
     
-    VSIFSeek( fpDDF, nOffset, SEEK_SET );
+    VSIFSeekL( fpDDF, nOffset, SEEK_SET );
 
     if( nOffset == nFirstRecordOffset && poRecord != NULL )
         poRecord->Clear();
