@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ddfrecord.cpp,v 1.29 2006/02/24 15:59:24 fwarmerdam Exp $
+ * $Id: ddfrecord.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  ISO 8211 Access
  * Purpose:  Implements the DDFRecord class.
@@ -25,104 +25,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ddfrecord.cpp,v $
- * Revision 1.29  2006/02/24 15:59:24  fwarmerdam
- * Added error checking on record buffer memory allocation.
- *
- * Revision 1.28  2005/10/16 04:09:20  fwarmerdam
- * Avoid warnings in sprintf()s.
- *
- * Revision 1.27  2004/06/24 00:35:59  warmerda
- * fixed bug in my last 'fix' for resizing records
- *
- * Revision 1.26  2004/06/23 19:03:59  warmerda
- * fixed some serious problems in ResizeField(), especially with shrinking
- *
- * Revision 1.25  2004/03/10 18:09:35  warmerda
- * Avoid warning on casting result of pointer subtraction.
- *
- * Revision 1.24  2004/02/18 14:09:42  warmerda
- * doc fixups
- *
- * Revision 1.23  2003/12/08 20:32:48  warmerda
- * Added some improved error checking in DDFRecord() on the leader values.
- * Try reading extra bytes to find field terminator if the last byte in
- * a record isn't the field terminator.
- *
- * Revision 1.22  2003/12/02 16:50:21  warmerda
- * fixed problems with writing some kinds of records
- *
- * Revision 1.21  2003/11/12 21:21:55  warmerda
- * fixed to include field terminators when creating fields
- *
- * Revision 1.20  2003/09/17 21:11:07  warmerda
- * try to create new default field instance when needed
- *
- * Revision 1.19  2003/09/15 20:46:53  warmerda
- * supressed some warnings in SetFieldRaw if field empty
- *
- * Revision 1.18  2003/09/11 19:56:35  warmerda
- * avoid warnings
- *
- * Revision 1.17  2003/09/03 20:36:26  warmerda
- * added subfield writing support
- *
- * Revision 1.16  2003/07/03 15:38:46  warmerda
- * some write capabilities added
- *
- * Revision 1.15  2002/08/08 12:39:18  warmerda
- * Added support for variable length records as per bugzilla bug 181.
- *
- * Revision 1.14  2001/08/30 21:08:19  warmerda
- * expand tabs
- *
- * Revision 1.13  2001/08/27 19:09:00  warmerda
- * added GetInstanceData() method on DDFField
- *
- * Revision 1.12  2001/08/24 19:41:19  warmerda
- * fixed cloning problems
- *
- * Revision 1.11  2001/08/24 16:30:55  warmerda
- * added DDFRecord update in place methods for S57 updating
- *
- * Revision 1.10  2001/07/18 04:51:57  warmerda
- * added CPL_CVSID
- *
- * Revision 1.9  2000/02/15 17:55:53  warmerda
- * Improved error message for corrupt DDF files.
- *
- * Revision 1.8  1999/11/18 19:03:04  warmerda
- * expanded tabs
- *
- * Revision 1.7  1999/09/03 14:14:39  warmerda
- * fix cloning
- *
- * Revision 1.6  1999/09/02 03:10:01  warmerda
- * fixed subtle problem with rewinding modules with reusable headers
- *
- * Revision 1.5  1999/05/08 20:16:20  warmerda
- * added some record validity testing
- *
- * Revision 1.4  1999/05/07 14:12:24  warmerda
- * added record cloning, and subfield value fetches
- *
- * Revision 1.3  1999/05/06 14:48:28  warmerda
- * Fixed EOF handling in files with reused headers
- *
- * Revision 1.2  1999/05/06 14:24:29  warmerda
- * minor optimization, don't emit an error on EOF
- *
- * Revision 1.1  1999/04/27 18:45:05  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "iso8211.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ddfrecord.cpp,v 1.29 2006/02/24 15:59:24 fwarmerdam Exp $");
+CPL_CVSID("$Id: ddfrecord.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static const size_t nLeaderSize = 24;
 
@@ -226,12 +134,12 @@ int DDFRecord::Read()
 /* -------------------------------------------------------------------- */
     size_t      nReadBytes;
 
-    nReadBytes = VSIFRead( pachData + nFieldOffset, 1,
-                           nDataSize - nFieldOffset,
-                           poModule->GetFP() );
+    nReadBytes = VSIFReadL( pachData + nFieldOffset, 1,
+                            nDataSize - nFieldOffset,
+                            poModule->GetFP() );
     if( nReadBytes != (size_t) (nDataSize - nFieldOffset)
         && nReadBytes == 0
-        && VSIFEof( poModule->GetFP() ) )
+        && VSIFEofL( poModule->GetFP() ) )
     {
         return FALSE;
     }
@@ -296,12 +204,12 @@ int DDFRecord::Write()
 /* -------------------------------------------------------------------- */
 /*      Write the leader.                                               */
 /* -------------------------------------------------------------------- */
-    VSIFWrite( szLeader, nLeaderSize, 1, poModule->GetFP() );
+    VSIFWriteL( szLeader, nLeaderSize, 1, poModule->GetFP() );
 
 /* -------------------------------------------------------------------- */
 /*      Write the remainder of the record.                              */
 /* -------------------------------------------------------------------- */
-    VSIFWrite( pachData, nDataSize, 1, poModule->GetFP() );
+    VSIFWriteL( pachData, nDataSize, 1, poModule->GetFP() );
     
     return TRUE;
 }
@@ -352,8 +260,8 @@ int DDFRecord::ReadHeader()
     char        achLeader[nLeaderSize];
     int         nReadBytes;
 
-    nReadBytes = VSIFRead(achLeader,1,nLeaderSize,poModule->GetFP());
-    if( nReadBytes == 0 && VSIFEof( poModule->GetFP() ) )
+    nReadBytes = VSIFReadL(achLeader,1,nLeaderSize,poModule->GetFP());
+    if( nReadBytes == 0 && VSIFEofL( poModule->GetFP() ) )
     {
         return FALSE;
     }
@@ -418,7 +326,7 @@ int DDFRecord::ReadHeader()
         nDataSize = _recLength - nLeaderSize;
         pachData = (char *) CPLMalloc(nDataSize);
 
-        if( VSIFRead( pachData, 1, nDataSize, poModule->GetFP()) !=
+        if( VSIFReadL( pachData, 1, nDataSize, poModule->GetFP()) !=
             (size_t) nDataSize )
         {
             CPLError( CE_Failure, CPLE_FileIO, 
@@ -436,7 +344,7 @@ int DDFRecord::ReadHeader()
             nDataSize++;
             pachData = (char *) CPLRealloc(pachData,nDataSize);
             
-            if( VSIFRead( pachData + nDataSize - 1, 1, 1, poModule->GetFP() )
+            if( VSIFReadL( pachData + nDataSize - 1, 1, 1, poModule->GetFP() )
                 != 1 )
             {
                 CPLError( CE_Failure, CPLE_FileIO, 
@@ -550,7 +458,7 @@ int DDFRecord::ReadHeader()
         do {
             // read an Entry:
             if(nFieldEntryWidth != 
-               (int) VSIFRead(tmpBuf, 1, nFieldEntryWidth, poModule->GetFP())) {
+               (int) VSIFReadL(tmpBuf, 1, nFieldEntryWidth, poModule->GetFP())) {
                 CPLError(CE_Failure, CPLE_FileIO,
                          "Data record is short on DDF file.");
                 return FALSE;
@@ -590,7 +498,7 @@ int DDFRecord::ReadHeader()
 
             // read an Entry:
             if(nFieldLength != 
-               (int) VSIFRead(tmpBuf, 1, nFieldLength, poModule->GetFP())) {
+               (int) VSIFReadL(tmpBuf, 1, nFieldLength, poModule->GetFP())) {
                 CPLError(CE_Failure, CPLE_FileIO,
                          "Data record is short on DDF file.");
                 return FALSE;
