@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: jp2kakdataset.cpp,v 1.39 2006/04/28 04:21:14 fwarmerdam Exp $
+ * $Id: jp2kakdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  JPEG-2000
  * Purpose:  Implementation of the ISO/IEC 15444-1 standard based on Kakadu.
@@ -25,73 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- * 
- * $Log: jp2kakdataset.cpp,v $
- * Revision 1.39  2006/04/28 04:21:14  fwarmerdam
- * cleanup GCPs
- *
- * Revision 1.38  2006/04/07 14:42:49  fwarmerdam
- * Overide builtin georeferencing with PAM info.
- *
- * Revision 1.37  2006/04/07 05:36:26  fwarmerdam
- * use new ReadAndParse() method
- *
- * Revision 1.36  2006/03/22 20:05:26  fwarmerdam
- * changed default tilesize rules, now tilexsize is at most 20K
- *
- * Revision 1.35  2006/03/22 19:50:38  fwarmerdam
- * Trim history.
- *
- * Revision 1.34  2006/03/22 19:50:00  fwarmerdam
- * Preliminary support for output with tiling.
- *
- * Revision 1.33  2006/03/16 17:11:39  fwarmerdam
- * Catch exceptions in allocator.finalize() to catch common out of memory
- * condition.
- *
- * Revision 1.32  2006/03/08 15:35:18  fwarmerdam
- * Fixed problem with bIsJPX test.
- *
- * Revision 1.31  2006/02/28 18:04:50  fwarmerdam
- * Make sure tiles get closed in case of an exception.
- *
- * Revision 1.30  2006/02/19 22:37:52  fwarmerdam
- * avoid warning with kakadu 4
- *
- * Revision 1.29  2005/07/05 22:09:50  fwarmerdam
- * use GDALJP2Metadata for geoinfo, better pam support
- *
- * Revision 1.28  2005/05/05 15:54:48  fwarmerdam
- * PAM Enabled
- *
- * Revision 1.27  2004/10/07 20:18:27  fwarmerdam
- * added support for greyscale+alpha
- *
- * Revision 1.26  2004/10/07 16:48:32  fwarmerdam
- * added simple CopyCreate RGBA output support
- *
- * Revision 1.25  2004/09/08 15:36:12  warmerda
- * fixed up initialization issues
- *
- * Revision 1.24  2004/07/29 16:38:28  warmerda
- * fixed up error trapping when opening subfile
- *
- * Revision 1.23  2004/07/28 14:39:48  warmerda
- * added jp2 subfile support
- *
- * Revision 1.22  2004/07/23 19:13:41  warmerda
- * Added J2K_SUBFILE support
- *
- * Revision 1.21  2004/04/02 17:26:46  warmerda
- * Added creation options.
- *
- * Revision 1.20  2004/01/15 21:21:04  warmerda
- * added lossless compression of 16bit values
- *
- * Revision 1.19  2004/01/12 19:50:09  warmerda
- * Disable jpip support on 4.1 till I get around to fixing it up.
- */
+ ****************************************************************************/
 
 #include "gdal_pam.h"
 #include "gdaljp2metadata.h"
@@ -138,7 +72,7 @@
 #  define kdu_client void
 #endif
 
-CPL_CVSID("$Id: jp2kakdataset.cpp,v 1.39 2006/04/28 04:21:14 fwarmerdam Exp $");
+CPL_CVSID("$Id: jp2kakdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 CPL_C_START
 CPLErr CPL_DLL GTIFMemBufFromWkt( const char *pszWKT, 
@@ -156,12 +90,6 @@ static void
 transfer_bytes(kdu_byte *dest, kdu_line_buf &src, int gap, int precision,
                GDALDataType eOutType );
 
-static const unsigned int jp2_uuid_box_type = 0x75756964;
-
-static unsigned char msi_uuid2[16] =
-{0xb1,0x4b,0xf8,0xbd,0x08,0x3d,0x4b,0x43,
- 0xa5,0xae,0x8c,0xd7,0xd5,0xa6,0xce,0x03}; 
-
 static unsigned char jp2_header[] = 
 {0x00,0x00,0x00,0x0c,0x6a,0x50,0x20,0x20,0x0d,0x0a,0x87,0x0a};
 
@@ -219,6 +147,8 @@ class JP2KAKRasterBand : public GDALPamRasterBand
 {
     friend class JP2KAKDataset;
 
+    JP2KAKDataset *poBaseDS;
+
     int         nDiscardLevels; 
 
     kdu_dims 	band_dims; 
@@ -239,7 +169,7 @@ class JP2KAKRasterBand : public GDALPamRasterBand
   public:
 
     		JP2KAKRasterBand( int, int, kdu_codestream, int, kdu_client *,
-                                  jp2_channels );
+                                  jp2_channels, JP2KAKDataset * );
     		~JP2KAKRasterBand();
     
     virtual CPLErr IReadBlock( int, int, void * );
@@ -254,9 +184,8 @@ class JP2KAKRasterBand : public GDALPamRasterBand
 
     void        ApplyPalette( jp2_palette oJP2Palette );
     void        ProcessYCbCrTile(kdu_tile tile, GByte *pabyBuffer, 
-                                 int nTileXOff, int nTileYOff );
-    void        ProcessTile(kdu_tile tile, GByte *pabyBuffer, 
-                            int nTileXOff, int nTileYOff );
+                                 int nBlockXOff, int nBlockYOff );
+    void        ProcessTile(kdu_tile tile, GByte *pabyBuffer );
 };
 
 /************************************************************************/
@@ -325,10 +254,13 @@ private:
 JP2KAKRasterBand::JP2KAKRasterBand( int nBand, int nDiscardLevels,
                                     kdu_codestream oCodeStream,
                                     int nResCount, kdu_client *jpip_client,
-                                    jp2_channels oJP2Channels )
+                                    jp2_channels oJP2Channels,
+                                    JP2KAKDataset *poBaseDSIn )
 
 {
     this->nBand = nBand;
+    this->poBaseDS = poBaseDSIn;
+
     bYCbCrReported = FALSE;
 
     if( oCodeStream.get_bit_depth(nBand-1) == 16
@@ -354,10 +286,10 @@ JP2KAKRasterBand::JP2KAKRasterBand( int nBand, int nDiscardLevels,
     this->nRasterYSize = band_dims.size.y;
 
 /* -------------------------------------------------------------------- */
-/*      Use a 512x128 "virtual" block size unless the file is small.    */
+/*      Use a 2048x128 "virtual" block size unless the file is small.    */
 /* -------------------------------------------------------------------- */
-    if( nRasterXSize >= 1024 )
-        nBlockXSize = 512;
+    if( nRasterXSize >= 2048 )
+        nBlockXSize = 2048;
     else
         nBlockXSize = nRasterXSize;
     
@@ -499,7 +431,8 @@ JP2KAKRasterBand::JP2KAKRasterBand( int nBand, int nDiscardLevels,
                                 sizeof(void*) * nOverviewCount );
                 papoOverviewBand[nOverviewCount-1] = 
                     new JP2KAKRasterBand( nBand, nDiscard, oCodeStream, 0,
-                                          jpip_client, oJP2Channels );
+                                          jpip_client, oJP2Channels,
+                                          poBaseDS );
             }
             else
             {
@@ -630,11 +563,11 @@ CPLErr JP2KAKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
                 try 
                 {
-
                     if( tile.get_ycc() && nBand < 4 )
-                        ProcessYCbCrTile( tile, pabyDest, offset.x, offset.y );
+                        ProcessYCbCrTile( tile, pabyDest, 
+                                          nBlockXOff, nBlockYOff );
                     else
-                        ProcessTile( tile, pabyDest, offset.x, offset.y );
+                        ProcessTile( tile, pabyDest );
                     tile.close();
                 }
                 catch( ... )
@@ -664,8 +597,7 @@ CPLErr JP2KAKRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 /*      buffer.                                                         */
 /************************************************************************/
 
-void JP2KAKRasterBand::ProcessTile( kdu_tile tile, GByte *pabyDest, 
-                                    int /*nTileXOff*/, int /*nTileYOff*/ )
+void JP2KAKRasterBand::ProcessTile( kdu_tile tile, GByte *pabyDest )
 
 {
     // Open tile-components and create processing engines and resources
@@ -717,7 +649,7 @@ void JP2KAKRasterBand::ProcessTile( kdu_tile tile, GByte *pabyDest,
 /************************************************************************/
 
 void JP2KAKRasterBand::ProcessYCbCrTile( kdu_tile tile, GByte *pabyDest, 
-                                         int /*nTileXOff*/, int /*nTileYOff*/ )
+                                         int nTileXOff, int nTileYOff )
 
 {
     // Open tile-components and create processing engines and resources
@@ -734,12 +666,62 @@ void JP2KAKRasterBand::ProcessYCbCrTile( kdu_tile tile, GByte *pabyDest,
     int  nWordSize = GDALGetDataTypeSize( eDataType ) / 8;
     int  iComp;
 
+/* -------------------------------------------------------------------- */
+/*      Report if we are doing ycbcr processing, but only report        */
+/*      once.                                                           */
+/* -------------------------------------------------------------------- */
     if( !bYCbCrReported && nBand == 1 )
     {
         bYCbCrReported = TRUE;
         CPLDebug( "JP2KAK", "Using ProcessYCbCrTile() for this dataset." );
     }
 
+/* -------------------------------------------------------------------- */
+/*      Fetch the other color component blocks, so we can fill all      */
+/*      three blocks at once.                                           */
+/* -------------------------------------------------------------------- */
+    int iBand;
+    GByte *apabyBandDest[3];
+    GDALRasterBlock *apoBlocks[3] = { NULL, NULL, NULL };
+
+    for( iBand=0; iBand < 3; iBand++ )
+    {
+        if( nBand == iBand+1 )
+            apabyBandDest[iBand] = pabyDest;
+        else
+        {
+            GDALRasterBand *poBaseBand = poBaseDS->GetRasterBand(iBand+1);
+            JP2KAKRasterBand *poBand;
+
+            if( nDiscardLevels == 0 )
+                poBand = (JP2KAKRasterBand *) poBaseBand;
+            else
+            {
+                int iOver;
+
+                for( iOver = 0; iOver < poBaseBand->GetOverviewCount(); iOver++ )
+                {
+                    poBand = (JP2KAKRasterBand *) 
+                        poBaseBand->GetOverview( iOver );
+                    if( poBand->nDiscardLevels == nDiscardLevels )
+                        break;
+                }
+                if( iOver == poBaseBand->GetOverviewCount() )
+                {
+                    CPLAssert( FALSE );
+                    return;
+                }
+            }
+
+            apoBlocks[iBand] = 
+                poBand->GetLockedBlockRef( nTileXOff, nTileYOff, TRUE );
+            apabyBandDest[iBand] = (GByte *) apoBlocks[iBand]->GetDataRef();
+        }
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Setup decoders.                                                 */
+/* -------------------------------------------------------------------- */
     res.get_dims(dims);
     
     bool use_shorts = (comp[0].get_bit_depth(true) <= 16);
@@ -762,8 +744,10 @@ void JP2KAKRasterBand::ProcessYCbCrTile( kdu_tile tile, GByte *pabyDest,
     for( iComp = 0; iComp < 3; iComp++ )
         line[iComp].create(); // Grabs resources from the allocator.
 
-    // Now walk through the lines of the buffer, recovering them from the
-    // relevant tile-component processing engines.
+/* -------------------------------------------------------------------- */
+/*      Now walk through the lines of the buffer, recovering them from the*/
+/*      relevant tile-component processing engines.                     */
+/* -------------------------------------------------------------------- */
 
     for( int y = 0; y < dims.size.y; y++ )
     {
@@ -773,13 +757,23 @@ void JP2KAKRasterBand::ProcessYCbCrTile( kdu_tile tile, GByte *pabyDest,
 
         kdu_convert_ycc_to_rgb(line[0], line[1], line[2]);
 
-        transfer_bytes(pabyDest + y * nBlockXSize * nWordSize,
-                       line[nBand-1], nWordSize, bit_depth, eDataType );
+        for( iBand = 0; iBand < 3; iBand++ )
+            transfer_bytes(apabyBandDest[iBand] + y * nBlockXSize * nWordSize,
+                           line[iBand], nWordSize, bit_depth, eDataType );
     }
 
+/* -------------------------------------------------------------------- */
+/*      Cleanup and release band locks.                                 */
+/* -------------------------------------------------------------------- */
     engine[0].destroy();
     engine[1].destroy();
     engine[2].destroy();
+
+    for( iBand = 0; iBand < 3; iBand++ )
+    {
+        if( apoBlocks[iBand] != NULL )
+            apoBlocks[iBand]->DropLock();
+    }
 }
 
 /************************************************************************/
@@ -1307,7 +1301,7 @@ GDALDataset *JP2KAKDataset::Open( GDALOpenInfo * poOpenInfo )
         {
             JP2KAKRasterBand *poBand = 
                 new JP2KAKRasterBand(iBand,0,poDS->oCodeStream, nResCount,
-                                     jpip_client, oJP2Channels );
+                                     jpip_client, oJP2Channels, poDS );
 
             if( iBand == 1 && oJP2Palette.exists() )
                 poBand->ApplyPalette( oJP2Palette );
@@ -1333,6 +1327,33 @@ GDALDataset *JP2KAKDataset::Open( GDALOpenInfo * poOpenInfo )
                 oJP2Geo.pasGCPList = NULL;
                 oJP2Geo.nGCPCount = 0;
             }
+
+/* -------------------------------------------------------------------- */
+/*      Do we have any XML boxes we would like to treat as special      */
+/*      domain metadata?                                                */
+/* -------------------------------------------------------------------- */
+            int iBox;
+
+            for( iBox = 0; 
+                 oJP2Geo.papszGMLMetadata
+                     && oJP2Geo.papszGMLMetadata[iBox] != NULL; 
+                 iBox++ )
+            {
+                char *pszName = NULL;
+                const char *pszXML = 
+                    CPLParseNameValue( oJP2Geo.papszGMLMetadata[iBox], 
+                                       &pszName );
+                CPLString osDomain;
+                char *apszMDList[2];
+
+                osDomain.Printf( "xml:%s", pszName );
+                apszMDList[0] = (char *) pszXML;
+                apszMDList[1] = NULL;
+
+                poDS->GDALPamDataset::SetMetadata( apszMDList, osDomain );
+
+                CPLFree( pszName );
+            }
         }
 
 /* -------------------------------------------------------------------- */
@@ -1569,58 +1590,41 @@ transfer_bytes(kdu_byte *dest, kdu_line_buf &src, int gap, int precision,
 }
 
 /************************************************************************/
-/*                       JP2KAKWriteGeoTIFFInfo()                       */
+/*                           JP2KAKWriteBox()                           */
+/*                                                                      */
+/*      Write out the passed box and delete it.                         */
 /************************************************************************/
 
-void JP2KAKWriteGeoTIFFInfo( jp2_target *jp2_out, GDALDataset *poSrcDS )
+static void JP2KAKWriteBox( jp2_target *jp2_out, GDALJP2Box *poBox )
 
 {
-/* -------------------------------------------------------------------- */
-/*      Prepare the memory buffer containing the degenerate GeoTIFF     */
-/*      file.                                                           */
-/* -------------------------------------------------------------------- */
-    const char *pszWKT;
-    double	adfGeoTransform[6];
-    int         nGTBufSize = 0;
-    unsigned char *pabyGTBuf = NULL;
+    GUInt32 nBoxType;
 
-    if( GDALGetGCPCount( poSrcDS ) > 0 )
-        pszWKT = poSrcDS->GetGCPProjection();
-    else
-        pszWKT = poSrcDS->GetProjectionRef();
-
-    poSrcDS->GetGeoTransform(adfGeoTransform);
-
-    if( GTIFMemBufFromWkt( pszWKT, adfGeoTransform, 
-                           poSrcDS->GetGCPCount(), poSrcDS->GetGCPs(), 
-                           &nGTBufSize, &pabyGTBuf ) != CE_None )
-        return;
-
-    if( nGTBufSize == 0 )
+    if( poBox == NULL )
         return;
 
+    memcpy( &nBoxType, poBox->GetType(), 4 );
+    
 /* -------------------------------------------------------------------- */
 /*      Write to a box on the JP2 file.                                 */
 /* -------------------------------------------------------------------- */
 #ifdef KAKADU4
-    jp2_out->open_next( jp2_uuid_box_type );
+    jp2_out->open_next( nBoxType );
 
-    jp2_out->write( (kdu_byte *) msi_uuid2, sizeof(msi_uuid2) );
-
-    jp2_out->write( (kdu_byte *) pabyGTBuf, nGTBufSize );
+    jp2_out->write( (kdu_byte *) poBox->GetWritableData(), 
+                    poBox->GetDataLength() );
 
     jp2_out->close();
 #else
-    jp2_output_box &uuid_box = jp2_out->open_box( jp2_uuid_box_type );
+    jp2_output_box &uuid_box = jp2_out->open_box( nBoxType );
 
-    uuid_box.write( (kdu_byte *) msi_uuid2, sizeof(msi_uuid2) );
-
-    uuid_box.write( (kdu_byte *) pabyGTBuf, nGTBufSize );
+    uuid_box.write( (kdu_byte *) poBox->GetWritableData(), 
+                    poBox->GetDataLength() );
 
     uuid_box.close();
 #endif
 
-    CPLFree( pabyGTBuf );
+    delete poBox;
 }
 
 /************************************************************************/
@@ -1634,7 +1638,8 @@ JP2KAKCreateCopy_WriteTile( GDALDataset *poSrcDS, kdu_tile &oTile,
                             int bReversible, GDALDataType eType,
                             kdu_codestream &oCodeStream, int bFlushEnabled,
                             kdu_long *layer_bytes, int layer_count,
-                            GDALProgressFunc pfnProgress, void * pProgressData)
+                            GDALProgressFunc pfnProgress, void * pProgressData,
+                            bool bComseg )
 
 {                                       
 /* -------------------------------------------------------------------- */
@@ -1785,7 +1790,8 @@ JP2KAKCreateCopy_WriteTile( GDALDataset *poSrcDS, kdu_tile &oTile,
                       "Calling oCodeStream.flush() at line %d",
                       MIN(nYSize,iLine+CHUNK_SIZE) );
             
-            oCodeStream.flush( layer_bytes, layer_count );
+            oCodeStream.flush( layer_bytes, layer_count, NULL,
+                               true, bComseg );
         }
         else if( bFlushEnabled )
             CPLDebug( "JP2KAK", 
@@ -1950,6 +1956,13 @@ JP2KAKCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
         nTileYSize = MIN(nYSize,
                          atoi(CSLFetchNameValue( papszOptions, "BLOCKYSIZE")));
 
+/* -------------------------------------------------------------------- */
+/*      Do we want a comment segment emitted?                           */
+/* -------------------------------------------------------------------- */
+    bool bComseg;
+
+    bComseg = CSLFetchBoolean( papszOptions, "COMSEG", TRUE );
+
 /* -------------------------------------------------------------------- */
 /*      Establish the general image parameters.                         */
 /* -------------------------------------------------------------------- */
@@ -2085,11 +2098,15 @@ JP2KAKCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
     char *apszParms[] = { "Corder", "PCRL", 
                           "Cprecincts", "{512,512},{256,512},{128,512},{64,512},{32,512},{16,512},{8,512},{4,512},{2,512}",
                           "ORGgen_plt", "yes", 
+                          "ORGgen_tlm", NULL,
+                          "Qguard", NULL, 
                           "Cmodes", NULL, 
                           "Clevels", NULL,
+                          "Cblk", NULL,
                           "Rshift", NULL,
                           "Rlevels", NULL,
                           "Rweight", NULL,
+                          "Sprofile", NULL,
                           NULL, NULL };
 
     for( iParm = 0; apszParms[iParm] != NULL; iParm += 2 )
@@ -2198,8 +2215,8 @@ JP2KAKCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 #endif
 
 /* -------------------------------------------------------------------- */
-/*      Set the GeoTIFF box if georeferencing is available, and this    */
-/*      is a JP2 file.                                                  */
+/*      Set the GeoTIFF and GML boxes if georeferencing is available,   */
+/*      and this is a JP2 file.                                         */
 /* -------------------------------------------------------------------- */
     double	adfGeoTransform[6];
     if( bIsJP2
@@ -2212,7 +2229,48 @@ JP2KAKCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                  || ABS(adfGeoTransform[5]) != 1.0))
             || poSrcDS->GetGCPCount() > 0) )
     {
-        JP2KAKWriteGeoTIFFInfo( &jp2_out, poSrcDS );
+        GDALJP2Metadata oJP2MD;
+
+        if( poSrcDS->GetGCPCount() > 0 )
+        {
+            oJP2MD.SetProjection( poSrcDS->GetGCPProjection() );
+            oJP2MD.SetGCPs( poSrcDS->GetGCPCount(), poSrcDS->GetGCPs() );
+        }
+        else
+        {
+            oJP2MD.SetProjection( poSrcDS->GetProjectionRef() );
+            oJP2MD.SetGeoTransform( adfGeoTransform );
+        }
+
+        
+        if( CSLFetchBoolean( papszOptions, "GMLJP2", TRUE ) )
+            JP2KAKWriteBox( &jp2_out, oJP2MD.CreateGMLJP2(nXSize,nYSize) );
+        if( CSLFetchBoolean( papszOptions, "GeoJP2", TRUE ) )
+            JP2KAKWriteBox( &jp2_out, oJP2MD.CreateJP2GeoTIFF() );
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Do we have any XML boxes we want to preserve?                   */
+/* -------------------------------------------------------------------- */
+    int iBox;
+    
+    for( iBox = 0; TRUE; iBox++ )
+    {
+        CPLString oName;
+        char **papszMD;
+
+        oName.Printf( "xml:BOX_%d", iBox );
+        papszMD = poSrcDS->GetMetadata( oName );
+        
+        if( papszMD == NULL || CSLCount(papszMD) != 1 )
+            break;
+
+        GDALJP2Box *poXMLBox = new GDALJP2Box();
+
+        poXMLBox->SetType( "xml " );
+        poXMLBox->SetWritableData( strlen(papszMD[0])+1, 
+                                   (GByte *) papszMD[0] );
+        JP2KAKWriteBox( &jp2_out, poXMLBox );
     }
 
 /* -------------------------------------------------------------------- */
@@ -2269,7 +2327,7 @@ JP2KAKCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                                              oCodeStream, bFlushEnabled,
                                              layer_bytes, layer_count,
                                              GDALScaledProgress, 
-                                             pScaledProgressData ) )
+                                             pScaledProgressData, bComseg ) )
             {
                 GDALDestroyScaledProgress( pScaledProgressData );
 
@@ -2289,7 +2347,7 @@ JP2KAKCreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
 /* -------------------------------------------------------------------- */
 /*      Finish flushing out results.                                    */
 /* -------------------------------------------------------------------- */
-    oCodeStream.flush(layer_bytes, layer_count);
+    oCodeStream.flush(layer_bytes, layer_count, NULL, true, bComseg );
     oCodeStream.destroy();
 
     CPLFree( layer_bytes );
@@ -2349,13 +2407,19 @@ void GDALRegister_JP2KAK()
 "   <Option name='QUALITY' type='integer' description='1-100, 100 is lossless'/>"
 "   <Option name='BLOCKXSIZE' type='int' description='Tile Width'/>"
 "   <Option name='BLOCKYSIZE' type='int' description='Tile Height'/>"
+"   <Option name='GeoJP2' type='boolean' description='defaults to ON'/>"
+"   <Option name='GMLJP2' type='boolean' description='defaults to ON'/>"
 "   <Option name='LAYERS' type='integer'/>"
 "   <Option name='ROI' type='string'/>"
+"   <Option name='COMSEG' type='boolean' />"
 "   <Option name='Corder' type='string'/>"
 "   <Option name='Cprecincts' type='string'/>"
-"   <Option name='ORGgen_plt' type='string'/>"
 "   <Option name='Cmodes' type='string'/>"
 "   <Option name='Clevels' type='string'/>"
+"   <Option name='ORGgen_plt' type='string'/>"
+"   <Option name='ORGgen_tlm' type='string'/>"
+"   <Option name='Qguard' type='integer'/>"
+"   <Option name='Sprofile' type='integer'/>"
 "   <Option name='Rshift' type='string'/>"
 "   <Option name='Rlevels' type='string'/>"
 "   <Option name='Rweight' type='string'/>"
