@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: jpgdataset.cpp,v 1.41 2006/03/21 19:46:37 fwarmerdam Exp $
+ * $Id: jpgdataset.cpp 10990 2007-03-16 17:54:00Z warmerdam $
  *
  * Project:  JPEG JFIF Driver
  * Purpose:  Implement GDAL JPEG Support based on IJG libjpeg.
@@ -8,6 +8,9 @@
  ******************************************************************************
  * Copyright (c) 2000, Frank Warmerdam
  *
+ * Portions Copyright (c) Her majesty the Queen in right of Canada as
+ * represented by the Minister of National Defence, 2006.
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -25,147 +28,16 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- * 
- * $Log: jpgdataset.cpp,v $
- * Revision 1.41  2006/03/21 19:46:37  fwarmerdam
- * Avoid calling jpeg_finish_compress() if there was an error or
- * libjpeg is likely to issue an error and call exit()!
- *
- * Revision 1.40  2006/02/26 14:32:32  fwarmerdam
- * Added accelerated dataset rasterio for common case c/o Mike Mazzella.
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=1046
- *
- * Revision 1.39  2006/02/02 01:07:36  fwarmerdam
- * Added support for finding EXIF information when APP1 chunk appears
- * after an APP0 JFIF chunk.  ie. albania.jpg
- *
- * Revision 1.38  2005/10/14 21:52:26  fwarmerdam
- * Slightly safer version of last fix, retaining ASCII setting for ucomments.
- *
- * Revision 1.37  2005/10/14 21:50:03  fwarmerdam
- * Don't flip out of usercomment is less than 8 characters.
- *
- * Revision 1.36  2005/09/26 14:52:57  fwarmerdam
- * Fixed UserComments to skip language encoding.  Force some new tags
- * to ASCII.
- *
- * Revision 1.35  2005/09/23 20:01:56  fwarmerdam
- * Fixed progress reporting.
- *
- * Revision 1.34  2005/09/22 14:59:47  fwarmerdam
- * Fixed bug with unrecognised exif tags sometimes being written over
- * metadata of a previous tag.
- * Hack so that UserComments is treated as ASCII.
- *
- * Revision 1.33  2005/09/11 17:15:33  fwarmerdam
- * direct io through VSI, use large file API
- *
- * Revision 1.32  2005/09/11 16:36:54  fwarmerdam
- * Turn raw EXIF printf() into a debug statement.
- *
- * Revision 1.31  2005/09/05 22:37:52  fwarmerdam
- * Added progress monitor support in createcopy.
- *
- * Revision 1.30  2005/08/10 20:53:24  dnadeau
- * correct GPS and logic problems for EXIF
- *
- * Revision 1.29  2005/08/09 20:19:19  dnadeau
- * add EXIF GPS IFD Tags
- *
- * Revision 1.28  2005/07/27 02:00:39  dnadeau
- * correct leak problem. Free poTIFFDir.
- *
- * Revision 1.27  2005/07/19 19:36:15  fwarmerdam
- * fixed libtiff conflicts
- *
- * Revision 1.26  2005/07/19 18:06:28  dnadeau
- * fix loop problem finding tags
- *
- * Revision 1.25  2005/07/19 15:38:33  fwarmerdam
- * Disable a few less interesting tags.
- *
- * Revision 1.24  2005/07/19 15:33:53  fwarmerdam
- * Removed /Exif in longname.
- *
- * Revision 1.23  2005/07/19 15:21:57  dnadeau
- * added exif support
- *
- * Revision 1.22  2005/05/23 06:57:12  fwarmerdam
- * Use lockedblockrefs
- *
- * Revision 1.21  2005/04/27 16:35:58  fwarmerdam
- * PAM enable
- *
- * Revision 1.20  2005/03/23 00:27:32  fwarmerdam
- * First pass try at support for the libjpeg Mk1 library.
- *
- * Revision 1.19  2005/03/22 21:48:11  fwarmerdam
- * Added preliminary Mk1 libjpeg support.  Compress still not working.
- *
- * Revision 1.18  2005/02/25 15:17:35  fwarmerdam
- * Use dataset io to fetch to provide (potentially) faster interleaved
- * reads.
- *
- * Revision 1.17  2003/10/31 00:58:55  warmerda
- * Added support for .jpgw as per request from Markus.
- *
- * Revision 1.16  2003/04/04 13:45:12  warmerda
- * Made casting to JSAMPLE explicit.
- *
- * Revision 1.15  2002/11/23 18:57:06  warmerda
- * added CREATIONOPTIONS metadata on driver
- *
- * Revision 1.14  2002/09/04 06:50:37  warmerda
- * avoid static driver pointers
- *
- * Revision 1.13  2002/07/13 04:16:39  warmerda
- * added WORLDFILE support
- *
- * Revision 1.12  2002/06/20 19:57:04  warmerda
- * ensure GetGeoTransform always sets geotransform.
- *
- * Revision 1.11  2002/06/18 02:50:20  warmerda
- * fixed multiline string constants
- *
- * Revision 1.10  2002/06/12 21:12:25  warmerda
- * update to metadata based driver info
- *
- * Revision 1.9  2001/12/11 16:50:16  warmerda
- * try to push green and blue values into cache when reading red
- *
- * Revision 1.8  2001/11/11 23:51:00  warmerda
- * added required class keyword to friend declarations
- *
- * Revision 1.7  2001/08/22 17:11:30  warmerda
- * added support for .wld world files
- *
- * Revision 1.6  2001/07/18 04:51:57  warmerda
- * added CPL_CVSID
- *
- * Revision 1.5  2001/06/01 14:13:12  warmerda
- * Improved magic number testing.
- *
- * Revision 1.4  2001/05/01 18:18:28  warmerda
- * added world file support
- *
- * Revision 1.3  2001/01/12 21:19:25  warmerda
- * added progressive support
- *
- * Revision 1.2  2000/07/07 15:11:01  warmerda
- * added QUALITY=n creation option
- *
- * Revision 1.1  2000/04/28 20:57:57  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal_pam.h"
 #include "cpl_string.h"
 #include "gdalexif.h"
 
+#include <setjmp.h>
 
-CPL_CVSID("$Id: jpgdataset.cpp,v 1.41 2006/03/21 19:46:37 fwarmerdam Exp $");
+
+CPL_CVSID("$Id: jpgdataset.cpp 10990 2007-03-16 17:54:00Z warmerdam $");
 
 CPL_C_START
 #include "jpeglib.h"
@@ -192,11 +64,14 @@ class JPGDataset : public GDALPamDataset
 
     struct jpeg_decompress_struct sDInfo;
     struct jpeg_error_mgr sJErr;
+    jmp_buf setjmp_buffer;
 
     int	   bGeoTransformValid;
     double adfGeoTransform[6];
 
     FILE   *fpImage;
+    int    nSubfileOffset;
+
     int    nLoadedScanline;
     GByte  *pabyScanline;
 
@@ -219,8 +94,10 @@ class JPGDataset : public GDALPamDataset
     void   EXIFPrintShort(char *, const char*, TIFFDirEntry*);
     void   EXIFPrintData(char *, GUInt16, GUInt32, unsigned char* );
 
-    
+    int    nQLevel;
+    void   LoadDefaultTables(int);
 
+    static void ErrorExit(j_common_ptr cinfo);
   public:
                  JPGDataset();
                  ~JPGDataset();
@@ -243,6 +120,8 @@ class JPGRasterBand : public GDALPamRasterBand
 {
     friend class JPGDataset;
 
+    JPGDataset 	   *poGDS;
+
   public:
 
                    JPGRasterBand( JPGDataset *, int );
@@ -251,7 +130,6 @@ class JPGRasterBand : public GDALPamRasterBand
     virtual GDALColorInterp GetColorInterpretation();
 };
 
-
 /************************************************************************/
 /*                         EXIFPrintByte()                              */
 /************************************************************************/
@@ -418,8 +296,6 @@ void JPGDataset::EXIFPrintData(char* pszData, GUInt16 type,
   }
 }
 
-
-
 /************************************************************************/
 /*                        EXIFInit()                                    */
 /*                                                                      */
@@ -529,6 +405,10 @@ CPLErr JPGDataset::EXIFExtractMetadata(FILE *fp, int nOffset)
     if (bSwabflag)
         TIFFSwabShort(&nEntryCount);
 
+    // Some apps write empty directories - see bug 1523.
+    if( nEntryCount == 0 )  
+        return CE_None;
+
     poTIFFDir = (TIFFDirEntry *)CPLMalloc(nEntryCount * sizeof(TIFFDirEntry));
 
     if (poTIFFDir == NULL) 
@@ -734,7 +614,8 @@ CPLErr JPGDataset::EXIFExtractMetadata(FILE *fp, int nOffset)
 JPGRasterBand::JPGRasterBand( JPGDataset *poDS, int nBand )
 
 {
-    this->poDS = poDS;
+    this->poDS = poGDS = poDS;
+
     this->nBand = nBand;
     if( poDS->sDInfo.data_precision == 12 )
         eDataType = GDT_UInt16;
@@ -753,7 +634,6 @@ CPLErr JPGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
                                   void * pImage )
 
 {
-    JPGDataset	*poGDS = (JPGDataset *) poDS;
     CPLErr      eErr;
     int         nXSize = GetXSize();
     int         nWordSize = GDALGetDataTypeSize(eDataType) / 8;
@@ -821,8 +701,6 @@ CPLErr JPGRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 GDALColorInterp JPGRasterBand::GetColorInterpretation()
 
 {
-    JPGDataset	*poGDS = (JPGDataset *) poDS;
-
     if( poGDS->nBands == 1 )
         return GCI_GrayIndex;
 
@@ -887,6 +765,7 @@ JPGDataset::~JPGDataset()
     if( papszMetadata != NULL )
       CSLDestroy( papszMetadata );
 }
+
 /************************************************************************/
 /*                            LoadScanline()                            */
 /************************************************************************/
@@ -901,6 +780,10 @@ CPLErr JPGDataset::LoadScanline( int iLine )
         pabyScanline = (GByte *)
             CPLMalloc(GetRasterCount() * GetRasterXSize() * 2);
 
+    // setup to trap a fatal error.
+    if (setjmp(setjmp_buffer)) 
+        return CE_Failure;
+
     if( iLine < nLoadedScanline )
         Restart();
         
@@ -916,6 +799,178 @@ CPLErr JPGDataset::LoadScanline( int iLine )
     return CE_None;
 }
 
+/************************************************************************/
+/*                         LoadDefaultTables()                          */
+/************************************************************************/
+
+const static int Q1table[256] = 
+{
+    8,    72,     72,     72,    72,    72,    72,    72,    72,    72,
+    78,    74,     76,    74,    78,    89,    81,    84,    84,    81,
+    89,    106,    93,    94,    99,    94,    93,    106,    129,    111,
+    108,    116,    116,    108,    111,    129,    135,    128,    136,    
+    145,    136,    128,    135,    155,    160,    177,    177,    160,
+    155,    193,    213,    228,    213,    193,    255,    255,    255,    
+    255
+};
+
+const static int Q2table[64] = 
+{ 
+    8, 36, 36, 36,
+    36, 36, 36, 36, 36, 36, 39, 37, 38, 37, 39, 45, 41, 42, 42, 41, 45, 53,
+    47, 47, 50, 47, 47, 53, 65, 56, 54, 59, 59, 54, 56, 65, 68, 64, 69, 73,
+    69, 64, 68, 78, 81, 89, 89, 81, 78, 98,108,115,108, 98,130,144,144,130,
+    178,190,178,243,243,255
+};
+
+const static int Q3table[64] = 
+{ 
+     8, 10, 10, 10,
+    10, 10, 10, 10, 10, 10, 11, 10, 11, 10, 11, 13, 11, 12, 12, 11, 13, 15, 
+    13, 13, 14, 13, 13, 15, 18, 16, 15, 16, 16, 15, 16, 18, 19, 18, 19, 21, 
+    19, 18, 19, 22, 23, 25, 25, 23, 22, 27, 30, 32, 30, 27, 36, 40, 40, 36, 
+    50, 53, 50, 68, 68, 91 
+}; 
+
+const static int Q4table[64] = 
+{
+    8, 7, 7, 7,
+    7, 7, 7, 7, 7, 7, 8, 7, 8, 7, 8, 9, 8, 8, 8, 8, 9, 11, 
+    9, 9, 10, 9, 9, 11, 13, 11, 11, 12, 12, 11, 11, 13, 14, 13, 14, 15, 
+    14, 13, 14, 16, 16, 18, 18, 16, 16, 20, 22, 23, 22, 20, 26, 29, 29, 26, 
+    36, 38, 36, 49, 49, 65
+};
+
+const static int Q5table[64] = 
+{
+    4, 4, 4, 4, 
+    4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 6,
+    5, 5, 6, 5, 5, 6, 7, 6, 6, 6, 6, 6, 6, 7, 8, 7, 8, 8, 
+    8, 7, 8, 9, 9, 10, 10, 9, 9, 11, 12, 13, 12, 11, 14, 16, 16, 14, 
+    20, 21, 20, 27, 27, 36
+};
+
+static const int AC_BITS[16] = 
+{ 0, 2, 1, 3, 3, 2, 4, 3, 5, 5, 4, 4, 0, 0, 1, 125 };
+
+static const int AC_HUFFVAL[256] = {
+    0x01, 0x02, 0x03, 0x00, 0x04, 0x11, 0x05, 0x12,          
+    0x21, 0x31, 0x41, 0x06, 0x13, 0x51, 0x61, 0x07,
+    0x22, 0x71, 0x14, 0x32, 0x81, 0x91, 0xA1, 0x08,
+    0x23, 0x42, 0xB1, 0xC1, 0x15, 0x52, 0xD1, 0xF0,
+    0x24, 0x33, 0x62, 0x72, 0x82, 0x09, 0x0A, 0x16,
+    0x17, 0x18, 0x19, 0x1A, 0x25, 0x26, 0x27, 0x28,
+    0x29, 0x2A, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39,
+    0x3A, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49,
+    0x4A, 0x53, 0x54, 0x55, 0x56, 0x57, 0x58, 0x59,
+    0x5A, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69,
+    0x6A, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79,
+    0x7A, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
+    0x8A, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98,
+    0x99, 0x9A, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7,
+    0xA8, 0xA9, 0xAA, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6,
+    0xB7, 0xB8, 0xB9, 0xBA, 0xC2, 0xC3, 0xC4, 0xC5,
+    0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xD2, 0xD3, 0xD4,
+    0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xE1, 0xE2,
+    0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9, 0xEA,
+    0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6, 0xF7, 0xF8,
+    0xF9, 0xFA, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
+
+static const int DC_BITS[16] = 
+{ 0, 1, 5, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0 };
+
+static const int DC_HUFFVAL[256] = {
+    0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 
+    0x08, 0x09, 0x0A, 0x0B };
+
+
+void JPGDataset::LoadDefaultTables( int n )
+
+{
+    if( nQLevel < 1 )
+        return;
+
+/* -------------------------------------------------------------------- */
+/*      Load quantization table						*/
+/* -------------------------------------------------------------------- */
+    int i;
+    JQUANT_TBL  *quant_ptr;
+    const int *panQTable;
+
+    if( nQLevel == 1 )
+        panQTable = Q1table;
+    else if( nQLevel == 2 )
+        panQTable = Q2table;
+    else if( nQLevel == 3 )
+        panQTable = Q3table;
+    else if( nQLevel == 4 )
+        panQTable = Q4table;
+    else if( nQLevel == 5 )
+        panQTable = Q5table;
+    else
+        return;
+
+    if (sDInfo.quant_tbl_ptrs[n] == NULL)
+        sDInfo.quant_tbl_ptrs[n] = 
+            jpeg_alloc_quant_table((j_common_ptr) &(sDInfo));
+    
+    quant_ptr = sDInfo.quant_tbl_ptrs[n];	/* quant_ptr is JQUANT_TBL* */
+    for (i = 0; i < 64; i++) {
+        /* Qtable[] is desired quantization table, in natural array order */
+        quant_ptr->quantval[i] = panQTable[i];
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Load AC huffman table.                                          */
+/* -------------------------------------------------------------------- */
+    JHUFF_TBL  *huff_ptr;
+
+    if (sDInfo.ac_huff_tbl_ptrs[n] == NULL)
+        sDInfo.ac_huff_tbl_ptrs[n] =
+            jpeg_alloc_huff_table((j_common_ptr)&sDInfo);
+
+    huff_ptr = sDInfo.ac_huff_tbl_ptrs[n];	/* huff_ptr is JHUFF_TBL* */
+
+    for (i = 1; i <= 16; i++) {
+        /* counts[i] is number of Huffman codes of length i bits, i=1..16 */
+        huff_ptr->bits[i] = AC_BITS[i-1];
+    }
+
+    for (i = 0; i < 256; i++) {
+        /* symbols[] is the list of Huffman symbols, in code-length order */
+        huff_ptr->huffval[i] = AC_HUFFVAL[i];
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Load DC huffman table.                                          */
+/* -------------------------------------------------------------------- */
+    if (sDInfo.dc_huff_tbl_ptrs[n] == NULL)
+        sDInfo.dc_huff_tbl_ptrs[n] =
+            jpeg_alloc_huff_table((j_common_ptr)&sDInfo);
+
+    huff_ptr = sDInfo.dc_huff_tbl_ptrs[n];	/* huff_ptr is JHUFF_TBL* */
+
+    for (i = 1; i <= 16; i++) {
+        /* counts[i] is number of Huffman codes of length i bits, i=1..16 */
+        huff_ptr->bits[i] = DC_BITS[i-1];
+    }
+
+    for (i = 0; i < 256; i++) {
+        /* symbols[] is the list of Huffman symbols, in code-length order */
+        huff_ptr->huffval[i] = DC_HUFFVAL[i];
+    }
+
+}
+
 /************************************************************************/
 /*                              Restart()                               */
 /*                                                                      */
@@ -929,7 +984,15 @@ void JPGDataset::Restart()
     jpeg_destroy_decompress( &sDInfo );
     jpeg_create_decompress( &sDInfo );
 
-    VSIRewindL( fpImage );
+    LoadDefaultTables( 0 );
+    LoadDefaultTables( 1 );
+    LoadDefaultTables( 2 );
+    LoadDefaultTables( 3 );
+
+/* -------------------------------------------------------------------- */
+/*      restart io.                                                     */
+/* -------------------------------------------------------------------- */
+    VSIFSeekL( fpImage, nSubfileOffset, SEEK_SET );
 
     jpeg_vsiio_src( &sDInfo, fpImage );
     jpeg_read_header( &sDInfo, TRUE );
@@ -1025,32 +1088,137 @@ CPLErr JPGDataset::IRasterIO( GDALRWFlag eRWFlag,
 GDALDataset *JPGDataset::Open( GDALOpenInfo * poOpenInfo )
 
 {
+    int    bIsSubfile = FALSE;
+    GByte  *pabyHeader = NULL;
+    int    subfile_offset = 0, subfile_size;
+    int    nHeaderBytes = poOpenInfo->nHeaderBytes;
+    const char *real_filename = poOpenInfo->pszFilename;
+    GByte abySubfileHeader[16];
+    int nQLevel = -1;
+
+/* -------------------------------------------------------------------- */
+/*      If it is a subfile, read the JPEG header.                       */
+/* -------------------------------------------------------------------- */
+    if( ( poOpenInfo->fp == NULL ) &&
+        ( EQUALN(poOpenInfo->pszFilename,"JPEG_SUBFILE:",13) ) )
+    {
+        /* static GByte abySubfileHeader[16]; */
+        FILE *file;
+        int bScan;
+
+        if( EQUALN(poOpenInfo->pszFilename,"JPEG_SUBFILE:Q",14) )
+            bScan = sscanf( poOpenInfo->pszFilename, "JPEG_SUBFILE:Q%d,%d,%d", 
+                            &nQLevel, &subfile_offset, &subfile_size ) == 3;
+        else
+            bScan = sscanf( poOpenInfo->pszFilename, "JPEG_SUBFILE:%d,%d", 
+                            &subfile_offset, &subfile_size ) == 2;
+
+        if( !bScan ) 
+        {
+            CPLError( CE_Failure, CPLE_OpenFailed, 
+                      "Corrupt subfile definition: %s", 
+                      poOpenInfo->pszFilename );
+            return NULL;
+        }
+
+        real_filename = strstr(poOpenInfo->pszFilename,",");
+        if( real_filename != NULL )
+            real_filename = strstr(real_filename+1,",");
+        if( real_filename != NULL && nQLevel != -1 )
+            real_filename = strstr(real_filename+1,",");
+        if( real_filename != NULL )
+            real_filename++;
+        else
+        {
+            CPLError( CE_Failure, CPLE_OpenFailed, 
+                      "Could not find filename in subfile definition.");
+            return NULL;
+        }
+
+        CPLDebug( "JPG",
+                  "real_filename %s, offset=%d, size=%d\n", 
+                  real_filename, subfile_offset, subfile_size);
+
+        file = VSIFOpenL( real_filename, "rb" );
+        if( file == NULL )
+        {
+            CPLError( CE_Failure, CPLE_OpenFailed, 
+                      "Unable to open compressed data file %s",
+                      real_filename);
+            return NULL;
+        }
+
+        /* seek to beginning of JPEG data */
+        if( VSIFSeekL( file, subfile_offset, SEEK_SET ) != 0 )
+        {
+            CPLError( CE_Failure, CPLE_OpenFailed, 
+                      "Unable to seek in file %s",
+                      real_filename);
+            return NULL;
+        }
+
+        /* read header, then close file */
+        nHeaderBytes = VSIFReadL(abySubfileHeader, 1, (size_t) 16, file);
+        VSIFCloseL(file);
+
+        pabyHeader = abySubfileHeader;
+
+        bIsSubfile = TRUE;
+    }
+    else
+    {
+        pabyHeader = poOpenInfo->pabyHeader;
+    }
+
 /* -------------------------------------------------------------------- */
 /*	First we check to see if the file has the expected header	*/
 /*	bytes.								*/    
 /* -------------------------------------------------------------------- */
-    if( poOpenInfo->nHeaderBytes < 10 )
+    if( nHeaderBytes < 10 )
         return NULL;
 
-    if( poOpenInfo->pabyHeader[0] != 0xff
-        || poOpenInfo->pabyHeader[1] != 0xd8
-        || poOpenInfo->pabyHeader[2] != 0xff )
+    if( pabyHeader[0] != 0xff
+        || pabyHeader[1] != 0xd8
+        || pabyHeader[2] != 0xff )
         return NULL;
 
-    if( poOpenInfo->pabyHeader[3] == 0xe0
-        && poOpenInfo->pabyHeader[6] == 'J'
-        && poOpenInfo->pabyHeader[7] == 'F'
-        && poOpenInfo->pabyHeader[8] == 'I'
-        && poOpenInfo->pabyHeader[9] == 'F' )
-        /* OK */;
-    else if( poOpenInfo->pabyHeader[3] == 0xe1
-             && poOpenInfo->pabyHeader[6] == 'E'
-             && poOpenInfo->pabyHeader[7] == 'x'
-             && poOpenInfo->pabyHeader[8] == 'i'
-             && poOpenInfo->pabyHeader[9] == 'f' )
-        /* OK */;
-    else
-        return NULL;
+    int iChunkOff;
+
+    for( iChunkOff = 2; 
+         iChunkOff < nHeaderBytes-10; 
+         iChunkOff += 10 ) 
+    {
+        if( pabyHeader[0+iChunkOff] != 0xff
+            || (pabyHeader[1+iChunkOff] & 0xf0) < 0xe0 )
+            return NULL;
+
+        if( pabyHeader[1+iChunkOff] == 0xe0
+            && pabyHeader[4+iChunkOff] == 'J'
+            && pabyHeader[5+iChunkOff] == 'F'
+            && pabyHeader[6+iChunkOff] == 'I'
+            && pabyHeader[7+iChunkOff] == 'F' )
+            break;
+        else if( pabyHeader[1+iChunkOff] == 0xe1
+                 && pabyHeader[4+iChunkOff] == 'E'
+                 && pabyHeader[5+iChunkOff] == 'x'
+                 && pabyHeader[6+iChunkOff] == 'i'
+                 && pabyHeader[7+iChunkOff] == 'f' )
+            break;
+        else if( pabyHeader[1+iChunkOff] == 0xe6
+                 && pabyHeader[4+iChunkOff] == 'N'
+                 && pabyHeader[5+iChunkOff] == 'I'
+                 && pabyHeader[6+iChunkOff] == 'T'
+                 && pabyHeader[7+iChunkOff] == 'F' )
+            break;
+        else if( subfile_offset != 0 )
+            /* ok - we don't require app segment on "subfiles" which 
+               can be just part of a multi-image stream - ie. in nitf files */
+            break;
+        else if( CSLTestBoolean(
+                     CPLGetConfigOption( "SIMPLE_JPEG_MAGIC", "NO" ) ) )
+            /* Explicit request to allow non-app segment files */
+            break;
+    } /* next chunk */
 
     if( poOpenInfo->eAccess == GA_Update )
     {
@@ -1066,19 +1234,24 @@ GDALDataset *JPGDataset::Open( GDALOpenInfo * poOpenInfo )
     JPGDataset 	*poDS;
 
     poDS = new JPGDataset();
+    poDS->nQLevel = nQLevel;
 
 /* -------------------------------------------------------------------- */
 /*      Open the file using the large file api.                         */
 /* -------------------------------------------------------------------- */
-    poDS->fpImage = VSIFOpenL( poOpenInfo->pszFilename, "rb" );
+    poDS->fpImage = VSIFOpenL( real_filename, "rb" );
+    
     if( poDS->fpImage == NULL )
     {
         CPLError( CE_Failure, CPLE_OpenFailed, 
                   "VSIFOpenL(%s) failed unexpectedly in jpgdataset.cpp", 
-                  poOpenInfo->pszFilename );
+                  real_filename );
         return NULL;
     }
 
+    poDS->nSubfileOffset = subfile_offset;
+    VSIFSeekL( poDS->fpImage, poDS->nSubfileOffset, SEEK_SET );
+
 /* -------------------------------------------------------------------- */
 /*      Take care of EXIF Metadata                                      */
 /* -------------------------------------------------------------------- */
@@ -1101,13 +1274,31 @@ GDALDataset *JPGDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->eAccess = GA_ReadOnly;
 
     poDS->sDInfo.err = jpeg_std_error( &(poDS->sJErr) );
+    poDS->sJErr.error_exit = JPGDataset::ErrorExit;
+    poDS->sDInfo.client_data = (void *) poDS;
 
     jpeg_create_decompress( &(poDS->sDInfo) );
 
+/* -------------------------------------------------------------------- */
+/*      Preload default NITF JPEG quantization tables.                  */
+/* -------------------------------------------------------------------- */
+    poDS->LoadDefaultTables( 0 );
+    poDS->LoadDefaultTables( 1 );
+    poDS->LoadDefaultTables( 2 );
+    poDS->LoadDefaultTables( 3 );
+
+/* -------------------------------------------------------------------- */
+/*      If a fatal error occurs after this, we will return NULL but     */
+/*      not try to cleanup.  Cleaning up after a longjmp() can be       */
+/*      pretty risky.                                                   */
+/* -------------------------------------------------------------------- */
+    if (setjmp(poDS->setjmp_buffer)) 
+        return NULL;
+
 /* -------------------------------------------------------------------- */
 /*	Read pre-image data after ensuring the file is rewound.         */
 /* -------------------------------------------------------------------- */
-    VSIFSeekL( poDS->fpImage, 0, SEEK_SET );
+    VSIFSeekL( poDS->fpImage, poDS->nSubfileOffset, SEEK_SET );
 
     jpeg_vsiio_src( &(poDS->sDInfo), poDS->fpImage );
     jpeg_read_header( &(poDS->sDInfo), TRUE );
@@ -1165,7 +1356,9 @@ GDALDataset *JPGDataset::Open( GDALOpenInfo * poOpenInfo )
 /*      Initialize any PAM information.                                 */
 /* -------------------------------------------------------------------- */
     poDS->SetDescription( poOpenInfo->pszFilename );
-    poDS->TryLoadXML();
+    
+    if( !bIsSubfile )
+        poDS->TryLoadXML();
 
 /* -------------------------------------------------------------------- */
 /*      Check for world file.                                           */
@@ -1181,6 +1374,25 @@ GDALDataset *JPGDataset::Open( GDALOpenInfo * poOpenInfo )
     return poDS;
 }
 
+/************************************************************************/
+/*                             ErrorExit()                              */
+/************************************************************************/
+
+void JPGDataset::ErrorExit(j_common_ptr cinfo)
+{
+    JPGDataset *poDS = (JPGDataset *) cinfo->client_data;
+    char buffer[JMSG_LENGTH_MAX];
+
+    /* Create the message */
+    (*cinfo->err->format_message) (cinfo, buffer);
+
+    CPLError( CE_Failure, CPLE_AppDefined,
+              "libjpeg: %s", buffer );
+
+    /* Return control to the setjmp point */
+    longjmp(poDS->setjmp_buffer, 1);
+}
+
 /************************************************************************/
 /*                           JPEGCreateCopy()                           */
 /************************************************************************/
