@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vsidataio.cpp,v 1.3 2005/09/14 14:19:55 dron Exp $
+ * $Id: vsidataio.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  JPEG JFIF Driver
  * Purpose:  Implement JPEG read/write io indirection through VSI.
@@ -26,23 +26,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- * 
- * $Log: vsidataio.cpp,v $
- * Revision 1.3  2005/09/14 14:19:55  dron
- * Use sizeof() function instead of SIZEOF macro.
- *
- * Revision 1.2  2005/09/11 17:59:01  fwarmerdam
- * Fixed use of ferror().
- *
- * Revision 1.1  2005/09/11 17:15:16  fwarmerdam
- * New
- *
- */
+ ****************************************************************************/
 
 #include "cpl_vsi.h"
 
-CPL_CVSID("$Id: vsidataio.cpp,v 1.3 2005/09/14 14:19:55 dron Exp $");
+CPL_CVSID("$Id: vsidataio.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 CPL_C_START
 #include "jpeglib.h"
