@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: jpeg2000dataset.cpp,v 1.21 2006/04/07 05:37:51 fwarmerdam Exp $
+ * $Id: jpeg2000dataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  JPEG-2000
  * Purpose:  Partial implementation of the ISO/IEC 15444-1 standard
@@ -25,73 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- * 
- * $Log: jpeg2000dataset.cpp,v $
- * Revision 1.21  2006/04/07 05:37:51  fwarmerdam
- * modify to use GDALJP2Metadata class
- *
- * Revision 1.20  2005/05/05 15:54:48  fwarmerdam
- * PAM Enabled
- *
- * Revision 1.19  2004/01/20 16:23:44  dron
- * Don't close streams before deleting the JPEG2000Dataset object.
- *
- * Revision 1.18  2003/05/13 14:00:44  warmerda
- * added mimetype and extension
- *
- * Revision 1.17  2003/04/06 10:13:23  dron
- * Support for pseudocoloured images with LUT.
- *
- * Revision 1.16  2003/03/06 18:08:39  dron
- * Unneeded macro removed.
- *
- * Revision 1.15  2003/02/24 11:06:11  dron
- * Updated to JasPer 1.700.2.
- *
- * Revision 1.14  2003/02/14 11:26:04  dron
- * Colour interpretation improved.
- *
- * Revision 1.13  2003/02/13 21:59:24  dron
- * Fixed problem with cleaning formats before using.
- *
- * Revision 1.12  2003/02/03 19:37:45  dron
- * Fixed problem with component type determining.
- *
- * Revision 1.11  2003/02/03 15:13:53  warmerda
- * avoid warning
- *
- * Revision 1.10  2003/01/30 16:25:35  dron
- * Support for reading and writing GeoJP2 information via hacked JasPer.
- *
- * Revision 1.9  2003/01/30 11:59:15  dron
- * Fixes in component color space handling; several memory leaks removed.
- *
- * Revision 1.8  2003/01/28 14:44:11  dron
- * Temporary hack for reading JP2 boxes (due to lack of appropriate JasPer API).
- *
- * Revision 1.7  2002/11/23 18:54:17  warmerda
- * added CREATIONDATATYPES metadata for drivers
- *
- * Revision 1.6  2002/10/10 11:31:12  dron
- * Fix for buiding GDAL with JasPer software under Windows.
- *
- * Revision 1.5  2002/10/08 07:58:05  dron
- * Added encoding options.
- *
- * Revision 1.4  2002/10/04 15:13:25  dron
- * WORLDFILE support
- *
- * Revision 1.3  2002/09/23 15:47:00  dron
- * CreateCopy() enabled.
- *
- * Revision 1.2  2002/09/19 14:50:02  warmerda
- * added debug statement
- *
- * Revision 1.1  2002/09/18 16:49:01  dron
- * Initial release
- *
- */
+ ****************************************************************************/
 
 #include "gdal_pam.h"
 #include "cpl_string.h"
@@ -99,20 +33,14 @@
 
 #include <jasper/jasper.h>
 
-CPL_CVSID("$Id: jpeg2000dataset.cpp,v 1.21 2006/04/07 05:37:51 fwarmerdam Exp $");
+CPL_CVSID("$Id: jpeg2000dataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 CPL_C_START
 void    GDALRegister_JPEG2000(void);
-#ifdef HAVE_JASPER_UUID
-CPLErr CPL_DLL GTIFMemBufFromWkt( const char *pszWKT, 
-                                  const double *padfGeoTransform,
-                                  int nGCPCount, const GDAL_GCP *pasGCPList,
-                                  int *pnSize, unsigned char **ppabyBuffer );
-#endif
 CPL_C_END
 
 // XXX: Part of code below extracted from the JasPer internal headers and
-// must be in sync with JasPer version (this one works with JasPer 1.700.2)
+// must be in sync with JasPer version (this one works with JasPer 1.900.0)
 #define JP2_FTYP_MAXCOMPATCODES 32
 #define JP2_BOX_IHDR    0x69686472      /* Image Header */
 #define JP2_BOX_BPCC    0x62706363      /* Bits Per Component */
@@ -177,7 +105,7 @@ typedef struct {
 
 #ifdef HAVE_JASPER_UUID
 typedef struct {
-        uint_fast32_t data_len;
+        uint_fast32_t datalen;
         uint_fast8_t uuid[16];
         uint_fast8_t *data;
 } jp2_uuid_t;
@@ -190,10 +118,12 @@ typedef struct {
         struct jp2_boxinfo_s *info;
 
         uint_fast32_t type;
+
+        /* The length of the box including the (variable-length) header. */
         uint_fast32_t len;
-#ifdef HAVE_JASPER_UUID
-        uint_fast32_t data_len;
-#endif
+
+        /* The length of the box data. */
+        uint_fast32_t datalen;
 
         union {
                 jp2_jp_t jp;
@@ -210,6 +140,7 @@ typedef struct {
         } data;
 
 } jp2_box_t;
+
 typedef struct jp2_boxops_s {
         void (*init)(jp2_box_t *box);
         void (*destroy)(jp2_box_t *box);
@@ -229,13 +160,6 @@ int jp2_encode_uuid(jas_image_t *image, jas_stream_t *out,
 }
 // XXX: End of JasPer header.
 
-#ifdef HAVE_JASPER_UUID
-// Magick sequence for GeoJP2 box
-static unsigned char msi_uuid2[16] =
-        {0xb1,0x4b,0xf8,0xbd,0x08,0x3d,0x4b,0x43,
-         0xa5,0xae,0x8c,0xd7,0xd5,0xa6,0xce,0x03}; 
-#endif
-
 /************************************************************************/
 /* ==================================================================== */
 /*                              JPEG2000Dataset                         */
@@ -495,7 +419,7 @@ JPEG2000Dataset::JPEG2000Dataset()
 }
 
 /************************************************************************/
-/*                            ~JPEG2000Dataset()                         */
+/*                            ~JPEG2000Dataset()                        */
 /************************************************************************/
 
 JPEG2000Dataset::~JPEG2000Dataset()
@@ -512,10 +436,7 @@ JPEG2000Dataset::~JPEG2000Dataset()
         CPLFree( pszProjection );
     if( nGCPCount > 0 )
     {
-        for( int i = 0; i < nGCPCount; i++ )
-            if ( pasGCPList[i].pszId )
-                CPLFree( pasGCPList[i].pszId );
-
+        GDALDeinitGCPs( nGCPCount, pasGCPList );
         CPLFree( pasGCPList );
     }
     if( fp != NULL )
@@ -666,6 +587,7 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
                     }
                 }
                 break;
+
                 case JP2_BOX_BPCC:
                 CPLDebug( "JPEG2000", "BPCC box found. Dump:" );
                 if ( !paiDepth && !pabSignedness )
@@ -684,6 +606,7 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
                     }
                 }
                 break;
+
                 case JP2_BOX_PCLR:
                 CPLDebug( "JPEG2000",
                           "PCLR box found. Dump: number of LUT entries=%d, "
@@ -770,13 +693,15 @@ GDALDataset *JPEG2000Dataset::Open( GDALOpenInfo * poOpenInfo )
     
     if( oJP2Geo.ReadAndParse( poOpenInfo->pszFilename ) )
     {
+        if ( poDS->pszProjection )
+            CPLFree( poDS->pszProjection );
         poDS->pszProjection = CPLStrdup(oJP2Geo.pszProjection);
         poDS->bGeoTransformValid = oJP2Geo.bHaveGeoTransform;
         memcpy( poDS->adfGeoTransform, oJP2Geo.adfGeoTransform, 
                 sizeof(double) * 6 );
         poDS->nGCPCount = oJP2Geo.nGCPCount;
-        poDS->pasGCPList = oJP2Geo.pasGCPList;
-        oJP2Geo.pasGCPList = NULL;
+        poDS->pasGCPList =
+            GDALDuplicateGCPs( oJP2Geo.nGCPCount, oJP2Geo.pasGCPList );
     }
 
 /* -------------------------------------------------------------------- */
@@ -1036,26 +961,27 @@ JPEG2000CreateCopy( const char * pszFilename, GDALDataset *poSrcDS,
                      || ABS(adfGeoTransform[5]) != 1.0))
                 || poSrcDS->GetGCPCount() > 0) )
         {
-            // Prepare the memory buffer containing the degenerate GeoTIFF file.
-            const char  *pszWKT;
-            int         nGTBufSize = 0;
-            unsigned char *pabyGTBuf = NULL;
-            jp2_box_t   *box = jp2_box_create( JP2_BOX_UUID );
+            GDALJP2Metadata oJP2Geo;
 
-            if( GDALGetGCPCount( poSrcDS ) > 0 )
-                pszWKT = poSrcDS->GetGCPProjection();
+            if( poSrcDS->GetGCPCount() > 0 )
+            {
+                oJP2Geo.SetProjection( poSrcDS->GetGCPProjection() );
+                oJP2Geo.SetGCPs( poSrcDS->GetGCPCount(), poSrcDS->GetGCPs() );
+            }
             else
-                pszWKT = poSrcDS->GetProjectionRef();
+            {
+                oJP2Geo.SetProjection( poSrcDS->GetProjectionRef() );
+                oJP2Geo.SetGeoTransform( adfGeoTransform );
+            }
 
-            GTIFMemBufFromWkt( pszWKT, adfGeoTransform,
-                               poSrcDS->GetGCPCount(), poSrcDS->GetGCPs(),
-                               &nGTBufSize, &pabyGTBuf );
- 
-            memcpy( box->data.uuid.uuid, msi_uuid2, sizeof(msi_uuid2) );
-            box->data.uuid.data_len = nGTBufSize;
-            box->data.uuid.data = (uint_fast8_t *)jas_malloc( nGTBufSize );
-            memcpy( box->data.uuid.data, pabyGTBuf, nGTBufSize );
-            CPLFree( pabyGTBuf );
+            GDALJP2Box *poBox = oJP2Geo.CreateJP2GeoTIFF();
+            jp2_box_t  *box = jp2_box_create( JP2_BOX_UUID );
+            memcpy( box->data.uuid.uuid, poBox->GetUUID(), 16 );
+            box->data.uuid.datalen = poBox->GetDataLength() - 16;
+            box->data.uuid.data =
+                (uint_fast8_t *)jas_malloc( poBox->GetDataLength() - 16 );
+            memcpy( box->data.uuid.data, poBox->GetWritableData() + 16,
+                    poBox->GetDataLength() - 16 );
             if ( jp2_encode_uuid( psImage, psStream, pszOptionBuf, box) < 0 )
             {
                 CPLError( CE_Failure, CPLE_FileIO,
