@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: msg_basic_types.cpp,v 1.3 2006/04/04 12:59:28 fwarmerdam Exp $
+ * $Id: msg_basic_types.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  MSG Native Reader
  * Purpose:  Basic types implementation.
@@ -25,18 +25,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: msg_basic_types.cpp,v $
- * Revision 1.3  2006/04/04 12:59:28  fwarmerdam
- * added headers with explicit permission from Frans
- *
- */
+ ****************************************************************************/
 
 #include "msg_basic_types.h"
 #include "cpl_port.h"
 
-CPL_CVSID("$Id: msg_basic_types.cpp,v 1.3 2006/04/04 12:59:28 fwarmerdam Exp $");
+CPL_CVSID("$Id: msg_basic_types.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 #include <stdio.h>
 
