@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: nitfdataset.cpp,v 1.51 2006/01/05 23:12:03 fwarmerdam Exp $
+ * $Id: nitfdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  NITF Read/Write Translator
  * Purpose:  GDALDataset/GDALRasterBand implementation on top of "nitflib".
@@ -8,6 +8,9 @@
  ******************************************************************************
  * Copyright (c) 2002, Frank Warmerdam
  *
+ * Portions Copyright (c) Her majesty the Queen in right of Canada as
+ * represented by the Minister of National Defence, 2006.
+ *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
  * to deal in the Software without restriction, including without limitation
@@ -25,94 +28,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: nitfdataset.cpp,v $
- * Revision 1.51  2006/01/05 23:12:03  fwarmerdam
- * Added subdataset support to access multiple images in one file.
- *
- * Revision 1.50  2005/12/21 01:06:29  fwarmerdam
- * Added IMAGE_STRUCTURE COMPRESSION support.
- *
- * Revision 1.49  2005/10/03 17:39:45  fwarmerdam
- * fixed memory leak of GCPs as per bug 920
- *
- * Revision 1.48  2005/09/19 21:55:24  gwalter
- * Avoid creating gcps when geotransform already found.
- *
- * Revision 1.47  2005/09/14 12:54:40  dron
- * Avoid possible using of uninitialized value.
- *
- * Revision 1.46  2005/09/14 01:12:52  fwarmerdam
- * Removed complex types from creatable formats, apparently not
- * currently supported.
- *
- * Revision 1.45  2005/07/28 20:00:57  fwarmerdam
- * upgrade to support 2-4GB files, use large file api
- *
- * Revision 1.44  2005/05/23 06:57:36  fwarmerdam
- * fix flushing to go through pam
- *
- * Revision 1.43  2005/05/05 15:54:49  fwarmerdam
- * PAM Enabled
- *
- * Revision 1.42  2005/04/18 17:58:12  fwarmerdam
- * fixed up ICORDS=' ' case to mean no geotransform
- *
- * Revision 1.41  2005/04/18 15:45:17  gwalter
- * Check for Space Imaging style .hdr + .nfw, and
- * default to these rather than in-file geocoding
- * information if both files are present and have
- * recognizable information.
- *
- * Revision 1.40  2005/04/18 13:40:22  fwarmerdam
- * Remove the default geotransform if we don't have a valid geotransform.
- *
- * Revision 1.39  2005/03/21 16:25:41  fwarmerdam
- * When writing IGEOLO adjust for center of corner pixels instead of
- * outer image corners.  Also support generating geotransform from
- * rotated IGEOLO values.
- *
- * Revision 1.38  2005/03/17 19:34:31  fwarmerdam
- * Added IC and IMODE metadata.
- *
- * Revision 1.37  2005/03/01 14:28:06  fwarmerdam
- * Added support for overriding JPEG2000 profile.
- *
- * Revision 1.36  2005/02/25 17:01:18  fwarmerdam
- * forward AdviseRead and IRasterIO on dataset to j2kdataset if needed
- *
- * Revision 1.35  2005/02/24 15:11:23  fwarmerdam
- * fixed improper j2kdataset cleanup in ~NITFDataset
- *
- * Revision 1.34  2005/02/22 08:20:58  fwarmerdam
- * ensure we can capture color interp on jpeg2000 streams
- *
- * Revision 1.33  2005/02/18 19:29:16  fwarmerdam
- * added SetColorIntepretation support: bug 752
- *
- * Revision 1.32  2005/02/10 04:49:31  fwarmerdam
- * added color interpretation setting on J2K bands
- *
- * Revision 1.31  2005/02/08 05:46:33  fwarmerdam
- * Fixed up ~NITFDataset() image length/COMRAT patching.
- *
- * Revision 1.30  2005/02/08 05:20:16  fwarmerdam
- * preliminary implementation of JPEG2000 support via Create()
- *
- * Revision 1.29  2005/01/28 03:47:26  fwarmerdam
- * Fixed return value of SetGeoTransform().
- *
- * Revision 1.28  2005/01/15 07:47:20  fwarmerdam
- * enable NPJE profile
- */
+ ****************************************************************************/
 
 #include "gdal_pam.h"
 #include "nitflib.h"
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: nitfdataset.cpp,v 1.51 2006/01/05 23:12:03 fwarmerdam Exp $");
+CPL_CVSID("$Id: nitfdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static void NITFPatchImageLength( const char *pszFilename,
                                   long nImageOffset, 
@@ -141,6 +64,8 @@ class NITFDataset : public GDALPamDataset
     GDALDataset *poJ2KDataset;
     int         bJP2Writing;
 
+    GDALDataset *poJPEGDataset;
+
     int         bGotGeoTransform;
     double      adfGeoTransform[6];
 
@@ -150,6 +75,21 @@ class NITFDataset : public GDALPamDataset
     GDAL_GCP    *pasGCPList;
     char        *pszGCPProjection;
 
+    char       **papszCGMMetadata;
+
+    void         InitializeCGMMetadata();
+
+
+    int         *panJPEGBlockOffset;
+    GByte       *pabyJPEGBlock;
+    int          nQLevel;
+
+    int          ScanJPEGQLevel( GUInt32 *pnDataStart );
+    CPLErr       ScanJPEGBlocks( void );
+    CPLErr       ReadJPEGBlock( int, int );
+
+    CPLString    osNITFFilename;
+
   public:
                  NITFDataset();
                  ~NITFDataset();
@@ -172,6 +112,9 @@ class NITFDataset : public GDALPamDataset
     virtual const char *GetGCPProjection();
     virtual const GDAL_GCP *GetGCPs();
 
+    virtual char      **GetMetadata( const char * pszDomain = "" );
+    virtual const char *GetMetadataItem( const char * pszName,
+                                         const char * pszDomain = "" );
     virtual void   FlushCache();
 
     static GDALDataset *Open( GDALOpenInfo * );
@@ -317,6 +260,25 @@ CPLErr NITFRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
 
 {
     int  nBlockResult;
+    NITFDataset *poGDS = (NITFDataset *) poDS;
+
+/* -------------------------------------------------------------------- */
+/*      Special case for JPEG blocks.                                   */
+/* -------------------------------------------------------------------- */
+    if( EQUAL(psImage->szIC,"C3") )
+    {
+        CPLErr eErr = poGDS->ReadJPEGBlock( nBlockXOff, nBlockYOff );
+        int nBlockBandSize = psImage->nBlockWidth*psImage->nBlockHeight;
+
+        if( eErr != CE_None )
+            return eErr;
+
+        memcpy( pImage, 
+                poGDS->pabyJPEGBlock + (nBand - 1) * nBlockBandSize, 
+                nBlockBandSize );
+
+        return eErr;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Read the line/block                                             */
@@ -560,6 +522,11 @@ NITFDataset::NITFDataset()
     pszProjection = CPLStrdup("");
     poJ2KDataset = NULL;
     bJP2Writing = FALSE;
+    poJPEGDataset = NULL;
+
+    panJPEGBlockOffset = NULL;
+    pabyJPEGBlock = NULL;
+    nQLevel = 0;
 
     nGCPCount = 0;
     pasGCPList = NULL;
@@ -571,6 +538,8 @@ NITFDataset::NITFDataset()
     adfGeoTransform[3] = 0.0;
     adfGeoTransform[4] = 0.0;
     adfGeoTransform[5] = 1.0;
+
+    papszCGMMetadata = NULL;
 }
 
 /************************************************************************/
@@ -647,6 +616,25 @@ NITFDataset::~NITFDataset()
         NITFPatchImageLength( GetDescription(), nImageStart, nPixelCount );
     }
 
+/* -------------------------------------------------------------------- */
+/*      If we have a jpeg output file, make sure it gets closed         */
+/*      and flushed out.                                                */
+/* -------------------------------------------------------------------- */
+    if( poJPEGDataset != NULL )
+    {
+        int i;
+
+        GDALClose( (GDALDatasetH) poJPEGDataset );
+        
+        // the bands are really jpeg bands ... remove them 
+        // from the NITF list so they won't get destroyed twice.
+        for( i = 0; i < nBands && papoBands != NULL; i++ )
+            papoBands[i] = NULL;
+    }
+
+    CSLDestroy( papszCGMMetadata );
+    CPLFree( panJPEGBlockOffset );
+    CPLFree( pabyJPEGBlock );
 }
 
 /************************************************************************/
@@ -737,12 +725,10 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if( psImage == NULL )
     {
-        CPLError( CE_Failure, CPLE_AppDefined, 
+        CPLError( CE_Warning, CPLE_AppDefined, 
                   "The file %s appears to be an NITF file, but no image\n"
-                  "blocks were found on it.  GDAL cannot utilize non-image\n"
-                  "NITF files.", 
+                  "blocks were found on it.", 
                   poOpenInfo->pszFilename );
-        return NULL;
     }
     
 /* -------------------------------------------------------------------- */
@@ -755,18 +741,30 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
     poDS->psFile = psFile;
     poDS->psImage = psImage;
     poDS->eAccess = poOpenInfo->eAccess;
+    poDS->osNITFFilename = pszFilename;
 
-    poDS->nRasterXSize = psImage->nCols;
-    poDS->nRasterYSize = psImage->nRows;
+    if( psImage )
+    {
+        poDS->nRasterXSize = psImage->nCols;
+        poDS->nRasterYSize = psImage->nRows;
+    }
+    else
+    {
+        poDS->nRasterXSize = 1;
+        poDS->nRasterYSize = 1;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      If the image is JPEG2000 (C8) compressed, we will need to       */
 /*      open the image data as a JPEG2000 dataset.                      */
 /* -------------------------------------------------------------------- */
-    int nUsableBands = psImage->nBands;
+    int nUsableBands = 0;
     int		iBand;
 
-    if( EQUAL(psImage->szIC,"C8") )
+    if( psImage )
+        nUsableBands = psImage->nBands;
+
+    if( psImage != NULL && EQUAL(psImage->szIC,"C8") )
     {
         char *pszDSName = CPLStrdup( 
             CPLSPrintf( "J2K_SUBFILE:%d,%d,%s", 
@@ -828,18 +826,98 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
         }
     }
 
+/* -------------------------------------------------------------------- */
+/*      If the image is JPEG (C3) compressed, we will need to open      */
+/*      the image data as a JPEG dataset.                               */
+/* -------------------------------------------------------------------- */
+    else if( psImage != NULL
+             && EQUAL(psImage->szIC,"C3") 
+             && psImage->nBlocksPerRow == 1
+             && psImage->nBlocksPerColumn == 1 )
+    {
+        GUInt32 nJPEGStart = psFile->pasSegmentInfo[iSegment].nSegmentStart;
+
+        poDS->nQLevel = poDS->ScanJPEGQLevel( &nJPEGStart );
+
+        char *pszDSName = CPLStrdup( 
+            CPLSPrintf( "JPEG_SUBFILE:Q%d,%d,%d,%s", 
+                        poDS->nQLevel,
+                        nJPEGStart,
+                        psFile->pasSegmentInfo[iSegment].nSegmentSize
+                        - (nJPEGStart - psFile->pasSegmentInfo[iSegment].nSegmentStart),
+                        pszFilename ) );
+
+        CPLDebug( "GDAL", 
+                  "NITFDataset::Open() as IC=C3 (JPEG compressed)\n");
+
+        poDS->poJPEGDataset = (GDALDataset *) GDALOpen( pszDSName, GA_ReadOnly );
+        CPLFree( pszDSName );
+
+        if( poDS->poJPEGDataset == NULL )
+        {
+            CPLError( CE_Failure, CPLE_AppDefined, 
+                      "Unable to open JPEG image within NITF file.\n"
+                      "Is the JPEG driver available?" );
+            delete poDS;
+            return NULL;
+        }
+
+        if( poDS->poJPEGDataset->GetRasterCount() < nUsableBands )
+        {
+            CPLError( CE_Warning, CPLE_AppDefined, 
+                      "JPEG data stream has less useful bands than expected, likely\n"
+                      "because some channels have differing resolutions." );
+            
+            nUsableBands = poDS->poJPEGDataset->GetRasterCount();
+        }
+
+        // Force NITF derived color space info 
+        for( iBand = 0; iBand < nUsableBands; iBand++ )
+        {
+            NITFBandInfo *psBandInfo = psImage->pasBandInfo + iBand;
+            GDALRasterBand *poBand=poDS->poJPEGDataset->GetRasterBand(iBand+1);
+            
+            if( EQUAL(psBandInfo->szIREPBAND,"R") )
+                poBand->SetColorInterpretation( GCI_RedBand );
+            if( EQUAL(psBandInfo->szIREPBAND,"G") )
+                poBand->SetColorInterpretation( GCI_GreenBand );
+            if( EQUAL(psBandInfo->szIREPBAND,"B") )
+                poBand->SetColorInterpretation( GCI_BlueBand );
+            if( EQUAL(psBandInfo->szIREPBAND,"M") )
+                poBand->SetColorInterpretation( GCI_GrayIndex );
+            if( EQUAL(psBandInfo->szIREPBAND,"Y") )
+                poBand->SetColorInterpretation( GCI_YCbCr_YBand );
+            if( EQUAL(psBandInfo->szIREPBAND,"Cb") )
+                poBand->SetColorInterpretation( GCI_YCbCr_CbBand );
+            if( EQUAL(psBandInfo->szIREPBAND,"Cr") )
+                poBand->SetColorInterpretation( GCI_YCbCr_CrBand );
+        }
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
     for( iBand = 0; iBand < nUsableBands; iBand++ )
     {
-        if( poDS->poJ2KDataset == NULL )
-            poDS->SetBand( iBand+1, new NITFRasterBand( poDS, iBand+1 ) );
-        else
-        {
+        if( poDS->poJ2KDataset != NULL )
             poDS->SetBand( iBand+1, 
                            poDS->poJ2KDataset->GetRasterBand(iBand+1) );
-        }
+        else if( poDS->poJPEGDataset != NULL )
+            poDS->SetBand( iBand+1, 
+                           poDS->poJPEGDataset->GetRasterBand(iBand+1) );
+        else
+            poDS->SetBand( iBand+1, new NITFRasterBand( poDS, iBand+1 ) );
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Report problems with odd bit sizes.                             */
+/* -------------------------------------------------------------------- */
+    if( psImage != NULL 
+        && (psImage->nBitsPerSample < 8 || psImage->nBitsPerSample % 8 != 0) )
+    {
+        CPLError( CE_Warning, CPLE_AppDefined, 
+                  "Image with %d bits per sample will not be interpreted properly.", 
+                  psImage->nBitsPerSample );
     }
 
 /* -------------------------------------------------------------------- */
@@ -847,7 +925,11 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     OGRSpatialReference oSRSWork;
 
-    if( psImage->chICORDS == 'G'  )
+    if( psImage == NULL )
+    {
+        /* nothing */
+    }
+    else if( psImage->chICORDS == 'G' || psImage->chICORDS == 'D' )
     {
         CPLFree( poDS->pszProjection );
         poDS->pszProjection = NULL;
@@ -901,8 +983,9 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Try looking for a .nfw file.                                    */
 /* -------------------------------------------------------------------- */
-    if( GDALReadWorldFile( pszFilename, "nfw", 
-                           poDS->adfGeoTransform ) )
+    if( psImage
+        && GDALReadWorldFile( pszFilename, "nfw", 
+                              poDS->adfGeoTransform ) )
     {
         const char *pszHDR;
         FILE *fpHDR;
@@ -980,7 +1063,7 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
     int nGCPCount = 0;
     GDAL_GCP    *psGCPs = NULL;
 
-    if( ( poDS->bGotGeoTransform == FALSE ) && ( psImage->chICORDS != ' ' ) )
+    if( psImage && !poDS->bGotGeoTransform && psImage->chICORDS != ' ' )
     {
         nGCPCount = 4;
 
@@ -1011,9 +1094,14 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Convert the GCPs into a geotransform definition, if possible.   */
 /* -------------------------------------------------------------------- */
-    if(  ( poDS->bGotGeoTransform == FALSE ) && nGCPCount > 0 && 
-         GDALGCPsToGeoTransform( nGCPCount, psGCPs, 
-                                 poDS->adfGeoTransform, TRUE ) )
+    if( !psImage )
+    {
+        /* nothing */
+    }
+    else if( poDS->bGotGeoTransform == FALSE 
+             && nGCPCount > 0 
+             && GDALGCPsToGeoTransform( nGCPCount, psGCPs, 
+                                        poDS->adfGeoTransform, TRUE ) )
     {	
         poDS->bGotGeoTransform = TRUE;
     } 
@@ -1082,51 +1170,87 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
 
     // File and Image level metadata.
     papszMergedMD = CSLDuplicate( poDS->psFile->papszMetadata );
-    papszMergedMD = CSLInsertStrings( papszMergedMD, 
-                                      CSLCount( papszMergedMD ),
-                                      psImage->papszMetadata );
 
-    // Comments.
-    if( psImage->pszComments != NULL && strlen(psImage->pszComments) != 0 )
-        papszMergedMD = CSLSetNameValue( 
-            papszMergedMD, "NITF_IMAGE_COMMENTS", psImage->pszComments );
-
-    // Compression code. 
-    papszMergedMD = CSLSetNameValue( papszMergedMD, "NITF_IC", 
-                                     psImage->szIC );
-
-    // IMODE
-    char szIMODE[2];
-    szIMODE[0] = psImage->chIMODE;
-    szIMODE[1] = '\0';
-    papszMergedMD = CSLSetNameValue( papszMergedMD, "NITF_IMODE", szIMODE );
-
-    // USE00A 
-    papszUSE00A_MD = NITFReadUSE00A( psImage );
-    if( papszUSE00A_MD != NULL )
+    if( psImage )
     {
         papszMergedMD = CSLInsertStrings( papszMergedMD, 
-                                          CSLCount( papszUSE00A_MD ),
-                                          papszUSE00A_MD );
-        CSLDestroy( papszUSE00A_MD );
-    }
-    
-    papszUSE00A_MD = NITFReadSTDIDC( psImage );
-    if( papszUSE00A_MD != NULL )
-    {
-        papszMergedMD = CSLInsertStrings( papszMergedMD, 
-                                          CSLCount( papszUSE00A_MD ),
-                                          papszUSE00A_MD );
-        CSLDestroy( papszUSE00A_MD );
-    }
+                                          CSLCount( papszMergedMD ),
+                                          psImage->papszMetadata );
 
+        // Comments.
+        if( psImage->pszComments != NULL && strlen(psImage->pszComments) != 0 )
+            papszMergedMD = CSLSetNameValue( 
+                papszMergedMD, "NITF_IMAGE_COMMENTS", psImage->pszComments );
+        
+        // Compression code. 
+        papszMergedMD = CSLSetNameValue( papszMergedMD, "NITF_IC", 
+                                         psImage->szIC );
+        
+        // IMODE
+        char szIMODE[2];
+        szIMODE[0] = psImage->chIMODE;
+        szIMODE[1] = '\0';
+        papszMergedMD = CSLSetNameValue( papszMergedMD, "NITF_IMODE", szIMODE );
+
+        // ILOC/Attachment info
+        if( psImage->nILOCRow != 0 )
+        {
+            papszMergedMD = 
+                CSLSetNameValue( papszMergedMD, "NITF_IDLVL", 
+                                 CPLString().Printf("%d",psImage->nIDLVL) );
+            papszMergedMD = 
+                CSLSetNameValue( papszMergedMD, "NITF_IALVL", 
+                                 CPLString().Printf("%d",psImage->nIALVL) );
+            papszMergedMD = 
+                CSLSetNameValue( papszMergedMD, "NITF_ILOC_ROW", 
+                                 CPLString().Printf("%d",psImage->nILOCRow) );
+            papszMergedMD = 
+                CSLSetNameValue( papszMergedMD, "NITF_ILOC_COLUMN", 
+                                CPLString().Printf("%d",psImage->nILOCColumn));
+            papszMergedMD = 
+                CSLSetNameValue( papszMergedMD, "NITF_IMAG", 
+                                 psImage->szIMAG );
+        }
+
+        // USE00A 
+        papszUSE00A_MD = NITFReadUSE00A( psImage );
+        if( papszUSE00A_MD != NULL )
+        {
+            papszMergedMD = CSLInsertStrings( papszMergedMD, 
+                                              CSLCount( papszUSE00A_MD ),
+                                              papszUSE00A_MD );
+            CSLDestroy( papszUSE00A_MD );
+        }
+        
+        // BLOCKA 
+        papszUSE00A_MD = NITFReadBLOCKA( psImage );
+        if( papszUSE00A_MD != NULL )
+        {
+            papszMergedMD = CSLInsertStrings( papszMergedMD, 
+                                              CSLCount( papszUSE00A_MD ),
+                                              papszUSE00A_MD );
+            CSLDestroy( papszUSE00A_MD );
+        }
+        
+        papszUSE00A_MD = NITFReadSTDIDC( psImage );
+        if( papszUSE00A_MD != NULL )
+        {
+            papszMergedMD = CSLInsertStrings( papszMergedMD, 
+                                              CSLCount( papszUSE00A_MD ),
+                                              papszUSE00A_MD );
+            CSLDestroy( papszUSE00A_MD );
+        }
+    }
+        
     poDS->SetMetadata( papszMergedMD );
     CSLDestroy( papszMergedMD );
 
 /* -------------------------------------------------------------------- */
 /*      Image structure metadata.                                       */
 /* -------------------------------------------------------------------- */
-    if( psImage->szIC[1] == '1' )
+    if( psImage == NULL )
+        /* do nothing */;
+    else if( psImage->szIC[1] == '1' )
         poDS->SetMetadataItem( "COMPRESSION", "BILEVEL", 
                                "IMAGE_STRUCTURE" );
     else if( psImage->szIC[1] == '2' )
@@ -1150,7 +1274,8 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     NITFRPC00BInfo sRPCInfo;
 
-    if( NITFReadRPC00B( psImage, &sRPCInfo ) && sRPCInfo.SUCCESS )
+    if( psImage
+        && NITFReadRPC00B( psImage, &sRPCInfo ) && sRPCInfo.SUCCESS )
     {
         char szValue[1280];
         int  i;
@@ -1231,7 +1356,8 @@ GDALDataset *NITFDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     NITFICHIPBInfo sChipInfo;
 
-    if( NITFReadICHIPB( psImage, &sChipInfo ) && sChipInfo.XFRM_FLAG == 0 )
+    if( psImage
+        && NITFReadICHIPB( psImage, &sChipInfo ) && sChipInfo.XFRM_FLAG == 0 )
     {
         char szValue[1280];
 
@@ -1366,6 +1492,11 @@ CPLErr NITFDataset::AdviseRead( int nXOff, int nYOff, int nXSize, int nYSize,
                                         nBufXSize, nBufYSize, eDT, 
                                         nBandCount, panBandList, 
                                         papszOptions);
+    else if( poJPEGDataset != NULL )
+        return poJPEGDataset->AdviseRead( nXOff, nYOff, nXSize, nYSize, 
+                                          nBufXSize, nBufYSize, eDT, 
+                                          nBandCount, panBandList, 
+                                          papszOptions);
     else
         return poJ2KDataset->AdviseRead( nXOff, nYOff, nXSize, nYSize, 
                                          nBufXSize, nBufYSize, eDT, 
@@ -1385,13 +1516,18 @@ CPLErr NITFDataset::IRasterIO( GDALRWFlag eRWFlag,
                                int nPixelSpace, int nLineSpace, int nBandSpace)
     
 {
-    if( poJ2KDataset == NULL )
-        return GDALDataset::IRasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
+    if( poJ2KDataset != NULL )
+        return poJ2KDataset->RasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
                                        pData, nBufXSize, nBufYSize, eBufType,
                                        nBandCount, panBandMap, 
                                        nPixelSpace, nLineSpace, nBandSpace );
-    else
-        return poJ2KDataset->RasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
+    else if( poJPEGDataset != NULL )
+        return poJPEGDataset->RasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
+                                        pData, nBufXSize, nBufYSize, eBufType,
+                                        nBandCount, panBandMap, 
+                                        nPixelSpace, nLineSpace, nBandSpace );
+    else 
+        return GDALDataset::IRasterIO( eRWFlag, nXOff, nYOff, nXSize, nYSize,
                                        pData, nBufXSize, nBufYSize, eBufType,
                                        nBandCount, panBandMap, 
                                        nPixelSpace, nLineSpace, nBandSpace );
@@ -1459,6 +1595,149 @@ const char *NITFDataset::GetProjectionRef()
         return "";
 }
 
+/************************************************************************/
+/*                       InitializeCGMMetadata()                        */
+/************************************************************************/
+
+void NITFDataset::InitializeCGMMetadata()
+
+{
+    int iSegment;
+    int iCGM = 0;
+
+    if( papszCGMMetadata != NULL )
+        return;
+
+    papszCGMMetadata = 
+        CSLSetNameValue( papszCGMMetadata, "SEGMENT_COUNT", "0" );
+
+/* ==================================================================== */
+/*      Process all graphics segments.                                  */
+/* ==================================================================== */
+    for( iSegment = 0; iSegment < psFile->nSegmentCount; iSegment++ )
+    {
+        NITFSegmentInfo *psSegment = psFile->pasSegmentInfo + iSegment;
+
+        if( !EQUAL(psSegment->szSegmentType,"GR") 
+            && !EQUAL(psSegment->szSegmentType,"SY") )
+            continue;
+
+/* -------------------------------------------------------------------- */
+/*      Load the graphic subheader.                                     */
+/* -------------------------------------------------------------------- */
+        char achSubheader[298];
+        int  nSTYPEOffset;
+
+        if( VSIFSeekL( psFile->fp, psSegment->nSegmentHeaderStart, 
+                       SEEK_SET ) != 0 
+            || VSIFReadL( achSubheader, 1, sizeof(achSubheader), 
+                          psFile->fp ) < 258 )
+        {
+            CPLError( CE_Warning, CPLE_FileIO, 
+                      "Failed to read graphic subheader at %d.", 
+                      psSegment->nSegmentHeaderStart );
+            return;
+        }
+
+        // NITF 2.0. (also works for NITF 2.1)
+        nSTYPEOffset = 200;
+        if( EQUALN(achSubheader+193,"999998",6) )
+            nSTYPEOffset += 40;
+
+        // We don't want bitmaps or anything other than cgm for now.
+        if( achSubheader[nSTYPEOffset] != 'C' )
+            continue;
+
+        char szSLOC_R[6], szSLOC_C[6];
+
+        strncpy( szSLOC_R, achSubheader + nSTYPEOffset + 20, 5 );
+        strncpy( szSLOC_C, achSubheader + nSTYPEOffset + 20 + 5, 5 );
+        
+        szSLOC_R[5] = '\0';
+        szSLOC_C[5] = '\0';
+
+        papszCGMMetadata = 
+            CSLSetNameValue( papszCGMMetadata, 
+                             CPLString().Printf("SEGMENT_%d_SLOC_ROW", iCGM), 
+                             szSLOC_R );
+        papszCGMMetadata = 
+            CSLSetNameValue( papszCGMMetadata, 
+                             CPLString().Printf("SEGMENT_%d_SLOC_COL", iCGM), 
+                             szSLOC_C );
+
+/* -------------------------------------------------------------------- */
+/*      Load the raw CGM data itself.                                   */
+/* -------------------------------------------------------------------- */
+        char *pabyCGMData, *pszEscapedCGMData;
+
+        pabyCGMData = (char *) CPLCalloc(1,psSegment->nSegmentSize);
+        if( VSIFSeekL( psFile->fp, psSegment->nSegmentStart, 
+                       SEEK_SET ) != 0 
+            || VSIFReadL( pabyCGMData, 1, psSegment->nSegmentSize, 
+                          psFile->fp ) != psSegment->nSegmentSize )
+        {
+            CPLError( CE_Warning, CPLE_FileIO, 
+                      "Failed to read %d bytes of graphic data at %d.", 
+                      psSegment->nSegmentSize,
+                      psSegment->nSegmentStart );
+            return;
+        }
+
+        pszEscapedCGMData = CPLEscapeString( pabyCGMData, 
+                                             psSegment->nSegmentSize, 
+                                             CPLES_BackslashQuotable );
+
+        papszCGMMetadata = 
+            CSLSetNameValue( papszCGMMetadata, 
+                             CPLString().Printf("SEGMENT_%d_DATA", iCGM), 
+                             pszEscapedCGMData );
+        CPLFree( pszEscapedCGMData );
+        CPLFree( pabyCGMData );
+
+        iCGM++;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Record the CGM segment count.                                   */
+/* -------------------------------------------------------------------- */
+    papszCGMMetadata = 
+        CSLSetNameValue( papszCGMMetadata, 
+                         "SEGMENT_COUNT", 
+                         CPLString().Printf( "%d", iCGM ) );
+}
+
+/************************************************************************/
+/*                            GetMetadata()                             */
+/************************************************************************/
+
+char **NITFDataset::GetMetadata( const char * pszDomain )
+
+{
+    if( pszDomain == NULL || !EQUAL(pszDomain,"CGM") )
+        return GDALPamDataset::GetMetadata( pszDomain );
+
+    InitializeCGMMetadata();
+
+    return papszCGMMetadata;
+}
+
+/************************************************************************/
+/*                          GetMetadataItem()                           */
+/************************************************************************/
+
+const char *NITFDataset::GetMetadataItem(const char * pszName,
+                                         const char * pszDomain )
+
+{
+    if( pszDomain == NULL || !EQUAL(pszDomain,"CGM") )
+        return GDALPamDataset::GetMetadataItem( pszName, pszDomain );
+
+    InitializeCGMMetadata();
+
+    return CSLFetchNameValue( papszCGMMetadata, pszName );
+}
+
+
 /************************************************************************/
 /*                            GetGCPCount()                             */
 /************************************************************************/
@@ -1492,6 +1771,208 @@ const GDAL_GCP *NITFDataset::GetGCPs()
     return pasGCPList;
 }
 
+/************************************************************************/
+/*                           ScanJPEGQLevel()                           */
+/*                                                                      */
+/*      Search the NITF APP header in the jpeg data stream to find      */
+/*      out what predefined Q level tables should be used (or -1 if     */
+/*      they are inline).                                               */
+/************************************************************************/
+
+int NITFDataset::ScanJPEGQLevel( GUInt32 *pnDataStart )
+
+{
+    GByte abyHeader[100];
+
+    if( VSIFSeekL( psFile->fp, *pnDataStart,
+                   SEEK_SET ) != 0 )
+    {
+        CPLError( CE_Failure, CPLE_FileIO, 
+                  "Seek error to jpeg data stream." );
+        return 0;
+    }
+        
+    if( VSIFReadL( abyHeader, 1, sizeof(abyHeader), psFile->fp ) 
+        < sizeof(abyHeader) )
+    {
+        CPLError( CE_Failure, CPLE_FileIO, 
+                  "Read error to jpeg data stream." );
+        return 0;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Scan ahead for jpeg magic code.  In some files (eg. NSIF)       */
+/*      there seems to be some extra junk before the image data stream. */
+/* -------------------------------------------------------------------- */
+    GUInt32 nOffset = 0;
+    while( nOffset < sizeof(abyHeader) - 23 
+           && (abyHeader[nOffset+0] != 0xff
+               || abyHeader[nOffset+1] != 0xd8
+               || abyHeader[nOffset+2] != 0xff) )
+        nOffset++;
+
+    if( nOffset >= sizeof(abyHeader) - 23 )
+        return 0;
+
+    *pnDataStart += nOffset;
+
+    if( nOffset > 0 )
+        CPLDebug( "NITF", 
+                  "JPEG data stream at offset %d from start of data segement, NSIF?", 
+                  nOffset );
+
+/* -------------------------------------------------------------------- */
+/*      Do we have an NITF app tag?  If so, pull out the Q level.       */
+/* -------------------------------------------------------------------- */
+    if( !EQUAL((char *)abyHeader+nOffset+6,"NITF") )
+        return 0;
+
+    return abyHeader[22+nOffset];
+}
+
+/************************************************************************/
+/*                           ScanJPEGBlocks()                           */
+/************************************************************************/
+
+CPLErr NITFDataset::ScanJPEGBlocks()
+
+{
+    int iBlock;
+    GUInt32 nJPEGStart = 
+        psFile->pasSegmentInfo[psImage->iSegment].nSegmentStart;
+
+    nQLevel = ScanJPEGQLevel( &nJPEGStart );
+
+/* -------------------------------------------------------------------- */
+/*      Allocate offset array                                           */
+/* -------------------------------------------------------------------- */
+    panJPEGBlockOffset = (int *) 
+        CPLCalloc(sizeof(int),
+                  psImage->nBlocksPerRow*psImage->nBlocksPerColumn+1);
+    panJPEGBlockOffset[0] = nJPEGStart;
+
+    for( iBlock = psImage->nBlocksPerRow * psImage->nBlocksPerColumn - 1;
+         iBlock > 0; iBlock-- )
+        panJPEGBlockOffset[iBlock] = -1;
+    
+/* -------------------------------------------------------------------- */
+/*      Scan through the whole image data stream identifying all        */
+/*      block boundaries.  Each block starts with 0xFFD8 (SOI).         */
+/*      They also end with 0xFFD9, but we don't currently look for      */
+/*      that.                                                           */
+/* -------------------------------------------------------------------- */
+    int iNextBlock = 1;
+    int iSegOffset = 2;
+    int iSegSize = psFile->pasSegmentInfo[psImage->iSegment].nSegmentSize
+        - (nJPEGStart - psFile->pasSegmentInfo[psImage->iSegment].nSegmentStart);
+    GByte abyBlock[512];
+
+    while( iSegOffset < iSegSize-1 )
+    {
+        int nReadSize = MIN((int)sizeof(abyBlock),iSegSize - iSegOffset);
+        int i;
+
+        if( VSIFSeekL( psFile->fp, panJPEGBlockOffset[0] + iSegOffset, 
+                       SEEK_SET ) != 0 )
+        {
+            CPLError( CE_Failure, CPLE_FileIO, 
+                      "Seek error to jpeg data stream." );
+            return CE_Failure;
+        }
+        
+        if( VSIFReadL( abyBlock, 1, nReadSize, psFile->fp ) < (size_t)nReadSize)
+        {
+            CPLError( CE_Failure, CPLE_FileIO, 
+                      "Read error to jpeg data stream." );
+            return CE_Failure;
+        }
+        
+        for( i = 0; i < nReadSize-1; i++ )
+        {
+            if( abyBlock[i] == 0xff && abyBlock[i+1] == 0xd8 )
+            {
+                CPLAssert( iNextBlock 
+                          <= psImage->nBlocksPerRow*psImage->nBlocksPerColumn );
+
+                panJPEGBlockOffset[iNextBlock++] 
+                    = panJPEGBlockOffset[0] + iSegOffset + i; 
+            }
+        }
+
+        iSegOffset += nReadSize - 1;
+    }
+
+    return CE_None;
+}
+
+/************************************************************************/
+/*                           ReadJPEGBlock()                            */
+/************************************************************************/
+
+CPLErr NITFDataset::ReadJPEGBlock( int iBlockX, int iBlockY )
+
+{
+    CPLErr eErr;
+
+/* -------------------------------------------------------------------- */
+/*      If this is our first request, do a scan for block boundaries.   */
+/* -------------------------------------------------------------------- */
+    if( panJPEGBlockOffset == NULL )
+    {
+        eErr = ScanJPEGBlocks();
+        if( eErr != CE_None )
+            return eErr;
+    }
+    
+/* -------------------------------------------------------------------- */
+/*    Allocate image data block (where the uncompressed image will go)  */
+/* -------------------------------------------------------------------- */
+    if( pabyJPEGBlock == NULL )
+    {
+        // this is really 8bit only for now. 
+        pabyJPEGBlock = (GByte *) 
+            CPLCalloc(psImage->nBands,
+                      psImage->nBlockWidth * psImage->nBlockHeight);
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Read JPEG Chunk.                                                */
+/* -------------------------------------------------------------------- */
+    CPLString osFilename;
+    int iBlock = iBlockX + iBlockY * psImage->nBlocksPerRow;
+    GDALDataset *poDS;
+    int anBands[3] = { 1, 2, 3 };
+
+    osFilename.Printf( "JPEG_SUBFILE:Q%d,%d,%d,%s", 
+                       nQLevel,
+                       panJPEGBlockOffset[iBlock], 0, 
+                       osNITFFilename.c_str() );
+
+    poDS = (GDALDataset *) GDALOpen( osFilename, GA_ReadOnly );
+    if( poDS == NULL )
+        return CE_Failure;
+
+    if( poDS->GetRasterXSize() != psImage->nBlockWidth
+        || poDS->GetRasterYSize() != psImage->nBlockHeight )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "JPEG block %d not same size as NITF blocksize.", 
+                  iBlock );
+        return CE_Failure;
+    }
+                       
+    eErr = poDS->RasterIO( GF_Read, 
+                           0, 0, 
+                           psImage->nBlockWidth, psImage->nBlockHeight,
+                           pabyJPEGBlock, 
+                           psImage->nBlockWidth, psImage->nBlockHeight,
+                           GDT_Byte, psImage->nBands, anBands, 0, 0, 0 );
+
+    delete poDS;
+
+    return eErr;
+}
+
 /************************************************************************/
 /*                         GDALToNITFDataType()                         */
 /************************************************************************/
