@@ -33,7 +33,10 @@
  */
 
 #include "pgchip.h"
+#include <cassert>
 
+/* Define to enable debugging info */
+/*#define PGCHIP_DEBUG 1*/
 
 CPL_C_START
 void	GDALRegister_PGCHIP(void);
@@ -85,14 +88,8 @@ PGCHIPDataset::~PGCHIPDataset(){
     CPLFree(pszConnectionString);
     CPLFree(pszDBName);
     CPLFree(pszName);
-
-    
-    if(PGCHIP->data)
-        CPLFree(PGCHIP->data);
-        
-    if(PGCHIP)
-        CPLFree(PGCHIP);
-    
+    //CPLFree(PGCHIP->data); ?? 
+    CPLFree(PGCHIP);
 }
 
 /************************************************************************/
@@ -199,7 +196,7 @@ CPLErr PGCHIPDataset::SetProjection( const char * pszNewProjection ){
         return CE_Failure;
     }
     
-    CPLFree( pszProjection );
+    CPLFree( pszProjection ); pszProjection = NULL;
         
 /* -------------------------------------------------------------------- */
 /*      Reading SRID                                                    */
@@ -281,10 +278,10 @@ GDALDataset *PGCHIPDataset::Open( GDALOpenInfo * poOpenInfo ){
 
     char                szCommand[1024];
     PGresult            *hResult = NULL;
-    PGCHIPDataset 	*poDS = NULL;
+    PGCHIPDataset 	    *poDS = NULL;
     char                *chipStringHex;
 
-    unsigned char        *chipdata;
+    unsigned char       *chipdata;
     char                *layerName;
     int                 t;
        
@@ -404,17 +401,17 @@ GDALDataset *PGCHIPDataset::Open( GDALOpenInfo * poOpenInfo ){
             
     
 /* -------------------------------------------------------------------- */
-/*      Read the chip header                                            */
+/*      Read the chip                                                   */
 /* -------------------------------------------------------------------- */
-    
-    
+
+
     hResult = PQexec(poDS->hPGConn, "BEGIN");
     
     if( hResult && PQresultStatus(hResult) == PGRES_COMMAND_OK )
     {
         PQclear( hResult );
         sprintf( szCommand, 
-                 "SELECT raster FROM %s",
+                 "SELECT raster FROM %s LIMIT 1",
                  poDS->pszName);
                          
         hResult = PQexec(poDS->hPGConn,szCommand);
@@ -435,11 +432,15 @@ GDALDataset *PGCHIPDataset::Open( GDALOpenInfo * poOpenInfo ){
     chipdata = (unsigned char *) CPLMalloc(stringlen/2);
                       	
     for (t=0;t<stringlen/2;t++){
-	chipdata[t] = parse_hex( &chipStringHex[t*2]) ;
+	    chipdata[t] = parse_hex( &chipStringHex[t*2]) ;
     }
     
     // Chip assigment
     poDS->PGCHIP = (CHIP *)chipdata;
+
+#ifdef PGCHIP_DEBUG
+    poDS->printChipInfo(*(poDS->PGCHIP));
+#endif
            
     if( hResult )
         PQclear( hResult );
@@ -447,6 +448,28 @@ GDALDataset *PGCHIPDataset::Open( GDALOpenInfo * poOpenInfo ){
     hResult = PQexec(poDS->hPGConn, "COMMIT");
     PQclear( hResult );
     
+/* -------------------------------------------------------------------- */
+/*      Verify that there's no unknown field set.                       */
+/* -------------------------------------------------------------------- */
+
+    if ( poDS->PGCHIP->future[0] != 0 ||
+         poDS->PGCHIP->future[1] != 0 ||
+         poDS->PGCHIP->future[2] != 0 ||
+         poDS->PGCHIP->future[3] != 0 )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "Unsupported CHIP format (future field bytes != 0)\n");
+        delete poDS;
+        return NULL;
+    }
+
+    if ( poDS->PGCHIP->compression != 0 )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "Compressed CHIP unsupported\n");
+        delete poDS;
+        return NULL;
+    }
     
 /* -------------------------------------------------------------------- */
 /*      Set some information from the file that is of interest.         */
@@ -454,9 +477,34 @@ GDALDataset *PGCHIPDataset::Open( GDALOpenInfo * poOpenInfo ){
 
     poDS->nRasterXSize = poDS->PGCHIP->width;
     poDS->nRasterYSize = poDS->PGCHIP->height;
-    poDS->nBands = (int)poDS->PGCHIP->future[0];
-    poDS->nBitDepth = (int)poDS->PGCHIP->future[1];
-    poDS->nColorType = (int)poDS->PGCHIP->future[2];
+    poDS->nBands = 1; // (int)poDS->PGCHIP->future[0];
+    //poDS->nBitDepth = (int)poDS->PGCHIP->future[1];
+    poDS->nColorType = PGCHIP_COLOR_TYPE_GRAY; //(int)poDS->PGCHIP->future[2];
+
+    switch (poDS->PGCHIP->datatype)
+    {
+        case 5: // 24bit integer
+            poDS->nBitDepth = 24;
+            break;
+        case 6: // 16bit integer
+            poDS->nBitDepth = 16;
+            break;
+        case 8:
+            poDS->nBitDepth = 8;
+            break;
+        case 1: // float32
+        case 7: // 16bit ???
+        case 101: // float32 (NDR)
+        case 105: // 24bit integer (NDR)
+        case 106: // 16bit integer (NDR)
+        case 107: // 16bit ??? (NDR)
+        case 108: // 8bit ??? (NDR) [ doesn't make sense ]
+        default :
+             CPLError( CE_Failure, CPLE_AppDefined,
+                "Under development : CHIP datatype %d unsupported.\n",
+                poDS->PGCHIP->datatype);
+            break;   
+    }
     
         
 /* -------------------------------------------------------------------- */
@@ -470,6 +518,7 @@ GDALDataset *PGCHIPDataset::Open( GDALOpenInfo * poOpenInfo ){
 /*      Is there a palette?  Note: we should also read back and         */
 /*      apply transparency values if available.                         */
 /* -------------------------------------------------------------------- */
+    assert (poDS->nColorType != PGCHIP_COLOR_TYPE_PALETTE);
     if( poDS->nColorType == PGCHIP_COLOR_TYPE_PALETTE )
     {
         unsigned char *pPalette;
@@ -494,7 +543,7 @@ GDALDataset *PGCHIPDataset::Open( GDALOpenInfo * poOpenInfo ){
             poDS->poColorTable->SetColorEntry( iColor, &oEntry );
         }
     }
-    
+
     return( poDS );
 }
 
@@ -705,7 +754,7 @@ static GDALDataset * PGCHIPCreateCopy( const char * pszFilename, GDALDataset *po
     else        
         pszName = CPLStrdup("unknown_layer");
             
-    CPLFree(layerName);
+    CPLFree(layerName); layerName = NULL;
     
     
     // First allocation is small
@@ -745,7 +794,7 @@ static GDALDataset * PGCHIPCreateCopy( const char * pszFilename, GDALDataset *po
     else {
         CPLError( CE_Failure, CPLE_AppDefined, 
                   "%s", PQerrorMessage(hPGConn) );
-        CPLFree(szCommand);
+        CPLFree(szCommand); szCommand = NULL;
         return NULL;
     }
     
@@ -833,7 +882,7 @@ static GDALDataset * PGCHIPCreateCopy( const char * pszFilename, GDALDataset *po
     int		bHaveNoData = FALSE;
     double	dfNoDataValue = -1;
     int nbColors = 0,bFoundTrans = FALSE;
-    int sizePalette = 0;
+    size_t sizePalette = 0;
     
     if( nColorType == PGCHIP_COLOR_TYPE_PALETTE )
     {
@@ -881,43 +930,46 @@ static GDALDataset * PGCHIPCreateCopy( const char * pszFilename, GDALDataset *po
     
     PGCHIP.factor = 1.0;
     PGCHIP.endian_hint = 1;
-    PGCHIP.compression = nbColors; // To cope with palette extra information  : <header><palette><data>
+    PGCHIP.compression = 0; // nbColors; // To cope with palette extra information  : <header><palette><data>
     PGCHIP.height = nYSize;
     PGCHIP.width = nXSize;
     PGCHIP.SRID = SRID;
-    PGCHIP.future[0] = nBands; //nBands is stored in future variable
-    PGCHIP.future[1] = nBitDepth; //nBitDepth is stored in future variable
-    PGCHIP.future[2] = nColorType; //nBitDepth is stored in future variable
-    PGCHIP.future[3] = nbColors; // Useless as we store nbColors in the "compression" integer
+    PGCHIP.future[0] = 0; // nBands; //nBands is stored in future variable
+    PGCHIP.future[1] = 0; // nBitDepth; //nBitDepth is stored in future variable
+    PGCHIP.future[2] = 0; // nColorType; //nBitDepth is stored in future variable
+    PGCHIP.future[3] = 0; // nbColors; // Useless as we store nbColors in the "compression" integer
     PGCHIP.data = NULL; // Serialized Form
     
     // PGCHIP.size changes if there is a palette.
     // Is calculated by Postgis when inserting anyway
-    PGCHIP.size = sizeof(CHIP) + (nYSize * nXSize * storageChunk * nBands) + sizePalette;
+    PGCHIP.size = sizeof(CHIP) - sizeof(void*) + (nYSize * nXSize * storageChunk * nBands) + sizePalette;
     
-    switch(storageChunk*nBands){
-        case 1 :
-            PGCHIP.datatype = 8;
+    switch(nBitDepth)
+    {
+        case 8:
+            PGCHIP.datatype = 8; // NDR|XDR ?
             break;
-        case 2 :
-            PGCHIP.datatype = 6;
+        case 16:
+            PGCHIP.datatype = 6; // NDR|XDR ?
             break;
-        case 4 :
-            // Postgis sets data_size to 4 by default anyway
-            PGCHIP.datatype = 0;
+        case 24:
+            PGCHIP.datatype = 5; // NDR|XDR ?
             break;
-        default :
+        default:
              CPLError( CE_Failure, CPLE_AppDefined,"Under development : ERROR STORAGE CHUNK SIZE NOT SUPPORTED\n");
             break;   
     }
     
+#ifdef PGCHIP_DEBUG
+    PGCHIPDataset::printChipInfo(PGCHIP);
+#endif
                 
 /* -------------------------------------------------------------------- */
 /*      Loop over image                                                 */
 /* -------------------------------------------------------------------- */
        
     CPLErr      eErr;
-    int lineSize = nXSize * storageChunk * nBands;
+    size_t lineSize = nXSize * storageChunk * nBands;
     
     // allocating data buffer
     GByte *data = (GByte *) CPLMalloc( nYSize * lineSize);
@@ -941,10 +993,10 @@ static GDALDataset * PGCHIPCreateCopy( const char * pszFilename, GDALDataset *po
 /* -------------------------------------------------------------------- */    
     
     char *result;
-    int j=0;
+    size_t j=0;
     
     // Calculating result length (*2 -> Hex form, +1 -> end string) 
-    int size_result = (PGCHIP.size * 2) + 1;
+    size_t size_result = (PGCHIP.size * 2) + 1;
         
     // memory allocation
     result = (char *) CPLMalloc( size_result * sizeof(char));
@@ -953,22 +1005,22 @@ static GDALDataset * PGCHIPCreateCopy( const char * pszFilename, GDALDataset *po
     GByte *header = (GByte *)&PGCHIP;
         
     // Copy header into result string 
-    for(j=0;j<(int)sizeof(PGCHIP);j++){
-        deparse_hex( ((unsigned char *) header)[j], (unsigned char *)&result[j*2]);  
+    for(j=0; j<sizeof(PGCHIP)-sizeof(void*); j++) {
+        pgch_deparse_hex( header[j], (unsigned char*)&(result[j*2]));
     }  
     
     // Copy Palette into result string if required
-    int offsetPalette = (int)sizeof(PGCHIP) * 2;
+    size_t offsetPalette = (sizeof(PGCHIP)-sizeof(void*)) * 2;
     if(nColorType == PGCHIP_COLOR_TYPE_PALETTE && sizePalette>0){
         for(j=0;j<sizePalette;j++){
-            deparse_hex( ((unsigned char *) pPalette)[j], (unsigned char *)&result[offsetPalette + (j*2)]);     
+            pgch_deparse_hex( pPalette[j], (unsigned char *)&result[offsetPalette + (j*2)]);     
         }                   
     }
     
     // Copy data into result string
-    int offsetData = offsetPalette + sizePalette * 2;
+    size_t offsetData = offsetPalette + sizePalette * 2;
     for(j=0;j<(nYSize * lineSize);j++){
-         deparse_hex( ((unsigned char *) data)[j], (unsigned char *)&result[offsetData + (j*2)]);
+         pgch_deparse_hex( data[j], (unsigned char *)&result[offsetData + (j*2)]);
     }
    
     
@@ -1005,17 +1057,17 @@ static GDALDataset * PGCHIPCreateCopy( const char * pszFilename, GDALDataset *po
     else {
         CPLError( CE_Failure, CPLE_AppDefined, 
                   "%s", PQerrorMessage(hPGConn) );
-        CPLFree(szCommand);
+        CPLFree(szCommand); szCommand = NULL;
         return NULL;
     }
         
     hResult = PQexec(hPGConn, "COMMIT");
     PQclear( hResult );
             
-    CPLFree( szCommand );
-    CPLFree( pPalette );
-    CPLFree( data );       
-    CPLFree( result );
+    CPLFree( szCommand ); szCommand = NULL;
+    CPLFree( pPalette ); pPalette = NULL;
+    CPLFree( data ); data = NULL;
+    CPLFree( result ); result = NULL;
              
     return (GDALDataset *)GDALOpen(pszFilename,GA_Update);
 }
@@ -1024,20 +1076,20 @@ static GDALDataset * PGCHIPCreateCopy( const char * pszFilename, GDALDataset *po
 /************************************************************************/
 /*                          Display CHIP information                    */
 /************************************************************************/
-void     PGCHIPDataset::printChipInfo(){
+void     PGCHIPDataset::printChipInfo(const CHIP& c){
 
-    if(this->PGCHIP != NULL){
+    //if(this->PGCHIP != NULL){
         printf("\n---< CHIP INFO >----\n");
-        printf("CHIP.datatype = %d\n",this->PGCHIP->datatype);
-        printf("CHIP.compression = %d\n",this->PGCHIP->compression);
-        printf("CHIP.size = %d\n",this->PGCHIP->size);
-        printf("CHIP.factor = %f\n",this->PGCHIP->factor);
-        printf("CHIP.width = %d\n",this->PGCHIP->width);
-        printf("CHIP.height = %d\n",this->PGCHIP->height);
-        printf("CHIP.nBands = %d\n",(int)this->PGCHIP->future[0]);
-        printf("CHIP.nBitDepth = %d\n",(int)this->PGCHIP->future[1]);
+        printf("CHIP.datatype = %d\n", c.datatype);
+        printf("CHIP.compression = %d\n", c.compression);
+        printf("CHIP.size = %d\n", c.size);
+        printf("CHIP.factor = %f\n", c.factor);
+        printf("CHIP.width = %d\n", c.width);
+        printf("CHIP.height = %d\n", c.height);
+        //printf("CHIP.future[0] (nBands?) = %d\n", (int)c.future[0]);
+        //printf("CHIP.future[1] (nBitDepth?) = %d\n", (int)c.future[1]);
         printf("--------------------\n");
-     }
+     //}
 }
 
 
