@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ehdrdataset.cpp,v 1.37 2006/03/03 02:35:23 fwarmerdam Exp $
+ * $Id: ehdrdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  ESRI .hdr Driver
  * Purpose:  Implementation of EHdrDataset
@@ -25,78 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ehdrdataset.cpp,v $
- * Revision 1.37  2006/03/03 02:35:23  fwarmerdam
- * Be careful in constructor about zero band images.
- *
- * Revision 1.36  2006/02/08 17:10:52  fwarmerdam
- * Improved NODATA precision.
- *
- * Revision 1.35  2006/01/27 18:42:08  fwarmerdam
- * added the ability to save nodata and colortable information
- *
- * Revision 1.34  2006/01/20 13:33:24  fwarmerdam
- * Fallback to an assumption of nbits=8 if not specified.
- *
- * Revision 1.33  2006/01/17 13:16:20  fwarmerdam
- * as per Cees suggestions, correct handling of XLL/YLLCORNER
- * which is not at the center of the pixel.
- *
- * Revision 1.32  2006/01/17 12:28:27  fwarmerdam
- * Applied changes from Cees to default to signed, not unsigned and
- * for 32bit data to default to float, not integer.
- *
- * Revision 1.31  2006/01/13 00:18:12  fwarmerdam
- * Fixed error on failed open.
- *
- * Revision 1.30  2005/12/21 00:41:48  fwarmerdam
- * added support for reading/writing PIXELTYPE.
- * Added support for reading/writing writing geotransform and projection.
- *
- * Revision 1.29  2005/11/10 17:36:15  dron
- * _Always_ use 64-bit pointers when parsing header file.
- *
- * Revision 1.28  2005/11/10 17:24:48  dron
- * Use 64-bit pointers when parsing file header.
- *
- * Revision 1.27  2005/10/04 05:25:13  fwarmerdam
- * Added support for XLLCENTER/YLLCENTER.
- *
- * Revision 1.26  2005/05/05 13:55:41  fwarmerdam
- * PAM Enable
- *
- * Revision 1.25  2005/02/25 19:40:27  fwarmerdam
- * Support lower case byteorder values too.
- *
- * Revision 1.24  2004/07/10 12:19:01  dron
- * Read color table, when available (.clr file).
- *
- * Revision 1.23  2004/05/11 18:07:01  warmerda
- * Treat NBITS=32 and 64 as floating point cases.
- *
- * Revision 1.22  2004/02/12 10:15:33  dron
- * Fixed problem when computing band offset for band sequental files.
- *
- * Revision 1.21  2004/01/23 22:16:09  warmerda
- * fixed issue with static buffer overwriting in forming prj name
- *
- * Revision 1.20  2004/01/23 04:56:01  warmerda
- * support worldfiles if no corners in .hdr
- *
- * Revision 1.19  2004/01/20 16:22:56  dron
- * More clean usage of CPLFormCIFilename().
- *
- * Revision 1.18  2004/01/04 21:17:11  warmerda
- * fixed up to preserve path in CPLFormCIFilename calls
- */
+ ****************************************************************************/
 
 #include "rawdataset.h"
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ehdrdataset.cpp,v 1.37 2006/03/03 02:35:23 fwarmerdam Exp $");
+CPL_CVSID("$Id: ehdrdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 CPL_C_START
 void	GDALRegister_EHdr(void);
@@ -108,8 +43,12 @@ CPL_C_END
 /* ==================================================================== */
 /************************************************************************/
 
+class EHdrRasterBand;
+
 class EHdrDataset : public RawDataset
 {
+    friend class EHdrRasterBand;
+
     FILE	*fpImage;	// image data file.
 
     int         bGotTransform;
@@ -119,13 +58,18 @@ class EHdrDataset : public RawDataset
     int         bHDRDirty;
     char      **papszHDR;
 
+    int         bSTXDirty;
+
+    CPLErr      ReadSTX();
+    CPLErr      RewriteSTX();
     CPLErr      RewriteHDR();
     void        ResetKeyValue( const char *pszKey, const char *pszValue );
+    const char *GetKeyValue( const char *pszKey, const char *pszDefault = "" );
     void        RewriteColorTable( GDALColorTable * );
 
   public:
-    		EHdrDataset();
-    	        ~EHdrDataset();
+    EHdrDataset();
+    ~EHdrDataset();
     
     virtual CPLErr GetGeoTransform( double * padfTransform );
     virtual CPLErr SetGeoTransform( double *padfTransform );
@@ -136,8 +80,318 @@ class EHdrDataset : public RawDataset
     static GDALDataset *Create( const char * pszFilename,
                                 int nXSize, int nYSize, int nBands,
                                 GDALDataType eType, char ** papszParmList );
+    static GDALDataset *CreateCopy( const char * pszFilename, 
+                                    GDALDataset * poSrcDS, 
+                                    int bStrict, char ** papszOptions,
+                                    GDALProgressFunc pfnProgress,
+                                    void * pProgressData );
 };
 
+/************************************************************************/
+/* ==================================================================== */
+/*                          EHdrRasterBand                              */
+/* ==================================================================== */
+/************************************************************************/
+
+class EHdrRasterBand : public RawRasterBand
+{
+   friend class EHdrDataset;
+
+    int            nBits;
+    long           nStartBit;
+    int            nPixelOffsetBits;
+    int            nLineOffsetBits;
+
+    double         dfMin;
+    double         dfMax;
+    double         dfMean;
+    double         dfStdDev;
+
+    int            minmaxmeanstddev;
+
+    virtual CPLErr IRasterIO( GDALRWFlag, int, int, int, int,
+                              void *, int, int, GDALDataType,
+                              int, int );
+
+  public:
+    EHdrRasterBand( GDALDataset *poDS, int nBand, FILE * fpRaw, 
+                    vsi_l_offset nImgOffset, int nPixelOffset,
+                    int nLineOffset,
+                    GDALDataType eDataType, int bNativeOrder,
+                    int nBits);
+
+    virtual CPLErr IReadBlock( int, int, void * );
+    virtual CPLErr IWriteBlock( int, int, void * );
+
+    virtual double GetMinimum( int *pbSuccess = NULL );
+    virtual double GetMaximum(int *pbSuccess = NULL );
+    virtual CPLErr GetStatistics( int bApproxOK, int bForce,
+                                  double *pdfMin, double *pdfMax, 
+                                  double *pdfMean, double *pdfStdDev );
+    virtual CPLErr SetStatistics( double dfMin, double dfMax, 
+                                  double dfMean, double dfStdDev );
+
+};
+
+/************************************************************************/
+/*                           EHdrRasterBand()                           */
+/************************************************************************/
+
+EHdrRasterBand::EHdrRasterBand( GDALDataset *poDS,
+                                int nBand, FILE * fpRaw, 
+                                vsi_l_offset nImgOffset, int nPixelOffset,
+                                int nLineOffset,
+                                GDALDataType eDataType, int bNativeOrder,
+                                int nBits)
+: RawRasterBand( poDS, nBand, fpRaw, nImgOffset, nPixelOffset, nLineOffset, 
+                         eDataType, bNativeOrder, TRUE ),
+  nBits(nBits),
+  dfMin(0),
+  dfMax(0),
+  minmaxmeanstddev(0)
+{
+    if (nBits < 8)
+    {
+        EHdrDataset* poEDS = (EHdrDataset*)poDS;
+        nStartBit = atoi(poEDS->GetKeyValue("SKIPBYTES")) * 8;
+        if (nBand >= 2)
+        {
+            long nRowBytes = atoi(poEDS->GetKeyValue("BANDROWBYTES"));
+            if (nRowBytes == 0)
+                nRowBytes = (nBits * poDS->GetRasterXSize() + 7) / 8;
+
+            nStartBit += nRowBytes * (nBand-1) * 8;
+        }
+
+        nPixelOffsetBits = nBits;
+        nLineOffsetBits = atoi(poEDS->GetKeyValue("TOTALROWBYTES")) * 8;
+        if( nLineOffsetBits == 0 )
+            nLineOffsetBits = nPixelOffsetBits * poDS->GetRasterXSize();
+
+        nBlockXSize = poDS->GetRasterXSize();
+        nBlockYSize = 1;
+
+        SetMetadataItem( "NBITS", 
+                         CPLString().Printf( "%ld", nBits ) );
+    }
+}
+
+/************************************************************************/
+/*                             IReadBlock()                             */
+/************************************************************************/
+
+CPLErr EHdrRasterBand::IReadBlock( int nBlockXOff, int nBlockYOff,
+                                   void * pImage )
+
+{
+    if (nBits >= 8)
+      return RawRasterBand::IReadBlock(nBlockXOff, nBlockYOff, pImage);
+
+    vsi_l_offset   nLineStart;
+    unsigned int   nLineBytes;
+    int            iBitOffset;
+    GByte         *pabyBuffer;
+
+/* -------------------------------------------------------------------- */
+/*      Establish desired position.                                     */
+/* -------------------------------------------------------------------- */
+    nLineBytes = (nPixelOffsetBits*nBlockXSize + 7)/8;
+    nLineStart = (nStartBit + ((vsi_l_offset)nLineOffsetBits) * nBlockYOff) / 8;
+    iBitOffset = 
+        (nStartBit + ((vsi_l_offset)nLineOffsetBits) * nBlockYOff) % 8;
+
+/* -------------------------------------------------------------------- */
+/*      Read data into buffer.                                          */
+/* -------------------------------------------------------------------- */
+    pabyBuffer = (GByte *) CPLCalloc(nLineBytes,1);
+
+    if( VSIFSeekL( GetFP(), nLineStart, SEEK_SET ) != 0 
+        || VSIFReadL( pabyBuffer, 1, nLineBytes, GetFP() ) != nLineBytes )
+    {
+        CPLError( CE_Failure, CPLE_FileIO,
+                  "Failed to read %d bytes at offset %d.\n%s",
+                  nLineBytes, nLineStart, 
+                  VSIStrerror( errno ) );
+        return CE_Failure;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Copy data, promoting to 8bit.                                   */
+/* -------------------------------------------------------------------- */
+    int iPixel = 0, iX;
+
+    for( iX = 0; iX < nBlockXSize; iX++ )
+    {
+        int  nOutWord = 0, iBit;
+
+        for( iBit = 0; iBit < nBits; iBit++ )
+        {
+            if( pabyBuffer[iBitOffset>>3]  & (0x80 >>(iBitOffset & 7)) )
+                nOutWord |= (1 << (nBits - 1 - iBit));
+            iBitOffset++;
+        } 
+
+        iBitOffset = iBitOffset + nPixelOffsetBits - nBits;
+                
+        ((GByte *) pImage)[iPixel++] = nOutWord;
+    }
+
+    CPLFree( pabyBuffer );
+
+    return CE_None;
+}
+
+/************************************************************************/
+/*                            IWriteBlock()                             */
+/************************************************************************/
+
+CPLErr EHdrRasterBand::IWriteBlock( int nBlockXOff, int nBlockYOff,
+                                   void * pImage )
+
+{
+    if (nBits >= 8)
+      return RawRasterBand::IWriteBlock(nBlockXOff, nBlockYOff, pImage);
+
+    vsi_l_offset   nLineStart;
+    unsigned int   nLineBytes;
+    int            iBitOffset;
+    GByte         *pabyBuffer;
+
+/* -------------------------------------------------------------------- */
+/*      Establish desired position.                                     */
+/* -------------------------------------------------------------------- */
+    nLineBytes = (nPixelOffsetBits*nBlockXSize + 7)/8;
+    nLineStart = (nStartBit + ((vsi_l_offset)nLineOffsetBits) * nBlockYOff) / 8;
+    iBitOffset = 
+        (nStartBit + ((vsi_l_offset)nLineOffsetBits) * nBlockYOff) % 8;
+
+/* -------------------------------------------------------------------- */
+/*      Read data into buffer.                                          */
+/* -------------------------------------------------------------------- */
+    pabyBuffer = (GByte *) CPLCalloc(nLineBytes,1);
+
+    if( VSIFSeekL( GetFP(), nLineStart, SEEK_SET ) != 0 )
+    {
+        CPLError( CE_Failure, CPLE_FileIO,
+                  "Failed to read %d bytes at offset %d.\n%s",
+                  nLineBytes, nLineStart, 
+                  VSIStrerror( errno ) );
+        return CE_Failure;
+    }
+
+    VSIFReadL( pabyBuffer, 1, nLineBytes, GetFP() );
+
+/* -------------------------------------------------------------------- */
+/*      Copy data, promoting to 8bit.                                   */
+/* -------------------------------------------------------------------- */
+    int iPixel = 0, iX;
+
+    for( iX = 0; iX < nBlockXSize; iX++ )
+    {
+        int iBit;
+        int  nOutWord = ((GByte *) pImage)[iPixel++];
+
+        for( iBit = 0; iBit < nBits; iBit++ )
+        {
+            if( nOutWord & (1 << (nBits - 1 - iBit)) )
+                pabyBuffer[iBitOffset>>3] |= (0x80 >>(iBitOffset & 7));
+            else
+                pabyBuffer[iBitOffset>>3] &= ~((0x80 >>(iBitOffset & 7)));
+
+            iBitOffset++;
+        } 
+
+        iBitOffset = iBitOffset + nPixelOffsetBits - nBits;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Write the data back out.                                        */
+/* -------------------------------------------------------------------- */
+    if( VSIFSeekL( GetFP(), nLineStart, SEEK_SET ) != 0 
+        || VSIFWriteL( pabyBuffer, 1, nLineBytes, GetFP() ) != nLineBytes )
+    {
+        CPLError( CE_Failure, CPLE_FileIO,
+                  "Failed to write %d bytes at offset %d.\n%s",
+                  nLineBytes, nLineStart, 
+                  VSIStrerror( errno ) );
+        return CE_Failure;
+    }
+
+    CPLFree( pabyBuffer );
+
+    return CE_None;
+}
+
+/************************************************************************/
+/*                             IRasterIO()                              */
+/************************************************************************/
+
+CPLErr EHdrRasterBand::IRasterIO( GDALRWFlag eRWFlag,
+                                  int nXOff, int nYOff, int nXSize, int nYSize,
+                                  void * pData, int nBufXSize, int nBufYSize,
+                                  GDALDataType eBufType,
+                                  int nPixelSpace, int nLineSpace )
+
+{
+    // Defer to RawRasterBand
+    if (nBits >= 8)
+        return RawRasterBand::IRasterIO( eRWFlag, 
+                                         nXOff, nYOff, nXSize, nYSize,
+                                         pData, nBufXSize, nBufYSize, 
+                                         eBufType, nPixelSpace, nLineSpace );
+
+    // Force use of IReadBlock() and IWriteBlock()
+    else
+        return GDALRasterBand::IRasterIO( eRWFlag, 
+                                          nXOff, nYOff, nXSize, nYSize,
+                                          pData, nBufXSize, nBufYSize, 
+                                          eBufType, nPixelSpace, nLineSpace );
+}
+
+/************************************************************************/
+/*                              OSR_GDS()                               */
+/************************************************************************/
+
+static const char*OSR_GDS( char **papszNV, const char * pszField, 
+                           const char *pszDefaultValue )
+
+{
+    int         iLine;
+
+    if( papszNV == NULL || papszNV[0] == NULL )
+        return pszDefaultValue;
+
+    for( iLine = 0; 
+         papszNV[iLine] != NULL && 
+             !EQUALN(papszNV[iLine],pszField,strlen(pszField));
+         iLine++ ) {}
+
+    if( papszNV[iLine] == NULL )
+        return pszDefaultValue;
+    else
+    {
+        static char     szResult[80];
+        char    **papszTokens;
+        
+        papszTokens = CSLTokenizeString(papszNV[iLine]);
+
+        if( CSLCount(papszTokens) > 1 )
+            strncpy( szResult, papszTokens[1], sizeof(szResult));
+        else
+            strncpy( szResult, pszDefaultValue, sizeof(szResult));
+        
+        CSLDestroy( papszTokens );
+        return szResult;
+    }
+}
+
+
+/************************************************************************/
+/* ==================================================================== */
+/*				EHdrDataset				*/
+/* ==================================================================== */
+/************************************************************************/
+
 /************************************************************************/
 /*                            EHdrDataset()                             */
 /************************************************************************/
@@ -155,6 +409,7 @@ EHdrDataset::EHdrDataset()
     adfGeoTransform[5] = 1.0;
     papszHDR = NULL;
     bHDRDirty = FALSE;
+    bSTXDirty = FALSE;
 }
 
 /************************************************************************/
@@ -184,6 +439,9 @@ EHdrDataset::~EHdrDataset()
 
         if( bHDRDirty )
             RewriteHDR();
+
+        if( bSTXDirty )
+            RewriteSTX();
     }
 
     if( fpImage != NULL )
@@ -193,6 +451,32 @@ EHdrDataset::~EHdrDataset()
     CSLDestroy( papszHDR );
 }
 
+/************************************************************************/
+/*                            GetKeyValue()                             */
+/************************************************************************/
+
+const char *EHdrDataset::GetKeyValue( const char *pszKey, 
+                                      const char *pszDefault )
+
+{
+    int i;
+
+    for( i = 0; papszHDR[i] != NULL; i++ )
+    {
+        if( EQUALN(pszKey,papszHDR[i],strlen(pszKey))
+            && isspace(papszHDR[i][strlen(pszKey)]) )
+        {
+            const char *pszValue = papszHDR[i] + strlen(pszKey);
+            while( isspace(*pszValue) )
+                pszValue++;
+            
+            return pszValue;
+        }
+    }
+
+    return pszDefault;
+}
+
 /************************************************************************/
 /*                           ResetKeyValue()                            */
 /*                                                                      */
@@ -276,7 +560,10 @@ void EHdrDataset::RewriteColorTable( GDALColorTable *poTable )
 const char *EHdrDataset::GetProjectionRef()
 
 {
-    return pszProjection;
+    if (pszProjection && strlen(pszProjection) > 0)
+        return pszProjection;
+
+    return GDALPamDataset::GetProjectionRef();
 }
 
 /************************************************************************/
@@ -438,6 +725,104 @@ CPLErr EHdrDataset::RewriteHDR()
     return CE_None;
 }
 
+/************************************************************************/
+/*                             RewriteSTX()                             */
+/************************************************************************/
+    
+CPLErr EHdrDataset::RewriteSTX()
+{
+    CPLString osPath = CPLGetPath( GetDescription() );
+    CPLString osName = CPLGetBasename( GetDescription() );
+    CPLString osSTXFilename = CPLFormCIFilename( osPath, osName, "stx" );
+
+/* -------------------------------------------------------------------- */
+/*      Write .stx file.                                                */
+/* -------------------------------------------------------------------- */
+    FILE	*fp;
+    fp = VSIFOpen( osSTXFilename, "wt" );
+    if( fp == NULL )
+    {
+        CPLError( CE_Failure, CPLE_OpenFailed, 
+                  "Failed to rewrite .stx file %s.", 
+                  osSTXFilename.c_str() );
+        return CE_Failure;
+    }
+
+    for (int i = 0; i < nBands; ++i)
+    {
+        EHdrRasterBand* poBand = (EHdrRasterBand*)papoBands[i];
+        VSIFPrintf( fp, "%d %.10f %.10f ", i+1, poBand->dfMin, poBand->dfMax );
+        if ( poBand->minmaxmeanstddev & 0x4 )
+            VSIFPrintf( fp, "%.10f ", poBand->dfMean);
+        else
+            VSIFPrintf( fp, "# ");
+
+        if ( poBand->minmaxmeanstddev & 0x8 )
+            VSIFPrintf( fp, "%.10f\n", poBand->dfStdDev);
+        else
+            VSIFPrintf( fp, "#\n");
+    }
+
+    VSIFClose( fp );
+
+    bSTXDirty = FALSE;
+
+    return CE_None;
+}
+
+/************************************************************************/
+/*                              ReadSTX()                               */
+/************************************************************************/
+
+CPLErr EHdrDataset::ReadSTX()
+{
+    CPLString osPath = CPLGetPath( GetDescription() );
+    CPLString osName = CPLGetBasename( GetDescription() );
+    CPLString osSTXFilename = CPLFormCIFilename( osPath, osName, "stx" );
+
+/* -------------------------------------------------------------------- */
+/*      Read .stx file.                                                 */
+/* -------------------------------------------------------------------- */
+    FILE	*fp;
+    if ((fp = VSIFOpen( osSTXFilename, "rt" )))
+    {
+      const char *	pszLine;
+      while( (pszLine = CPLReadLine( fp )) )
+      {
+          char	**papszTokens;
+          papszTokens = CSLTokenizeStringComplex( pszLine, " \t", TRUE, FALSE );
+          if( CSLCount( papszTokens ) >= 5 )
+          {
+            int i = atoi(papszTokens[0]);
+            if (i > 0 && i <= nBands)
+            {
+              EHdrRasterBand* poBand = (EHdrRasterBand*)papoBands[i-1];
+              poBand->dfMin = atof(papszTokens[1]);
+              poBand->dfMax = atof(papszTokens[2]);
+              poBand->minmaxmeanstddev = 0x3;
+              // reads optional mean and stddev
+              if ( !EQUAL(papszTokens[3], "#") )
+              {
+                poBand->dfMean   = atof(papszTokens[3]);
+                poBand->minmaxmeanstddev |= 0x4;
+              }
+              if ( !EQUAL(papszTokens[4], "#") )
+              {
+                poBand->dfStdDev = atof(papszTokens[4]);
+                poBand->minmaxmeanstddev |= 0x8;
+              }
+            }
+          }
+
+          CSLDestroy( papszTokens );
+      }
+
+      VSIFClose( fp );
+    }
+
+    return CE_None;
+}
+
 /************************************************************************/
 /*                                Open()                                */
 /************************************************************************/
@@ -451,7 +836,7 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*	We assume the user is pointing to the binary (ie. .bil) file.	*/
 /* -------------------------------------------------------------------- */
-    if( poOpenInfo->fp == NULL )
+    if( poOpenInfo->nHeaderBytes < 2 )
         return NULL;
 
 /* -------------------------------------------------------------------- */
@@ -660,24 +1045,25 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
     if( nBits == 16 )
     {
-        if ( chPixelType == 'U' )
-            eDataType = GDT_UInt16;
+        if ( chPixelType == 'S' )
+            eDataType = GDT_Int16;
         else
-            eDataType = GDT_Int16; // default
+            eDataType = GDT_UInt16; // default
     }
     else if( nBits == 32 )
     {
         if( chPixelType == 'S' )
             eDataType = GDT_Int32;
-        else if( chPixelType == 'U' )
-            eDataType = GDT_UInt32;
         else if( chPixelType == 'F' )
             eDataType = GDT_Float32;
         else
-            eDataType = GDT_Float32; // apparently common usage? 
+            eDataType = GDT_UInt32; // default 
     }
     else if( nBits == 8 || nBits == -1 )
         eDataType = GDT_Byte;
+
+    else if( nBits < 8 && nBits >= 1 )
+        eDataType = GDT_Byte;
     
     else
     {
@@ -714,55 +1100,34 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
         nBandOffset = (vsi_l_offset)nItemSize * nCols;
     }
     
+    poDS->SetDescription( poOpenInfo->pszFilename );
+    poDS->PamInitialize();
+
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
 /* -------------------------------------------------------------------- */
     poDS->nBands = nBands;
     for( i = 0; i < poDS->nBands; i++ )
     {
-        RawRasterBand	*poBand;
-
+        EHdrRasterBand	*poBand;
+            
         poBand = 
-            new RawRasterBand( poDS, i+1, poDS->fpImage,
-                               nSkipBytes + nBandOffset * i, 
-                               nPixelOffset, nLineOffset, eDataType,
+            new EHdrRasterBand( poDS, i+1, poDS->fpImage,
+                                nSkipBytes + nBandOffset * i, 
+                                nPixelOffset, nLineOffset, eDataType,
 #ifdef CPL_LSB                               
-                               chByteOrder == 'I' || chByteOrder == 'L',
+                                chByteOrder == 'I' || chByteOrder == 'L',
 #else
-                               chByteOrder == 'M',
+                                chByteOrder == 'M',
 #endif        
-                               TRUE );
+                                nBits);
 
         if( bNoDataSet )
             poBand->StoreNoDataValue( dfNoData );
-
+            
         poDS->SetBand( i+1, poBand );
     }
 
-/* -------------------------------------------------------------------- */
-/*      Check for a .prj file.                                          */
-/* -------------------------------------------------------------------- */
-    const char  *pszPrjFilename = CPLFormCIFilename( pszPath, pszName, "prj" );
-
-    fp = VSIFOpen( pszPrjFilename, "r" );
-    if( fp != NULL )
-    {
-        char	**papszLines;
-        OGRSpatialReference oSRS;
-
-        VSIFClose( fp );
-        
-        papszLines = CSLLoad( pszPrjFilename );
-
-        if( oSRS.importFromESRI( papszLines ) == OGRERR_NONE )
-        {
-            CPLFree( poDS->pszProjection );
-            oSRS.exportToWkt( &(poDS->pszProjection) );
-        }
-
-        CSLDestroy( papszLines );
-    }
-
 /* -------------------------------------------------------------------- */
 /*      If we didn't get bounds in the .hdr, look for a worldfile.      */
 /* -------------------------------------------------------------------- */
@@ -800,7 +1165,7 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
     
     if( !poDS->bGotTransform )
         poDS->bGotTransform = 
-            GDALReadWorldFile( poOpenInfo->pszFilename, "blw", 
+            GDALReadWorldFile( poOpenInfo->pszFilename, 0, 
                                poDS->adfGeoTransform );
 
     if( !poDS->bGotTransform )
@@ -808,6 +1173,43 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
             GDALReadWorldFile( poOpenInfo->pszFilename, "wld", 
                                poDS->adfGeoTransform );
 
+/* -------------------------------------------------------------------- */
+/*      Check for a .prj file.                                          */
+/* -------------------------------------------------------------------- */
+    const char  *pszPrjFilename = CPLFormCIFilename( pszPath, pszName, "prj" );
+
+    fp = VSIFOpen( pszPrjFilename, "r" );
+    if( fp != NULL )
+    {
+        char	**papszLines;
+        OGRSpatialReference oSRS;
+
+        VSIFClose( fp );
+        
+        papszLines = CSLLoad( pszPrjFilename );
+
+        if( oSRS.importFromESRI( papszLines ) == OGRERR_NONE )
+        {
+            // If geographic values are in seconds, we must transform. 
+            // Is there a code for minutes too? 
+            if( oSRS.IsGeographic() 
+                && EQUAL(OSR_GDS( papszLines, "Units", ""), "DS") )
+            {
+                poDS->adfGeoTransform[0] /= 3600.0;
+                poDS->adfGeoTransform[1] /= 3600.0;
+                poDS->adfGeoTransform[2] /= 3600.0;
+                poDS->adfGeoTransform[3] /= 3600.0;
+                poDS->adfGeoTransform[4] /= 3600.0;
+                poDS->adfGeoTransform[5] /= 3600.0;
+            }
+
+            CPLFree( poDS->pszProjection );
+            oSRS.exportToWkt( &(poDS->pszProjection) );
+        }
+
+        CSLDestroy( papszLines );
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Check for a color table.                                        */
 /* -------------------------------------------------------------------- */
@@ -824,6 +1226,9 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
             if ( !pszLine )
                 break;
 
+            if( *pszLine == '#' )
+                continue;
+
             char	**papszValues = CSLTokenizeString2(pszLine, "\t ",
                                                            CSLT_HONOURSTRINGS);
             GDALColorEntry oEntry;
@@ -851,6 +1256,11 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
         }
     }
 
+/* -------------------------------------------------------------------- */
+/*      Read statistics (.STX)                                          */
+/* -------------------------------------------------------------------- */
+    poDS->ReadSTX();
+
 /* -------------------------------------------------------------------- */
 /*      Check for overviews.                                            */
 /* -------------------------------------------------------------------- */
@@ -859,7 +1269,6 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
 /* -------------------------------------------------------------------- */
 /*      Initialize any PAM information.                                 */
 /* -------------------------------------------------------------------- */
-    poDS->SetDescription( poOpenInfo->pszFilename );
     poDS->TryLoadXML();
     
     CPLFree( pszName );
@@ -875,7 +1284,7 @@ GDALDataset *EHdrDataset::Open( GDALOpenInfo * poOpenInfo )
 GDALDataset *EHdrDataset::Create( const char * pszFilename,
                                   int nXSize, int nYSize, int nBands,
                                   GDALDataType eType,
-                                  char ** /* papszParmList */ )
+                                  char **papszParmList )
 
 {
 /* -------------------------------------------------------------------- */
@@ -934,20 +1343,29 @@ GDALDataset *EHdrDataset::Create( const char * pszFilename,
         CPLFree( pszHdrFilename );
         return NULL;
     }
+
+/* -------------------------------------------------------------------- */
+/*      Decide how many bits the file should have.                      */
+/* -------------------------------------------------------------------- */
+    int nRowBytes;
+    int nBits = GDALGetDataTypeSize(eType);
+
+    if( CSLFetchNameValue( papszParmList, "NBITS" ) != NULL )
+        nBits = atoi(CSLFetchNameValue( papszParmList, "NBITS" ));
+
+    nRowBytes = (nBits * nXSize + 7) / 8;
     
 /* -------------------------------------------------------------------- */
 /*      Write out the raw definition for the dataset as a whole.        */
 /* -------------------------------------------------------------------- */
-    int nItemSize = GDALGetDataTypeSize(eType) / 8;
-
     VSIFPrintf( fp, "BYTEORDER      I\n" );
     VSIFPrintf( fp, "LAYOUT         BIL\n" );
     VSIFPrintf( fp, "NROWS          %d\n", nYSize );
     VSIFPrintf( fp, "NCOLS          %d\n", nXSize );
     VSIFPrintf( fp, "NBANDS         %d\n", nBands );
-    VSIFPrintf( fp, "NBITS          %d\n", GDALGetDataTypeSize(eType) );
-    VSIFPrintf( fp, "BANDROWBYTES   %d\n", nItemSize * nXSize );
-    VSIFPrintf( fp, "TOTALROWBYTES  %d\n", nItemSize * nXSize * nBands );
+    VSIFPrintf( fp, "NBITS          %d\n", nBits );
+    VSIFPrintf( fp, "BANDROWBYTES   %d\n", nRowBytes );
+    VSIFPrintf( fp, "TOTALROWBYTES  %d\n", nRowBytes * nBands );
     
     if( eType == GDT_Float32 )
         VSIFPrintf( fp, "PIXELTYPE      FLOAT\n");
@@ -966,6 +1384,137 @@ GDALDataset *EHdrDataset::Create( const char * pszFilename,
     return (GDALDataset *) GDALOpen( pszFilename, GA_Update );
 }
 
+/************************************************************************/
+/*                             CreateCopy()                             */
+/************************************************************************/
+
+GDALDataset *EHdrDataset::CreateCopy( const char * pszFilename, 
+                                      GDALDataset * poSrcDS, 
+                                      int bStrict, char ** papszOptions,
+                                      GDALProgressFunc pfnProgress,
+                                      void * pProgressData )
+
+{
+    char **papszAdjustedOptions = CSLDuplicate( papszOptions );
+    GDALDataset *poOutDS;
+
+    if( poSrcDS->GetRasterBand(1)->GetMetadataItem( "NBITS" ) != NULL 
+        && CSLFetchNameValue( papszOptions, "NBITS" ) == NULL )
+    {
+        papszAdjustedOptions = 
+            CSLSetNameValue( papszAdjustedOptions, 
+                             "NBITS", 
+                             poSrcDS->GetRasterBand(1)->GetMetadataItem("NBITS") );
+    }
+    
+    GDALDriver	*poDriver = (GDALDriver *) GDALGetDriverByName( "EHdr" );
+
+    poOutDS = poDriver->DefaultCreateCopy( pszFilename, poSrcDS, bStrict,
+                                           papszAdjustedOptions, 
+                                           pfnProgress, pProgressData );
+    CSLDestroy( papszAdjustedOptions );
+
+    return poOutDS;
+}
+    
+/************************************************************************/
+/*                           GetMinimum()                               */
+/************************************************************************/
+
+double EHdrRasterBand::GetMinimum( int *pbSuccess )
+{
+    if( pbSuccess != NULL )
+        *pbSuccess = (minmaxmeanstddev & 0x1) != 0;
+
+    if( minmaxmeanstddev & 0x1 )
+      return dfMin;
+
+    return RawRasterBand::GetMinimum( pbSuccess );
+}
+
+/************************************************************************/
+/*                           GetMaximum()                               */
+/************************************************************************/
+
+double EHdrRasterBand::GetMaximum( int *pbSuccess )
+{
+    if( pbSuccess != NULL )
+        *pbSuccess = (minmaxmeanstddev & 0x2) != 0;
+
+    if( minmaxmeanstddev & 0x2 )
+      return dfMax;
+
+    return RawRasterBand::GetMaximum( pbSuccess );
+}
+
+/************************************************************************/
+/*                           GetStatistics()                            */
+/************************************************************************/
+
+CPLErr EHdrRasterBand::GetStatistics( int bApproxOK, int bForce, double *pdfMin, double *pdfMax, double *pdfMean, double *pdfStdDev )
+{
+    if( (minmaxmeanstddev & 0xf) == 0xf)
+    {
+        if ( pdfMin ) *pdfMin = dfMin;
+        if ( pdfMax ) *pdfMax = dfMax;
+        if ( pdfMean ) *pdfMean = dfMean;
+        if ( pdfStdDev ) *pdfStdDev = dfStdDev;
+        return CE_None;
+    }
+
+    CPLErr eErr = RawRasterBand::GetStatistics( bApproxOK, bForce, 
+                                                pdfMin, pdfMax, 
+                                                pdfMean, pdfStdDev );
+
+    if( eErr == CE_None )
+    {
+        EHdrDataset* poEDS = (EHdrDataset *) poDS;
+
+        if( pdfMin && pdfMax )
+        {
+            dfMin = *pdfMin;
+            dfMax = *pdfMax;
+            
+            minmaxmeanstddev |= 0x3;
+            poEDS->bSTXDirty = TRUE;
+        }
+
+        if( *pdfMean )
+        {
+            dfMean = *pdfMean;
+            minmaxmeanstddev |= 0x4;
+            poEDS->bSTXDirty = TRUE;
+        }
+
+        if( *pdfStdDev )
+        {
+            dfStdDev = *pdfStdDev;
+            minmaxmeanstddev |= 0x8;
+            poEDS->bSTXDirty = TRUE;
+        }
+    }
+
+    return eErr;
+}
+
+/************************************************************************/
+/*                           SetStatistics()                            */
+/************************************************************************/
+
+CPLErr EHdrRasterBand::SetStatistics( double dfMin, double dfMax, double dfMean, double dfStdDev )
+{
+    this->dfMin = dfMin;
+    this->dfMax = dfMax;
+    this->dfMean = dfMean;
+    this->dfStdDev = dfStdDev;
+    // marks stats valid
+    minmaxmeanstddev = 0xf;
+    // marks dataset stats dirty
+    ((EHdrDataset*)poDS)->bSTXDirty = TRUE;
+   
+    return CE_None;
+}
+
 /************************************************************************/
 /*                         GDALRegister_EHdr()                          */
 /************************************************************************/
@@ -987,8 +1536,14 @@ void GDALRegister_EHdr()
         poDriver->SetMetadataItem( GDAL_DMD_CREATIONDATATYPES, 
                                    "Byte Int16 UInt16 Int32 UInt32 Float32" );
 
+        poDriver->SetMetadataItem( GDAL_DMD_CREATIONOPTIONLIST, 
+"<CreationOptionList>"
+"   <Option name='NBITS' type='int' description='Special pixel bits (1-7)'/>"
+"</CreationOptionList>" );
+
         poDriver->pfnOpen = EHdrDataset::Open;
         poDriver->pfnCreate = EHdrDataset::Create;
+        poDriver->pfnCreateCopy = EHdrDataset::CreateCopy;
 
         GetGDALDriverManager()->RegisterDriver( poDriver );
     }
