@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: envidataset.cpp,v 1.31 2005/11/10 18:03:27 dron Exp $
+ * $Id: envidataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  ENVI .hdr Driver
  * Purpose:  Implementation of ENVI .hdr labelled raw raster support.
@@ -25,119 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: envidataset.cpp,v $
- * Revision 1.31  2005/11/10 18:03:27  dron
- * Added creation options descriptions in the driver's metadata.
- *
- * Revision 1.30  2005/09/21 19:57:37  fwarmerdam
- * 2 bytes in headerbytes is enough, that is what Create() writes to the
- * image file.
- *
- * Revision 1.29  2005/09/21 16:24:36  fwarmerdam
- * fixed to avoid using openinfo->fp or large files may not work
- *
- * Revision 1.28  2005/09/14 01:09:00  fwarmerdam
- * Fixed creation data types.
- *
- * Revision 1.27  2005/08/05 17:40:45  fwarmerdam
- * Reformat for clarity.
- *
- * Revision 1.26  2005/07/25 19:52:43  ssoule
- * Changed GDALMajorObject's char *pszDescription to std::string sDescription.
- *
- * Revision 1.25  2005/05/05 13:55:42  fwarmerdam
- * PAM Enable
- *
- * Revision 1.24  2005/03/16 11:04:00  lichun
- * FlushCache() now always writes the whole header file
- * in the case of classes the type will be Classified and the class names and colors are written
- *
- * Revision 1.23  2005/02/28 15:42:05  fwarmerdam
- * Call RawDataset::FlushCache() instead of GDALDataset::FlushCache()
- *
- * Revision 1.22  2005/02/08 15:52:44  gwalter
- * Avoid overwriting header when it hasn't changed.
- *
- * Revision 1.21  2004/11/02 20:21:26  fwarmerdam
- * added support for category names and colormaps
- *
- * Revision 1.20  2004/10/21 15:08:19  fwarmerdam
- * Added warning about file compression.
- *
- * Revision 1.19  2004/08/18 15:31:46  warmerda
- * I've fixed a bug in ENVI driver that happened when creating a band with a
- * start offset greater that 2 Gbytes.  I've also added support for geotransform
- * parameters on creation even if there are no projection information.
- * Sebastien Grignard (sebastien dot grignard at archivideo dot com).
- *
- * Revision 1.18  2004/07/28 14:33:27  warmerda
- * Added Geograpic lat/lon.
- *
- * Revision 1.17  2004/01/27 22:38:37  aamici
- * Remove CInt* data types from GDAL_DMD_CREATIONDATATYPES since they are
- * no supported by this driver.
- *
- * Revision 1.16  2003/07/24 18:29:35  warmerda
- * added oSRS.Fixup() to insert meter as linear units (bug 370)
- *
- * Revision 1.15  2003/06/05 15:41:08  dron
- * Fixed problem with projection name determining.
- *
- * Revision 1.14  2003/04/30 09:47:29  dron
- * Few memory leaks fixed.
- *
- * Revision 1.13  2003/03/11 16:34:49  gwalter
- * Test header before opening dataset to avoid "band names = {" being written
- * (during FlushCache in dataset deletion) to non-envi format files that also have
- * .hdr header files.
- *
- * Revision 1.12  2003/03/05 12:52:19  dron
- * Added flag for large file creation.
- *
- * Revision 1.11  2003/02/27 21:17:09  dron
- * Writing map info implemented.
- *
- * Revision 1.10  2003/02/27 09:50:41  dron
- * SUFFIX and INTERLEAVE creation options added; reference pixels supported.
- *
- * Revision 1.9  2003/02/17 21:30:02  dron
- * Write support via Create() method added.
- *
- * Revision 1.8  2003/02/13 14:57:08  warmerda
- * Corrected bug in north/south handling for UTM.
- * Fix submitted by Vladimir Slepnev.
- *
- * Revision 1.7  2003/01/27 14:21:28  warmerda
- * Fixed byte_order test.
- *
- * Revision 1.6  2003/01/24 15:05:01  warmerda
- * Fixed byte order detection bugs as per bugzilla 261.
- *
- * Revision 1.5  2003/01/24 15:00:07  warmerda
- * Added additional data types as per Bugzilla 262.
- *
- * Revision 1.4  2002/09/04 06:50:37  warmerda
- * avoid static driver pointers
- *
- * Revision 1.3  2002/06/12 21:12:25  warmerda
- * update to metadata based driver info
- *
- * Revision 1.2  2002/05/11 16:22:01  warmerda
- * Also search for .hdr appended to full filename, not just .hdr replacing
- * the existing extension - as per Bugzilla bug 127 from Alessandro Amici.
- *
- * Revision 1.1  2002/03/04 21:52:48  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "rawdataset.h"
 #include "ogr_spatialref.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: envidataset.cpp,v 1.31 2005/11/10 18:03:27 dron Exp $");
+CPL_CVSID("$Id: envidataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 CPL_C_START
 void	GDALRegister_ENVI(void);
