@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: usgsdem_create.cpp,v 1.22 2005/05/05 15:54:49 fwarmerdam Exp $
+ * $Id: usgsdem_create.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  USGS DEM Driver
  * Purpose:  CreateCopy() implementation.
@@ -28,79 +28,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- * 
- * $Log: usgsdem_create.cpp,v $
- * Revision 1.22  2005/05/05 15:54:49  fwarmerdam
- * PAM Enabled
- *
- * Revision 1.21  2005/03/29 20:56:32  fwarmerdam
- * Ensure that DataSpecVersion can be overridden even if CDED50K product
- * is selected.
- *
- * Revision 1.20  2005/03/23 20:30:57  fwarmerdam
- * Fixed "east" check for NTS option case.
- *
- * Revision 1.19  2005/03/23 17:41:42  fwarmerdam
- * added NTS and INTERNALNAME options for Matt
- *
- * Revision 1.18  2004/12/17 19:50:03  fwarmerdam
- * Fixed last bug reference.
- *
- * Revision 1.17  2004/12/17 19:49:06  fwarmerdam
- * Applied patch for min/max elevation values per bug 708.
- *
- * Revision 1.16  2004/12/10 21:59:55  fwarmerdam
- * added ZRESOLUTION support from Hunter Blanks (bug 708)
- *
- * Revision 1.15  2004/11/21 22:13:54  fwarmerdam
- * use new pointer encode/decode functions
- *
- * Revision 1.14  2004/09/01 19:10:32  warmerda
- * added UTM support care of Hunter Blanks (newwireless.com)
- *
- * Revision 1.13  2004/05/11 00:40:45  warmerda
- * fixed single precision resolution fields
- *
- * Revision 1.12  2004/05/03 14:13:42  warmerda
- * fine tuned MSVC hack
- *
- * Revision 1.11  2004/05/03 13:45:33  warmerda
- * correct old double formatting of MSVC++ (3 digit exponent)
- *
- * Revision 1.10  2004/04/29 13:46:46  warmerda
- * flush CPLReadLine() memory
- *
- * Revision 1.9  2004/04/27 14:54:56  warmerda
- * fixed filename checking, and DATAPOINTER support
- *
- * Revision 1.8  2004/04/23 20:27:24  warmerda
- * completed mapsheet support for CDED50K
- *
- * Revision 1.7  2004/04/23 19:43:00  warmerda
- * added partial NTS mapsheet db support
- *
- * Revision 1.6  2004/04/15 17:34:11  warmerda
- * Left rather than right justify the origin code.
- *
- * Revision 1.5  2004/04/01 21:06:36  warmerda
- * Added creation options for PRODUCER, OriginCode, and ProcessCode.
- * Added support for reprojecting to NAD83.
- * Fixed bug in A/B/C zone computation.
- *
- * Revision 1.4  2004/04/01 18:36:01  warmerda
- * fixed rounding issue in testing quarter degree boundaries
- *
- * Revision 1.3  2004/03/28 21:23:06  warmerda
- * minor nodata item
- *
- * Revision 1.2  2004/03/28 19:30:05  warmerda
- * implement CDED50K product specs, use warper
- *
- * Revision 1.1  2004/03/27 17:02:00  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal_pam.h"
 #include "ogr_spatialref.h"
@@ -108,7 +36,7 @@
 #include "gdalwarper.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: usgsdem_create.cpp,v 1.22 2005/05/05 15:54:49 fwarmerdam Exp $");
+CPL_CVSID("$Id: usgsdem_create.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 typedef struct 
 {
