@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vrtdataset.cpp,v 1.25 2006/02/10 15:02:15 fwarmerdam Exp $
+ * $Id: vrtdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Virtual GDAL Datasets
  * Purpose:  Implementation of VRTDataset
@@ -25,93 +25,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: vrtdataset.cpp,v $
- * Revision 1.25  2006/02/10 15:02:15  fwarmerdam
- * Write via the large file API.
- *
- * Revision 1.24  2006/02/08 06:12:07  fwarmerdam
- * Override SetMetadata methods so that metadata can be preserved.
- * Support saving histograms in VRT per bug 1060.
- *
- * Revision 1.23  2005/10/28 16:59:50  pnagy
- * Added VRTDerivedBand support
- *
- * Revision 1.22  2005/06/28 14:50:44  fwarmerdam
- * avoid destroy psTree in case of failure - done at higher level
- *
- * Revision 1.21  2005/05/05 13:56:34  fwarmerdam
- * moved metadata handling to PAM
- *
- * Revision 1.20  2005/04/11 17:08:36  fwarmerdam
- * Fixed leak of xml tree when XMLInit() fails.
- *
- * Revision 1.19  2004/10/03 16:52:45  fwarmerdam
- * ensure overviews initialized in Create()
- *
- * Revision 1.18  2004/08/11 20:34:52  warmerda
- * Allow "user input" for projection information in .vrt file.
- *
- * Revision 1.17  2004/08/11 18:43:50  warmerda
- * restructure init to support derived class, add pszVRTPath to serialize
- *
- * Revision 1.16  2004/07/30 21:51:29  warmerda
- * added support for VRTRawRasterBand
- *
- * Revision 1.15  2004/07/27 21:59:48  warmerda
- * Enable .ovr support.
- *
- * Revision 1.14  2004/03/16 18:34:35  warmerda
- * added support for relativeToVRT attribute on SourceFilename
- *
- * Revision 1.13  2003/07/17 20:31:12  warmerda
- * moved out driver and VRTCreateCopy() code
- *
- * Revision 1.12  2003/06/10 19:58:35  warmerda
- * added support for AddFuncSource in AddBand() method for Imagine
- *
- * Revision 1.11  2002/12/05 22:17:19  warmerda
- * added support for opening directly from XML
- *
- * Revision 1.10  2002/11/30 16:55:49  warmerda
- * added OpenXML method
- *
- * Revision 1.9  2002/11/24 04:27:52  warmerda
- * CopyCreate() nows saves source image directly if it is a VRTDataset
- *
- * Revision 1.8  2002/11/23 18:54:17  warmerda
- * added CREATIONDATATYPES metadata for drivers
- *
- * Revision 1.7  2002/09/04 06:50:37  warmerda
- * avoid static driver pointers
- *
- * Revision 1.6  2002/06/12 21:12:25  warmerda
- * update to metadata based driver info
- *
- * Revision 1.5  2002/05/29 16:40:18  warmerda
- * dont provide default value for method argumentsvrtdataset.cpp
- *
- * Revision 1.4  2002/05/29 16:06:05  warmerda
- * complete detailed band metadata
- *
- * Revision 1.3  2002/04/19 19:43:38  warmerda
- * added CreateCopy method
- *
- * Revision 1.2  2001/11/18 15:46:45  warmerda
- * added SRS and GeoTransform
- *
- * Revision 1.1  2001/11/16 21:14:31  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "vrtdataset.h"
 #include "cpl_string.h"
 #include "cpl_minixml.h"
 #include "ogr_spatialref.h"
 
-CPL_CVSID("$Id: vrtdataset.cpp,v 1.25 2006/02/10 15:02:15 fwarmerdam Exp $");
+CPL_CVSID("$Id: vrtdataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                            VRTDataset()                             */
@@ -260,10 +181,10 @@ CPLXMLNode *VRTDataset::SerializeToXML( const char *pszVRTPath )
                                     adfGeoTransform[5] ) );
     }
 
-    /* -------------------------------------------------------------------- */
-    /*      Metadata                                                        */
-    /* -------------------------------------------------------------------- */
-    psMD = PamSerializeMetadata( this );
+/* -------------------------------------------------------------------- */
+/*      Metadata                                                        */
+/* -------------------------------------------------------------------- */
+    psMD = oMDMD.Serialize();
     if( psMD != NULL )
         CPLAddXMLChild( psDSTree, psMD );
 
@@ -431,7 +352,7 @@ CPLErr VRTDataset::XMLInit( CPLXMLNode *psTree, const char *pszVRTPath )
 /* -------------------------------------------------------------------- */
 /*      Apply any dataset level metadata.                               */
 /* -------------------------------------------------------------------- */
-    PamApplyMetadata( psTree, this );
+    oMDMD.XMLInit( psTree, TRUE );
 
 /* -------------------------------------------------------------------- */
 /*      Create band information objects.                                */
