@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vrtdriver.cpp,v 1.7 2004/08/11 18:45:56 warmerda Exp $
+ * $Id: vrtdriver.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Virtual GDAL Datasets
  * Purpose:  Implementation of VRTDriver
@@ -25,37 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: vrtdriver.cpp,v $
- * Revision 1.7  2004/08/11 18:45:56  warmerda
- * Use CopyCommonInfoFrom method
- *
- * Revision 1.6  2004/07/28 16:56:36  warmerda
- * updated to use VRTSourcedRasterBand
- *
- * Revision 1.5  2004/04/15 18:54:38  warmerda
- * added UnitType, Offset, Scale and CategoryNames support
- *
- * Revision 1.4  2004/04/02 17:20:06  warmerda
- * Added defaulte extension, and point help topic to VRT tutorial.
- *
- * Revision 1.3  2004/03/16 18:34:35  warmerda
- * added support for relativeToVRT attribute on SourceFilename
- *
- * Revision 1.2  2003/07/27 11:16:06  dron
- * Check NULL pointer in GetMetadata()/SetMetadata().
- *
- * Revision 1.1  2003/07/17 20:27:18  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "vrtdataset.h"
 #include "cpl_minixml.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: vrtdriver.cpp,v 1.7 2004/08/11 18:45:56 warmerda Exp $");
+CPL_CVSID("$Id: vrtdriver.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                             VRTDriver()                              */
@@ -132,7 +108,11 @@ VRTSource *VRTDriver::ParseSource( CPLXMLNode *psSrc, const char *pszVRTPath )
     const char *pszParserFunc;
 
     if( psSrc == NULL || psSrc->eType != CXT_Element )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "Corrupt or empty VRT source XML document." );
         return NULL;
+    }
 
     pszParserFunc = CSLFetchNameValue( papszSourceParsers, psSrc->pszValue );
     if( pszParserFunc == NULL )
