@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vrtsourcedrasterband.cpp,v 1.5 2005/10/28 16:59:51 pnagy Exp $
+ * $Id: vrtsourcedrasterband.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Virtual GDAL Datasets
  * Purpose:  Implementation of VRTSourcedRasterBand
@@ -25,32 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: vrtsourcedrasterband.cpp,v $
- * Revision 1.5  2005/10/28 16:59:51  pnagy
- * Added VRTDerivedBand support
- *
- * Revision 1.4  2005/06/28 14:51:05  fwarmerdam
- * added error reporting if there are no sources
- *
- * Revision 1.3  2004/08/25 14:21:00  warmerda
- * Added support for overviews in sourced band RasterIO() contributed
- * by Sebasien Grignard.
- *
- * Revision 1.2  2004/08/11 18:46:45  warmerda
- * pass pszVRTPath through serialize methods
- *
- * Revision 1.1  2004/07/28 16:56:56  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "vrtdataset.h"
 #include "cpl_minixml.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: vrtsourcedrasterband.cpp,v 1.5 2005/10/28 16:59:51 pnagy Exp $");
+CPL_CVSID("$Id: vrtsourcedrasterband.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -285,7 +266,10 @@ CPLErr VRTSourcedRasterBand::XMLInit( CPLXMLNode * psTree,
          psChild = psChild->psNext)
     {
         VRTSource *poSource;
-        
+
+        if( psChild->eType != CXT_Element )
+            continue;
+
         CPLErrorReset();
         poSource = poDriver->ParseSource( psChild, pszVRTPath );
         if( poSource != NULL )
@@ -542,6 +526,41 @@ char **VRTSourcedRasterBand::GetMetadata( const char *pszDomain )
         return GDALRasterBand::GetMetadata( pszDomain );
 }
 
+/************************************************************************/
+/*                          SetMetadataItem()                           */
+/************************************************************************/
+
+CPLErr VRTSourcedRasterBand::SetMetadataItem( const char *pszName, 
+                                              const char *pszValue, 
+                                              const char *pszDomain )
+
+{
+    CPLDebug( "VRT", "VRTSourcedRasterBand::SetMetadataItem(%s,%s,%s)\n",
+              pszName, pszValue, pszDomain );
+              
+    if( pszDomain != NULL
+        && EQUAL(pszDomain,"new_vrt_sources") )
+    {
+        VRTDriver *poDriver = (VRTDriver *) GDALGetDriverByName( "VRT" );
+
+        CPLXMLNode *psTree = CPLParseXMLString( pszValue );
+        VRTSource *poSource;
+        
+        if( psTree == NULL )
+            return CE_Failure;
+        
+        poSource = poDriver->ParseSource( psTree, NULL );
+        CPLDestroyXMLNode( psTree );
+        
+        if( poSource != NULL )
+            return AddSource( poSource );
+        else
+            return CE_Failure;
+    }
+    else
+        return VRTRasterBand::SetMetadataItem( pszName, pszValue, pszDomain );
+}
+
 /************************************************************************/
 /*                            SetMetadata()                             */
 /************************************************************************/
@@ -591,6 +610,6 @@ CPLErr VRTSourcedRasterBand::SetMetadata( char **papszNewMD, const char *pszDoma
         return CE_None;
     }
     else
-        return GDALRasterBand::SetMetadata( papszNewMD, pszDomain );
+        return VRTRasterBand::SetMetadata( papszNewMD, pszDomain );
 }
 
