@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: vrtsources.cpp,v 1.5 2004/08/11 18:46:45 warmerda Exp $
+ * $Id: vrtsources.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Virtual GDAL Datasets
  * Purpose:  Implementation of VRTSimpleSource, VRTFuncSource and 
@@ -26,33 +26,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: vrtsources.cpp,v $
- * Revision 1.5  2004/08/11 18:46:45  warmerda
- * pass pszVRTPath through serialize methods
- *
- * Revision 1.4  2004/03/16 18:36:02  warmerda
- * Don't use pszVRTPath if it is NULL.
- *
- * Revision 1.3  2004/03/16 18:34:35  warmerda
- * added support for relativeToVRT attribute on SourceFilename
- *
- * Revision 1.2  2003/09/11 23:00:04  aamici
- * add class constructors and destructors where needed in order to
- * let the mingw/cygwin binutils produce sensible partially linked objet files
- * with 'ld -r'.
- *
- * Revision 1.1  2003/07/17 20:27:18  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "vrtdataset.h"
 #include "cpl_minixml.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: vrtsources.cpp,v 1.5 2004/08/11 18:46:45 warmerda Exp $");
+CPL_CVSID("$Id: vrtsources.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /* ==================================================================== */
@@ -342,21 +322,21 @@ VRTSimpleSource::GetSrcDstWindow( int nXOff, int nYOff, int nXSize, int nYSize,
 /* -------------------------------------------------------------------- */
 /*      Clamp within the bounds of the available source data.           */
 /* -------------------------------------------------------------------- */
-    int bModified = FALSE;
+    int bModifiedX = FALSE, bModifiedY = FALSE;
 
     if( *pnReqXOff < 0 )
     {
         *pnReqXSize += *pnReqXOff;
         *pnReqXOff = 0;
 
-        bModified = TRUE;
+        bModifiedX = TRUE;
     }
     
     if( *pnReqYOff < 0 )
     {
         *pnReqYSize += *pnReqYOff;
         *pnReqYOff = 0;
-        bModified = TRUE;
+        bModifiedY = TRUE;
     }
 
     if( *pnReqXSize == 0 )
@@ -367,13 +347,13 @@ VRTSimpleSource::GetSrcDstWindow( int nXOff, int nYOff, int nXSize, int nYSize,
     if( *pnReqXOff + *pnReqXSize > poRasterBand->GetXSize() )
     {
         *pnReqXSize = poRasterBand->GetXSize() - *pnReqXOff;
-        bModified = TRUE;
+        bModifiedX = TRUE;
     }
 
     if( *pnReqYOff + *pnReqYSize > poRasterBand->GetYSize() )
     {
         *pnReqYSize = poRasterBand->GetYSize() - *pnReqYOff;
-        bModified = TRUE;
+        bModifiedY = TRUE;
     }
 
 /* -------------------------------------------------------------------- */
@@ -391,7 +371,7 @@ VRTSimpleSource::GetSrcDstWindow( int nXOff, int nYOff, int nXSize, int nYSize,
 /*      If we haven't had to modify the source rectangle, then the      */
 /*      destination rectangle must be the whole region.                 */
 /* -------------------------------------------------------------------- */
-    if( !bModified )
+    if( !bModifiedX && !bModifiedY )
         return TRUE;
 
 /* -------------------------------------------------------------------- */
@@ -406,39 +386,35 @@ VRTSimpleSource::GetSrcDstWindow( int nXOff, int nYOff, int nXSize, int nYSize,
     SrcToDst( *pnReqXOff + *pnReqXSize, *pnReqYOff + *pnReqYSize, 
               dfDstLRX, dfDstLRY );
 
-    dfScaleWinToBufX = nBufXSize / (double) nXSize;
-    dfScaleWinToBufY = nBufYSize / (double) nYSize;
+    if( bModifiedX )
+    {
+        dfScaleWinToBufX = nBufXSize / (double) nXSize;
 
-    *pnOutXOff = (int) ((dfDstULX - nXOff) * dfScaleWinToBufX+0.001);
-    *pnOutYOff = (int) ((dfDstULY - nYOff) * dfScaleWinToBufY+0.001);
-    *pnOutXSize = (int) ((dfDstLRX - nXOff) * dfScaleWinToBufX+0.001) 
-        - *pnOutXOff;
-    *pnOutYSize = (int) ((dfDstLRY - nYOff) * dfScaleWinToBufY+0.001) 
-        - *pnOutYOff;
+        *pnOutXOff = (int) ((dfDstULX - nXOff) * dfScaleWinToBufX+0.001);
+        *pnOutXSize = (int) ((dfDstLRX - nXOff) * dfScaleWinToBufX+0.001) 
+            - *pnOutXOff;
+
+        *pnOutXOff = MAX(0,*pnOutXOff);
+        if( *pnOutXOff + *pnOutXSize > nBufXSize )
+            *pnOutXSize = nBufXSize - *pnOutXOff;
+    }
+
+    if( bModifiedY )
+    {
+        dfScaleWinToBufY = nBufYSize / (double) nYSize;
+
+        *pnOutYOff = (int) ((dfDstULY - nYOff) * dfScaleWinToBufY+0.001);
+        *pnOutYSize = (int) ((dfDstLRY - nYOff) * dfScaleWinToBufY+0.001) 
+            - *pnOutYOff;
+
+        *pnOutYOff = MAX(0,*pnOutYOff);
+        if( *pnOutYOff + *pnOutYSize > nBufYSize )
+            *pnOutYSize = nBufYSize - *pnOutYOff;
+    }
 
-    *pnOutXOff = MAX(0,*pnOutXOff);
-    *pnOutYOff = MAX(0,*pnOutYOff);
-    if( *pnOutXOff + *pnOutXSize > nBufXSize )
-        *pnOutXSize = nBufXSize - *pnOutXOff;
-    if( *pnOutYOff + *pnOutYSize > nBufYSize )
-        *pnOutYSize = nBufYSize - *pnOutYOff;
 
     if( *pnOutXSize < 1 || *pnOutYSize < 1 )
-    {
-        if( nYOff == 26 )						
-        {
-            printf( "adjusted outsize == 0!\n" );
-            printf( "Dst = (%.16g,%.16g,%.16g,%.16g)\n", 
-                    dfDstULX, dfDstULY, dfDstLRX, dfDstLRY );
-            printf( "Out = (%d,%d,%d,%d)\n",
-                    *pnOutXOff, 
-                    *pnOutYOff, 
-                    *pnOutXSize, 
-                    *pnOutYSize );
-        }
-
         return FALSE;
-    }
     else
         return TRUE;
 }
