@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdal_priv.h,v 1.72 2006/04/13 03:16:01 fwarmerdam Exp $
+ * $Id: gdal_priv.h 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Name:     gdal_priv.h
  * Project:  GDAL Core
@@ -26,83 +26,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gdal_priv.h,v $
- * Revision 1.72  2006/04/13 03:16:01  fwarmerdam
- * keep track if an object is PAM enabled, bug 1135
- *
- * Revision 1.71  2006/02/07 19:07:07  fwarmerdam
- * applied some strategic improved outofmemory checking
- *
- * Revision 1.70  2005/11/17 22:02:32  fwarmerdam
- * avoid overwriting existing .aux file, overview filename now CPLString
- *
- * Revision 1.69  2005/10/13 01:19:57  fwarmerdam
- * moved GDALMultiDomainMetadata into GDALMajorObject
- *
- * Revision 1.68  2005/09/26 15:52:03  fwarmerdam
- * centralized .aux opening logic
- *
- * Revision 1.67  2005/09/23 20:52:21  fwarmerdam
- * added the GMO flags on GDALMajorObject
- *
- * Revision 1.66  2005/09/23 16:55:39  fwarmerdam
- * use CPLString instead of std::string
- *
- * Revision 1.65  2005/09/17 04:04:16  fwarmerdam
- * provide default implementation for RAT functions
- *
- * Revision 1.64  2005/09/17 03:46:37  fwarmerdam
- * added HFAAuxBuildOverviews
- *
- * Revision 1.63  2005/09/16 20:31:15  fwarmerdam
- * added bOvrIsAux, and RAT methods
- *
- * Revision 1.62  2005/07/25 21:24:28  ssoule
- * Changed GDALColorTable's "GDALColorEntry *paoEntries" to
- * "std::vector<GDALColorEntry> aoEntries".
- *
- * Revision 1.61  2005/07/25 19:52:43  ssoule
- * Changed GDALMajorObject's char *pszDescription to std::string sDescription.
- *
- * Revision 1.60  2005/07/11 21:08:17  fwarmerdam
- * Removed GetAge() method.
- *
- * Revision 1.59  2005/07/11 19:07:02  fwarmerdam
- * removed GDALRasterBlock nAge.
- *
- * Revision 1.58  2005/05/23 06:44:48  fwarmerdam
- * blockrefs now fetched locked
- *
- * Revision 1.57  2005/05/13 18:19:15  fwarmerdam
- * Added SetDefaultHistogram
- *
- * Revision 1.56  2005/05/10 15:30:28  fwarmerdam
- * export some overview functions
- *
- * Revision 1.55  2005/05/10 04:49:24  fwarmerdam
- * added getdefaulthistogram and GDALOvLevelAdjust
- *
- * Revision 1.54  2005/04/27 16:29:33  fwarmerdam
- * GetHistogram() made virtual.  Added methods for getting and setting
- * statistics.  Added SetUnitType method.
- *
- * Revision 1.53  2005/04/07 17:31:00  fwarmerdam
- * added some brief descriptions of classes
- *
- * Revision 1.52  2005/04/04 15:24:48  fwarmerdam
- * Most C entry points now CPL_STDCALL
- *
- * Revision 1.51  2005/02/17 22:16:12  fwarmerdam
- * changed to use two level block cache
- *
- * Revision 1.50  2005/01/15 16:09:37  fwarmerdam
- * added SetOffset, SetScale methods
- *
- * Revision 1.49  2005/01/04 21:14:01  fwarmerdam
- * added GDAL_FORCE_CACHING config variable
- */
+ ****************************************************************************/
 
 #ifndef GDAL_PRIV_H_INCLUDED
 #define GDAL_PRIV_H_INCLUDED
@@ -516,6 +440,10 @@ class CPL_DLL GDALRasterBand : public GDALMajorObject
     virtual CPLErr GetStatistics( int bApproxOK, int bForce,
                                   double *pdfMin, double *pdfMax, 
                                   double *pdfMean, double *padfStdDev );
+    virtual CPLErr ComputeStatistics( int bApproxOK, 
+                                      double *pdfMin, double *pdfMax, 
+                                      double *pdfMean, double *padfStdDev,
+                                      GDALProgressFunc, void *pProgressData );
     virtual CPLErr SetStatistics( double dfMin, double dfMax, 
                                   double dfMean, double dfStdDev );
 
@@ -608,6 +536,11 @@ class CPL_DLL GDALDriver : public GDALMajorObject
                                      GDALProgressFunc pfnProgress, 
                                      void * pProgressData );
     
+    GDALDataset         *DefaultCreateCopy( const char *, GDALDataset *, 
+                                            int, char **,
+                                            GDALProgressFunc pfnProgress, 
+                                            void * pProgressData );
+    
 /* -------------------------------------------------------------------- */
 /*      The following are semiprivate, not intended to be accessed      */
 /*      by anyone but the formats instantiating and populating the      */
@@ -679,6 +612,8 @@ CPL_C_END
 
 CPL_C_START
 
+#ifndef WIN32CE
+
 CPLErr CPL_DLL
 HFAAuxBuildOverviews( const char *pszOvrFilename, GDALDataset *poParentDS,
                       GDALDataset **ppoDS,
@@ -687,6 +622,9 @@ HFAAuxBuildOverviews( const char *pszOvrFilename, GDALDataset *poParentDS,
                       const char *pszResampling, 
                       GDALProgressFunc pfnProgress, 
                       void *pProgressData );
+
+#endif /* WIN32CE */
+
 CPLErr CPL_DLL 
 GTIFFBuildOverviews( const char * pszFilename,
                      int nBands, GDALRasterBand **papoBandList, 
@@ -711,6 +649,15 @@ int CPL_DLL GDALOvLevelAdjust( int nOvLevel, int nXSize );
 GDALDataset CPL_DLL *
 GDALFindAssociatedAuxFile( const char *pszBasefile, GDALAccess eAccess );
 
+/* ==================================================================== */
+/*      Misc functions.                                                 */
+/* ==================================================================== */
+
+CPLErr CPL_DLL GDALParseGMLCoverage( CPLXMLNode *psTree, 
+                                     int *pnXSize, int *pnYSize,
+                                     double *padfGeoTransform,
+                                     char **ppszProjection );
+                                  
 CPL_C_END
 
 #endif /* ndef GDAL_PRIV_H_INCLUDED */
