@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaldataset.cpp,v 1.57 2006/03/03 19:44:45 fwarmerdam Exp $
+ * $Id: gdaldataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GDAL Core
  * Purpose:  Base class for raster file formats.  
@@ -25,96 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gdaldataset.cpp,v $
- * Revision 1.57  2006/03/03 19:44:45  fwarmerdam
- * force C numeric locale in select locations
- *
- * Revision 1.56  2005/09/23 20:52:55  fwarmerdam
- * avoid issuing NotSupported errors if GMO_IGNORE_UNIMPLEMENTED is set
- *
- * Revision 1.55  2005/07/25 23:15:31  fwarmerdam
- * Fixed another typo.
- *
- * Revision 1.54  2005/07/25 23:13:38  fwarmerdam
- * Fixed typo in BuildOverviews docs.
- *
- * Revision 1.53  2005/07/11 17:10:02  fwarmerdam
- * moved in GDALOpen, GDALClose - make shared list thread safer
- *
- * Revision 1.52  2005/06/20 01:52:47  fwarmerdam
- * an illegal band in GetRasterBand() no longer fatal
- *
- * Revision 1.51  2005/05/23 06:42:16  fwarmerdam
- * added mutex to protect dataset list
- *
- * Revision 1.50  2005/04/20 16:39:04  fwarmerdam
- * Fixed last fix.
- *
- * Revision 1.49  2005/04/20 16:38:10  fwarmerdam
- * Undo last change, error is fatal.
- *
- * Revision 1.48  2005/04/20 16:32:06  fwarmerdam
- * Ensure GetRasterBand() returns NULL on illegal band.
- *
- * Revision 1.47  2005/04/04 15:24:48  fwarmerdam
- * Most C entry points now CPL_STDCALL
- *
- * Revision 1.46  2005/03/09 17:06:48  fwarmerdam
- * Added use of BlockBasedRasterIO() if bForceCachedIO turned on.  untested.
- *
- * Revision 1.45  2005/02/17 22:13:16  fwarmerdam
- * added bForceCachedIO on dataset
- *
- * Revision 1.44  2004/12/10 19:04:07  fwarmerdam
- * fix up docs a bit
- *
- * Revision 1.43  2004/12/02 20:32:28  fwarmerdam
- * added AdviseRead methods
- *
- * Revision 1.42  2004/10/01 13:28:26  fwarmerdam
- * Fixed osr_tutorial link.
- *
- * Revision 1.41  2003/05/21 04:45:39  warmerda
- * avoid warings about unused formal parameters
- *
- * Revision 1.40  2003/05/21 03:05:47  warmerda
- * remove unused variable
- *
- * Revision 1.39  2003/05/14 08:53:10  dron
- * Fixed mistake in RasterIO() description.
- *
- * Revision 1.38  2003/05/08 14:38:40  warmerda
- * added BlockBasedFlushCache
- *
- * Revision 1.37  2003/04/30 17:13:48  warmerda
- * added docs for many C functions
- *
- * Revision 1.36  2003/04/25 19:49:51  warmerda
- * first crack at RasterIO implementation
- *
- * Revision 1.35  2003/03/18 05:58:52  warmerda
- * Added GDALFlushCache().
- *
- * Revision 1.34  2003/02/13 16:18:58  warmerda
- * Fixed typo.
- *
- * Revision 1.33  2003/02/13 16:10:23  warmerda
- * Cleaned up GetProjectionRef() info.
- *
- * Revision 1.32  2003/01/28 16:54:49  warmerda
- * indicate required include file
- *
- * Revision 1.31  2003/01/28 16:07:31  warmerda
- * improved documentation
- */
+ ****************************************************************************/
 
 #include "gdal_priv.h"
 #include "cpl_string.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gdaldataset.cpp,v 1.57 2006/03/03 19:44:45 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdaldataset.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static volatile int nGDALDatasetCount = 0;
 static GDALDataset ** volatile papoGDALDatasetList = NULL;
@@ -193,7 +110,10 @@ GDALDataset::~GDALDataset()
 {
     int         i;
 
-    CPLDebug( "GDAL", "GDALClose(%s)\n", GetDescription() );
+    // we don't want to report destruction of datasets that 
+    // were never really open.
+    if( nBands != 0 || !EQUAL(GetDescription(),"") )
+        CPLDebug( "GDAL", "GDALClose(%s)", GetDescription() );
 
 /* -------------------------------------------------------------------- */
 /*      Remove dataset from the "open" dataset list.                    */
@@ -1308,7 +1228,7 @@ CPLErr GDALDataset::IRasterIO( GDALRWFlag eRWFlag,
  *
  * This method is the same as the C GDALDatasetRasterIO() function.
  *
- * @param eRWFlag Either GF_Read to read a region of data, or GT_Write to
+ * @param eRWFlag Either GF_Read to read a region of data, or GF_Write to
  * write a region of data.
  *
  * @param nXOff The pixel offset to the top left corner of the region
@@ -1539,10 +1459,10 @@ GDALDataset **GDALDataset::GetOpenDatasets( int *pnCount )
  * @see GDALDataset::GetOpenDatasets()
  */
 
-void CPL_STDCALL GDALGetOpenDatasets( GDALDatasetH ***hDS, int *pnCount )
+void CPL_STDCALL GDALGetOpenDatasets( GDALDatasetH **ppahDSList, int *pnCount )
 
 {
-    *hDS = (GDALDatasetH **) GDALDataset::GetOpenDatasets( pnCount);
+    *ppahDSList = (GDALDatasetH *) GDALDataset::GetOpenDatasets( pnCount);
 }
 
 /************************************************************************/
@@ -1702,7 +1622,7 @@ GDALOpen( const char * pszFilename, GDALAccess eAccess )
                 poDS->poDriver = poDriver;
 
             
-            CPLDebug( "GDAL", "GDALOpen(%s) succeeds as %s.\n",
+            CPLDebug( "GDAL", "GDALOpen(%s) succeeds as %s.",
                       pszFilename, poDriver->GetDescription() );
 
             return (GDALDatasetH) poDS;
