@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdaljp2metadata.cpp,v 1.6 2006/04/07 05:35:25 fwarmerdam Exp $
+ * $Id: gdaljp2metadata.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GDAL 
  * Purpose:  GDALJP2Metadata - Read GeoTIFF and/or GML georef info.
@@ -25,29 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- *****************************************************************************
- *
- * $Log: gdaljp2metadata.cpp,v $
- * Revision 1.6  2006/04/07 05:35:25  fwarmerdam
- * Added ReadAndParse() method, which includes worldfile reading.
- * Actually set HaveGeoTransform flag properly.
- *
- * Revision 1.5  2005/07/05 22:09:00  fwarmerdam
- * add preliminary support for MSIG boxes
- *
- * Revision 1.4  2005/05/23 06:45:34  fwarmerdam
- * make msi_uuid const
- *
- * Revision 1.3  2005/05/17 20:13:04  fwarmerdam
- * fix up to support relative and absolute gmljp2:// uri format
- *
- * Revision 1.2  2005/05/05 20:17:15  fwarmerdam
- * support dictionary lookups
- *
- * Revision 1.1  2005/05/03 21:10:59  fwarmerdam
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdaljp2metadata.h"
 #include "cpl_string.h"
@@ -56,7 +34,7 @@
 #include "ogr_geometry.h"
 #include "ogr_api.h"
 
-CPL_CVSID("$Id: gdaljp2metadata.cpp,v 1.6 2006/04/07 05:35:25 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdaljp2metadata.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static const unsigned char msi_uuid2[16] =
 {0xb1,0x4b,0xf8,0xbd,0x08,0x3d,0x4b,0x43,
@@ -229,6 +207,7 @@ int GDALJP2Metadata::ReadBoxes( FILE *fpVSIL )
 
 {
     GDALJP2Box oBox( fpVSIL );
+    int iBox = 0;
 
     oBox.ReadFirst(); 
 
@@ -281,6 +260,21 @@ int GDALJP2Metadata::ReadBoxes( FILE *fpVSIL )
             }
         }
 
+/* -------------------------------------------------------------------- */
+/*      Process simple xml boxes.                                       */
+/* -------------------------------------------------------------------- */
+        if( EQUAL(oBox.GetType(),"xml ") )
+        {
+            CPLString osBoxName;
+            char *pszXML = (char *) oBox.ReadBoxData();
+
+            osBoxName.Printf( "BOX_%d", iBox++ );
+            
+            papszGMLMetadata = CSLSetNameValue( papszGMLMetadata, 
+                                                osBoxName, pszXML );
+            CPLFree( pszXML );
+        }
+
         oBox.ReadNext();
     }
     
@@ -616,6 +610,13 @@ int GDALJP2Metadata::ParseGMLCoverageDesc()
         adfGeoTransform[3] = poOriginGeometry->getY();
         adfGeoTransform[4] = atof(papszOffset2Tokens[0]);
         adfGeoTransform[5] = atof(papszOffset2Tokens[1]);
+
+        // offset from center of pixel.
+        adfGeoTransform[0] -= adfGeoTransform[1]*0.5;
+        adfGeoTransform[0] -= adfGeoTransform[2]*0.5;
+        adfGeoTransform[3] -= adfGeoTransform[4]*0.5;
+        adfGeoTransform[3] -= adfGeoTransform[5]*0.5;
+
         bSuccess = TRUE;
         bHaveGeoTransform = TRUE;
     }
@@ -671,3 +672,238 @@ int GDALJP2Metadata::ParseGMLCoverageDesc()
     return pszProjection != NULL && bSuccess;
 }
 
+/************************************************************************/
+/*                           SetProjection()                            */
+/************************************************************************/
+
+void GDALJP2Metadata::SetProjection( const char *pszWKT )
+
+{
+    CPLFree( pszProjection );
+    pszProjection = CPLStrdup(pszWKT);
+}
+
+/************************************************************************/
+/*                              SetGCPs()                               */
+/************************************************************************/
+
+void GDALJP2Metadata::SetGCPs( int nCount, const GDAL_GCP *pasGCPsIn )
+
+{
+    if( nGCPCount > 0 )
+    {
+        GDALDeinitGCPs( nGCPCount, pasGCPList );
+        CPLFree( pasGCPList );
+    }
+
+    nGCPCount = nCount;
+    pasGCPList = GDALDuplicateGCPs(nGCPCount, pasGCPsIn);
+}
+
+/************************************************************************/
+/*                          SetGeoTransform()                           */
+/************************************************************************/
+
+void GDALJP2Metadata::SetGeoTransform( double *padfGT )
+
+{
+    memcpy( adfGeoTransform, padfGT, sizeof(double) * 6 );
+}
+
+/************************************************************************/
+/*                          CreateJP2GeoTIFF()                          */
+/************************************************************************/
+
+GDALJP2Box *GDALJP2Metadata::CreateJP2GeoTIFF()
+
+{
+/* -------------------------------------------------------------------- */
+/*      Prepare the memory buffer containing the degenerate GeoTIFF     */
+/*      file.                                                           */
+/* -------------------------------------------------------------------- */
+    int         nGTBufSize = 0;
+    unsigned char *pabyGTBuf = NULL;
+
+    if( GTIFMemBufFromWkt( pszProjection, adfGeoTransform, 
+                           nGCPCount, pasGCPList,
+                           &nGTBufSize, &pabyGTBuf ) != CE_None )
+        return NULL;
+
+    if( nGTBufSize == 0 )
+        return NULL;
+
+/* -------------------------------------------------------------------- */
+/*      Write to a box on the JP2 file.                                 */
+/* -------------------------------------------------------------------- */
+    GDALJP2Box *poBox;
+
+    poBox = GDALJP2Box::CreateUUIDBox( msi_uuid2, nGTBufSize, pabyGTBuf );
+    
+    CPLFree( pabyGTBuf );
+
+    return poBox;
+}
+
+/************************************************************************/
+/*                         PrepareCoverageBox()                         */
+/************************************************************************/
+
+GDALJP2Box *GDALJP2Metadata::CreateGMLJP2( int nXSize, int nYSize )
+
+{
+/* -------------------------------------------------------------------- */
+/*      Try do determine a PCS or GCS code we can use.                  */
+/* -------------------------------------------------------------------- */
+    OGRSpatialReference oSRS;
+    char *pszWKTCopy = (char *) pszProjection;
+    int nEPSGCode = 0;
+    char szSRSName[100];
+
+    if( oSRS.importFromWkt( &pszWKTCopy ) != OGRERR_NONE )
+        return NULL;
+
+    if( oSRS.IsProjected() )
+    {
+        const char *pszAuthName = oSRS.GetAuthorityName( "PROJCS" );
+
+        if( pszAuthName != NULL && EQUAL(pszAuthName,"epsg") )
+        {
+            nEPSGCode = atoi(oSRS.GetAuthorityCode( "PROJCS" ));
+        }
+    }
+    else if( oSRS.IsGeographic() )
+    {
+        const char *pszAuthName = oSRS.GetAuthorityName( "GEOGCS" );
+
+        if( pszAuthName != NULL && EQUAL(pszAuthName,"epsg") )
+        {
+            nEPSGCode = atoi(oSRS.GetAuthorityCode( "GEOGCS" ));
+        }
+    }
+
+    if( nEPSGCode != 0 )
+        sprintf( szSRSName, "urn:ogc:def:crs:EPSG::%d", nEPSGCode );
+    else
+        strcpy( szSRSName, 
+                "gmljp2://xml/CRSDictionary.gml#ogrcrs1" );
+
+/* -------------------------------------------------------------------- */
+/*      For now we hardcode for a minimal instance format.              */
+/* -------------------------------------------------------------------- */
+    CPLString osDoc;
+
+    osDoc.Printf( 
+"<gml:FeatureCollection\n"
+"   xmlns:gml=\"http://www.opengis.net/gml\"\n"
+"   xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
+"   xsi:schemaLocation=\"http://www.opengeospatial.net/gml http://schemas.opengis.net/gml/3.1.1/profiles/gmlJP2Profile/1.0.0/gmlJP2Profile.xsd\">\n"
+"  <gml:boundedBy>\n"
+"    <gml:Null>withheld</gml:Null>\n"
+"  </gml:boundedBy>\n"
+"  <gml:featureMember>\n"
+"    <gml:FeatureCollection>\n"
+"      <gml:featureMember>\n"
+"        <gml:RectifiedGridCoverage dimension=\"2\" gml:id=\"RGC0001\">\n"
+"          <gml:rectifiedGridDomain>\n"
+"            <gml:RectifiedGrid dimension=\"2\">\n"
+"              <gml:limits>\n"
+"                <gml:GridEnvelope>\n"
+"                  <gml:low>0 0</gml:low>\n"
+"                  <gml:high>%d %d</gml:high>\n"
+"                </gml:GridEnvelope>\n"
+"              </gml:limits>\n"
+"              <gml:axisName>x</gml:axisName>\n"
+"              <gml:axisName>y</gml:axisName>\n"
+"              <gml:origin>\n"
+"                <gml:Point gml:id=\"P0001\" srsName=\"%s\">\n"
+"                  <gml:pos>%.15g %.15g</gml:pos>\n"
+"                </gml:Point>\n"
+"              </gml:origin>\n"
+"              <gml:offsetVector srsName=\"%s\">%.15g %.15g</gml:offsetVector>\n"
+"              <gml:offsetVector srsName=\"%s\">%.15g %.15g</gml:offsetVector>\n"
+"            </gml:RectifiedGrid>\n"
+"          </gml:rectifiedGridDomain>\n"
+"          <gml:rangeSet>\n"
+"            <gml:File>\n"
+"              <gml:fileName>gmljp2://codestream/0</gml:fileName>\n"
+"              <gml:fileStructure>Record Interleaved</gml:fileStructure>\n"
+"            </gml:File>\n"
+"          </gml:rangeSet>\n"
+"        </gml:RectifiedGridCoverage>\n"
+"      </gml:featureMember>\n"
+"    </gml:FeatureCollection>\n"
+"  </gml:featureMember>\n"
+"</gml:FeatureCollection>\n",
+             nXSize-1, nYSize-1, szSRSName,
+             adfGeoTransform[0] + adfGeoTransform[1] * 0.5
+                                + adfGeoTransform[4] * 0.5, 
+             adfGeoTransform[3] + adfGeoTransform[2] * 0.5
+                                + adfGeoTransform[5] * 0.5,
+             szSRSName, 
+             adfGeoTransform[1], adfGeoTransform[2],
+             szSRSName,
+             adfGeoTransform[4], adfGeoTransform[5] );
+
+/* -------------------------------------------------------------------- */
+/*      If we need a user defined CRSDictionary entry, prepare it       */
+/*      here.                                                           */
+/* -------------------------------------------------------------------- */
+    CPLString osDictBox;
+
+    if( nEPSGCode == 0 )
+    {
+        char *pszGMLDef = NULL;
+
+        if( oSRS.exportToXML( &pszGMLDef, NULL ) == OGRERR_NONE )
+        {
+            osDictBox.Printf(  
+"<gml:Dictionary gml:id=\"CRSU1\" \n"
+"        xmlns:gml=\"http://www.opengis.net/gml\"\n"
+"        xmlns:xlink=\"http://www.w3.org/1999/xlink\"\n"
+"        xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\">\n"
+"  <gml:dictionaryEntry>\n"
+"%s\n"
+"  </gml:dictionaryEntry>\n"
+"</gml:Dictionary>\n",
+                     pszGMLDef );
+        }
+        CPLFree( pszGMLDef );
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Setup the gml.data label.                                       */
+/* -------------------------------------------------------------------- */
+    GDALJP2Box *apoGMLBoxes[5];
+    int nGMLBoxes = 0;
+
+    apoGMLBoxes[nGMLBoxes++] = GDALJP2Box::CreateLblBox( "gml.data" );
+
+/* -------------------------------------------------------------------- */
+/*      Setup gml.root-instance.                                        */
+/* -------------------------------------------------------------------- */
+    apoGMLBoxes[nGMLBoxes++] = 
+        GDALJP2Box::CreateLabelledXMLAssoc( "gml.root-instance", osDoc );
+
+/* -------------------------------------------------------------------- */
+/*      Add optional dictionary.                                        */
+/* -------------------------------------------------------------------- */
+    if( strlen(osDictBox) > 0 )
+        apoGMLBoxes[nGMLBoxes++] = 
+            GDALJP2Box::CreateLabelledXMLAssoc( "CRSDictionary.gml",
+                                                osDictBox );
+        
+/* -------------------------------------------------------------------- */
+/*      Bundle gml.data boxes into an association.                      */
+/* -------------------------------------------------------------------- */
+    GDALJP2Box *poGMLData = GDALJP2Box::CreateAsocBox( nGMLBoxes, apoGMLBoxes);
+
+/* -------------------------------------------------------------------- */
+/*      Cleanup working boxes.                                          */
+/* -------------------------------------------------------------------- */
+    while( nGMLBoxes > 0 )
+        delete apoGMLBoxes[--nGMLBoxes];
+
+    return poGMLData;
+}
+
+
