@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalrasterblock.cpp,v 1.22 2006/02/07 19:07:08 fwarmerdam Exp $
+ * $Id: gdalrasterblock.cpp 10798 2007-02-07 23:27:06Z warmerdam $
  *
  * Project:  GDAL Core
  * Purpose:  Implementation of GDALRasterBlock class and related global 
@@ -26,86 +26,15 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gdalrasterblock.cpp,v $
- * Revision 1.22  2006/02/07 19:07:08  fwarmerdam
- * applied some strategic improved outofmemory checking
- *
- * Revision 1.21  2006/01/13 20:40:47  fwarmerdam
- * grab mutex in raster block destructor while updating nCacheUsed.
- *
- * Revision 1.20  2005/07/11 19:06:36  fwarmerdam
- * Removed tile ticker, and GDALRasterBlock age.
- *
- * Revision 1.19  2005/05/23 14:28:21  fwarmerdam
- * fixed serious flaw in Detach() method
- *
- * Revision 1.18  2005/05/23 06:43:37  fwarmerdam
- * adding mutex for block list
- *
- * Revision 1.17  2005/04/04 15:24:48  fwarmerdam
- * Most C entry points now CPL_STDCALL
- *
- * Revision 1.16  2004/04/07 13:15:57  warmerda
- * Ensure that GDALSetCacheMax() keeps flushing till the new limit is
- * honoured. http://bugzilla.remotesensing.org/show_bug.cgi?id=542
- *
- * Revision 1.15  2004/04/06 19:16:16  dron
- * Remove GDALRasterBlock::IsCached() method in favor
- * of GDALRasterBand::IsBlockCached().
- *
- * Revision 1.14  2004/03/19 05:17:42  warmerda
- * Increase default cachesize to 10MB.
- *
- * Revision 1.13  2004/03/15 08:33:38  warmerda
- * Use CPLGetConfigOption() for cache max.
- *
- * Revision 1.12  2003/07/27 11:01:01  dron
- * GDALRasterBlock::IsCached() method added.
- *
- * Revision 1.11  2003/04/25 19:48:16  warmerda
- * added block locking to ensure in-use blocks arent flushed
- *
- * Revision 1.10  2003/02/21 20:07:55  warmerda
- * update header
- *
- * Revision 1.9  2003/01/28 16:51:24  warmerda
- * document cache functions
- *
- * Revision 1.8  2002/07/09 20:33:12  warmerda
- * expand tabs
- *
- * Revision 1.7  2001/09/27 16:33:41  warmerda
- * fixed problems with blocks larger than 2GB/8
- *
- * Revision 1.6  2001/07/18 04:04:30  warmerda
- * added CPL_CVSID
- *
- * Revision 1.5  2001/06/22 21:00:06  warmerda
- * fixed support for caching override by environment variable
- *
- * Revision 1.4  2001/06/22 20:09:13  warmerda
- * added GDAL_CACHEMAX environment variable support
- *
- * Revision 1.3  2000/03/31 13:42:49  warmerda
- * added debugging code
- *
- * Revision 1.2  2000/03/24 00:09:05  warmerda
- * rewrote cache management
- *
- * Revision 1.1  1998/12/31 18:52:58  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal_priv.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: gdalrasterblock.cpp,v 1.22 2006/02/07 19:07:08 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdalrasterblock.cpp 10798 2007-02-07 23:27:06Z warmerdam $");
 
 static int bCacheMaxInitialized = FALSE;
-static int nCacheMax = 10 * 1024*1024;
+static int nCacheMax = 40 * 1024*1024;
 static volatile int nCacheUsed = 0;
 
 static volatile GDALRasterBlock *poOldest = NULL;    /* tail */
@@ -167,7 +96,7 @@ int CPL_STDCALL GDALGetCacheMax()
         if( CPLGetConfigOption("GDAL_CACHEMAX",NULL) != NULL )
         {
             nCacheMax = atoi(CPLGetConfigOption("GDAL_CACHEMAX","10"));
-            if( nCacheMax < 1000 )
+            if( nCacheMax < 10000 )
                 nCacheMax *= 1024 * 1024;
         }
         bCacheMaxInitialized = TRUE;
