@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr2ogr.cpp,v 1.36 2006/03/31 17:57:32 fwarmerdam Exp $
+ * $Id: ogr2ogr.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Simple client for translating between formats.
@@ -25,118 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogr2ogr.cpp,v $
- * Revision 1.36  2006/03/31 17:57:32  fwarmerdam
- * header updates
- *
- * Revision 1.35  2006/02/09 05:22:07  fwarmerdam
- * Improve memory cleanup for leak testing.
- *
- * Revision 1.34  2006/02/09 05:03:09  fwarmerdam
- * added -overwrite switch, using DeleteLayer()
- *
- * Revision 1.33  2006/01/11 03:50:28  fwarmerdam
- * Fixed usage message to make it clear the layer is optional.
- *
- * Revision 1.32  2005/11/25 02:17:30  fwarmerdam
- * Added --help-general.
- *
- * Revision 1.31  2005/11/10 19:07:57  fwarmerdam
- * -append now creates layer if it does not already exist: bug 994.
- *
- * Revision 1.30  2005/10/16 01:59:06  cfis
- * Added declaration for OGRGeneralCmdLineProcessor to ogr_p.h, and included it into ogr2ogr.  Also changed call to CPL_DLL from CPL_STDCALL
- *
- * Revision 1.29  2005/10/16 01:39:07  cfis
- * Added support for --config, --debug, and --formats command line parameters similar to what GDAL utilities have.
- *
- * Revision 1.28  2005/04/14 14:20:24  fwarmerdam
- * More stuff to avoid feature leaks.
- *
- * Revision 1.27  2005/04/14 14:16:37  fwarmerdam
- * Fix from Julien for destroying features in error case.
- *
- * Revision 1.26  2005/03/15 21:18:47  fwarmerdam
- * Added -sql to usage message.
- *
- * Revision 1.25  2004/10/30 15:50:41  fwarmerdam
- * added -skipfailures doc
- *
- * Revision 1.24  2004/06/02 18:06:15  warmerda
- * Fix failure logic when output layer not found.
- *
- * Revision 1.23  2003/05/12 18:08:33  warmerda
- * added 25D settings for -nlt.
- *
- * Revision 1.22  2003/01/22 18:13:35  warmerda
- * use indirect (in DLL) feature creation and destruction
- *
- * Revision 1.21  2003/01/17 20:42:48  warmerda
- * added -preserve_fid and -fid commandline options
- *
- * Revision 1.20  2003/01/08 22:03:17  warmerda
- * Added code to force geometries to polygon or multipolygo if -nlt used
- *
- * Revision 1.19  2002/10/24 02:22:56  warmerda
- * added the -nlt flag
- *
- * Revision 1.18  2002/10/18 14:32:02  warmerda
- * Added -s_srs to usage option list.
- *
- * Revision 1.17  2002/10/03 13:20:31  warmerda
- * improve docs for -append and -update in ogr2ogr
- *
- * Revision 1.16  2002/05/09 16:32:29  warmerda
- * added -sql, -update and -append options
- *
- * Revision 1.15  2002/03/05 14:25:14  warmerda
- * expand tabs
- *
- * Revision 1.14  2002/01/18 04:46:38  warmerda
- * added -s_srs support
- *
- * Revision 1.13  2001/11/15 21:18:59  warmerda
- * added transaction grouping on the write side
- *
- * Revision 1.12  2001/10/25 22:33:16  danmo
- * Added support for -select option
- *
- * Revision 1.11  2001/09/27 14:50:10  warmerda
- * added untested -spat and -where support
- *
- * Revision 1.10  2001/09/21 16:19:50  warmerda
- * added support for -a_srs and -t_srs options
- *
- * Revision 1.9  2001/07/18 05:03:05  warmerda
- * added CPL_CVSID
- *
- * Revision 1.8  2001/06/26 20:58:20  warmerda
- * added -nln switch
- *
- * Revision 1.7  2000/12/05 23:09:05  warmerda
- * improved error testing, added lots of CPLResetError calls
- *
- * Revision 1.6  2000/10/17 02:23:33  warmerda
- * improve error reporting
- *
- * Revision 1.5  2000/06/19 18:46:59  warmerda
- * added -skipfailures
- *
- * Revision 1.4  2000/06/09 21:15:19  warmerda
- * made CreateField and SetFrom forgiving
- *
- * Revision 1.3  2000/03/14 21:37:35  warmerda
- * added support for passing options to create methods
- *
- * Revision 1.2  1999/11/18 19:02:19  warmerda
- * expanded tabs
- *
- * Revision 1.1  1999/11/04 21:07:53  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogrsf_frmts.h"
 #include "ogr_p.h"
@@ -144,7 +33,7 @@
 #include "cpl_string.h"
 #include "ogr_api.h"
 
-CPL_CVSID("$Id: ogr2ogr.cpp,v 1.36 2006/03/31 17:57:32 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr2ogr.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static void Usage();
 
@@ -552,12 +441,16 @@ int main( int nArgc, char ** papszArgv )
 /* -------------------------------------------------------------------- */
 /*      Close down.                                                     */
 /* -------------------------------------------------------------------- */
+    delete poOutputSRS;
+    delete poSourceSRS;
     delete poODS;
     delete poDS;
 
     CSLDestroy(papszSelFields);
     CSLDestroy( papszArgv );
     CSLDestroy( papszLayers );
+    CSLDestroy( papszDSCO );
+    CSLDestroy( papszLCO );
 
     OGRCleanupAll();
 
@@ -577,13 +470,13 @@ static void Usage()
 {
     OGRSFDriverRegistrar        *poR = OGRSFDriverRegistrar::GetRegistrar();
 
-    printf( "Usage: ogr2ogr [-skipfailures] [-append] [-update] [-f format_name]\n"
+    printf( "Usage: ogr2ogr [--help-general] [-skipfailures] [-append] [-update]\n"
             "               [-select field_list] [-where restricted_where] \n"
-            "               [-sql <sql statement>] [--help-general]\n" 
+            "               [-sql <sql statement>] \n" 
             "               [-spat xmin ymin xmax ymax] [-preserve_fid] [-fid FID]\n"
             "               [-a_srs srs_def] [-t_srs srs_def] [-s_srs srs_def]\n"
-            "               [[-dsco NAME=VALUE] ...] dst_datasource_name\n"
-            "               src_datasource_name\n"
+            "               [-f format_name] [-overwrite] [[-dsco NAME=VALUE] ...]\n"
+            "               dst_datasource_name src_datasource_name\n"
             "               [-lco NAME=VALUE] [-nln name] [-nlt type] [layer [layer ...]]\n"
             "\n"
             " -f format_name: output file format name, possible values are:\n");
@@ -740,7 +633,14 @@ static int TranslateLayer( OGRDataSource *poSrcDS,
         if( eGType == -2 )
             eGType = poFDefn->GetGeomType();
 
-        CPLAssert( poDstDS->TestCapability( ODsCCreateLayer ) );
+        if( !poDstDS->TestCapability( ODsCCreateLayer ) )
+        {
+            fprintf( stderr, 
+              "Layer %s not found, and CreateLayer not supported by driver.", 
+                     pszNewLayerName );
+            return FALSE;
+        }
+
         CPLErrorReset();
 
         poDstLayer = poDstDS->CreateLayer( pszNewLayerName, poOutputSRS,
@@ -902,6 +802,11 @@ static int TranslateLayer( OGRDataSource *poSrcDS,
     if( nGroupTransactions )
         poDstLayer->CommitTransaction();
 
+/* -------------------------------------------------------------------- */
+/*      Cleaning                                                        */
+/* -------------------------------------------------------------------- */
+    delete poCT;
+
     return TRUE;
 }
 
