@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_fromepsg.cpp,v 1.43 2006/04/21 04:12:29 fwarmerdam Exp $
+ * $Id: ogr_fromepsg.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Generate an OGRSpatialReference object based on an EPSG
@@ -26,144 +26,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogr_fromepsg.cpp,v $
- * Revision 1.43  2006/04/21 04:12:29  fwarmerdam
- * Removed debugging printf.
- *
- * Revision 1.42  2006/04/21 04:10:07  fwarmerdam
- * added deprecated PCS name if it is deprecated
- *
- * Revision 1.41  2006/03/01 03:12:51  fwarmerdam
- * added method 9823 (Equirectangular/Plate Caree/Equidistant Cylindrical
- *
- * Revision 1.40  2005/12/06 06:41:18  fwarmerdam
- * added slightly questionable translation for method 9829 (PS Variant B
- *
- * Revision 1.39  2005/04/06 00:02:05  fwarmerdam
- * various osr and oct functions now stdcall
- *
- * Revision 1.38  2004/09/10 21:02:41  fwarmerdam
- * added note about 9814
- *
- * Revision 1.37  2004/05/26 19:53:17  warmerda
- * Fixed poor NULL checking in auto-identify.
- *
- * Revision 1.36  2004/05/10 17:05:14  warmerda
- * added AutoIdentifyEPSG()
- *
- * Revision 1.35  2004/05/04 17:54:06  warmerda
- * keep longitudes greenwich relative
- *
- * Revision 1.34  2004/04/28 00:27:58  warmerda
- * Added Albers Conic Area Area translation (method=9822).
- *
- * Revision 1.33  2004/04/24 15:47:19  warmerda
- * Added uom 9122 improvements.
- *
- * Revision 1.32  2004/03/04 18:04:45  warmerda
- * added importFromDict() support
- *
- * Revision 1.31  2003/11/20 19:41:15  warmerda
- * added logic to set EPSG authority info if read from PROJ.4
- *
- * Revision 1.30  2003/11/19 20:40:58  warmerda
- * Add support for faking EPSG coordinate systems from PROJ.4 definitions
- *
- * Revision 1.29  2003/06/23 14:48:42  warmerda
- * make OGREPSTDatumNameMassage() public
- *
- * Revision 1.28  2003/05/30 18:35:15  warmerda
- * fixed stateplane.csv error msg and clear() before SetLocalCS call
- *
- * Revision 1.27  2003/05/30 17:48:57  warmerda
- * Set GEOGCS|UNIT AUTHORITY information.
- *
- * Revision 1.26  2003/05/30 15:39:53  warmerda
- * Added override units capability for SetStatePlane()
- *
- * Revision 1.25  2003/05/20 19:03:24  warmerda
- * suggest GDAL_DATA in error message
- *
- * Revision 1.24  2003/05/20 18:43:31  warmerda
- * added error reporting
- *
- * Revision 1.23  2003/02/14 22:15:04  warmerda
- * expand tabs
- *
- * Revision 1.22  2003/02/14 16:29:02  warmerda
- * Added LAEA support
- *
- * Revision 1.21  2003/02/06 04:53:53  warmerda
- * fixed handling of empty fields to avoid emitting debug warning
- *
- * Revision 1.20  2003/02/05 17:14:11  warmerda
- * work around a bug in EPSG 6.2.2 with scale factor units for PCS 2935/2936
- *
- * Revision 1.19  2003/01/08 18:23:45  warmerda
- * ensure order is fixed up
- *
- * Revision 1.18  2002/12/16 17:06:32  warmerda
- * ensure that prime meridian offsets are applied to angular proj parms
- *
- * Revision 1.17  2002/12/14 22:59:14  warmerda
- * added Krovak in ESRI compatible way
- *
- * Revision 1.16  2002/12/13 06:35:45  warmerda
- * fixed rotatin sign in TOWGS84 from method 9607
- *
- * Revision 1.15  2002/12/02 00:07:50  warmerda
- * set angular units properly in GEOGCS
- *
- * Revision 1.14  2002/11/30 15:45:07  warmerda
- * be careful about how UOM codes are linear
- *
- * Revision 1.13  2002/11/29 20:49:10  warmerda
- * fixed GetTRFInfo filename corruption
- *
- * Revision 1.12  2002/11/28 15:59:58  warmerda
- * rewritten to use EPSG 6.2.2 tables
- *
- * Revision 1.11  2001/08/23 13:20:07  warmerda
- * fixed serious bug in translating LCC projections from EPSG to WKT
- *
- * Revision 1.10  2001/07/18 05:03:05  warmerda
- * added CPL_CVSID
- *
- * Revision 1.9  2001/03/15 03:19:23  warmerda
- * added polyconic (9818) support
- *
- * Revision 1.8  2001/03/14 21:33:46  warmerda
- * treaat 9717 (Lambert Near Conformal) like LCC1SP
- *
- * Revision 1.7  2001/01/19 21:10:46  warmerda
- * replaced tabs
- *
- * Revision 1.6  2000/12/07 03:46:04  danmo
- * Made papszDatumEquiv[] const
- *
- * Revision 1.5  2000/10/23 13:01:59  warmerda
- * Fixed OSRSetStatePlane
- *
- * Revision 1.4  2000/10/20 04:19:38  warmerda
- * added setstateplane
- *
- * Revision 1.3  2000/03/20 22:39:08  warmerda
- * Added C function.
- *
- * Revision 1.2  2000/03/20 14:59:05  warmerda
- * fixed semi major axis check
- *
- * Revision 1.1  2000/03/16 19:04:14  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogr_spatialref.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogr_fromepsg.cpp,v 1.43 2006/04/21 04:12:29 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr_fromepsg.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 #ifndef PI
 #  define PI 3.14159265358979323846
