@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_esri.cpp,v 1.47 2006/04/24 23:21:07 fwarmerdam Exp $
+ * $Id: ogr_srs_esri.cpp 11161 2007-04-02 03:14:32Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  OGRSpatialReference translation to/from ESRI .prj definitions.
@@ -25,112 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogr_srs_esri.cpp,v $
- * Revision 1.47  2006/04/24 23:21:07  fwarmerdam
- * fixup some common ellipsoid names (from pgao)
- *
- * Revision 1.46  2006/03/31 17:44:20  fwarmerdam
- * header updates
- *
- * Revision 1.45  2006/01/16 15:19:03  fwarmerdam
- * Fixed datum.csv error message.
- *
- * Revision 1.44  2005/12/20 21:49:57  fwarmerdam
- * trim log
- *
- * Revision 1.43  2005/12/20 21:48:38  fwarmerdam
- * Fixed last fix which seriously hosted morphToESRI().
- *
- * Revision 1.42  2005/12/20 02:09:49  fwarmerdam
- * ESRI requires linear units.  Ensure morphToESRI() produces them.
- *
- * Revision 1.41  2005/10/27 23:12:16  fwarmerdam
- * morphToESRI() now maps a PROJCS of Unknown to the PROJECTION value
- *
- * Revision 1.40  2005/08/03 16:11:18  fwarmerdam
- * Added additional morphToESRI() rules to:
- *  o Change NAD27/NAD83/WGS84 GEOGCSes to GCS_ names used by ESRI.
- *  o Change the word Unnamed to Unknown in names.
- *  o Set exact ESRI names for UTM PROJCSes (for WGS84, NAD27 and NAD83).
- *
- * Revision 1.39  2005/05/04 14:29:20  fwarmerdam
- * convert Standard_Parallel_1 in ESRI Mercator to Latitude_Of_Origin.
- *
- * Revision 1.38  2005/02/18 21:52:44  fwarmerdam
- * Fixed support for .prj files with blank lines between the parameters.
- *
- * Revision 1.37  2005/01/13 16:32:27  fwarmerdam
- * added support for fipszone for stateplane
- *
- * Revision 1.36  2005/01/13 15:18:05  fwarmerdam
- * use SetLinearUnitsAndUpdateParameters()
- *
- * Revision 1.35  2004/09/23 16:20:40  fwarmerdam
- * added clode to cleanup datum mapping table: bug 613
- *
- * Revision 1.34  2004/09/10 21:03:55  fwarmerdam
- * Lots of changes to map Hotine_Oblique_Mercator_Azimuth_Center in ESRI format
- * to Hotine_Oblique_Mercator with a rectified_grid_angle of 90 (and back).
- * This is a special case for the swiss oblique mercator.
- * See http://bugzilla.remotesensing.org/show_bug.cgi?id=423
- *
- * Revision 1.33  2004/07/29 19:12:05  warmerda
- * support multiline ESRI .prj files in WKT format
- *
- * Revision 1.32  2004/05/04 13:11:49  warmerda
- * Added support for KRASOVSKY spheroid
- *
- * Revision 1.31  2004/04/24 15:45:04  warmerda
- * Added GRS80 spheroid support.
- *
- * Revision 1.30  2004/04/19 19:32:25  warmerda
- * added INTERNATIONAL1909
- *
- * Revision 1.29  2004/02/25 21:14:35  warmerda
- * added morph of spheroid names to ESRI
- *
- * Revision 1.28  2003/11/03 21:37:34  warmerda
- * fixed southern hemisphere UTM support
- *
- * Revision 1.27  2003/08/14 14:36:24  warmerda
- * some ESRI .prj files use Central_Parallel instead of latitude_of_origin
- *
- * Revision 1.26  2003/08/03 01:30:34  warmerda
- * Added mapping for EUR datum as per:
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=371
- *
- * Revision 1.25  2003/06/23 14:49:17  warmerda
- * added InitDatumMappingTable, and use of gdal_datum.csv file
- *
- * Revision 1.24  2003/06/17 14:39:00  warmerda
- * Translate Equirectangular to/from Equidistant_Cylindrical
- *
- * Revision 1.23  2003/05/21 02:59:41  warmerda
- * morphToEsri() now sets specific units constants for Foot_US and Degree.
- * morphToEsri() converts albers parameters to match expected ESRI values.
- *
- * Revision 1.22  2003/05/08 21:52:55  warmerda
- * fixed C morphToESRI func, added ESRI unit remapping
- *
- * Revision 1.21  2003/02/25 04:53:16  warmerda
- * Added support for the LAMBERT projection.  Added support for defining a
- * GEOGCS from the SPHEROID if there is no known DATUM.   Fixed bug with
- * GREATBRITIAN_GRID.
- *
- * Revision 1.20  2003/02/14 22:15:04  warmerda
- * expand tabs
- *
- * Revision 1.19  2003/01/24 20:15:34  warmerda
- * added polar stereographic support
- */
+ ****************************************************************************/
 
 #include "ogr_spatialref.h"
 #include "ogr_p.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogr_srs_esri.cpp,v 1.47 2006/04/24 23:21:07 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr_srs_esri.cpp 11161 2007-04-02 03:14:32Z warmerdam $");
 
 static char *apszProjMapping[] = {
     "Albers", SRS_PT_ALBERS_CONIC_EQUAL_AREA,
@@ -154,10 +55,19 @@ static char *apszAlbersMapping[] = {
     "Central_Parallel", SRS_PP_LATITUDE_OF_CENTER,
     NULL, NULL };
 
+static char *apszECMapping[] = {
+    SRS_PP_CENTRAL_MERIDIAN, SRS_PP_LONGITUDE_OF_CENTER, 
+    SRS_PP_LATITUDE_OF_ORIGIN, SRS_PP_LATITUDE_OF_CENTER, 
+    NULL, NULL };
+
 static char *apszMercatorMapping[] = {
     SRS_PP_STANDARD_PARALLEL_1, SRS_PP_LATITUDE_OF_ORIGIN,
     NULL, NULL };
 
+static char *apszPolarStereographicMapping[] = {
+    SRS_PP_STANDARD_PARALLEL_1, SRS_PP_LATITUDE_OF_ORIGIN,
+    NULL, NULL };
+
 static char **papszDatumMapping = NULL;
  
 static char *apszDefaultDatumMapping[] = {
@@ -917,7 +827,10 @@ OGRErr OGRSpatialReference::importFromESRI( char **papszPrj )
         if( pszValue == NULL )
             SetLinearUnitsAndUpdateParameters( SRS_UL_METER, 1.0 );
         else if( EQUAL(pszValue,"FEET") )
-            SetLinearUnitsAndUpdateParameters( SRS_UL_FOOT, atof(SRS_UL_FOOT_CONV) );
+            SetLinearUnitsAndUpdateParameters( SRS_UL_US_FOOT, atof(SRS_UL_US_FOOT_CONV) );
+        else if( atof(pszValue) != 0.0 )
+            SetLinearUnitsAndUpdateParameters( "user-defined", 
+                                               1.0 / atof(pszValue) );
         else
             SetLinearUnitsAndUpdateParameters( pszValue, 1.0 );
     }
@@ -980,6 +893,29 @@ OGRErr OGRSpatialReference::morphToESRI()
                  "Hotine_Oblique_Mercator_Azimuth_Center" );
 
         /* ideally we should strip out of the rectified_grid_angle */
+        pszProjection = GetAttrValue("PROJECTION");
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Polar_Stereographic maps to ESRI codes                          */
+/*      Stereographic_South_Pole or Stereographic_North_Pole based      */
+/*      on latitude.                                                    */
+/* -------------------------------------------------------------------- */
+    if( pszProjection != NULL
+        && EQUAL(pszProjection,SRS_PT_POLAR_STEREOGRAPHIC) )
+    {
+        if( GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN, 0.0 ) < 0.0 )
+        {
+            SetNode( "PROJCS|PROJECTION", 
+                     "Stereographic_South_Pole" );
+            pszProjection = GetAttrValue("PROJECTION");
+        }
+        else
+        {
+            SetNode( "PROJCS|PROJECTION", 
+                     "Stereographic_North_Pole" );
+            pszProjection = GetAttrValue("PROJECTION");
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -987,6 +923,7 @@ OGRErr OGRSpatialReference::morphToESRI()
 /* -------------------------------------------------------------------- */
     GetRoot()->applyRemapper( "PROJECTION", 
                               apszProjMapping+1, apszProjMapping, 2 );
+    pszProjection = GetAttrValue("PROJECTION");
 
 /* -------------------------------------------------------------------- */
 /*      Translate DATUM keywords that are misnamed.                     */
@@ -1134,9 +1071,22 @@ OGRErr OGRSpatialReference::morphToESRI()
         GetRoot()->applyRemapper( 
             "PARAMETER", apszAlbersMapping + 1, apszAlbersMapping + 0, 2 );
 
+    if( pszProjection != NULL 
+        && EQUAL(pszProjection,SRS_PT_EQUIDISTANT_CONIC) )
+        GetRoot()->applyRemapper( 
+            "PARAMETER", apszECMapping + 1, apszECMapping + 0, 2 );
+
     if( pszProjection != NULL && EQUAL(pszProjection,"Mercator") )
         GetRoot()->applyRemapper( 
-            "PARAMETER", apszAlbersMapping + 1, apszAlbersMapping + 0, 2 );
+            "PARAMETER", apszMercatorMapping + 1, apszMercatorMapping + 0, 2 );
+
+    if( pszProjection != NULL 
+        && EQUALN(pszProjection,"Stereographic_",14)
+        && EQUALN(pszProjection+strlen(pszProjection)-5,"_Pole",5) )
+        GetRoot()->applyRemapper( 
+            "PARAMETER", 
+            apszPolarStereographicMapping + 1, 
+            apszPolarStereographicMapping + 0, 2 );
 
 /* -------------------------------------------------------------------- */
 /*      Convert SPHEROID name to use underscores instead of spaces.     */
@@ -1252,13 +1202,18 @@ OGRErr OGRSpatialReference::morphFromESRI()
 /*      Split Lambert_Conformal_Conic into 1SP or 2SP form.             */
 /*                                                                      */
 /*      See bugzilla.remotesensing.org/show_bug.cgi?id=187              */
+/*                                                                      */
+/*      We decide based on whether it has 2SPs.  We used to assume      */
+/*      1SP if it had a scale factor but that turned out to be a        */
+/*      poor test.                                                      */
 /* -------------------------------------------------------------------- */
     const char *pszProjection = GetAttrValue("PROJECTION");
     
     if( pszProjection != NULL
         && EQUAL(pszProjection,"Lambert_Conformal_Conic") )
     {
-        if( GetProjParm( "Scale_Factor", 2.0 ) == 2.0 )
+        if( GetProjParm( SRS_PP_STANDARD_PARALLEL_1, 1000.0 ) != 1000.0 
+            && GetProjParm( SRS_PP_STANDARD_PARALLEL_2, 1000.0 ) != 1000.0 )
             SetNode( "PROJCS|PROJECTION", 
                      SRS_PT_LAMBERT_CONFORMAL_CONIC_2SP );
         else
@@ -1280,16 +1235,40 @@ OGRErr OGRSpatialReference::morphFromESRI()
     }
 
 /* -------------------------------------------------------------------- */
-/*      Remap albers and Mercator parameters.                           */
+/*      Remap Albers, Mercator and Polar Stereographic parameters.      */
 /* -------------------------------------------------------------------- */
     if( pszProjection != NULL && EQUAL(pszProjection,"Albers") )
         GetRoot()->applyRemapper( 
             "PARAMETER", apszAlbersMapping + 0, apszAlbersMapping + 1, 2 );
 
+    if( pszProjection != NULL 
+        && EQUAL(pszProjection,SRS_PT_EQUIDISTANT_CONIC) )
+        GetRoot()->applyRemapper( 
+            "PARAMETER", apszECMapping + 0, apszECMapping + 1, 2 );
+
     if( pszProjection != NULL && EQUAL(pszProjection,"Mercator") )
         GetRoot()->applyRemapper( 
             "PARAMETER", apszMercatorMapping + 0, apszMercatorMapping + 1, 2 );
 
+    if( pszProjection != NULL 
+        && EQUALN(pszProjection,"Stereographic_",14) 
+        && EQUALN(pszProjection+strlen(pszProjection)-5,"_Pole",5) )
+        GetRoot()->applyRemapper( 
+            "PARAMETER", 
+            apszPolarStereographicMapping + 0, 
+            apszPolarStereographicMapping + 1, 2 );
+
+/* -------------------------------------------------------------------- */
+/*      Remap south and north polar stereographic to one value.         */
+/* -------------------------------------------------------------------- */
+    if( pszProjection != NULL
+        && EQUALN(pszProjection,"Stereographic_",14)
+        && EQUALN(pszProjection+strlen(pszProjection)-5,"_Pole",5) )
+    {
+        SetNode( "PROJCS|PROJECTION", SRS_PT_POLAR_STEREOGRAPHIC );
+        pszProjection = GetAttrValue("PROJECTION");
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Translate PROJECTION keywords that are misnamed.                */
 /* -------------------------------------------------------------------- */
