@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_srs_validate.cpp,v 1.8 2005/12/01 04:59:46 fwarmerdam Exp $
+ * $Id: ogr_srs_validate.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implementation of the OGRSpatialReference::Validate() method and
@@ -26,39 +26,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogr_srs_validate.cpp,v $
- * Revision 1.8  2005/12/01 04:59:46  fwarmerdam
- * added two point equidistant support
- *
- * Revision 1.7  2005/10/03 17:34:19  fwarmerdam
- * Added a bunch of missing definitions as per 937.
- *
- * Revision 1.6  2004/11/11 18:28:45  fwarmerdam
- * added Bonne projection support
- *
- * Revision 1.5  2004/02/06 07:42:14  dron
- * Support for HOM projection, specified by two points on centerline.
- *
- * Revision 1.4  2002/03/05 14:25:14  warmerda
- * expand tabs
- *
- * Revision 1.3  2001/11/23 17:02:04  warmerda
- * add CT support to PROJCS children
- *
- * Revision 1.2  2001/11/21 03:43:00  warmerda
- * a variety of fixes to validate to allow for CT style parms
- *
- * Revision 1.1  2001/10/11 19:23:49  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogr_spatialref.h"
 #include "ogr_p.h"
 
-CPL_CVSID("$Id: ogr_srs_validate.cpp,v 1.8 2005/12/01 04:59:46 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogr_srs_validate.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /* why would fipszone and zone be paramers when they relate to a composite
    projection which renders done into a non-zoned projection? */
@@ -384,6 +357,14 @@ static char *papszProjWithParms[] = {
     SRS_PP_FALSE_NORTHING,
     NULL,
 
+    SRS_PT_OBLIQUE_STEREOGRAPHIC,
+    SRS_PP_LATITUDE_OF_ORIGIN,
+    SRS_PP_CENTRAL_MERIDIAN,
+    SRS_PP_SCALE_FACTOR,
+    SRS_PP_FALSE_EASTING,
+    SRS_PP_FALSE_NORTHING,
+    NULL,
+
     SRS_PT_VANDERGRINTEN,
     SRS_PP_CENTRAL_MERIDIAN,
     SRS_PP_FALSE_EASTING,
