@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpoint.cpp,v 1.37 2006/03/31 17:44:20 fwarmerdam Exp $
+ * $Id: ogrpoint.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  The Point geometry class.
@@ -25,127 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrpoint.cpp,v $
- * Revision 1.37  2006/03/31 17:44:20  fwarmerdam
- * header updates
- *
- * Revision 1.36  2006/01/06 19:04:13  fwarmerdam
- * fix clone to preserve coordinate dimension
- *
- * Revision 1.35  2005/09/12 01:43:40  fwarmerdam
- * ensure Z is initialized to zero in 2D constructor
- *
- * Revision 1.34  2005/08/04 17:18:59  fwarmerdam
- * now have separate 2D and 3D OGRPoint constructors
- *
- * Revision 1.33  2005/07/22 20:39:36  fwarmerdam
- * Fixed getGeometryType to look at coordinate dimension, expand wkt buffer.
- *
- * Revision 1.32  2005/07/20 01:43:51  fwarmerdam
- * upgraded OGR geometry dimension handling
- *
- * Revision 1.31  2005/07/12 17:34:00  fwarmerdam
- * updated to produce proper empty syntax and consume either
- *
- * Revision 1.30  2005/04/06 20:43:00  fwarmerdam
- * fixed a variety of method signatures for documentation
- *
- * Revision 1.29  2005/02/22 12:38:01  fwarmerdam
- * rename Equal/Intersect to Equals/Intersects
- *
- * Revision 1.28  2004/02/22 10:03:40  dron
- * Fix compirison casting problems in OGRPoint::Equal() method.
- *
- * Revision 1.27  2004/02/21 15:36:14  warmerda
- * const correctness updates for geometry: bug 289
- *
- * Revision 1.26  2004/01/16 21:57:17  warmerda
- * fixed up EMPTY support
- *
- * Revision 1.25  2004/01/16 21:20:00  warmerda
- * Added EMPTY support
- *
- * Revision 1.24  2003/08/27 15:40:37  warmerda
- * added support for generating DB2 V7.2 compatible WKB
- *
- * Revision 1.23  2003/06/09 13:48:54  warmerda
- * added DB2 V7.2 byte order hack
- *
- * Revision 1.22  2003/05/28 19:16:43  warmerda
- * fixed up argument names and stuff for docs
- *
- * Revision 1.21  2003/03/07 21:28:56  warmerda
- * support 0x8000 style 3D WKB flags
- *
- * Revision 1.20  2003/02/08 00:37:14  warmerda
- * try to improve documentation
- *
- * Revision 1.19  2002/09/11 13:47:17  warmerda
- * preliminary set of fixes for 3D WKB enum
- *
- * Revision 1.18  2002/05/02 19:45:36  warmerda
- * added flattenTo2D() method
- *
- * Revision 1.17  2001/11/01 17:20:33  warmerda
- * added DISABLE_OGRGEOM_TRANSFORM macro
- *
- * Revision 1.16  2001/11/01 17:01:28  warmerda
- * pass output buffer into OGRMakeWktCoordinate
- *
- * Revision 1.15  2001/09/21 16:24:20  warmerda
- * added transform() and transformTo() methods
- *
- * Revision 1.14  2001/07/18 05:03:05  warmerda
- * added CPL_CVSID
- *
- * Revision 1.13  1999/11/18 19:02:19  warmerda
- * expanded tabs
- *
- * Revision 1.12  1999/09/13 14:34:07  warmerda
- * updated to use wkbZ of 0x8000 instead of 0x80000000
- *
- * Revision 1.11  1999/09/13 02:27:33  warmerda
- * incorporated limited 2.5d support
- *
- * Revision 1.10  1999/07/27 00:48:11  warmerda
- * Added Equal() support
- *
- * Revision 1.9  1999/07/06 21:36:47  warmerda
- * tenatively added getEnvelope() and Intersect()
- *
- * Revision 1.8  1999/06/25 20:44:43  warmerda
- * implemented assignSpatialReference, carry properly
- *
- * Revision 1.7  1999/05/31 20:42:13  warmerda
- * added empty method
- *
- * Revision 1.6  1999/05/31 14:59:06  warmerda
- * added documentation
- *
- * Revision 1.5  1999/05/31 11:05:08  warmerda
- * added some documentation
- *
- * Revision 1.4  1999/05/23 05:34:40  warmerda
- * added support for clone(), multipolygons and geometry collections
- *
- * Revision 1.3  1999/05/20 14:35:44  warmerda
- * added support for well known text format
- *
- * Revision 1.2  1999/03/30 21:21:43  warmerda
- * added linearring/polygon support
- *
- * Revision 1.1  1999/03/29 21:21:10  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogr_geometry.h"
 #include "ogr_p.h"
 #include <assert.h>
 
-CPL_CVSID("$Id: ogrpoint.cpp,v 1.37 2006/03/31 17:44:20 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrpoint.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                              OGRPoint()                              */
@@ -272,6 +158,19 @@ void OGRPoint::flattenTo2D()
     nCoordDimension = 2;
 }
 
+/************************************************************************/
+/*                       setCoordinateDimension()                       */
+/************************************************************************/
+
+void OGRPoint::setCoordinateDimension( int nNewDimension )
+
+{
+    nCoordDimension = nNewDimension;
+    
+    if( nCoordDimension == 2 )
+        z = 0;
+}
+
 /************************************************************************/
 /*                              WkbSize()                               */
 /*                                                                      */
