@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogravcdatasource.cpp,v 1.2 2005/09/21 01:00:55 fwarmerdam Exp $
+ * $Id: ogravcdatasource.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OGR
  * Purpose:  Implements OGRAVCDataSource class.
@@ -25,20 +25,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogravcdatasource.cpp,v $
- * Revision 1.2  2005/09/21 01:00:55  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.1  2002/02/14 23:01:09  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogr_avc.h"
 
-CPL_CVSID("$Id: ogravcdatasource.cpp,v 1.2 2005/09/21 01:00:55 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogravcdatasource.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                          OGRAVCDataSource()                          */
