@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrcsvlayer.cpp,v 1.8 2006/04/08 05:37:23 fwarmerdam Exp $
+ * $Id: ogrcsvlayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  CSV Translator
  * Purpose:  Implements OGRCSVLayer class.
@@ -25,41 +25,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrcsvlayer.cpp,v $
- * Revision 1.8  2006/04/08 05:37:23  fwarmerdam
- * Avoid memory leak of fieldtypes.
- *
- * Revision 1.7  2006/03/15 15:12:21  fwarmerdam
- * Make sure that csvt column type parsing is case insensitive.
- *
- * Revision 1.6  2005/09/21 01:01:01  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.5  2005/06/20 17:54:04  fwarmerdam
- * added support for external csvt file
- *
- * Revision 1.4  2005/02/02 20:30:10  fwarmerdam
- * added m_nFeaturesRead support
- *
- * Revision 1.3  2004/08/17 15:40:40  warmerda
- * track capabilities and update mode better
- *
- * Revision 1.2  2004/08/16 21:29:48  warmerda
- * added output support
- *
- * Revision 1.1  2004/07/20 19:18:23  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogr_csv.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 #include "cpl_csv.h"
 
-CPL_CVSID("$Id: ogrcsvlayer.cpp,v 1.8 2006/04/08 05:37:23 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrcsvlayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                            OGRCSVLayer()                             */
