@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: gmlpropertydefn.cpp,v 1.2 2004/01/19 16:54:44 warmerda Exp $
+ * $Id: gmlpropertydefn.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GML Reader
  * Purpose:  Implementation of GMLPropertyDefn
@@ -25,17 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
  * DEALINGS IN THE SOFTWARE.
- **********************************************************************
- *
- * $Log: gmlpropertydefn.cpp,v $
- * Revision 1.2  2004/01/19 16:54:44  warmerda
- * added logic to capture field types
- *
- * Revision 1.1  2002/01/04 19:46:30  warmerda
- * New
- *
- *
- **********************************************************************/
+ ****************************************************************************/
 
 #include "gmlreader.h"
 #include "cpl_conv.h"
@@ -54,6 +44,7 @@ GMLPropertyDefn::GMLPropertyDefn( const char *pszName,
     else
         m_pszSrcElement = NULL;
     m_eType = GMLPT_Untyped;
+    m_nWidth = 0; 
 }
 
 /************************************************************************/
@@ -96,7 +87,14 @@ void GMLPropertyDefn::AnalysePropertyValue( const char *pszValue )
 /*      give up on changing.                                            */
 /* -------------------------------------------------------------------- */
     if( m_eType == GMLPT_String )
+    {
+        /* grow the Width to the length of the string passed in */
+        int nWidth;
+        nWidth = strlen(pszValue);
+        if ( m_nWidth < nWidth ) 
+            SetWidth( nWidth );
         return;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      If it is a zero length string, just return.  We can't deduce    */
