@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gmlreader.h,v 1.9 2005/01/27 04:05:35 fwarmerdam Exp $
+ * $Id: gmlreader.h 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GML Reader
  * Purpose:  Public Declarations for OGR free GML Reader code.
@@ -25,36 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gmlreader.h,v $
- * Revision 1.9  2005/01/27 04:05:35  fwarmerdam
- * added ParseXSD
- *
- * Revision 1.8  2004/01/19 16:54:44  warmerda
- * added logic to capture field types
- *
- * Revision 1.7  2003/05/21 03:48:35  warmerda
- * Expand tabs
- *
- * Revision 1.6  2002/03/06 20:08:02  warmerda
- * added tracking of extents, feature count and extrainfo
- *
- * Revision 1.5  2002/01/25 21:23:21  warmerda
- * handle IGMLReader destructor properly in gmlreader.cpp
- *
- * Revision 1.4  2002/01/25 21:17:34  warmerda
- * provided IGMLReader destructor implementation
- *
- * Revision 1.3  2002/01/25 20:38:01  warmerda
- * added prescan and resetreading methods
- *
- * Revision 1.2  2002/01/24 17:39:08  warmerda
- * added xml serialization and geometry support
- *
- * Revision 1.1  2002/01/04 19:46:30  warmerda
- * New
- */
+ ****************************************************************************/
 
 #ifndef _GMLREADER_H_INCLUDED
 #define _GMLREADER_H_INCLUDED
@@ -77,6 +48,7 @@ class CPL_DLL GMLPropertyDefn
 {
     char             *m_pszName;
     GMLPropertyType   m_eType;
+    int               m_nWidth;
     char             *m_pszSrcElement;
 
 public:
@@ -88,7 +60,8 @@ public:
 
     GMLPropertyType GetType() { return m_eType; } const
     void        SetType( GMLPropertyType eType ) { m_eType = eType; }
-
+    void        SetWidth( int nWidth) { m_nWidth = nWidth; }
+    int         GetWidth() { return m_nWidth; }
     void        SetSrcElement( const char *pszSrcElement );
     const char *GetSrcElement() { return m_pszSrcElement; }
 
