@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gmlreaderp.h,v 1.8 2005/01/27 04:06:47 fwarmerdam Exp $
+ * $Id: gmlreaderp.h 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GML Reader
  * Purpose:  Private Declarations for OGR free GML Reader code.
@@ -25,33 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gmlreaderp.h,v $
- * Revision 1.8  2005/01/27 04:06:47  fwarmerdam
- * added ParseXSD
- *
- * Revision 1.7  2003/06/27 17:11:23  warmerda
- * removed DOMString.hpp, and added XERCES namespace support
- *
- * Revision 1.6  2003/05/21 03:48:35  warmerda
- * Expand tabs
- *
- * Revision 1.5  2003/02/21 14:31:56  warmerda
- * hack for INFINITY conflict problems with math.h
- *
- * Revision 1.4  2002/03/06 20:08:02  warmerda
- * added tracking of extents, feature count and extrainfo
- *
- * Revision 1.3  2002/01/25 20:37:43  warmerda
- * added prescan and resetreading methods
- *
- * Revision 1.2  2002/01/24 17:38:37  warmerda
- * Converted TrString to C funcs, added various methods
- *
- * Revision 1.1  2002/01/04 19:46:30  warmerda
- * New
- */
+ ****************************************************************************/
 
 #ifndef _CPL_GMLREADERP_H_INCLUDED
 #define _CPL_GMLREADERP_H_INCLUDED
@@ -69,6 +43,7 @@
 #include <sax2/ContentHandler.hpp>
 #include <sax2/SAX2XMLReader.hpp>
 #include <sax2/XMLReaderFactory.hpp>
+#include <sax2/Attributes.hpp>
 
 #ifdef XERCES_CPP_NAMESPACE_USE
 XERCES_CPP_NAMESPACE_USE
@@ -94,7 +69,10 @@ class GMLHandler : public DefaultHandler
     GMLReader  *m_poReader;
 
     char       *m_pszCurField;
+
     char       *m_pszGeometry;
+    int        m_nGeomAlloc;
+    int        m_nGeomLen;
 
     int        m_nGeometryDepth;
     int        IsGeometryElement( const char * );
