@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ili1reader.cpp,v 1.14 2006/04/27 16:37:19 pka Exp $
+ * $Id: ili1reader.cpp 10653 2007-01-18 16:38:13Z warmerdam $
  *
  * Project:  Interlis 1 Reader
  * Purpose:  Implementation of ILI1Reader class.
@@ -25,60 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ili1reader.cpp,v $
- * Revision 1.14  2006/04/27 16:37:19  pka
- * Ili2 model reader fix
- * Support for multiple Ili2 models
- *
- * Revision 1.13  2006/04/24 16:49:48  pka
- * Fixed polyline feature with coordinate attribute
- * Float support for ARC_DEGREES
- *
- * Revision 1.12  2006/04/04 15:31:30  pka
- * No copy of original geometry on windows (file position errors)
- *
- * Revision 1.11  2006/03/24 17:51:00  fwarmerdam
- * Fixed syntax error with GEOS_C_API case.
- *
- * Revision 1.10  2006/03/23 18:04:39  pka
- * Add polygon geometry to area layer
- * Performance improvement area polygonizer
- *
- * Revision 1.9  2006/02/13 18:18:53  pka
- * Interlis 2: Support for nested attributes
- * Interlis 2: Arc interpolation
- *
- * Revision 1.8  2005/12/19 17:33:21  pka
- * Interlis 1: Support for 100 columns (unlimited, if model given)
- * Interlis 1: Fixes for output
- * Interlis: Examples in driver documentation
- *
- * Revision 1.7  2005/11/21 14:56:31  pka
- * Fix for call of GetNextFeature without ResetReading (Interlis 2)
- * Fix for polygonizer crash on Linux with GEOS 2.1.3 (Interlis 1)
- *
- * Revision 1.6  2005/11/18 23:40:53  fwarmerdam
- * enable support with GEOS_C_API
- *
- * Revision 1.5  2005/11/02 16:24:57  fwarmerdam
- * Implement C API based version of polygonize, and fix support for C++ API.
- *
- * Revision 1.4  2005/10/13 14:30:40  pka
- * Explicit point geometry type
- * ARC_DEGREES environment variable (Default 1 degree)
- *
- * Revision 1.3  2005/10/09 22:59:57  pka
- * ARC interpolation (Interlis 1)
- *
- * Revision 1.2  2005/08/06 22:21:53  pka
- * Area polygonizer added
- *
- * Revision 1.1  2005/07/08 22:10:57  pka
- * Initial import of OGR Interlis driver
- *
- */
+ ****************************************************************************/
 
 #include "ogr_ili1.h"
 #include "cpl_conv.h"
@@ -92,22 +39,16 @@
 #include "ili1readerp.h"
 
 #ifdef HAVE_GEOS
-#  ifdef GEOS_C_API
-#    define POLYGONIZE_AREAS
-#  else
-#    include "geos/version.h"
-#    if GEOS_VERSION_MAJOR*100+GEOS_VERSION_MINOR*10+GEOS_VERSION_PATCH >= 210
-#      include "geos/opPolygonize.h"
-#      define POLYGONIZE_AREAS
-#    endif
-#  endif
+#  define POLYGONIZE_AREAS
 #endif
 
 #ifndef POLYGONIZE_AREAS
-#  warning Interlis 1 Area polygonizing disabled. Needs GEOS >= 2.1.0
+#  if defined(__GNUC_PREREQ)
+#    warning Interlis 1 Area polygonizing disabled. Needs GEOS >= 2.1.0
+#  endif
 #endif
 
-CPL_CVSID("$Id: ili1reader.cpp,v 1.14 2006/04/27 16:37:19 pka Exp $");
+CPL_CVSID("$Id: ili1reader.cpp 10653 2007-01-18 16:38:13Z warmerdam $");
 
 
 //
@@ -123,6 +64,9 @@ ILI1Reader::ILI1Reader() {
   nAreaLayers = 0;
   papoAreaLayers = NULL;
   papoAreaLineLayers = NULL;
+  nSurfaceLayers = 0;
+  papoSurfaceLayers = NULL;
+  papoSurfacePolyLayers = NULL;
   SetArcDegrees(1);
 }
 
@@ -199,8 +143,8 @@ void ILI1Reader::AddField(OGRLayer* layer, IOM_BASKET model, IOM_OBJECT obj) {
   if (EQUAL(iom_getobjecttag(obj),"iom04.metamodel.LocalAttribute")) typenam = GetTypeName(model, obj);
   CPLDebug( "OGR_ILI", "Field %s: %s", iom_getattrvalue(obj, "name"), typenam);
   if (EQUAL(typenam, "iom04.metamodel.SurfaceType")) {
-    AddGeomTable(layer->GetLayerDefn()->GetName(), iom_getattrvalue(obj, "name"), wkbPolygon);
-    //TODO: combine geometry and attribute layer
+    OGRLayer* polyLayer = AddGeomTable(layer->GetLayerDefn()->GetName(), iom_getattrvalue(obj, "name"), wkbPolygon);
+    AddSurfaceLayer(layer, polyLayer);
     //TODO: add line attributes to geometry
   } else if (EQUAL(typenam, "iom04.metamodel.AreaType")) {
     IOM_OBJECT controlPointDomain = GetAttrObj(model, GetTypeObj(model, obj), "controlPointDomain");
@@ -380,6 +324,7 @@ int ILI1Reader::ReadFeatures() {
       }
       else if (EQUAL(firsttok, "ENDE"))
       {
+        JoinSurfaceLayers();
         PolygonizeAreaLayers();
         return TRUE;
       }
@@ -419,9 +364,9 @@ OGRMultiPolygon* ILI1Reader::Polygonize( OGRGeometryCollection* poLines )
 {
     OGRMultiPolygon *poPolygon = new OGRMultiPolygon();
 
-#if defined(POLYGONIZE_AREAS) && defined(GEOS_C_API)
-    GEOSGeom *ahInGeoms;
-    int       i;
+#if defined(POLYGONIZE_AREAS)
+    GEOSGeom *ahInGeoms = NULL;
+    int       i = 0;
     
     ahInGeoms = (GEOSGeom *) CPLCalloc(sizeof(void*),poLines->getNumGeometries());
     for( i = 0; i < poLines->getNumGeometries(); i++ )
@@ -444,22 +389,6 @@ OGRMultiPolygon* ILI1Reader::Polygonize( OGRGeometryCollection* poLines )
 
     return (OGRMultiPolygon *) poMP;
 
-#elif defined(POLYGONIZE_AREAS)
-    int i;
-    geos::Polygonizer* polygonizer = new geos::Polygonizer();
-    for (i=0; i<poLines->getNumGeometries(); i++)
-        polygonizer->add((geos::Geometry *)poLines->getGeometryRef(i)->exportToGEOS());
-
-    vector<geos::Polygon*> *poOtherGeosGeom = polygonizer->getPolygons();
-
-    for (unsigned i = 0; i < poOtherGeosGeom->size(); i++)
-    {
-       poPolygon->addGeometryDirectly(
-          OGRGeometryFactory::createFromGEOS((GEOSGeom)(*poOtherGeosGeom)[i]));
-    }
-
-    //delete poOtherGeosGeom;
-    delete polygonizer;
 #endif
 
     return poPolygon;
@@ -486,8 +415,9 @@ void ILI1Reader::PolygonizeAreaLayers()
       //associate polygon feature with data row according to centroid
       int i;
       OGRPolygon emptyPoly;
-#if defined(POLYGONIZE_AREAS) && defined(GEOS_C_API)
-      GEOSGeom *ahInGeoms;
+#if defined(POLYGONIZE_AREAS)
+      GEOSGeom *ahInGeoms = NULL;
+
       ahInGeoms = (GEOSGeom *) CPLCalloc(sizeof(void*),polys->getNumGeometries());
       for( i = 0; i < polys->getNumGeometries(); i++ )
       {
@@ -517,42 +447,33 @@ void ILI1Reader::PolygonizeAreaLayers()
       for( i = 0; i < polys->getNumGeometries(); i++ )
           GEOSGeom_destroy( ahInGeoms[i] );
       CPLFree( ahInGeoms );
-#elif defined(POLYGONIZE_AREAS)
-      geos::Geometry **ahInGeoms;
-      ahInGeoms = (geos::Geometry **) CPLCalloc(sizeof(void*),polys->getNumGeometries());
-      for( i = 0; i < polys->getNumGeometries(); i++ )
-      {
-          ahInGeoms[i] = (geos::Geometry *)polys->getGeometryRef(i)->exportToGEOS();
-          if (!ahInGeoms[i]->isValid()) ahInGeoms[i] = NULL;
-      }
-      poAreaLayer->ResetReading();
-      while (OGRFeature *feature = poAreaLayer->GetNextFeature())
-      {
-        geos::Geometry *point = (geos::Geometry *)feature->GetGeometryRef()->exportToGEOS();
-        for (i = 0; i < polys->getNumGeometries(); i++ )
-        {
-          if (ahInGeoms[i] && point->within(ahInGeoms[i]))
-          {
-            feature->SetGeometry( polys->getGeometryRef(i) );
-            break;
-          }
-        }
-        if (i == polys->getNumGeometries())
-        {
-          CPLDebug( "OGR_ILI", "Association between area and point failed.");
-          feature->SetGeometry( &emptyPoly );
-        }
-        delete point;
-      }
-      poAreaLayer->GetLayerDefn()->SetGeomType(wkbPolygon);
-      for( i = 0; i < polys->getNumGeometries(); i++ )
-          delete ahInGeoms[i];
-      CPLFree( ahInGeoms );
 #endif
     }
 }
 
 
+void ILI1Reader::JoinSurfaceLayers()
+{
+    for(int iLayer = 0; iLayer < nSurfaceLayers; iLayer++ )
+    {
+      OGRLayer *poSurfaceLayer = papoSurfaceLayers[iLayer];
+      OGRLayer *poPolyLayer = papoSurfacePolyLayers[iLayer];
+
+      poSurfaceLayer->GetLayerDefn()->SetGeomType(poPolyLayer->GetLayerDefn()->GetGeomType());
+      poSurfaceLayer->ResetReading();
+      poPolyLayer->ResetReading();
+      while (OGRFeature *feature = poSurfaceLayer->GetNextFeature()) {
+        //Assume same sequence -> Dangerous?
+        OGRFeature *polyfeature = poPolyLayer->GetNextFeature();
+        if (polyfeature) { //&& EQUAL(feature->GetFieldAsString(0), polyfeature->GetFieldAsString(0))
+          feature->SetGeometry(polyfeature->GetGeometryRef());
+        }
+
+      }
+    }
+}
+
+
 int ILI1Reader::ReadTable() {
     
     char **tokens = NULL;
@@ -778,6 +699,24 @@ void ILI1Reader::AddAreaLayer( OGRLayer * poAreaLayer,  OGRLayer * poLineLayer )
     papoAreaLineLayers[nAreaLayers-1] = poLineLayer;
 }
 
+/************************************************************************/
+/*                              AddSurfaceLayer()                              */
+/************************************************************************/
+
+void ILI1Reader::AddSurfaceLayer( OGRLayer * poDataLayer,  OGRLayer * poPolyLayer )
+
+{
+    ++nSurfaceLayers;
+
+    papoSurfaceLayers = (OGRLayer **)
+        CPLRealloc( papoSurfaceLayers, sizeof(void*) * nSurfaceLayers );
+    papoSurfacePolyLayers = (OGRLayer **)
+        CPLRealloc( papoSurfacePolyLayers, sizeof(void*) * nSurfaceLayers );
+    
+    papoSurfaceLayers[nSurfaceLayers-1] = poDataLayer;
+    papoSurfacePolyLayers[nSurfaceLayers-1] = poPolyLayer;
+}
+
 /************************************************************************/
 /*                              GetLayer()                              */
 /************************************************************************/
