@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ili1readerp.h,v 1.5 2006/03/23 18:04:39 pka Exp $
+ * $Id: ili1readerp.h 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Interlis 1 Reader
  * Purpose:  Private Declarations for Reader code.
@@ -25,27 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ili1readerp.h,v $
- * Revision 1.5  2006/03/23 18:04:39  pka
- * Add polygon geometry to area layer
- * Performance improvement area polygonizer
- *
- * Revision 1.4  2006/02/13 18:18:53  pka
- * Interlis 2: Support for nested attributes
- * Interlis 2: Arc interpolation
- *
- * Revision 1.3  2005/10/09 22:59:57  pka
- * ARC interpolation (Interlis 1)
- *
- * Revision 1.2  2005/08/06 22:21:53  pka
- * Area polygonizer added
- *
- * Revision 1.1  2005/07/08 22:10:57  pka
- * Initial import of OGR Interlis driver
- *
- */
+ ****************************************************************************/
 
 #ifndef _CPL_ILI1READERP_H_INCLUDED
 #define _CPL_ILI1READERP_H_INCLUDED
@@ -71,6 +51,9 @@ private:
     int          nAreaLayers;
     OGRLayer     **papoAreaLayers;
     OGRLayer     **papoAreaLineLayers;
+    int          nSurfaceLayers;
+    OGRLayer     **papoSurfaceLayers;
+    OGRLayer     **papoSurfacePolyLayers;
     OGRILI1Layer *curLayer;
     double       arcIncr;
 
@@ -88,9 +71,11 @@ public:
 
     void         AddLayer( OGRLayer * poNewLayer );
     void         AddAreaLayer( OGRLayer * poAreaLayer,  OGRLayer * poLineLayer );
+    void         AddSurfaceLayer( OGRLayer * poDataLayer,  OGRLayer * poPolyLayer );
     int          AddIliGeom(OGRFeature *feature, int iField, long fpos);
     OGRMultiPolygon* Polygonize( OGRGeometryCollection* poLines );
     void         PolygonizeAreaLayers();
+    void         JoinSurfaceLayers();
     OGRLayer     *GetLayer( int );
     OGRLayer     *GetLayerByName( const char* );
     int          GetLayerCount();
