@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ili2handler.cpp,v 1.2 2005/08/06 22:21:53 pka Exp $
+ * $Id: ili2handler.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Interlis 2 Reader
  * Purpose:  Implementation of ILI2Handler class.
@@ -25,16 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ili2handler.cpp,v $
- * Revision 1.2  2005/08/06 22:21:53  pka
- * Area polygonizer added
- *
- * Revision 1.1  2005/07/08 22:10:57  pka
- * Initial import of OGR Interlis driver
- *
- */
+ ****************************************************************************/
 
 #include "ogr_ili2.h"
 #include "cpl_conv.h"
@@ -42,7 +33,7 @@
 
 #include "ili2readerp.h"
 
-CPL_CVSID("$Id: ili2handler.cpp,v 1.2 2005/08/06 22:21:53 pka Exp $");
+CPL_CVSID("$Id: ili2handler.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 // 
 // constants
