@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrili2layer.cpp,v 1.4 2005/11/21 14:56:31 pka Exp $
+ * $Id: ogrili2layer.cpp 11178 2007-04-02 20:44:35Z warmerdam $
  *
  * Project:  Interlis 2 Translator
  * Purpose:  Implements OGRILI2Layer class.
@@ -25,29 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrili2layer.cpp,v $
- * Revision 1.4  2005/11/21 14:56:31  pka
- * Fix for call of GetNextFeature without ResetReading (Interlis 2)
- * Fix for polygonizer crash on Linux with GEOS 2.1.3 (Interlis 1)
- *
- * Revision 1.3  2005/09/21 00:59:36  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.2  2005/08/06 22:21:53  pka
- * Area polygonizer added
- *
- * Revision 1.1  2005/07/08 22:10:57  pka
- * Initial import of OGR Interlis driver
- *
- */
+ ****************************************************************************/
 
 #include "ogr_ili2.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrili2layer.cpp,v 1.4 2005/11/21 14:56:31 pka Exp $");
+CPL_CVSID("$Id: ogrili2layer.cpp 11178 2007-04-02 20:44:35Z warmerdam $");
 
 /************************************************************************/
 /*                           OGRILI2Layer()                              */
@@ -73,7 +57,6 @@ OGRILI2Layer::OGRILI2Layer( const char * pszName,
     poFeatureDefn->SetGeomType( eReqType );
 
     bWriter = bWriterIn;
-    listFeatureIt = 0;
 }
 
 /************************************************************************/
@@ -132,9 +115,8 @@ void OGRILI2Layer::ResetReading(){
 /************************************************************************/
 
 OGRFeature *OGRILI2Layer::GetNextFeature() {
-    if (listFeatureIt == 0) listFeatureIt = listFeature.begin();
     if (listFeatureIt != listFeature.end())
-        return *(listFeatureIt++);
+        return (*(listFeatureIt++))->Clone();
     return NULL;
 }
 
