@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mitab_coordsys.cpp,v 1.32 2005/08/07 21:00:38 fwarmerdam Exp $
+ * $Id: mitab_coordsys.cpp,v 1.34 2006/03/10 19:50:45 fwarmerdam Exp $
  *
  * Name:     mitab_coordsys.cpp
  * Project:  MapInfo TAB Read/Write library
@@ -31,6 +31,17 @@
  **********************************************************************
  *
  * $Log: mitab_coordsys.cpp,v $
+ * Revision 1.34  2006/03/10 19:50:45  fwarmerdam
+ * Coordsys false easting and northing are in the units of the coordsys, not
+ * necessarily meters.  Adjusted mitab_coordsys.cpp to reflect this.
+ * http://bugzilla.remotesensing.org/show_bug.cgi?id=1113
+ *
+ * Revision 1.33  2005/09/29 20:13:57  dmorissette
+ * MITABCoordSys2SpatialRef() patches from Anthony D (bug 1155):
+ * Improved support for modified TM projections 21-24.
+ * Added support for affine parameters (inside #ifdef MITAB_AFFINE_PARAMS since
+ * affine params cannot be stored directly in OGRSpatialReference)
+ *
  * Revision 1.32  2005/08/07 21:00:38  fwarmerdam
  * Initialize adfDatumParm[] to avoid warnings with gcc 4.
  *
@@ -78,60 +89,6 @@
  *
  * Revision 1.18  2001/01/19 21:56:18  warmerda
  * added untested support for Swiss Oblique Mercator
- *
- * Revision 1.17  2000/12/05 14:55:27  daniel
- * Added some missing unit name aliases in MITABSpatialRef2CoordSys()
- *
- * Revision 1.16  2000/10/16 21:44:50  warmerda
- * added nonearth support
- *
- * Revision 1.15  2000/10/12 22:01:01  daniel
- * Fixed test on number of bound params in MITABExtractCoordSysBounds()
- *
- * Revision 1.14  2000/09/28 16:39:44  warmerda
- * avoid warnings for unused, and unitialized variables
- *
- * Revision 1.13  2000/08/29 22:30:45  daniel
- * Made MITABCoordSys2SpatialRef() return NULL for a NonEarth coordsys since
- * returning an empty SpatialRef created confusion between LatLon and NonEarth
- *
- * Revision 1.12  2000/03/27 03:31:15  daniel
- * Fixed parsing of CoordSys NonEarth broken by previous change
- *
- * Revision 1.11  2000/03/19 23:26:27  daniel
- * Made MITABCoordSys2SpatialRef() remove leading spaces in coordsys string
- * and produce a CPLError() if it fails parsing string.
- *
- * Revision 1.10  2000/02/07 17:43:37  daniel
- * Fixed offset in parsing of custom datum string in SetSpatialRef()
- *
- * Revision 1.9  2000/01/15 22:30:43  daniel
- * Switch to MIT/X-Consortium OpenSource license
- *
- * Revision 1.8  2000/01/14 14:53:32  warmerda
- * fixed generation of MIF %d datum names
- *
- * Revision 1.7  1999/12/19 17:35:16  daniel
- * Fixed memory leaks
- *
- * Revision 1.6  1999/11/20 04:53:41  daniel
- * Fixed warning on return value in MITABExtractCoordSysBounds()
- *
- * Revision 1.5  1999/11/12 05:51:57  daniel
- * Added MITABExtractCoordSysBounds()
- *
- * Revision 1.4  1999/11/11 02:55:25  warmerda
- * fixed problems with stereographic and survey ft
- *
- * Revision 1.3  1999/11/10 20:13:12  warmerda
- * implement spheroid table
- *
- * Revision 1.2  1999/11/10 02:19:05  warmerda
- * fixed up datum support when reading MIF coord sys
- *
- * Revision 1.1  1999/11/09 22:29:38  warmerda
- * New
- *
  **********************************************************************/
 
 #include "mitab.h"
@@ -166,6 +123,12 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
     char        **papszFields;
     OGRSpatialReference *poSR;
 
+#ifdef MITAB_AFFINE_PARAMS  // See MITAB bug 1155
+    // Encom 2003
+    int nAffineUnits = 7;
+    double dAffineParams[6];
+#endif
+
     if( pszCoordSys == NULL )
         return NULL;
     
@@ -178,6 +141,40 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
     
     papszFields = CSLTokenizeStringComplex( pszCoordSys, " ,", TRUE, FALSE );
 
+#ifdef MITAB_AFFINE_PARAMS  // See MITAB bug 1155
+/* -------------------------------------------------------------------- */
+/*      Store and then clip off Affine information. - Encom 2003        */
+/* -------------------------------------------------------------------- */
+    int         iAffine = CSLFindString( papszFields, "Affine" );
+    int         nAffineIndex = 0;
+    int         nAffineFlag = 0;
+
+    while( iAffine != -1 && papszFields[iAffine] != NULL )
+    {
+        nAffineFlag = 1;
+        if (nAffineIndex<2)
+        {
+            // Ignore "Affine Units"
+        }
+        else if (nAffineIndex==2)
+        {
+            // Convert units to integer (TBD)
+            nAffineUnits = TABUnitIdFromString(papszFields[iAffine]);
+            if (nAffineUnits==-1) nAffineUnits = 7; // metres is default
+        }
+        else if (nAffineIndex<=8)
+        {
+            // Store affine params
+            dAffineParams[nAffineIndex-3] = atof(papszFields[iAffine]);
+        }
+        nAffineIndex++;
+
+        CPLFree( papszFields[iAffine] );
+        papszFields[iAffine] = NULL;
+        iAffine++;
+    }
+#endif // MITAB_AFFINE_PARAMS
+
 /* -------------------------------------------------------------------- */
 /*      Clip off Bounds information.                                    */
 /* -------------------------------------------------------------------- */
@@ -195,6 +192,25 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
 /* -------------------------------------------------------------------- */
     poSR = new OGRSpatialReference;
 
+#ifdef MITAB_AFFINE_PARAMS  // See MITAB bug 1155
+    // Encom 2003
+    if (nAffineFlag)
+    {
+        poSR->nAffineFlag = 1;
+        poSR->nAffineUnit = nAffineUnits;
+        poSR->dAffineParamA = dAffineParams[0];
+        poSR->dAffineParamB = dAffineParams[1];
+        poSR->dAffineParamC = dAffineParams[2];
+        poSR->dAffineParamD = dAffineParams[3];
+        poSR->dAffineParamE = dAffineParams[4];
+        poSR->dAffineParamF = dAffineParams[5];
+    }
+    else
+    {
+        poSR->nAffineFlag = 0; // Encom 2005
+    }
+#endif
+
 /* -------------------------------------------------------------------- */
 /*      Fetch the projection.                                           */
 /* -------------------------------------------------------------------- */
@@ -345,7 +361,11 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
     }
 
 /* -------------------------------------------------------------------- */
-/*      Handle the PROJCS style projections, but add the datum later.   */
+/*      Handle the PROJCS style projections, but add the datum          */
+/*      later.                                                          */
+/*                                                                      */
+/*      Note that per GDAL bug 1113 the false easting and north are     */
+/*      in local units, not necessarily meters.                         */
 /* -------------------------------------------------------------------- */
     switch( nProjection )
     {
@@ -376,8 +396,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
       case 2:
         poSR->SetCEA( GetMIFParm( papszNextField, 1, 0.0 ),
                       GetMIFParm( papszNextField, 0, 0.0 ),
-                      GetMIFParm( papszNextField, 2, 0.0 ) * dfUnitsConv,
-                      GetMIFParm( papszNextField, 3, 0.0 ) * dfUnitsConv );
+                      GetMIFParm( papszNextField, 2, 0.0 ),
+                      GetMIFParm( papszNextField, 3, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
@@ -388,8 +408,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
                       GetMIFParm( papszNextField, 3, 0.0 ),
                       GetMIFParm( papszNextField, 1, 0.0 ),
                       GetMIFParm( papszNextField, 0, 0.0 ),
-                      GetMIFParm( papszNextField, 4, 0.0 ) * dfUnitsConv,
-                      GetMIFParm( papszNextField, 5, 0.0 ) * dfUnitsConv );
+                      GetMIFParm( papszNextField, 4, 0.0 ),
+                      GetMIFParm( papszNextField, 5, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
@@ -420,8 +440,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
                      GetMIFParm( papszNextField, 3, 0.0 ),
                      GetMIFParm( papszNextField, 1, 0.0 ),
                      GetMIFParm( papszNextField, 0, 0.0 ),
-                     GetMIFParm( papszNextField, 4, 0.0 ) * dfUnitsConv,
-                     GetMIFParm( papszNextField, 5, 0.0 ) * dfUnitsConv );
+                     GetMIFParm( papszNextField, 4, 0.0 ),
+                     GetMIFParm( papszNextField, 5, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
@@ -433,23 +453,67 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
                       GetMIFParm( papszNextField, 2, 0.0 ),
                       90.0,
                       GetMIFParm( papszNextField, 3, 1.0 ),
-                      GetMIFParm( papszNextField, 4, 0.0 * dfUnitsConv ),
-                      GetMIFParm( papszNextField, 5, 0.0 * dfUnitsConv ) );
+                      GetMIFParm( papszNextField, 4, 0.0 ),
+                      GetMIFParm( papszNextField, 5, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
          * Transverse Mercator
          *-------------------------------------------------------------*/
       case 8:
-      case 21:
-      case 22:
-      case 23:
-      case 24:
         poSR->SetTM( GetMIFParm( papszNextField, 1, 0.0 ),
                      GetMIFParm( papszNextField, 0, 0.0 ),
                      GetMIFParm( papszNextField, 2, 1.0 ),
-                     GetMIFParm( papszNextField, 3, 0.0 ) * dfUnitsConv,
-                     GetMIFParm( papszNextField, 4, 0.0 ) * dfUnitsConv );
+                     GetMIFParm( papszNextField, 3, 0.0 ),
+                     GetMIFParm( papszNextField, 4, 0.0 ) );
+        break;
+
+        /*----------------------------------------------------------------
+         * Transverse Mercator,(modified for Danish System 34 Jylland-Fyn)
+         *---------------------------------------------------------------*/
+      case 21:
+        poSR->SetTMVariant( SRS_PT_TRANSVERSE_MERCATOR_MI_21,
+                            GetMIFParm( papszNextField, 1, 0.0 ),
+                            GetMIFParm( papszNextField, 0, 0.0 ),
+                            GetMIFParm( papszNextField, 2, 1.0 ),
+                            GetMIFParm( papszNextField, 3, 0.0 ),
+                            GetMIFParm( papszNextField, 4, 0.0 ));
+        break;
+
+        /*--------------------------------------------------------------
+         * Transverse Mercator,(modified for Danish System 34 Sjaelland)
+         *-------------------------------------------------------------*/
+      case 22:
+        poSR->SetTMVariant( SRS_PT_TRANSVERSE_MERCATOR_MI_22,
+                            GetMIFParm( papszNextField, 1, 0.0 ),
+                            GetMIFParm( papszNextField, 0, 0.0 ),
+                            GetMIFParm( papszNextField, 2, 1.0 ),
+                            GetMIFParm( papszNextField, 3, 0.0 ),
+                            GetMIFParm( papszNextField, 4, 0.0 ));
+        break;
+
+        /*----------------------------------------------------------------
+         * Transverse Mercator,(modified for Danish System 34/45 Bornholm)
+         *---------------------------------------------------------------*/
+      case 23:
+        poSR->SetTMVariant( SRS_PT_TRANSVERSE_MERCATOR_MI_23,
+                            GetMIFParm( papszNextField, 1, 0.0 ),
+                            GetMIFParm( papszNextField, 0, 0.0 ),
+                            GetMIFParm( papszNextField, 2, 1.0 ),
+                            GetMIFParm( papszNextField, 3, 0.0 ),
+                            GetMIFParm( papszNextField, 4, 0.0 ));
+        break;
+
+        /*--------------------------------------------------------------
+         * Transverse Mercator,(modified for Finnish KKJ)
+         *-------------------------------------------------------------*/
+      case 24:
+        poSR->SetTMVariant( SRS_PT_TRANSVERSE_MERCATOR_MI_24,
+                            GetMIFParm( papszNextField, 1, 0.0 ),
+                            GetMIFParm( papszNextField, 0, 0.0 ),
+                            GetMIFParm( papszNextField, 2, 1.0 ),
+                            GetMIFParm( papszNextField, 3, 0.0 ),
+                            GetMIFParm( papszNextField, 4, 0.0 ));
         break;
 
         /*--------------------------------------------------------------
@@ -460,8 +524,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
                        GetMIFParm( papszNextField, 3, 0.0 ),
                        GetMIFParm( papszNextField, 1, 0.0 ),
                        GetMIFParm( papszNextField, 0, 0.0 ),
-                       GetMIFParm( papszNextField, 4, 0.0 ) * dfUnitsConv,
-                       GetMIFParm( papszNextField, 5, 0.0 ) * dfUnitsConv );
+                       GetMIFParm( papszNextField, 4, 0.0 ),
+                       GetMIFParm( papszNextField, 5, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
@@ -533,8 +597,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
       case 18:
         poSR->SetNZMG( GetMIFParm( papszNextField, 1, 0.0 ),
                        GetMIFParm( papszNextField, 0, 0.0 ),
-                       GetMIFParm( papszNextField, 2, 0.0 ) * dfUnitsConv,
-                       GetMIFParm( papszNextField, 3, 0.0 ) * dfUnitsConv );
+                       GetMIFParm( papszNextField, 2, 0.0 ),
+                       GetMIFParm( papszNextField, 3, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
@@ -545,8 +609,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
                        GetMIFParm( papszNextField, 3, 0.0 ),
                        GetMIFParm( papszNextField, 1, 0.0 ),
                        GetMIFParm( papszNextField, 0, 0.0 ),
-                       GetMIFParm( papszNextField, 4, 0.0 ) * dfUnitsConv,
-                       GetMIFParm( papszNextField, 5, 0.0 ) * dfUnitsConv );
+                       GetMIFParm( papszNextField, 4, 0.0 ),
+                       GetMIFParm( papszNextField, 5, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
@@ -558,8 +622,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
             GetMIFParm( papszNextField, 1, 0.0 ),
             GetMIFParm( papszNextField, 0, 0.0 ),
             GetMIFParm( papszNextField, 2, 1.0 ),
-            GetMIFParm( papszNextField, 3, 0.0 ) * dfUnitsConv,
-            GetMIFParm( papszNextField, 4, 0.0 ) * dfUnitsConv );
+            GetMIFParm( papszNextField, 3, 0.0 ),
+            GetMIFParm( papszNextField, 4, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
@@ -568,8 +632,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
       case 25:
         poSR->SetSOC( GetMIFParm( papszNextField, 1, 0.0 ),
                       GetMIFParm( papszNextField, 0, 0.0 ),
-                      GetMIFParm( papszNextField, 2, 0.0 ) * dfUnitsConv,
-                      GetMIFParm( papszNextField, 3, 0.0 ) * dfUnitsConv );
+                      GetMIFParm( papszNextField, 2, 0.0 ),
+                      GetMIFParm( papszNextField, 3, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
@@ -587,8 +651,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
       case 27:
         poSR->SetPolyconic( GetMIFParm( papszNextField, 1, 0.0 ), 
                             GetMIFParm( papszNextField, 0, 0.0 ),
-                          GetMIFParm( papszNextField, 2, 0.0 ) * dfUnitsConv,
-                          GetMIFParm( papszNextField, 3, 0.0 ) * dfUnitsConv );
+                          GetMIFParm( papszNextField, 2, 0.0 ),
+                          GetMIFParm( papszNextField, 3, 0.0 ) );
         break;
 
         /*--------------------------------------------------------------
@@ -598,8 +662,8 @@ OGRSpatialReference *MITABCoordSys2SpatialRef( const char * pszCoordSys )
         poSR->SetCS( 
             GetMIFParm( papszNextField, 1, 0.0 ),
             GetMIFParm( papszNextField, 0, 0.0 ),
-            GetMIFParm( papszNextField, 2, 0.0 ) * dfUnitsConv,
-            GetMIFParm( papszNextField, 3, 0.0 ) * dfUnitsConv );
+            GetMIFParm( papszNextField, 2, 0.0 ),
+            GetMIFParm( papszNextField, 3, 0.0 ) );
         break;
 
       default:
@@ -812,8 +876,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_CENTER,0.0);
         parms[2] = poSR->GetProjParm(SRS_PP_STANDARD_PARALLEL_1,0.0);
         parms[3] = poSR->GetProjParm(SRS_PP_STANDARD_PARALLEL_2,0.0);
-        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 6;
     }
 
@@ -858,8 +922,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_CENTER,0.0);
         parms[2] = poSR->GetProjParm(SRS_PP_STANDARD_PARALLEL_1,0.0);
         parms[3] = poSR->GetProjParm(SRS_PP_STANDARD_PARALLEL_2,0.0);
-        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 6;
     }
 
@@ -877,8 +941,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_CENTER,0.0);
         parms[2] = poSR->GetProjParm(SRS_PP_AZIMUTH,0.0);
         parms[3] = poSR->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 6;
     }
 
@@ -901,8 +965,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSR->GetProjParm(SRS_PP_STANDARD_PARALLEL_1,0.0);
         parms[3] = poSR->GetProjParm(SRS_PP_STANDARD_PARALLEL_2,0.0);
-        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 6;
     }
 
@@ -913,8 +977,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSR->GetProjParm(SRS_PP_STANDARD_PARALLEL_1,0.0);
         parms[3] = poSR->GetProjParm(SRS_PP_STANDARD_PARALLEL_2,0.0);
-        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 6;
     }
 
@@ -951,8 +1015,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         nProjection = 25;
         parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
-        parms[2] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[2] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 4;
     }
 
@@ -976,8 +1040,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSR->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 5;
     }
 
@@ -987,8 +1051,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSR->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 5;
     }
 
@@ -999,8 +1063,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
        parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
        parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
        parms[2] = poSR->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-       parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-       parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+       parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+       parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
        nParmCount = 5;
     }
 
@@ -1011,8 +1075,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
        parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
        parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
        parms[2] = poSR->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-       parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-       parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+       parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+       parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
        nParmCount = 5;
     }
 
@@ -1023,8 +1087,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
        parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
        parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
        parms[2] = poSR->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-       parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-       parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+       parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+       parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
        nParmCount = 5;
     }
 
@@ -1035,8 +1099,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
        parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
        parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
        parms[2] = poSR->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-       parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-       parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+       parms[3] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+       parms[4] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
        nParmCount = 5;
     }
 
@@ -1045,8 +1109,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         nProjection = 30;
         parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
-        parms[2] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[2] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 4;
     }
 
@@ -1055,8 +1119,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         nProjection = 18;
         parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
-        parms[2] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[2] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 4;
     }
 
@@ -1065,8 +1129,8 @@ char *MITABSpatialRef2CoordSys( OGRSpatialReference * poSR )
         nProjection = 27;
         parms[0] = poSR->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSR->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
-        parms[2] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[2] = poSR->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[3] = poSR->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
         nParmCount = 4;
     }
 
