@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mitab_spatialref.cpp,v 1.44 2005/09/29 20:15:36 dmorissette Exp $
+ * $Id: mitab_spatialref.cpp,v 1.47 2006/07/10 17:58:48 fwarmerdam Exp $
  *
  * Name:     mitab_spatialref.cpp
  * Project:  MapInfo TAB Read/Write library
@@ -30,6 +30,17 @@
  **********************************************************************
  *
  * $Log: mitab_spatialref.cpp,v $
+ * Revision 1.47  2006/07/10 17:58:48  fwarmerdam
+ * North_American_Datum_1927 support
+ *
+ * Revision 1.46  2006/07/07 19:41:32  dmorissette
+ * Fixed problem with uninitialized sTABProj.nAffineFlag (bug 1254,1319)
+ *
+ * Revision 1.45  2006/05/09 20:21:29  fwarmerdam
+ * Coordsys false easting and northing are in the units of the coordsys, not
+ * necessarily meters.  Adjusted mitab_spatialref.cpp to reflect this.
+ * http://bugzilla.remotesensing.org/show_bug.cgi?id=1113
+ *
  * Revision 1.44  2005/09/29 20:15:36  dmorissette
  * More improvements to handling of modified TM projections 21-24.
  * Added correct name stings to all datum definitions (Anthony D, bug 1155)
@@ -251,6 +262,7 @@ MapInfoDatumInfo asDatumInfoList[] =
 {60, "Nahrwan_Saudi_Arabia",        6, -231, -196, 482, 0, 0, 0, 0, 0},
 {61, "Naparima_1972",               4, -2,   374,  172, 0, 0, 0, 0, 0},
 {62, "NAD_1927",                    7, -8,   160,  176, 0, 0, 0, 0, 0},
+{62, "North_American_Datum_1927",   7, -8,   160,  176, 0, 0, 0, 0, 0},
 {63, "NAD_27_Alaska",               7, -5,   135,  172, 0, 0, 0, 0, 0},
 {64, "NAD_27_Bahamas",              7, -4,   154,  178, 0, 0, 0, 0, 0},
 {65, "NAD_27_San_Salvador",         7, 1,    140,  165, 0, 0, 0, 0, 0},
@@ -585,8 +597,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
       case 2:
         m_poSpatialRef->SetCEA( sTABProj.adProjParams[1],
                                 sTABProj.adProjParams[0],
-                                sTABProj.adProjParams[2] * dfConv,
-                                sTABProj.adProjParams[3] * dfConv );
+                                sTABProj.adProjParams[2],
+                                sTABProj.adProjParams[3] );
         break;
 
         /*--------------------------------------------------------------
@@ -597,8 +609,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                 sTABProj.adProjParams[3],
                                 sTABProj.adProjParams[1],
                                 sTABProj.adProjParams[0],
-                                sTABProj.adProjParams[4] * dfConv,
-                                sTABProj.adProjParams[5] * dfConv );
+                                sTABProj.adProjParams[4],
+                                sTABProj.adProjParams[5] );
         break;
 
         /*--------------------------------------------------------------
@@ -629,8 +641,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                sTABProj.adProjParams[3],
                                sTABProj.adProjParams[1],
                                sTABProj.adProjParams[0],
-                               sTABProj.adProjParams[4] * dfConv,
-                               sTABProj.adProjParams[5] * dfConv );
+                               sTABProj.adProjParams[4],
+                               sTABProj.adProjParams[5] );
         break;
 
         /*--------------------------------------------------------------
@@ -642,8 +654,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                 sTABProj.adProjParams[2],
                                 90.0, 
                                 sTABProj.adProjParams[3],
-                                sTABProj.adProjParams[4] * dfConv,
-                                sTABProj.adProjParams[5] * dfConv );
+                                sTABProj.adProjParams[4],
+                                sTABProj.adProjParams[5] );
         break;
 
         /*--------------------------------------------------------------
@@ -653,8 +665,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
         m_poSpatialRef->SetTM( sTABProj.adProjParams[1],
                                sTABProj.adProjParams[0],
                                sTABProj.adProjParams[2],
-                               sTABProj.adProjParams[3] * dfConv,
-                               sTABProj.adProjParams[4] * dfConv );
+                               sTABProj.adProjParams[3],
+                               sTABProj.adProjParams[4] );
         break;
 
         /*----------------------------------------------------------------
@@ -665,8 +677,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                        sTABProj.adProjParams[1],
                                        sTABProj.adProjParams[0],
                                        sTABProj.adProjParams[2],
-                                       sTABProj.adProjParams[3] * dfConv,
-                                       sTABProj.adProjParams[4] * dfConv );
+                                       sTABProj.adProjParams[3],
+                                       sTABProj.adProjParams[4] );
          break;
 
         /*--------------------------------------------------------------
@@ -677,8 +689,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                        sTABProj.adProjParams[1],
                                        sTABProj.adProjParams[0],
                                        sTABProj.adProjParams[2],
-                                       sTABProj.adProjParams[3] * dfConv,
-                                       sTABProj.adProjParams[4] * dfConv );
+                                       sTABProj.adProjParams[3],
+                                       sTABProj.adProjParams[4] );
          break;
 
         /*----------------------------------------------------------------
@@ -689,8 +701,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                        sTABProj.adProjParams[1],
                                        sTABProj.adProjParams[0],
                                        sTABProj.adProjParams[2],
-                                       sTABProj.adProjParams[3] * dfConv,
-                                       sTABProj.adProjParams[4] * dfConv );
+                                       sTABProj.adProjParams[3],
+                                       sTABProj.adProjParams[4] );
          break;
 
         /*--------------------------------------------------------------
@@ -701,8 +713,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                        sTABProj.adProjParams[1],
                                        sTABProj.adProjParams[0],
                                        sTABProj.adProjParams[2],
-                                       sTABProj.adProjParams[3] * dfConv,
-                                       sTABProj.adProjParams[4] * dfConv );
+                                       sTABProj.adProjParams[3],
+                                       sTABProj.adProjParams[4] );
          break;
 
         /*--------------------------------------------------------------
@@ -713,8 +725,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                  sTABProj.adProjParams[3],
                                  sTABProj.adProjParams[1],
                                  sTABProj.adProjParams[0],
-                                 sTABProj.adProjParams[4] * dfConv,
-                                 sTABProj.adProjParams[5] * dfConv );
+                                 sTABProj.adProjParams[4],
+                                 sTABProj.adProjParams[5] );
         break;
 
         /*--------------------------------------------------------------
@@ -783,8 +795,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
       case 18:
         m_poSpatialRef->SetNZMG( sTABProj.adProjParams[1],
                                  sTABProj.adProjParams[0],
-                                 sTABProj.adProjParams[2] * dfConv,
-                                 sTABProj.adProjParams[3] * dfConv );
+                                 sTABProj.adProjParams[2],
+                                 sTABProj.adProjParams[3] );
         break;
 
         /*--------------------------------------------------------------
@@ -795,8 +807,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
                                  sTABProj.adProjParams[3],
                                  sTABProj.adProjParams[1],
                                  sTABProj.adProjParams[0],
-                                 sTABProj.adProjParams[4] * dfConv,
-                                 sTABProj.adProjParams[5] * dfConv );
+                                 sTABProj.adProjParams[4],
+                                 sTABProj.adProjParams[5] );
         break;
 
         /*--------------------------------------------------------------
@@ -807,8 +819,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
         m_poSpatialRef->SetStereographic( sTABProj.adProjParams[1],
                                           sTABProj.adProjParams[0],
                                           sTABProj.adProjParams[2],
-                                          sTABProj.adProjParams[3] * dfConv,
-                                          sTABProj.adProjParams[4] * dfConv );
+                                          sTABProj.adProjParams[3],
+                                          sTABProj.adProjParams[4] );
         break;
 
         /*--------------------------------------------------------------
@@ -817,8 +829,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
       case 25:
         m_poSpatialRef->SetSOC( sTABProj.adProjParams[1],
                                 sTABProj.adProjParams[0],
-                                sTABProj.adProjParams[2] * dfConv,
-                                sTABProj.adProjParams[3] * dfConv );
+                                sTABProj.adProjParams[2],
+                                sTABProj.adProjParams[3] );
         break;
 
         /*--------------------------------------------------------------
@@ -836,8 +848,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
       case 27:
         m_poSpatialRef->SetPolyconic( sTABProj.adProjParams[1],
                                       sTABProj.adProjParams[0],
-                                      sTABProj.adProjParams[2] * dfConv,
-                                      sTABProj.adProjParams[3] * dfConv );
+                                      sTABProj.adProjParams[2],
+                                      sTABProj.adProjParams[3] );
         break;
 
         /*--------------------------------------------------------------
@@ -846,8 +858,8 @@ OGRSpatialReference *TABFile::GetSpatialRef()
       case 30:
         m_poSpatialRef->SetCS( sTABProj.adProjParams[1],
                                sTABProj.adProjParams[0],
-                               sTABProj.adProjParams[2] * dfConv,
-                               sTABProj.adProjParams[3] * dfConv );
+                               sTABProj.adProjParams[2],
+                               sTABProj.adProjParams[3] );
         break;
 
       default:
@@ -1098,7 +1110,6 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
     sTABProj.adDatumParams[3] = 0.0;
     sTABProj.adDatumParams[4] = 0.0;
 
-#ifdef MITAB_AFFINE_PARAMS  // See MITAB bug 1155
     // Encom 2003
     sTABProj.nAffineFlag   = 0;
     sTABProj.nAffineUnits  = 7;
@@ -1108,7 +1119,6 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
     sTABProj.dAffineParamD = 0.0;
     sTABProj.dAffineParamE = 0.0;
     sTABProj.dAffineParamF = 0.0;
-#endif // MITAB_AFFINE_PARAMS
     
     /*-----------------------------------------------------------------
      * Get the linear units and conversion.
@@ -1144,8 +1154,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_CENTER,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_STANDARD_PARALLEL_1,0.0);
         parms[3] = poSpatialRef->GetProjParm(SRS_PP_STANDARD_PARALLEL_2,0.0);
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_AZIMUTHAL_EQUIDISTANT) )
@@ -1185,8 +1195,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_CENTER,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_STANDARD_PARALLEL_1,0.0);
         parms[3] = poSpatialRef->GetProjParm(SRS_PP_STANDARD_PARALLEL_2,0.0);
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_GALL_STEREOGRAPHIC) )
@@ -1202,8 +1212,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_CENTER,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_AZIMUTH,0.0);
         parms[3] = poSpatialRef->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_LAMBERT_AZIMUTHAL_EQUAL_AREA) )
@@ -1224,8 +1234,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_STANDARD_PARALLEL_1,0.0);
         parms[3] = poSpatialRef->GetProjParm(SRS_PP_STANDARD_PARALLEL_2,0.0);
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_LAMBERT_CONFORMAL_CONIC_2SP_BELGIUM) )
@@ -1235,8 +1245,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_STANDARD_PARALLEL_1,0.0);
         parms[3] = poSpatialRef->GetProjParm(SRS_PP_STANDARD_PARALLEL_2,0.0);
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[5] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_MERCATOR_1SP) )
@@ -1267,8 +1277,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         sTABProj.nProjId = 18;
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
-        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_SWISS_OBLIQUE_CYLINDRICAL) )
@@ -1276,8 +1286,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         sTABProj.nProjId = 25;
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
-        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_ROBINSON) )
@@ -1298,8 +1308,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR) )
@@ -1308,8 +1318,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR_MI_21) ) // Encom 2003
@@ -1318,8 +1328,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR_MI_22) ) // Encom 2003
@@ -1328,8 +1338,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR_MI_23) ) // Encom 2003
@@ -1338,8 +1348,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_TRANSVERSE_MERCATOR_MI_24) ) // Encom 2003
@@ -1348,8 +1358,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
         parms[2] = poSpatialRef->GetProjParm(SRS_PP_SCALE_FACTOR,1.0);
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) / dfLinearConv;
-        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) / dfLinearConv;
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[4] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_CASSINI_SOLDNER) )
@@ -1357,10 +1367,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         sTABProj.nProjId = 30;
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
-        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) 
-            / dfLinearConv;
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) 
-            / dfLinearConv;
+        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_NEW_ZEALAND_MAP_GRID) )
@@ -1368,10 +1376,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         sTABProj.nProjId = 18;
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
-        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) 
-            / dfLinearConv;
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) 
-            / dfLinearConv;
+        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     else if( EQUAL(pszProjection,SRS_PT_POLYCONIC) )
@@ -1379,10 +1385,8 @@ int TABFile::SetSpatialRef(OGRSpatialReference *poSpatialRef)
         sTABProj.nProjId = 27;
         parms[0] = poSpatialRef->GetProjParm(SRS_PP_CENTRAL_MERIDIAN,0.0);
         parms[1] = poSpatialRef->GetProjParm(SRS_PP_LATITUDE_OF_ORIGIN,0.0);
-        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0) 
-            / dfLinearConv;
-        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0) 
-            / dfLinearConv;
+        parms[2] = poSpatialRef->GetProjParm(SRS_PP_FALSE_EASTING,0.0);
+        parms[3] = poSpatialRef->GetProjParm(SRS_PP_FALSE_NORTHING,0.0);
     }
 
     /* ==============================================================
