@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: mitab_tabfile.cpp,v 1.56 2004/09/17 19:26:23 fwarmerdam Exp $
+ * $Id: mitab_tabfile.cpp,v 1.59 2006/02/08 05:02:58 dmorissette Exp $
  *
  * Name:     mitab_tabfile.cpp
  * Project:  MapInfo TAB Read/Write library
@@ -32,6 +32,20 @@
  **********************************************************************
  *
  * $Log: mitab_tabfile.cpp,v $
+ * Revision 1.59  2006/02/08 05:02:58  dmorissette
+ * Fixed crash when attempting to write TABPolyline object with an invalid
+ * geometry (GDAL bug 1059)
+ *
+ * Revision 1.58  2005/10/13 20:12:03  fwarmerdam
+ * layers with just regions can't be set as type wkbPolygon because they may
+ * have multipolygons (bug GDAL:958)
+ *     http://bugzilla.remotesensing.org/show_bug.cgi?id=958
+ *
+ * Revision 1.57  2005/10/04 15:44:31  dmorissette
+ * First round of support for Collection objects. Currently supports reading
+ * from .TAB/.MAP and writing to .MIF. Still lacks symbol support and write
+ * support. (Based in part on patch and docs from Jim Hope, bug 1126)
+ *
  * Revision 1.56  2004/09/17 19:26:23  fwarmerdam
  * Several fixes in error/failure cases in TABFile::Open() care of
  * Stephen Cheesman of GeoSoft.
@@ -508,8 +522,6 @@ int TABFile::Open(const char *pszFname, const char *pszAccess,
             m_poDefn->SetGeomType( wkbPoint );
         else if( numPoints == 0 && numLines > 0 && numRegions == 0 )
             m_poDefn->SetGeomType( wkbLineString );
-        else if( numPoints == 0 && numLines == 0 && numRegions > 0 )
-            m_poDefn->SetGeomType( wkbPolygon );
         else
             /* we leave it unknown indicating a mixture */;
     }
@@ -1310,6 +1322,10 @@ TABFeature *TABFile::GetFeatureRef(int nFeatureId)
       case TAB_GEOM_MULTIPOINT:
         m_poCurFeature = new TABMultiPoint(m_poDefn);
         break;
+      case TAB_GEOM_COLLECTION_C:
+      case TAB_GEOM_COLLECTION:
+        m_poCurFeature = new TABCollection(m_poDefn);  
+        break;
       default:
         /*-------------------------------------------------------------
          * Unsupported feature type... we still return a valid feature
@@ -1456,6 +1472,21 @@ int TABFile::SetFeature(TABFeature *poFeature, int nFeatureId /*=-1*/)
                              nFeatureId);
     TABMAPObjectBlock *poObjBlock = NULL;
 
+    
+    /*-----------------------------------------------------------------
+     * ValidateMapInfoType() may have returned TAB_GEOM_NONE if feature
+     * contained an invalid geometry for its class. Need to catch that
+     * case and return the error.
+     *----------------------------------------------------------------*/
+    if (poObjHdr->m_nType == TAB_GEOM_NONE &&
+        poFeature->GetFeatureClass() != TABFCNoGeomFeature )
+    {
+        CPLError(CE_Failure, CPLE_FileIO,
+                 "Invalid geometry for feature id %d in %s",
+                 nFeatureId, m_pszFname);
+        return -1;
+    }
+
     if ( poObjHdr == NULL || m_poMAPFile == NULL ||
         m_poMAPFile->PrepareNewObj(nFeatureId, poObjHdr->m_nType) != 0 ||
          poFeature->WriteGeometryToMAPFile(m_poMAPFile, poObjHdr) != 0 )
