@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmysqldatasource.cpp,v 1.20 2006/03/01 19:40:40 fwarmerdam Exp $
+ * $Id: ogrmysqldatasource.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMySQLDataSource class.
@@ -26,86 +26,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrmysqldatasource.cpp,v $
- * Revision 1.20  2006/03/01 19:40:40  fwarmerdam
- * Change MYSQL_GEOM_COLUMN to GEOMETRY_NAME.
- *
- * Revision 1.19  2006/02/27 01:52:50  hobu
- * tweak error message
- *
- * Revision 1.18  2006/02/27 00:29:27  hobu
- * Allow the setting of timeouts via environment variable.
- * Connect with CLIENT_INTERACTIVE mode.
- * Tweak the error message in ReportError to make sure
- * we can see what MySQL tells us (bug 1087).
- *
- * Revision 1.17  2006/02/21 03:50:11  fwarmerdam
- * Fixed selection of new SRID values when no match is found.
- *
- * Revision 1.16  2006/02/16 20:59:32  fwarmerdam
- * added ENGINE layer creation option
- *
- * Revision 1.15  2006/02/16 18:00:11  fwarmerdam
- * Don't make F_TABLE_NAME, it doesn't work on MySQL 4.x
- * Set COORD_DIMENSION.  Leave SRID NULL if not known.  Support wkbNone
- * in CreateLayer() as having no geometry field.  Added support for
- * SPATIAL_INDEX creation option to disable spatial index and associated
- * NOT NULL constraint.
- *
- * Revision 1.14  2006/02/16 16:28:00  fwarmerdam
- * Improved InitializeMetadataTables() error reporting, and removed
- * UNIQUE constraint on F_TABLE_NAME as it causes problems in mysql 4.x
- *
- * Revision 1.13  2006/02/13 04:15:04  hobu
- * major formatting cleanup
- * Added myself as an author
- *
- * Revision 1.12  2006/02/13 01:44:25  hobu
- * Define an Initialize() method for TableLayer and
- * make sure to use it when we after we construct
- * a new one.  This is to prevent ResetReading from
- * causing segfault in the case where we were given
- * an invalid table name
- *
- * Revision 1.11  2006/02/12 06:07:27  hobu
- * Make sure to create a spatial index
- *
- * Revision 1.10  2006/02/11 18:08:03  hobu
- * implemented CreateLayer
- * moved FetchSRS to here like PG
- *
- * Revision 1.9  2006/02/10 04:58:37  hobu
- * InitializeMetadataTables implementation
- *
- * Revision 1.8  2006/02/10 04:46:32  fwarmerdam
- * format cleanup
- *
- * Revision 1.7  2006/02/09 05:54:52  hobu
- * start on CreateLayer
- *
- * Revision 1.6  2006/01/16 16:04:58  hobu
- * comment typo
- *
- * Revision 1.5  2005/09/21 01:00:01  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.4  2005/08/02 13:01:29  fwarmerdam
- * Removed old postgis create logic.
- *
- * Revision 1.3  2004/10/12 16:59:31  fwarmerdam
- * rearrange include files for win32
- *
- * Revision 1.2  2004/10/08 20:50:05  fwarmerdam
- * Implemented ExecuteSQL().
- * Added support for getting host, password, port, and user from the
- * datasource name or from the mysql init files (ie. ~/.my.cnf).
- *
- * Revision 1.1  2004/10/07 20:56:15  fwarmerdam
- * New
- *
- */
+ ****************************************************************************/
 
 
 #include <string>
@@ -115,7 +36,7 @@
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrmysqldatasource.cpp,v 1.20 2006/03/01 19:40:40 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrmysqldatasource.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 /************************************************************************/
 /*                         OGRMySQLDataSource()                         */
 /************************************************************************/
