@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrmysqltablelayer.cpp,v 1.33 2006/04/13 03:57:40 hobu Exp $
+ * $Id: ogrmysqltablelayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRMySQLTableLayer class.
@@ -26,130 +26,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrmysqltablelayer.cpp,v $
- * Revision 1.33  2006/04/13 03:57:40  hobu
- * add William's patches for tinyint and smallint detection (bug 1155)
- *
- * Revision 1.32  2006/04/03 01:48:53  fwarmerdam
- * improve date support
- *
- * Revision 1.31  2006/03/17 01:29:33  hobu
- * Was too greedy on setting widths and precisions and
- * we were picking up columns of type "text" and setting a
- * width when they don't have one.
- *
- * Revision 1.30  2006/02/27 02:29:13  hobu
- * Provide a good error message when we try to insert a feature
- * that is larger than the max_allowed_packet
- *
- * Revision 1.29  2006/02/16 17:56:03  fwarmerdam
- * Use COORD_DIMENSION when working out layer geometry type.
- *
- * Revision 1.28  2006/02/16 17:21:00  fwarmerdam
- * Added date support.
- *
- * Revision 1.27  2006/02/13 04:15:04  hobu
- * major formatting cleanup
- * Added myself as an author
- *
- * Revision 1.26  2006/02/13 01:44:25  hobu
- * Define an Initialize() method for TableLayer and
- * make sure to use it when we after we construct
- * a new one.  This is to prevent ResetReading from
- * causing segfault in the case where we were given
- * an invalid table name
- *
- * Revision 1.25  2006/02/12 06:22:45  hobu
- * Implement SetFeature
- *
- * Revision 1.24  2006/02/12 06:17:25  hobu
- * Implement DeleteFeature
- *
- * Revision 1.23  2006/02/12 01:16:42  hobu
- * remove debugging lint and handle
- * double(width, precision) fields
- *
- * Revision 1.22  2006/02/12 00:40:57  hobu
- * Implement a somewhat working CreateFeature
- *
- * Revision 1.21  2006/02/11 18:08:34  hobu
- * Moved FetchSRS to happen on the datasource like PG
- * Implemented CreateField for TableLayer
- *
- * Revision 1.19  2006/02/07 18:17:15  hobu
- * Always return geometries as "native"
- *
- * Revision 1.18  2006/02/02 01:24:17  hobu
- * make sure we properly sett hResultSet to NULL after all frees.
- *
- * Revision 1.17  2006/02/01 01:40:09  hobu
- * separate fetching of SRID
- *
- * Revision 1.16  2006/01/31 06:17:15  hobu
- * move SRS fetching to mysqllayer.cpp
- *
- * Revision 1.15  2006/01/27 01:08:29  fwarmerdam
- * Various fixes for when the geometry columns or srid tables are
- * missing.  Avoid issue errors in this case, they are optional.
- *
- * Revision 1.14  2006/01/19 06:31:44  hobu
- * Use MBRIntersects for spatial filters.
- * Test the envelope rather than the entire geometry.
- *
- * Revision 1.13  2006/01/19 03:51:23  hobu
- * Handle spatial filters
- *
- * Revision 1.12  2006/01/16 16:08:39  hobu
- * Detect geometry column.
- * Query geometry column.
- * Detect geometry type.
- * Query geometry type.
- * Detect spatial reference.
- * Query spatial reference.
- *
- * Revision 1.11  2005/12/28 06:24:52  hobu
- * remove debugging lint
- *
- * Revision 1.10  2005/12/28 06:22:08  hobu
- * ReadTableDefinitions() wasn't setting widths properly for varchar, char, and decimal types.
- * Also, varchar handling case was never being selected.
- *
- * Revision 1.9  2005/09/21 01:00:01  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.8  2005/08/30 23:53:16  fwarmerdam
- * implement binary field support
- *
- * Revision 1.7  2005/07/25 16:08:50  fwarmerdam
- * Fixed recognision of some integer types, such as mediumint.
- *
- * Revision 1.6  2005/07/25 14:39:55  fwarmerdam
- * Added FID column related debug statement.
- *
- * Revision 1.5  2005/06/27 17:00:50  fwarmerdam
- * Fixed bug with cleaning up hResultSet in GetFeature(id).
- *
- * Revision 1.4  2005/02/24 21:54:36  fwarmerdam
- * added support for decimal fields.
- *
- * Revision 1.3  2005/02/22 12:54:27  fwarmerdam
- * use OGRLayer base spatial filter support
- *
- * Revision 1.2  2004/10/08 20:50:48  fwarmerdam
- * avoid leak in GetFeatureCount()
- *
- * Revision 1.1  2004/10/07 20:56:15  fwarmerdam
- * New
- *
- */
+ ****************************************************************************/
 
 #include "cpl_conv.h"
 #include "cpl_string.h"
 #include "ogr_mysql.h"
 
-CPL_CVSID("$Id: ogrmysqltablelayer.cpp,v 1.33 2006/04/13 03:57:40 hobu Exp $");
+CPL_CVSID("$Id: ogrmysqltablelayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                         OGRMySQLTableLayer()                         */
