@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrntfdriver.cpp,v 1.5 2006/03/28 22:59:55 fwarmerdam Exp $
+ * $Id: ogrntfdriver.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  UK NTF Reader
  * Purpose:  Implements OGRNTFDriver
@@ -25,30 +25,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrntfdriver.cpp,v $
- * Revision 1.5  2006/03/28 22:59:55  fwarmerdam
- * updated contact info
- *
- * Revision 1.4  2001/07/18 04:55:16  warmerda
- * added CPL_CSVID
- *
- * Revision 1.3  2001/01/19 20:31:12  warmerda
- * expand tabs
- *
- * Revision 1.2  1999/11/04 21:11:52  warmerda
- * Added TestCapability().
- *
- * Revision 1.1  1999/08/28 03:13:35  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ntf.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: ogrntfdriver.cpp,v 1.5 2006/03/28 22:59:55 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrntfdriver.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /* ==================================================================== */
