@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrocitablelayer.cpp,v 1.35 2005/11/23 00:49:13 fwarmerdam Exp $
+ * $Id: ogrocitablelayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  Oracle Spatial Driver
  * Purpose:  Implementation of the OGROCITableLayer class.  This class provides
@@ -27,130 +27,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrocitablelayer.cpp,v $
- * Revision 1.35  2005/11/23 00:49:13  fwarmerdam
- * Clear error if table describe fails.
- *
- * Revision 1.34  2005/10/27 13:42:26  fwarmerdam
- * preliminary DeleteFeature implementation
- *
- * Revision 1.33  2005/06/23 16:07:55  fwarmerdam
- * Use polygon instead of rect object for sdo_filter.  Apparently this is needed
- * for queries against geodetic coordinates.  Patch provided by Lorenzo
- * Sassolini (http://www.maior.it).
- *
- * Revision 1.32  2005/02/22 12:57:51  fwarmerdam
- * use OGRLayer base spatial filter support
- *
- * Revision 1.31  2005/02/10 15:46:02  fwarmerdam
- * added GEOMETRY_NAME layer creation option
- *
- * Revision 1.30  2004/11/22 19:24:15  fwarmerdam
- * added support for a list of tables in the datasource name
- *
- * Revision 1.29  2004/11/09 02:49:12  fwarmerdam
- * Don't add user_sdo_geom_metadata record till finalize (layer load complete)
- * time as that is when we have all the diminfo details.  Inserts of metadata
- * records with NULL diminfos fails in Oracle 10 due to new constraints.
- *
- * Revision 1.28  2004/09/28 17:19:25  fwarmerdam
- * Added untested support for OCI_FID config variable for FID column name.
- *
- * Revision 1.27  2003/09/18 15:32:59  warmerda
- * Added view support as per bug 394
- *
- * Revision 1.26  2003/09/17 16:36:33  warmerda
- * fixed setting of dimension for point objects
- *
- * Revision 1.25  2003/09/13 03:48:36  warmerda
- * actually use the papaeFieldInd to write NULL fields properly in bound create
- *
- * Revision 1.24  2003/05/21 03:54:01  warmerda
- * expand tabs
- *
- * Revision 1.23  2003/04/22 19:37:51  warmerda
- * Added sync to disk
- *
- * Revision 1.22  2003/04/11 16:27:42  warmerda
- * add support for bound feature writing in OGROCITableLayer
- *
- * Revision 1.21  2003/04/04 06:18:08  warmerda
- * first pass implementation of loader support
- *
- * Revision 1.20  2003/02/06 21:16:37  warmerda
- * restructure to handle elem_info indirectly like ordinals
- *
- * Revision 1.19  2003/01/15 05:47:06  warmerda
- * fleshed out SRID writing support in newly created geometries
- *
- * Revision 1.18  2003/01/15 05:36:14  warmerda
- * start work on creating feature geometry with SRID
- *
- * Revision 1.17  2003/01/14 22:15:02  warmerda
- * added logic to order rings properly
- *
- * Revision 1.16  2003/01/14 16:59:03  warmerda
- * added field truncation support
- *
- * Revision 1.15  2003/01/14 15:31:08  warmerda
- * added fallback support if no spatial index available
- *
- * Revision 1.14  2003/01/14 15:11:00  warmerda
- * Added layer creation options caching on layer.
- * Set SRID in spatial query.
- * Support user override of DIMINFO bounds in FinalizeNewLayer().
- * Support user override of indexing options, or disabling of indexing.
- *
- * Revision 1.13  2003/01/13 13:50:13  warmerda
- * dont quote table names, it doesnt seem to work with userid.tablename
- *
- * Revision 1.12  2003/01/10 22:31:53  warmerda
- * no longer encode ordinates into SQL statement when creating features
- *
- * Revision 1.11  2003/01/09 21:19:12  warmerda
- * improved data type support, get/set precision
- *
- * Revision 1.10  2003/01/07 22:24:35  warmerda
- * added SRS support
- *
- * Revision 1.9  2003/01/07 21:14:20  warmerda
- * implement GetFeature() and SetFeature()
- *
- * Revision 1.8  2003/01/07 18:16:01  warmerda
- * implement spatial filtering in Oracle, re-enable index build
- *
- * Revision 1.7  2003/01/06 18:00:34  warmerda
- * Restructure geometry translation ... collections now supported.
- * Dimension is now a layer wide attribute.
- * Update dimension info in USER_SDO_GEOM_METADATA on close.
- *
- * Revision 1.6  2003/01/02 21:51:05  warmerda
- * fix quote escaping
- *
- * Revision 1.5  2002/12/29 19:43:59  warmerda
- * avoid some warnings
- *
- * Revision 1.4  2002/12/29 03:48:58  warmerda
- * fixed memory bug in CreateFeature()
- *
- * Revision 1.3  2002/12/28 20:06:31  warmerda
- * minor CreateFeature improvements
- *
- * Revision 1.2  2002/12/28 04:38:36  warmerda
- * converted to unix file conventions
- *
- * Revision 1.1  2002/12/28 04:07:27  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogr_oci.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrocitablelayer.cpp,v 1.35 2005/11/23 00:49:13 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrocitablelayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static int nDiscarded = 0;
 static int nHits = 0;
