@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrodbclayer.cpp,v 1.13 2005/11/18 16:50:46 fwarmerdam Exp $
+ * $Id: ogrodbclayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRODBCLayer class, code shared between 
@@ -26,56 +26,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrodbclayer.cpp,v $
- * Revision 1.13  2005/11/18 16:50:46  fwarmerdam
- * added ODBC_OGR_FID config variable
- *
- * Revision 1.12  2005/09/21 00:56:55  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.11  2005/09/09 05:02:12  fwarmerdam
- * added wkb support
- *
- * Revision 1.10  2005/09/05 19:56:42  fwarmerdam
- * added binary field support
- *
- * Revision 1.9  2005/03/21 20:36:31  fwarmerdam
- * set precision for decimal fields - MS bug 1255
- *
- * Revision 1.8  2005/02/22 12:53:56  fwarmerdam
- * use OGRLayer base spatial filter support
- *
- * Revision 1.7  2005/02/02 20:54:27  fwarmerdam
- * track m_nFeaturesRead
- *
- * Revision 1.6  2004/03/04 17:16:05  warmerda
- * Cleanup featuredefn on exit.
- *
- * Revision 1.5  2004/01/05 22:38:17  warmerda
- * stripped out some junk
- *
- * Revision 1.4  2003/10/29 16:47:38  warmerda
- * Improved width/precision handling for real numbers.
- *
- * Revision 1.3  2003/10/08 16:00:16  warmerda
- * provide default GetFeature(int) implementation
- *
- * Revision 1.2  2003/09/26 20:03:03  warmerda
- * initialize pszFIDColumn
- *
- * Revision 1.1  2003/09/25 17:08:37  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "cpl_conv.h"
 #include "ogr_odbc.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrodbclayer.cpp,v 1.13 2005/11/18 16:50:46 fwarmerdam Exp $");
-
+CPL_CVSID("$Id: ogrodbclayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                            OGRODBCLayer()                            */
@@ -89,6 +46,7 @@ OGRODBCLayer::OGRODBCLayer()
     bGeomColumnWKB = FALSE;
     pszGeomColumn = NULL;
     pszFIDColumn = NULL;
+    panFieldOrdinals = NULL;
 
     poStmt = NULL;
 
@@ -107,27 +65,30 @@ OGRODBCLayer::~OGRODBCLayer()
 {
     if( m_nFeaturesRead > 0 && poFeatureDefn != NULL )
     {
-        CPLDebug( "ODBC", "%d features read on layer '%s'.",
+        CPLDebug( "OGR_ODBC", "%d features read on layer '%s'.",
                   (int) m_nFeaturesRead, 
                   poFeatureDefn->GetName() );
     }
 
-    if( poStmt != NULL )
+    if( poStmt )
     {
         delete poStmt;
         poStmt = NULL;
     }
 
-    if( pszGeomColumn != NULL )
+    if( pszGeomColumn )
         CPLFree( pszGeomColumn );
 
-    if( poFeatureDefn != NULL )
+    if ( panFieldOrdinals )
+        CPLFree( panFieldOrdinals );
+
+    if( poFeatureDefn )
     {
         poFeatureDefn->Release();
         poFeatureDefn = NULL;
     }
 
-    if( poSRS != NULL )
+    if( poSRS )
         poSRS->Release();
 }
 
@@ -158,33 +119,33 @@ CPLErr OGRODBCLayer::BuildFeatureDefn( const char *pszLayerName,
         if( pszGeomColumn != NULL 
             && EQUAL(poStmt->GetColName(iCol),pszGeomColumn) )
             continue;
-        
-        switch( poStmt->GetColType(iCol) )
+       
+        switch( CPLODBCStatement::GetTypeMapping(poStmt->GetColType(iCol)) )
         {
-          case SQL_INTEGER:
-            oField.SetType( OFTInteger );
-            break;
+            case SQL_C_SSHORT:
+            case SQL_C_USHORT:
+            case SQL_C_SLONG:
+            case SQL_C_ULONG:
+                oField.SetType( OFTInteger );
+                break;
 
-          case SQL_BINARY:
-          case SQL_VARBINARY:
-          case SQL_LONGVARBINARY:
-            oField.SetType( OFTBinary );
-            break;
+            case SQL_C_BINARY:
+                oField.SetType( OFTBinary );
+                break;
 
-          case SQL_DECIMAL:
-            oField.SetType( OFTReal );
-            oField.SetPrecision( poStmt->GetColPrecision(iCol) );
-            break;
+            case SQL_C_NUMERIC:
+                oField.SetType( OFTReal );
+                oField.SetPrecision( poStmt->GetColPrecision(iCol) );
+                break;
 
-          case SQL_FLOAT:
-          case SQL_REAL:
-          case SQL_DOUBLE:
-            oField.SetType( OFTReal );
-            oField.SetWidth( 0 );
-            break;
+            case SQL_C_FLOAT:
+            case SQL_C_DOUBLE:
+                oField.SetType( OFTReal );
+                oField.SetWidth( 0 );
+                break;
 
-          default:
-            /* leave it as OFTString */;
+            default:
+                /* leave it as OFTString */;
         }
 
         poFeatureDefn->AddFieldDefn( &oField );
@@ -309,20 +270,44 @@ OGRFeature *OGRODBCLayer::GetNextRawFeature()
         int iField = poStmt->GetColId( pszGeomColumn );
         const char *pszGeomText = poStmt->GetColData( iField );
         OGRGeometry *poGeom = NULL;
+        OGRErr eErr = OGRERR_NONE;
 
         if( pszGeomText != NULL && !bGeomColumnWKB )
         {
-            OGRGeometryFactory::createFromWkt( (char **) &pszGeomText,
-                                               NULL, &poGeom );
+            eErr =
+                OGRGeometryFactory::createFromWkt((char **) &pszGeomText,
+                                                  NULL, &poGeom);
         }
         else if( pszGeomText != NULL && bGeomColumnWKB )
         {
             int nLength = poStmt->GetColDataLength( iField );
 
-            OGRGeometryFactory::createFromWkb( (unsigned char *) pszGeomText,
-                                               NULL, &poGeom, nLength );
+            eErr =
+                OGRGeometryFactory::createFromWkb((unsigned char *) pszGeomText,
+                                                  NULL, &poGeom, nLength);
         }
         
+        if ( eErr != OGRERR_NONE )
+        {
+            const char *pszMessage;
+
+            switch ( eErr )
+            {
+                case OGRERR_NOT_ENOUGH_DATA:
+                    pszMessage = "Not enough data to deserialize";
+                    break;
+                case OGRERR_UNSUPPORTED_GEOMETRY_TYPE:
+                    pszMessage = "Unsupported geometry type";
+                    break;
+                case OGRERR_CORRUPT_DATA:
+                    pszMessage = "Corrupt data";
+                default:
+                    pszMessage = "Unrecognized error";
+            }
+            CPLError(CE_Failure, CPLE_AppDefined,
+                     "GetNextRawFeature(): %s", pszMessage);
+        }
+
         if( poGeom != NULL )
             poFeature->SetGeometryDirectly( poGeom );
     }
@@ -374,3 +359,4 @@ OGRSpatialReference *OGRODBCLayer::GetSpatialRef()
 {
     return poSRS;
 }
+
