@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpglayer.cpp,v 1.31 2006/04/05 03:15:13 fwarmerdam Exp $
+ * $Id: ogrpglayer.cpp 11226 2007-04-08 14:54:41Z mloskot $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGLayer class  which implements shared handling
@@ -27,113 +27,17 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrpglayer.cpp,v $
- * Revision 1.31  2006/04/05 03:15:13  fwarmerdam
- * use CPLString for command buffer to handle large commands
- *
- * Revision 1.30  2006/04/02 18:47:39  fwarmerdam
- * added detailed date type support
- *
- * Revision 1.29  2006/02/15 04:26:17  fwarmerdam
- * added date support
- *
- * Revision 1.28  2006/01/27 00:10:32  fwarmerdam
- * added Get{FID,Geometry}Column() support
- *
- * Revision 1.27  2006/01/16 15:36:33  fwarmerdam
- * disable PQfformat calls in pre7.4 client libs
- *
- * Revision 1.26  2005/12/16 07:42:08  osemykin
- * Added regular data fields support for binary cursor
- *
- * Revision 1.25  2005/11/18 12:42:55  dron
- * Handle varchar arrays when reading feature definition fields.
- *
- * Revision 1.24  2005/10/24 23:50:16  fwarmerdam
- * fixed extraction of geomType in GeometryToHex()
- *
- * Revision 1.23  2005/10/16 01:38:34  cfis
- * Updates that add support for using COPY for inserting data to Postgresql.  COPY is less robust than INSERT, but signficantly faster.
- *
- * Revision 1.22  2005/09/30 19:11:16  fwarmerdam
- * Fixed Hextobinary conversion (a and A mistranslated).
- *
- * Revision 1.21  2005/09/21 00:55:42  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.20  2005/08/06 14:49:27  osemykin
- * Added BINARY CURSOR support
- * Use it with 'PGB:dbname=...' instead 'PG:dbname=...'
- *
- * Revision 1.19  2005/07/20 01:45:01  fwarmerdam
- * added PostGIS 8.0 hex geometry format support
- *
- * Revision 1.18  2005/05/05 20:47:52  dron
- * Override GetExtent() method for PostGIS layers with PostGIS standard function
- * extent() (Oleg Semykin <oleg.semykin@gmail.com>
- *
- * Revision 1.17  2005/02/22 12:54:05  fwarmerdam
- * use OGRLayer base spatial filter support
- *
- * Revision 1.16  2005/02/02 20:54:27  fwarmerdam
- * track m_nFeaturesRead
- *
- * Revision 1.15  2004/07/10 04:46:24  warmerda
- * initialize nResultOffset, use soft transactions
- *
- * Revision 1.14  2004/05/08 02:14:49  warmerda
- * added GetFeature() on table, generalize FID support a bit
- *
- * Revision 1.13  2003/05/21 03:59:42  warmerda
- * expand tabs
- *
- * Revision 1.12  2003/02/01 07:55:48  warmerda
- * avoid dependence on libpq-fs.h
- *
- * Revision 1.11  2003/01/08 22:07:14  warmerda
- * Added support for integer and real list field types
- *
- * Revision 1.10  2002/05/09 16:03:19  warmerda
- * major upgrade to support SRS better and add ExecuteSQL
- *
- * Revision 1.9  2001/11/15 21:19:47  warmerda
- * added soft transaction semantics, handle null fields properly
- *
- * Revision 1.8  2001/11/15 16:10:12  warmerda
- * fixed some escaping issues with string field values
- *
- * Revision 1.7  2001/09/28 04:03:52  warmerda
- * partially upraded to PostGIS 0.6
- *
- * Revision 1.6  2001/07/18 04:55:16  warmerda
- * added CPL_CSVID
- *
- * Revision 1.5  2001/06/26 20:59:13  warmerda
- * implement efficient spatial and attribute query support
- *
- * Revision 1.4  2001/06/19 22:29:12  warmerda
- * upgraded to include PostGIS support
- *
- * Revision 1.3  2001/06/19 15:50:23  warmerda
- * added feature attribute query support
- *
- * Revision 1.2  2000/11/23 06:03:35  warmerda
- * added Oid support
- *
- * Revision 1.1  2000/10/17 17:46:51  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
-#include "cpl_conv.h"
 #include "ogr_pg.h"
+#include "ogrpgutility.h"
+#include "ogr_p.h"
+#include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrpglayer.cpp,v 1.31 2006/04/05 03:15:13 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrpglayer.cpp 11226 2007-04-08 14:54:41Z mloskot $");
 
-#define CURSOR_PAGE     1
+#define CURSOR_PAGE     500
 
 // These originally are defined in libpq-fs.h.
 
@@ -221,14 +125,14 @@ void OGRPGLayer::ResetReading()
 
     if( hCursorResult != NULL )
     {
-        PQclear( hCursorResult );
+        OGRPGClearResult( hCursorResult );
 
         if( bCursorActive )
         {
             sprintf( szCommand, "CLOSE %s", pszCursorName );
 
             hCursorResult = PQexec(hPGConn, szCommand);
-            PQclear( hCursorResult );
+            OGRPGClearResult( hCursorResult );
         }
 
         poDS->FlushSoftTransaction();
@@ -363,6 +267,9 @@ OGRFeature *OGRPGLayer::RecordToFeature( int iRecord )
         {
             GByte * pabyWkb = (GByte *)PQgetvalue( hCursorResult,
                                                    iRecord, iField);
+
+
+
             OGRGeometry * poGeom = NULL;
             OGRGeometryFactory::createFromWkb(pabyWkb,NULL,&poGeom);
             poFeature->SetGeometryDirectly( poGeom );
@@ -622,7 +529,7 @@ OGRFeature *OGRPGLayer::GetNextRawFeature()
         CPLDebug( "OGR_PG", "PQexec(%s)", osCommand.c_str() );
 
         hCursorResult = PQexec(hPGConn, osCommand );
-        PQclear( hCursorResult );
+        OGRPGClearResult( hCursorResult );
 
         osCommand.Printf( "FETCH %d in %s", CURSOR_PAGE, pszCursorName );
         hCursorResult = PQexec(hPGConn, osCommand );
@@ -638,6 +545,10 @@ OGRFeature *OGRPGLayer::GetNextRawFeature()
     if( hCursorResult == NULL
         || PQresultStatus(hCursorResult) != PGRES_TUPLES_OK )
     {
+        CPLDebug( "OGR_PG", "PQclear() on an error condition");
+
+        OGRPGClearResult( hCursorResult );
+
         iNextShapeId = MAX(1,iNextShapeId);
         return NULL;
     }
@@ -648,8 +559,8 @@ OGRFeature *OGRPGLayer::GetNextRawFeature()
     if( nResultOffset >= PQntuples(hCursorResult)
         && bCursorActive )
     {
-        PQclear( hCursorResult );
-
+        OGRPGClearResult( hCursorResult );
+        
         osCommand.Printf( "FETCH %d in %s", CURSOR_PAGE, pszCursorName );
         hCursorResult = PQexec(hPGConn, osCommand );
 
@@ -662,14 +573,14 @@ OGRFeature *OGRPGLayer::GetNextRawFeature()
 /* -------------------------------------------------------------------- */
     if( nResultOffset >= PQntuples(hCursorResult) )
     {
-        PQclear( hCursorResult );
+        OGRPGClearResult( hCursorResult );
 
         if( bCursorActive )
         {
             osCommand.Printf( "CLOSE %s", pszCursorName );
 
             hCursorResult = PQexec(hPGConn, osCommand);
-            PQclear( hCursorResult );
+            OGRPGClearResult( hCursorResult );
         }
 
         poDS->FlushSoftTransaction();
@@ -701,7 +612,11 @@ OGRFeature *OGRPGLayer::GetNextRawFeature()
 OGRFeature *OGRPGLayer::GetFeature( long nFeatureId )
 
 {
-    /* This should be implemented! */
+    /*
+     * TODO: This should be implemented!
+     * See related Bug 1445
+     * http://bugzilla.remotesensing.org/show_bug.cgi?id=1445
+     */
 
     return NULL;
 }
@@ -713,13 +628,18 @@ OGRFeature *OGRPGLayer::GetFeature( long nFeatureId )
 OGRGeometry *OGRPGLayer::HEXToGeometry( const char *pszBytea )
 
 {
-    GByte       *pabyWKB;
-    int iSrc=0, iDst=0;
-    OGRGeometry *poGeometry;
+    GByte   *pabyWKB = NULL;
+    int     iSrc=0;
+    int     iDst=0;
+    OGRGeometry *poGeometry = NULL;
+    unsigned int ewkbFlags = 0;
 
     if( pszBytea == NULL )
         return NULL;
 
+/* -------------------------------------------------------------------- */
+/*      Convert hex to binary.                                          */
+/* -------------------------------------------------------------------- */
     pabyWKB = (GByte *) CPLMalloc(strlen(pszBytea)+1);
     while( pszBytea[iSrc] != '\0' )
     {
@@ -749,6 +669,45 @@ OGRGeometry *OGRPGLayer::HEXToGeometry( const char *pszBytea )
         iDst++;
     }
 
+/* -------------------------------------------------------------------- */
+/*      Detect XYZM variant of PostGIS EWKB                             */
+/*                                                                      */
+/*      OGR does not support parsing M coordinate,                      */
+/*      so we return NULL geometry.                                     */
+/* -------------------------------------------------------------------- */
+    memcpy(&ewkbFlags, pabyWKB+1, 4);
+    OGRwkbByteOrder eByteOrder = (pabyWKB[0] == 0 ? wkbXDR : wkbNDR);
+    if( OGR_SWAP( eByteOrder ) )
+        ewkbFlags= CPL_SWAP32(ewkbFlags);
+
+    if (ewkbFlags & 0x40000000)
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+            "Reading EWKB with 4-dimensional coordinates (XYZM) is not supported" );
+
+        CPLFree( pabyWKB );
+        return NULL;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      PostGIS EWKB format includes an  SRID, but this won't be        */
+/*      understood by OGR, so if the SRID flag is set, we remove the    */
+/*      SRID (bytes at offset 5 to 8).                                  */
+/* -------------------------------------------------------------------- */
+    if( (pabyWKB[0] == 0 /* big endian */ && (pabyWKB[1] & 0x20) )
+        || (pabyWKB[0] != 0 /* little endian */ && (pabyWKB[4] & 0x20)) )
+    {
+        memmove( pabyWKB+5, pabyWKB+9, iDst-9 );
+        iDst -= 4;
+        if( pabyWKB[0] == 0 )
+            pabyWKB[1] &= (~0x20);
+        else
+            pabyWKB[4] &= (~0x20);
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Try to ingest the geometry.                                     */
+/* -------------------------------------------------------------------- */
     poGeometry = NULL;
     OGRGeometryFactory::createFromWkb( pabyWKB, NULL, &poGeometry, iDst );
 
