@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrpgtablelayer.cpp,v 1.52 2006/04/02 18:47:39 fwarmerdam Exp $
+ * $Id: ogrpgtablelayer.cpp 11225 2007-04-08 14:51:34Z mloskot $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRPGTableLayer class, access to an existing table.
@@ -25,114 +25,16 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrpgtablelayer.cpp,v $
- * Revision 1.52  2006/04/02 18:47:39  fwarmerdam
- * added detailed date type support
- *
- * Revision 1.51  2006/02/15 04:26:17  fwarmerdam
- * added date support
- *
- * Revision 1.50  2006/01/21 03:45:58  fwarmerdam
- * Changed to use CPLString for various command buffers in an attempt
- * to avoid issues with limited size buffers.
- *
- * Revision 1.49  2006/01/13 13:21:57  fwarmerdam
- * Ensure that column widths for VARCHAR are used if available.  Bug 1037
- *
- * Revision 1.48  2005/12/05 11:34:22  dron
- * Handle 'double precision[]' arrays in ReadTableDefinition().
- *
- * Revision 1.47  2005/11/28 16:18:32  osemykin
- * TestCapability() modified for right result of SetFeature() and DeleteFeature() capabilities;
- * Added system variable PGSQL_OGR_FID support for detecting FID column
- *
- * Revision 1.46  2005/11/18 12:42:55  dron
- * Handle varchar arrays when reading feature definition fields.
- *
- * Revision 1.45  2005/10/24 23:50:50  fwarmerdam
- * moved bUseCopy test into layer on creation of first feature
- *
- * Revision 1.44  2005/10/24 21:01:09  fwarmerdam
- * added PG_PRE74 define for old copy functions
- *
- * Revision 1.43  2005/10/24 06:38:27  cfis
- * Added code to support COPY on postgresql 7.3 - however it currently commented out until the appropriate #ifdefs are defined.
- *
- * Revision 1.42  2005/10/19 02:49:56  cfis
- * When using COPY geometries were always sent to NULL when sending data to postgresql, even when a geometry existed.  This is now fixed.
- *
- * Revision 1.41  2005/10/16 08:33:50  cfis
- * Temporary fix for bug 962.  If the same table name is in two different schemas then the number of fields for each table will be doubled.  It its in three, then the number of fields will be tripled, etc.  The appropriate solution is to make ogr postgresql schema aware.
- *
- * Revision 1.40  2005/10/16 04:25:42  cfis
- * Use the bCopyActive flag instead of checking the data source.
- *
- * Revision 1.39  2005/10/16 03:39:25  fwarmerdam
- * cleanup COPY support somewhat
- *
- * Revision 1.38  2005/10/16 01:38:34  cfis
- * Updates that add support for using COPY for inserting data to Postgresql.  COPY is less robust than INSERT, but signficantly faster.
- *
- * Revision 1.37  2005/09/26 04:37:17  cfis
- * If inserting a feature into postgresql failed, the program would crash since the wrong number of parameters were sent to CPLError.  Fixed by passing pszCommand.
- *
- * Revision 1.36  2005/09/21 00:55:42  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.35  2005/08/17 16:25:39  fwarmerdam
- * Patched error output to include command if CreateFeature() fails.
- *
- * Revision 1.34  2005/08/06 14:49:27  osemykin
- * Added BINARY CURSOR support
- * Use it with 'PGB:dbname=...' instead 'PG:dbname=...'
- *
- * Revision 1.33  2005/08/05 13:37:40  fwarmerdam
- * Bug 902: set pszGeomColumn if WKB_GEOMETRY found.
- *
- * Revision 1.32  2005/08/04 07:28:28  osemykin
- * Fixes for support postgis version < 0.9.0
- * geom as EWKT for PostGIS >= 1.0
- * geom as Text for PostGIS < 1.0
- *
- * Revision 1.31  2005/08/03 21:27:32  osemykin
- * GetExtent fixes for old PostGIS version support
- *
- * Revision 1.30  2005/08/03 13:22:50  fwarmerdam
- * Patched GetExtent method to support BOX3D return types as reported
- * by Markus on gdal-dev.
- *
- * Revision 1.29  2005/07/20 01:45:30  fwarmerdam
- * improved geometry dimension support, PostGIS 8 EWKT upgrades
- *
- * Revision 1.28  2005/05/05 20:47:52  dron
- * Override GetExtent() method for PostGIS layers with PostGIS standard function
- * extent() (Oleg Semykin <oleg.semykin@gmail.com>
- *
- * Revision 1.27  2005/05/04 19:14:27  dron
- * Determine layer geometry type from Geometry_Columns standard OGC table.
- * Works only for PostGIS layers. (Oleg Semykin <oleg.semykin@gmail.com>)
- *
- * Revision 1.26  2005/04/29 17:08:58  dron
- * Move nSRSId checking into constructor. Change spatial filter query syntax
- * to match PostGIS 1.0.0 syntax (Oleg Semykin <oleg.semykin@gmail.com>)
- *
- * Revision 1.25  2005/02/22 12:54:05  fwarmerdam
- * use OGRLayer base spatial filter support
- *
- * Revision 1.24  2005/02/11 22:17:10  fwarmerdam
- * Applied fix for bug 681.  The truncation logic was kicking
- * inappropriately for integerlist and stringlist values.
- */
+ ****************************************************************************/
 
+#include "ogr_pg.h"
+#include "ogrpgutility.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
-#include "ogr_pg.h"
+#include "cpl_error.h"
 
-CPL_CVSID("$Id: ogrpgtablelayer.cpp,v 1.52 2006/04/02 18:47:39 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrpgtablelayer.cpp 11225 2007-04-08 14:51:34Z mloskot $");
 
-#define CURSOR_PAGE     1
 #define USE_COPY_UNSET  -10
 
 /************************************************************************/
@@ -140,7 +42,8 @@ CPL_CVSID("$Id: ogrpgtablelayer.cpp,v 1.52 2006/04/02 18:47:39 fwarmerdam Exp $"
 /************************************************************************/
 
 OGRPGTableLayer::OGRPGTableLayer( OGRPGDataSource *poDSIn,
-                                  const char * pszTableName,
+                                  const char * pszTableNameIn,
+                                  const char * pszSchemaNameIn,
                                   int bUpdate, int nSRSIdIn )
 
 {
@@ -154,18 +57,28 @@ OGRPGTableLayer::OGRPGTableLayer( OGRPGDataSource *poDSIn,
 
     nSRSId = nSRSIdIn;
 
-    poFeatureDefn = ReadTableDefinition( pszTableName );
-
-    ResetReading();
-
     bLaunderColumnNames = TRUE;
     bCopyActive = FALSE;
-
-    // check SRID if it's necessary
-    if( nSRSId == -2 )
-        GetSpatialRef();
-
     bUseCopy = USE_COPY_UNSET;  // unknown
+
+    pszTableName = CPLStrdup( pszTableNameIn );
+    if (pszSchemaNameIn)
+      pszSchemaName = CPLStrdup( pszSchemaNameIn );
+    else
+      pszSchemaName = NULL;
+
+    pszSqlTableName = (char*)CPLMalloc(255); //set in ReadTableDefinition
+
+    poFeatureDefn = ReadTableDefinition( pszTableName, pszSchemaName );
+
+    if( poFeatureDefn )
+    {
+        ResetReading();
+        
+        // check SRID if it's necessary
+        if( nSRSId == -2 )
+            GetSpatialRef();
+    }
 }
 
 //************************************************************************/
@@ -176,6 +89,9 @@ OGRPGTableLayer::~OGRPGTableLayer()
 
 {
     EndCopy();
+    CPLFree( pszSqlTableName );
+    CPLFree( pszTableName );
+    CPLFree( pszSchemaName );
 }
 
 /************************************************************************/
@@ -185,12 +101,14 @@ OGRPGTableLayer::~OGRPGTableLayer()
 /*      catalog.                                                        */
 /************************************************************************/
 
-OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
+OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTableIn,
+                                                      const char * pszSchemaNameIn )
 
 {
     PGresult            *hResult;
     CPLString           osCommand;
     CPLString           osPrimaryKey;
+    CPLString           osCurrentSchema;
     PGconn              *hPGConn = poDS->GetPGConn();
 
     poDS->FlushSoftTransaction();
@@ -204,6 +122,18 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
     /* -------------------------------------------- */
     osPrimaryKey = CPLGetConfigOption( "PGSQL_OGR_FID", "ogc_fid" );
 
+    // 
+    /* -------------------------------------------- */
+    /*          Get the current schema              */
+    /* -------------------------------------------- */
+    hResult = PQexec(hPGConn,"SELECT current_schema()");
+    if ( hResult && PQntuples(hResult) == 1 && !PQgetisnull(hResult,0,0) )
+    {
+        osCurrentSchema = PQgetvalue(hResult,0,0);
+
+        OGRPGClearResult( hResult );
+    }
+
     /* TODO make changes corresponded to Frank issues
        
     sprintf ( szCommand,
@@ -212,7 +142,7 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
               "WHERE c.contype='p' AND c.conrelid=cl.oid "
               "AND a.attnum = c.conkey[1] AND a.attrelid=cl.oid "
               "AND cl.relname = '%s'",
-              pszTable );
+              pszTableIn );
 
     hResult = PQexec(hPGConn, szCommand );
 
@@ -230,50 +160,89 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
     }*/
 
 /* -------------------------------------------------------------------- */
-/*      Fire off commands to get back the schema of the table.          */
+/*      Fire off commands to get back the columns of the table.          */
 /* -------------------------------------------------------------------- */
     hResult = PQexec(hPGConn, "BEGIN");
 
     if( hResult && PQresultStatus(hResult) == PGRES_COMMAND_OK )
     {
-        PQclear( hResult );
+        OGRPGClearResult( hResult );
+
+        CPLString osSchemaClause;
+        if (pszSchemaNameIn)
+          osSchemaClause.Printf("AND n.nspname='%s'", pszSchemaNameIn);
+
         osCommand.Printf(
                  "DECLARE mycursor CURSOR for "
                  "SELECT DISTINCT a.attname, t.typname, a.attlen,"
                  "       format_type(a.atttypid,a.atttypmod) "
-                 "FROM pg_class c, pg_attribute a, pg_type t "
+                 "FROM pg_class c, pg_attribute a, pg_type t, pg_namespace n "
                  "WHERE c.relname = '%s' "
                  "AND a.attnum > 0 AND a.attrelid = c.oid "
-                 "AND a.atttypid = t.oid",
-                 pszTable );
+                 "AND a.atttypid = t.oid "
+                 "AND c.relnamespace=n.oid "
+                 "%s",
+                 pszTableIn, osSchemaClause.c_str());
 
         hResult = PQexec(hPGConn, osCommand.c_str() );
     }
 
     if( hResult && PQresultStatus(hResult) == PGRES_COMMAND_OK )
     {
-        PQclear( hResult );
+        OGRPGClearResult( hResult );
         hResult = PQexec(hPGConn, "FETCH ALL in mycursor" );
     }
 
     if( !hResult || PQresultStatus(hResult) != PGRES_TUPLES_OK )
     {
+        OGRPGClearResult( hResult );
+
         CPLError( CE_Failure, CPLE_AppDefined,
                   "%s", PQerrorMessage(hPGConn) );
         return NULL;
     }
 
+    if( PQntuples(hResult) == 0 )
+    {
+        OGRPGClearResult( hResult );
+
+        hResult = PQexec(hPGConn, "CLOSE mycursor");
+        OGRPGClearResult( hResult );
+
+        hResult = PQexec(hPGConn, "COMMIT");
+        OGRPGClearResult( hResult );
+
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "No field definitions found for '%s', is it a table?",
+                  pszTableIn );
+        return NULL;
+    }
+
 /* -------------------------------------------------------------------- */
 /*      Parse the returned table information.                           */
 /* -------------------------------------------------------------------- */
-    OGRFeatureDefn *poDefn = new OGRFeatureDefn( pszTable );
+    char szLayerName[256];
+    if ( pszSchemaNameIn && osCurrentSchema != pszSchemaNameIn )
+    {
+        sprintf( szLayerName, "%s.%s", pszSchemaNameIn, pszTableIn );
+        sprintf( pszSqlTableName, "%s.\"%s\"", pszSchemaNameIn, pszTableIn );
+    }
+    else
+    {
+        //no prefix for current_schema in layer name, for backwards compatibility
+        strcpy( szLayerName, pszTableIn );
+        sprintf( pszSqlTableName, "\"%s\"", pszTableIn );
+    }
+
+    OGRFeatureDefn *poDefn = new OGRFeatureDefn( szLayerName );
     int            iRecord;
 
     poDefn->Reference();
 
     for( iRecord = 0; iRecord < PQntuples(hResult); iRecord++ )
     {
-        const char      *pszType, *pszFormatType;
+        const char      *pszType = NULL;
+        const char      *pszFormatType = NULL;
         OGRFieldDefn    oField( PQgetvalue( hResult, iRecord, 0 ), OFTString);
 
         pszType = PQgetvalue(hResult, iRecord, 1 );
@@ -284,7 +253,7 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
         {
             bHasFid = TRUE;
             pszFIDColumn = CPLStrdup(oField.GetNameRef());
-            CPLDebug("OGR_PG","Using column '%s' as FID for table '%s'", pszFIDColumn, pszTable );
+            CPLDebug("OGR_PG","Using column '%s' as FID for table '%s'", pszFIDColumn, pszTableIn );
             continue;
         }
         else if( EQUAL(pszType,"geometry") )
@@ -388,22 +357,23 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
         poDefn->AddFieldDefn( &oField );
     }
 
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     hResult = PQexec(hPGConn, "CLOSE mycursor");
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     hResult = PQexec(hPGConn, "COMMIT");
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     // get layer geometry type (for PostGIS dataset)
     if ( bHasPostGISGeometry )
     {
         osCommand.Printf(
             "SELECT type, coord_dimension FROM geometry_columns WHERE f_table_name='%s'",
-            pszTable);
+            pszTableIn);
 
         hResult = PQexec(hPGConn,osCommand);
+
         if ( hResult && PQntuples(hResult) == 1 && !PQgetisnull(hResult,0,0) )
         {
             char * pszType = PQgetvalue(hResult,0,0);
@@ -431,13 +401,13 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
                 nGeomType = (OGRwkbGeometryType) (nGeomType | wkb25DBit);
 
             CPLDebug("OGR_PG","Layer '%s' geometry type: %s:%s, Dim=%d",
-                     pszTable, pszType, OGRGeometryTypeToName(nGeomType),
+                     pszTableIn, pszType, OGRGeometryTypeToName(nGeomType),
                      nCoordDimension );
 
             poDefn->SetGeomType( nGeomType );
-
-            PQclear( hResult );
         }
+
+        OGRPGClearResult( hResult );
     }
 
     return poDefn;
@@ -450,12 +420,12 @@ OGRFeatureDefn *OGRPGTableLayer::ReadTableDefinition( const char * pszTable )
 void OGRPGTableLayer::SetSpatialFilter( OGRGeometry * poGeomIn )
 
 {
-    if( !InstallFilter( poGeomIn ) )
-        return;
+    if( InstallFilter( poGeomIn ) )
+    {
+        BuildWhere();
 
-    BuildWhere();
-
-    ResetReading();
+        ResetReading();
+    }
 }
 
 /************************************************************************/
@@ -472,6 +442,8 @@ void OGRPGTableLayer::BuildWhere()
 
     if( m_poFilterGeom != NULL && bHasPostGISGeometry )
     {
+        CPLDebug( "PG", "bHasPostGISGeometry == TRUE" );
+
         OGREnvelope  sEnvelope;
 
         m_poFilterGeom->getEnvelope( &sEnvelope );
@@ -485,13 +457,22 @@ void OGRPGTableLayer::BuildWhere()
     if( strlen(osQuery) > 0 )
     {
         if( strlen(osWHERE) == 0 )
+        {
             osWHERE.Printf( "WHERE %s ", osQuery.c_str()  );
+        }
         else	
         {
             osWHERE += "AND ";
             osWHERE += osQuery;
         }
     }
+
+    // XXX - mloskot - some debugging logic, can be removed
+    if( bHasPostGISGeometry )
+        CPLDebug( "PG", "OGRPGTableLayer::BuildWhere returns: %s",
+                  osWHERE.c_str() );
+    else
+        CPLDebug( "PG", "PostGIS is NOT available!" );
 }
 
 /************************************************************************/
@@ -511,10 +492,10 @@ void OGRPGTableLayer::BuildFullQueryStatement()
 
     pszQueryStatement = (char *)
         CPLMalloc(strlen(pszFields)+strlen(osWHERE)
-                  +strlen(poFeatureDefn->GetName()) + 40);
+                  +strlen(pszSqlTableName) + 40);
     sprintf( pszQueryStatement,
-             "SELECT %s FROM \"%s\" %s",
-             pszFields, poFeatureDefn->GetName(), osWHERE.c_str() );
+             "SELECT %s FROM %s %s",
+             pszFields, pszSqlTableName, osWHERE.c_str() );
 
     CPLFree( pszFields );
 }
@@ -543,8 +524,9 @@ void OGRPGTableLayer::ResetReading()
 char *OGRPGTableLayer::BuildFields()
 
 {
-    int         i, nSize;
-    char        *pszFieldList;
+    int     i = 0;
+    int     nSize = 0;
+    char    *pszFieldList = NULL;
 
     nSize = 25;
     if( pszGeomColumn )
@@ -633,9 +615,9 @@ OGRErr OGRPGTableLayer::SetAttributeFilter( const char *pszQuery )
 OGRErr OGRPGTableLayer::DeleteFeature( long nFID )
 
 {
-    PGconn              *hPGConn = poDS->GetPGConn();
-    PGresult            *hResult;
-    CPLString            osCommand;
+    PGconn      *hPGConn = poDS->GetPGConn();
+    PGresult    *hResult = NULL;
+    CPLString   osCommand;
 
 /* -------------------------------------------------------------------- */
 /*      We can only delete features if we have a well defined FID       */
@@ -654,11 +636,11 @@ OGRErr OGRPGTableLayer::DeleteFeature( long nFID )
 /* -------------------------------------------------------------------- */
 /*      Form the statement to drop the record.                          */
 /* -------------------------------------------------------------------- */
-    osCommand.Printf( "DELETE FROM \"%s\" WHERE \"%s\" = %ld",
-                      poFeatureDefn->GetName(), pszFIDColumn, nFID );
+    osCommand.Printf( "DELETE FROM %s WHERE \"%s\" = %ld",
+                      pszSqlTableName, pszFIDColumn, nFID );
 
 /* -------------------------------------------------------------------- */
-/*      Execute the insert.                                             */
+/*      Execute the delete.                                             */
 /* -------------------------------------------------------------------- */
     OGRErr eErr;
 
@@ -676,14 +658,19 @@ OGRErr OGRPGTableLayer::DeleteFeature( long nFID )
                   "DeleteFeature() DELETE statement failed.\n%s",
                   PQerrorMessage(hPGConn) );
 
-        PQclear( hResult );
+        OGRPGClearResult( hResult );
 
         poDS->SoftRollback();
+        eErr = OGRERR_FAILURE;
+    }
+    else
+    {
+        OGRPGClearResult( hResult );
 
-        return OGRERR_FAILURE;
+        eErr = poDS->SoftCommit();
     }
 
-    return poDS->SoftCommit();
+    return eErr;
 }
 
 /************************************************************************/
@@ -697,13 +684,20 @@ OGRErr OGRPGTableLayer::DeleteFeature( long nFID )
 OGRErr OGRPGTableLayer::SetFeature( OGRFeature *poFeature )
 
 {
-    OGRErr eErr;
+    OGRErr eErr(OGRERR_FAILURE);
+
+    if( NULL == poFeature )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "NULL pointer to OGRFeature passed to SetFeature()." );
+        return eErr;
+    }
 
     if( poFeature->GetFID() == OGRNullFID )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "FID required on features given to SetFeature()." );
-        return OGRERR_FAILURE;
+        return eErr;
     }
 
     eErr = DeleteFeature( poFeature->GetFID() );
@@ -716,8 +710,16 @@ OGRErr OGRPGTableLayer::SetFeature( OGRFeature *poFeature )
 /************************************************************************/
 /*                           CreateFeature()                            */
 /************************************************************************/
+
 OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
-{
+{ 
+    if( NULL == poFeature )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "NULL pointer to OGRFeature passed to CreateFeature()." );
+        return OGRERR_FAILURE;
+    }
+
     // We avoid testing the config option too often. 
     if( bUseCopy == USE_COPY_UNSET )
         bUseCopy = CSLTestBoolean( CPLGetConfigOption( "PG_USE_COPY", "NO") );
@@ -738,23 +740,34 @@ OGRErr OGRPGTableLayer::CreateFeature( OGRFeature *poFeature )
 /************************************************************************/
 /*                       CreateFeatureViaInsert()                       */
 /************************************************************************/
+
 OGRErr OGRPGTableLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
 
 {
     PGconn              *hPGConn = poDS->GetPGConn();
-    PGresult            *hResult;
+    PGresult            *hResult = NULL;
     CPLString           osCommand;
-    int                 i, bNeedComma = FALSE;
-    OGRErr              eErr;
+    int                 i = 0;
+    int                 bNeedComma = FALSE;
+    OGRErr              eErr = OGRERR_FAILURE;
+    
+    if( NULL == poFeature )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined,
+                  "NULL pointer to OGRFeature passed to CreateFeatureViaInsert()." );
+        return eErr;
+    }
 
     eErr = poDS->SoftStartTransaction();
     if( eErr != OGRERR_NONE )
+    {
         return eErr;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Form the INSERT command.                                        */
 /* -------------------------------------------------------------------- */
-    osCommand.Printf( "INSERT INTO \"%s\" (", poFeatureDefn->GetName() );
+    osCommand.Printf( "INSERT INTO %s (", pszSqlTableName );
 
     if( bHasWkb && poFeature->GetGeometryRef() != NULL )
     {
@@ -830,7 +843,7 @@ OGRErr OGRPGTableLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
 
         if( pszBytea != NULL )
         {
-            osCommand += osCommand + "'" + pszBytea + "'";
+            osCommand = osCommand + "'" + pszBytea + "'";
             CPLFree( pszBytea );
         }
         else
@@ -860,6 +873,10 @@ OGRErr OGRPGTableLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
 
     for( i = 0; i < poFeatureDefn->GetFieldCount(); i++ )
     {
+        // Flag indicating NULL or not-a-date date value
+        // e.g. 0000-00-00 - there is no year 0
+        OGRBoolean bIsDateNull = FALSE;
+
         const char *pszStrValue = poFeature->GetFieldAsString(i);
         char *pszNeedToFree = NULL;
 
@@ -911,8 +928,19 @@ OGRErr OGRPGTableLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
             pszStrValue = pszNeedToFree;
         }
 
+        // Check if date is NULL: 0000-00-00
+        if( poFeatureDefn->GetFieldDefn(i)->GetType() == OFTDate )
+        {
+            if( EQUALN( pszStrValue, "0000", 4 ) )
+            {
+                pszStrValue = "NULL";
+                bIsDateNull = TRUE;
+            }
+        }
+
         if( poFeatureDefn->GetFieldDefn(i)->GetType() != OFTInteger
-                 && poFeatureDefn->GetFieldDefn(i)->GetType() != OFTReal )
+            && poFeatureDefn->GetFieldDefn(i)->GetType() != OFTReal
+            && !bIsDateNull )
         {
             int         iChar;
 
@@ -939,7 +967,9 @@ OGRErr OGRPGTableLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
                     osCommand += pszStrValue[iChar];
                 }
                 else
+                {
                     osCommand += pszStrValue[iChar];
+                }
             }
 
             osCommand += "'";
@@ -967,7 +997,7 @@ OGRErr OGRPGTableLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
                   "INSERT command for new feature failed.\n%s\nCommand: %s",
                   PQerrorMessage(hPGConn), osCommand.c_str() );
 
-        PQclear( hResult );
+        OGRPGClearResult( hResult );
 
         poDS->SoftRollback();
 
@@ -980,7 +1010,8 @@ OGRErr OGRPGTableLayer::CreateFeatureViaInsert( OGRFeature *poFeature )
     Oid nNewOID = PQoidValue( hResult );
     printf( "nNewOID = %d\n", (int) nNewOID );
 #endif
-    PQclear( hResult );
+
+    OGRPGClearResult( hResult );
 
     return poDS->SoftCommit();
 }
@@ -997,7 +1028,7 @@ OGRErr OGRPGTableLayer::CreateFeatureViaCopy( OGRFeature *poFeature )
     OGRGeometry *poGeometry = (OGRGeometry *) poFeature->GetGeometryRef();
     
     char *pszGeom = NULL;
-    if ( poGeometry )
+    if ( NULL != poGeometry )
     {
         poGeometry->closeRings();
         poGeometry->setCoordinateDimension( nCoordDimension );
@@ -1225,7 +1256,7 @@ OGRErr OGRPGTableLayer::CreateField( OGRFieldDefn *poFieldIn, int bApproxOK )
 
 {
     PGconn              *hPGConn = poDS->GetPGConn();
-    PGresult            *hResult;
+    PGresult            *hResult = NULL;
     CPLString           osCommand;
     char                szFieldType[256];
     OGRFieldDefn        oField( poFieldIn );
@@ -1318,10 +1349,10 @@ OGRErr OGRPGTableLayer::CreateField( OGRFieldDefn *poFieldIn, int bApproxOK )
 /* -------------------------------------------------------------------- */
     poDS->FlushSoftTransaction();
     hResult = PQexec(hPGConn, "BEGIN");
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
-    osCommand.Printf( "ALTER TABLE \"%s\" ADD COLUMN \"%s\" %s",
-                      poFeatureDefn->GetName(), oField.GetNameRef(), szFieldType );
+    osCommand.Printf( "ALTER TABLE %s ADD COLUMN \"%s\" %s",
+                      pszSqlTableName, oField.GetNameRef(), szFieldType );
     hResult = PQexec(hPGConn, osCommand);
     if( PQresultStatus(hResult) != PGRES_COMMAND_OK )
     {
@@ -1330,17 +1361,18 @@ OGRErr OGRPGTableLayer::CreateField( OGRFieldDefn *poFieldIn, int bApproxOK )
                   osCommand.c_str(), 
                   PQerrorMessage(hPGConn) );
 
-        PQclear( hResult );
+        OGRPGClearResult( hResult );
+
         hResult = PQexec( hPGConn, "ROLLBACK" );
-        PQclear( hResult );
+        OGRPGClearResult( hResult );
 
         return OGRERR_FAILURE;
     }
 
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     hResult = PQexec(hPGConn, "COMMIT");
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     poFeatureDefn->AddFieldDefn( &oField );
 
@@ -1366,7 +1398,7 @@ OGRFeature *OGRPGTableLayer::GetFeature( long nFeatureId )
 /*      Issue query for a single record.                                */
 /* -------------------------------------------------------------------- */
     OGRFeature  *poFeature = NULL;
-    PGresult    *hResult;
+    PGresult    *hResult = NULL;
     PGconn      *hPGConn = poDS->GetPGConn();
     char        *pszFieldList = BuildFields();
     char        *pszCommand = (char *) CPLMalloc(strlen(pszFieldList)+2000);
@@ -1376,8 +1408,8 @@ OGRFeature *OGRPGTableLayer::GetFeature( long nFeatureId )
 
     sprintf( pszCommand,
              "DECLARE getfeaturecursor CURSOR for "
-             "SELECT %s FROM \"%s\" WHERE %s = %ld",
-             pszFieldList, poFeatureDefn->GetName(), pszFIDColumn,
+             "SELECT %s FROM %s WHERE %s = %ld",
+             pszFieldList, pszSqlTableName, pszFIDColumn,
              nFeatureId );
     CPLFree( pszFieldList );
 
@@ -1386,7 +1418,8 @@ OGRFeature *OGRPGTableLayer::GetFeature( long nFeatureId )
 
     if( hResult && PQresultStatus(hResult) == PGRES_COMMAND_OK )
     {
-        PQclear( hResult );
+        OGRPGClearResult( hResult );
+
         hResult = PQexec(hPGConn, "FETCH ALL in getfeaturecursor" );
 
         if( hResult && PQresultStatus(hResult) == PGRES_TUPLES_OK )
@@ -1400,14 +1433,13 @@ OGRFeature *OGRPGTableLayer::GetFeature( long nFeatureId )
 /* -------------------------------------------------------------------- */
 /*      Cleanup                                                         */
 /* -------------------------------------------------------------------- */
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     hResult = PQexec(hPGConn, "CLOSE getfeaturecursor");
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     poDS->FlushSoftTransaction();
 
-
     return poFeature;
 }
 
@@ -1437,38 +1469,38 @@ int OGRPGTableLayer::GetFeatureCount( int bForce )
 /*      application when working against a database.                    */
 /* -------------------------------------------------------------------- */
     PGconn              *hPGConn = poDS->GetPGConn();
-    PGresult            *hResult;
+    PGresult            *hResult = NULL;
     CPLString           osCommand;
     int                 nCount = 0;
 
     poDS->FlushSoftTransaction();
     hResult = PQexec(hPGConn, "BEGIN");
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     osCommand.Printf(
         "DECLARE countCursor CURSOR for "
-        "SELECT count(*) FROM \"%s\" "
+        "SELECT count(*) FROM %s "
         "%s",
-        poFeatureDefn->GetName(), osWHERE.c_str() );
+        pszSqlTableName, osWHERE.c_str() );
 
     CPLDebug( "OGR_PG", "PQexec(%s)\n",
               osCommand.c_str() );
 
     hResult = PQexec(hPGConn, osCommand);
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     hResult = PQexec(hPGConn, "FETCH ALL in countCursor");
     if( hResult != NULL && PQresultStatus(hResult) == PGRES_TUPLES_OK )
         nCount = atoi(PQgetvalue(hResult,0,0));
     else
         CPLDebug( "OGR_PG", "%s; failed.", osCommand.c_str() );
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     hResult = PQexec(hPGConn, "CLOSE countCursor");
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     hResult = PQexec(hPGConn, "COMMIT");
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
 
     return nCount;
 }
@@ -1486,9 +1518,9 @@ OGRSpatialReference *OGRPGTableLayer::GetSpatialRef()
 {
     if( nSRSId == -2 )
     {
-        PGconn          *hPGConn = poDS->GetPGConn();
-        PGresult        *hResult;
-        char            szCommand[1024];
+        PGconn      *hPGConn = poDS->GetPGConn();
+        PGresult    *hResult = NULL;
+        char        szCommand[1024];
 
         nSRSId = -1;
 
@@ -1496,8 +1528,8 @@ OGRSpatialReference *OGRPGTableLayer::GetSpatialRef()
 
         sprintf( szCommand,
                  "SELECT srid FROM geometry_columns "
-                 "WHERE f_table_name = '%s'",
-                 poFeatureDefn->GetName() );
+                 "WHERE f_table_name = '%s' AND f_table_schema = '%s'",
+                 pszTableName, pszSchemaName );
         hResult = PQexec(hPGConn, szCommand );
 
         if( hResult
@@ -1506,7 +1538,27 @@ OGRSpatialReference *OGRPGTableLayer::GetSpatialRef()
         {
             nSRSId = atoi(PQgetvalue(hResult,0,0));
         }
-        PQclear( hResult );
+        else // I think perhaps an older version used f_schema_name.
+        {
+            OGRPGClearResult( hResult );
+
+            poDS->SoftCommit();
+            poDS->SoftStartTransaction();
+
+            sprintf( szCommand,
+                     "SELECT srid FROM geometry_columns "
+                     "WHERE f_table_name = '%s' AND f_schema_name = '%s'",
+                     pszTableName, pszSchemaName );
+            hResult = PQexec(hPGConn, szCommand );
+            if( hResult
+                && PQresultStatus(hResult) == PGRES_TUPLES_OK
+                && PQntuples(hResult) == 1 )
+            {
+                nSRSId = atoi(PQgetvalue(hResult,0,0));
+            }
+        }
+
+        OGRPGClearResult( hResult );
 
         poDS->SoftCommit();
     }
@@ -1528,17 +1580,17 @@ OGRErr OGRPGTableLayer::GetExtent( OGREnvelope *psExtent, int bForce )
 
     if ( bHasPostGISGeometry )
     {
-        PGconn          *hPGConn = poDS->GetPGConn();
-        PGresult        *hResult;
-        CPLString       osCommand;
+        PGconn      *hPGConn = poDS->GetPGConn();
+        PGresult    *hResult = NULL;
+        CPLString   osCommand;
 
-        osCommand.Printf( "SELECT Extent(\"%s\") FROM \"%s\"", 
-                          pszGeomColumn, poFeatureDefn->GetName() );
+        osCommand.Printf( "SELECT Extent(\"%s\") FROM %s", 
+                          pszGeomColumn, pszSqlTableName );
 
         hResult = PQexec( hPGConn, osCommand );
         if( ! hResult || PQresultStatus(hResult) != PGRES_TUPLES_OK || PQgetisnull(hResult,0,0) )
         {
-            PQclear( hResult );
+            OGRPGClearResult( hResult );
             CPLDebug("OGR_PG","Unable to get extent by PostGIS. Using standard OGRLayer method.");
             return OGRPGLayer::GetExtent( psExtent, bForce );
         }
@@ -1560,7 +1612,8 @@ OGRErr OGRPGTableLayer::GetExtent( OGREnvelope *psExtent, int bForce )
             CPLError( CE_Failure, CPLE_IllegalArg,
                       "Bad extent representation: '%s'", pszBox);
             CSLDestroy(papszTokens);
-            PQclear(hResult);
+
+            OGRPGClearResult( hResult );
             return OGRERR_FAILURE;
         }
 
@@ -1576,8 +1629,8 @@ OGRErr OGRPGTableLayer::GetExtent( OGREnvelope *psExtent, int bForce )
         psExtent->MaxY = CPLScanDouble(papszTokens[nTokenCnt/2+1],strlen(papszTokens[nTokenCnt/2+1]),"C");
 
         CSLDestroy(papszTokens);
+        OGRPGClearResult( hResult );
 
-        PQclear( hResult );
         return OGRERR_NONE;
     }
 
@@ -1598,12 +1651,12 @@ OGRErr OGRPGTableLayer::StartCopy()
 
     char *pszFields = BuildCopyFields();
 
-    int size = strlen(pszFields) +  strlen(poFeatureDefn->GetName()) + 100;
+    int size = strlen(pszFields) +  strlen(pszSqlTableName) + 100;
     char *pszCommand = (char *) CPLMalloc(size);
 
     sprintf( pszCommand,
-             "COPY \"%s\" (%s) FROM STDIN;",
-             poFeatureDefn->GetName(), pszFields );
+             "COPY %s (%s) FROM STDIN;",
+             pszSqlTableName, pszFields );
 
     CPLFree( pszFields );
 
@@ -1620,7 +1673,7 @@ OGRErr OGRPGTableLayer::StartCopy()
     else
         bCopyActive = TRUE;
 
-    PQclear( hResult );
+    OGRPGClearResult( hResult );
     CPLFree( pszCommand );
 
     return OGRERR_NONE;
@@ -1645,7 +1698,6 @@ OGRErr OGRPGTableLayer::EndCopy()
 
     bCopyActive = FALSE;
 
-
     /* This is for postgresql 7.4 and higher */
 #if !defined(PG_PRE74)
     int copyResult = PQputCopyEnd(hPGConn, NULL);
@@ -1685,7 +1737,7 @@ OGRErr OGRPGTableLayer::EndCopy()
         result = OGRERR_FAILURE;
     }
 
-    PQclear(hResult);
+    OGRPGClearResult( hResult );
 
     bUseCopy = USE_COPY_UNSET;
 
@@ -1698,8 +1750,9 @@ OGRErr OGRPGTableLayer::EndCopy()
 
 char *OGRPGTableLayer::BuildCopyFields()
 {
-    int         i, nSize;
-    char        *pszFieldList;
+    int     i = 0;
+    int     nSize = 0;
+    char    *pszFieldList;
         
     nSize = 25;
     if( pszGeomColumn )
