@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrs57datasource.cpp,v 1.29 2006/04/02 18:48:26 fwarmerdam Exp $
+ * $Id: ogrs57datasource.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  S-57 Translator
  * Purpose:  Implements OGRS57DataSource class
@@ -25,103 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrs57datasource.cpp,v $
- * Revision 1.29  2006/04/02 18:48:26  fwarmerdam
- * contact info updated
- *
- * Revision 1.28  2006/02/15 18:04:45  fwarmerdam
- * implemented DSID feature support
- *
- * Revision 1.27  2005/09/21 00:54:43  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.26  2004/08/30 20:11:51  warmerda
- * keep the S57ClassRegistrar on the driver, not the datasource
- *
- * Revision 1.25  2003/11/15 21:50:52  warmerda
- * Added limited creation support
- *
- * Revision 1.24  2003/11/12 21:24:12  warmerda
- * updates to new featuredefn generators
- *
- * Revision 1.23  2003/09/15 20:52:12  warmerda
- * added RETURN_LINKAGES
- *
- * Revision 1.22  2003/09/12 21:06:31  warmerda
- * use SetWellKnownGeogCS() to establish WGS84 SRS
- *
- * Revision 1.21  2003/09/05 19:12:05  warmerda
- * added RETURN_PRIMITIVES support to get low level prims
- *
- * Revision 1.20  2002/09/09 18:38:58  warmerda
- * Initialize module list.
- *
- * Revision 1.19  2002/09/09 18:20:58  warmerda
- * fixed serious leak ... S57Readers never destroyed!
- *
- * Revision 1.18  2002/05/14 21:33:30  warmerda
- * use macros for options, pass PRESERVE_EMPTY_NUMBERS opt
- *
- * Revision 1.17  2002/03/05 14:25:43  warmerda
- * expanded tabs
- *
- * Revision 1.16  2001/12/19 22:44:53  warmerda
- * added ADD_SOUNDG_DEPTH support
- *
- * Revision 1.15  2001/12/19 22:03:58  warmerda
- * iniitialize bExtentsSet
- *
- * Revision 1.14  2001/12/17 22:34:43  warmerda
- * ensure SPLIT_MULTIPOINT option is passed to S57Reader
- *
- * Revision 1.13  2001/12/14 19:40:18  warmerda
- * added optimized feature counting, and extents collection
- *
- * Revision 1.12  2001/11/21 14:35:44  warmerda
- * dont managed updates directly anymore
- *
- * Revision 1.11  2001/08/30 21:05:32  warmerda
- * added support for generic object if not recognised
- *
- * Revision 1.10  2001/08/30 03:48:43  warmerda
- * preliminary implementation of S57 Update Support
- *
- * Revision 1.9  2001/07/18 04:55:16  warmerda
- * added CPL_CSVID
- *
- * Revision 1.8  2000/06/16 18:10:05  warmerda
- * expanded tabs
- *
- * Revision 1.7  2000/06/07 20:50:58  warmerda
- * make CSV location configurable with env variable
- *
- * Revision 1.6  1999/11/29 14:04:53  warmerda
- * use LNAM_REFS
- *
- * Revision 1.5  1999/11/26 16:18:33  warmerda
- * OGRFeatureDefn generators no longer static
- *
- * Revision 1.4  1999/11/18 19:01:25  warmerda
- * expanded tabs
- *
- * Revision 1.3  1999/11/16 21:47:31  warmerda
- * updated class occurance collection
- *
- * Revision 1.2  1999/11/08 22:23:00  warmerda
- * added object class support
- *
- * Revision 1.1  1999/11/03 22:12:43  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogr_s57.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrs57datasource.cpp,v 1.29 2006/04/02 18:48:26 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrs57datasource.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                          OGRS57DataSource()                          */
@@ -240,21 +150,21 @@ int OGRS57DataSource::Open( const char * pszFilename, int bTestOpen )
         FILE    *fp;
         char    pachLeader[10];
 
-        fp = VSIFOpen( pszFilename, "rb" );
+        fp = VSIFOpenL( pszFilename, "rb" );
         if( fp == NULL )
             return FALSE;
         
-        if( VSIFRead( pachLeader, 1, 10, fp ) != 10
+        if( VSIFReadL( pachLeader, 1, 10, fp ) != 10
             || (pachLeader[5] != '1' && pachLeader[5] != '2'
                 && pachLeader[5] != '3' )
             || pachLeader[6] != 'L'
             || (pachLeader[8] != '1' && pachLeader[8] != ' ') )
         {
-            VSIFClose( fp );
+            VSIFCloseL( fp );
             return FALSE;
         }
 
-        VSIFClose( fp );
+        VSIFCloseL( fp );
     }
 
 /* -------------------------------------------------------------------- */
