@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsdelayer.cpp,v 1.11 2006/04/11 02:31:32 fwarmerdam Exp $
+ * $Id: ogrsdelayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRSDELayer class.
@@ -25,49 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrsdelayer.cpp,v $
- * Revision 1.11  2006/04/11 02:31:32  fwarmerdam
- * added multilinestring
- *
- * Revision 1.10  2006/04/11 01:37:53  fwarmerdam
- * fixed up typos
- *
- * Revision 1.9  2006/04/11 01:26:10  fwarmerdam
- * First draft multipoint support, fixing multipolygons.
- *
- * Revision 1.8  2006/03/09 22:50:43  fwarmerdam
- * Added  support for multipolygons
- *
- * Revision 1.7  2006/03/08 00:22:46  fwarmerdam
- * preliminary update to support rowid-less tables
- *
- * Revision 1.6  2005/11/30 01:53:31  fwarmerdam
- * added OGR_SDE_SEARCHORDER config option
- *
- * Revision 1.5  2005/11/30 01:34:02  fwarmerdam
- * Ensure layerinfo is loaded for GetSpatialRef().
- *
- * Revision 1.4  2005/11/25 18:55:15  fwarmerdam
- * layer implementation essentially complete
- *
- * Revision 1.3  2005/11/25 05:58:27  fwarmerdam
- * preliminary operation of feature reading
- *
- * Revision 1.2  2005/11/22 17:09:17  fwarmerdam
- * Preinitalize poFeature in GetNextFeature().
- *
- * Revision 1.1  2005/11/22 17:01:48  fwarmerdam
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogr_sde.h"
 #include "cpl_conv.h"
 #include "cpl_string.h"
 
-CPL_CVSID("$Id: ogrsdelayer.cpp,v 1.11 2006/04/11 02:31:32 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrsdelayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                            OGRSDELayer()                             */
@@ -153,6 +117,19 @@ int OGRSDELayer::Initialize( const char *pszTableName,
     poFeatureDefn = new OGRFeatureDefn( pszTableName );
     poFeatureDefn->Reference();
 
+/* -------------------------------------------------------------------- */
+/*  If the OGR_SDE_GETLAYERTYPE option is set to TRUE, then the layer   */
+/*  is tested to see if it contains one, and only one, type of geometry */
+/*  then that geometry type is placed into the LayerDefn.               */
+/* -------------------------------------------------------------------- */
+    const char *pszLayerType = CPLGetConfigOption( "OGR_SDE_GETLAYERTYPE", 
+                                                   "FALSE" );
+
+    if( CSLTestBoolean(pszLayerType) != FALSE )
+    {
+        poFeatureDefn->SetGeomType(DiscoverLayerType());
+    }
+
     for( iCol = 0; iCol < nColumnCount; iCol++ )
     {
         OGRFieldType eOGRType = OFTIntegerList; // dummy
@@ -301,6 +278,151 @@ int OGRSDELayer::NeedLayerInfo()
     return TRUE;
 }
 
+/************************************************************************/
+/*                          DiscoverLayerType()                         */
+/************************************************************************/
+
+OGRwkbGeometryType OGRSDELayer::DiscoverLayerType()
+{
+    if( !NeedLayerInfo() )
+        return wkbUnknown;
+
+    int nSDEErr;
+    long nShapeTypeMask = 0;
+  
+/* -------------------------------------------------------------------- */
+/*      Check layerinfo flags to establish what geometry types may      */
+/*      occur.                                                          */
+/* -------------------------------------------------------------------- */
+    nSDEErr = SE_layerinfo_get_shape_types( hLayerInfo, &nShapeTypeMask );
+    if (nSDEErr != SE_SUCCESS)
+    {
+        CPLDebug( "OGR_SDE",
+                  "Unable to read the layer type information, defaulting to wkbUnknown:  error=%d.",
+                  nSDEErr );
+
+        return wkbUnknown;
+    }
+
+    int bIsMultipart = ( nShapeTypeMask & SE_MULTIPART_TYPE_MASK ? 1 : 0);
+    nShapeTypeMask &= ~SE_MULTIPART_TYPE_MASK;
+
+    // Since we assume that all layers can bear a NULL geometry, 
+    // throw the flag away.
+    nShapeTypeMask &= ~SE_NIL_TYPE_MASK;
+
+    int nTypeCount = 0;
+    if ( nShapeTypeMask & SE_POINT_TYPE_MASK )
+        nTypeCount++;
+    if ( nShapeTypeMask & SE_LINE_TYPE_MASK 
+         || nShapeTypeMask & SE_SIMPLE_LINE_TYPE_MASK )
+        nTypeCount++;
+
+    if ( nShapeTypeMask & SE_AREA_TYPE_MASK )
+        nTypeCount++;
+
+/* -------------------------------------------------------------------- */
+/*      When the flags indicate multiple geometry types are             */
+/*      possible, we examine the layer statistics to see if in          */
+/*      reality only one geometry type does occur.  This is somewhat    */
+/*      expensive, so we avoid it if we can.                            */
+/* -------------------------------------------------------------------- */
+    if ( nTypeCount == 0 )
+    {
+        CPLDebug( "OGR_SDE", "There is no layer type indicated for the current layer." );
+        return wkbUnknown;
+    }
+    else if ( nTypeCount > 1 )
+    {
+        CPLDebug( "OGR_SDE", "More than one layer type is indicated for this layer, gathering layer statistics are being gathered." );
+        SE_LAYER_STATS layerstats = {0};
+        char szTableName[SE_QUALIFIED_TABLE_NAME];
+        char szShapeColumn[SE_MAX_COLUMN_LEN];
+
+        szTableName[0] = '\0';
+        szShapeColumn[0] = '\0';
+
+        nSDEErr = SE_layerinfo_get_spatial_column( hLayerInfo, szTableName, 
+                                                   szShapeColumn );
+        if ( nSDEErr != SE_SUCCESS )
+        {
+            poDS->IssueSDEError( nSDEErr, "SE_layerinfo_get_spatial_column" );
+            return wkbUnknown;
+        }
+
+        nSDEErr = SE_layer_get_statistics( poDS->GetConnection(), szTableName, 
+                                           szShapeColumn,
+                                           &layerstats);
+        if( nSDEErr != SE_SUCCESS )
+        {
+            poDS->IssueSDEError( nSDEErr, "SE_layer_get_statistics" );
+            return wkbUnknown;
+        }
+
+        if ( nShapeTypeMask & SE_POINT_TYPE_MASK && 
+             ( layerstats.POINTs + layerstats.MultiPOINTs ) == 0 )
+            nShapeTypeMask &= ~SE_POINT_TYPE_MASK;
+
+        if ( nShapeTypeMask & SE_LINE_TYPE_MASK &&
+             ( layerstats.LINEs + layerstats.MultiLINEs ) == 0 )
+            nShapeTypeMask &= ~SE_LINE_TYPE_MASK;
+
+        if ( nShapeTypeMask & SE_SIMPLE_LINE_TYPE_MASK &&
+             ( layerstats.SIMPLE_LINEs + layerstats.MultiSIMPLE_LINEs ) == 0 )
+            nShapeTypeMask &= ~SE_SIMPLE_LINE_TYPE_MASK;
+
+        if ( nShapeTypeMask & SE_AREA_TYPE_MASK &&
+             ( layerstats.AREAs + layerstats.MultiAREAs ) == 0 )
+            nShapeTypeMask &= ~SE_AREA_TYPE_MASK;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Select a geometry type based on the remaining flags.  If        */
+/*      there is a mix we will fall through to the default (wkbUknown). */
+/* -------------------------------------------------------------------- */
+    OGRwkbGeometryType eGeoType;
+    char *pszTypeName;
+    switch (nShapeTypeMask)
+    {
+      case SE_POINT_TYPE_MASK:
+        if (bIsMultipart)
+            eGeoType = wkbMultiPoint;
+        else
+            eGeoType = wkbPoint;
+        pszTypeName = "point";
+        break;
+
+      case (SE_SIMPLE_LINE_TYPE_MASK | SE_LINE_TYPE_MASK):
+      case SE_SIMPLE_LINE_TYPE_MASK:
+      case SE_LINE_TYPE_MASK:
+        if (bIsMultipart)
+            eGeoType = wkbMultiLineString;
+        else
+            eGeoType = wkbLineString;
+        pszTypeName = "line";
+        break;
+      
+      case SE_AREA_TYPE_MASK:
+        if (bIsMultipart)
+            eGeoType = wkbMultiPolygon;
+        else
+            eGeoType = wkbPolygon;
+        pszTypeName = "polygon";
+        break;
+      
+      default:
+        eGeoType = wkbUnknown;
+        pszTypeName = "unknown";
+        break;
+    }
+
+    CPLDebug( "OGR_SDE", 
+              "DiscoverLayerType is returning type=%d (%s), multipart=%d.",
+              eGeoType, pszTypeName, bIsMultipart );
+
+    return eGeoType;
+}
+
 /************************************************************************/
 /*                            ResetReading()                            */
 /************************************************************************/
@@ -644,7 +766,7 @@ OGRGeometry *OGRSDELayer::TranslateSDEGeometry( SE_SHAPE hShape )
               else
                   nLineVertCount = panSubParts[iPart+1] - panSubParts[iPart];
 
-              poLine->setNumPoints( nPointCount );
+              poLine->setNumPoints( nLineVertCount );
               
               for( i = 0; i < nLineVertCount; i++ )
               {
@@ -1054,6 +1176,45 @@ OGRFeature *OGRSDELayer::GetFeature( long nFeatureId )
 int OGRSDELayer::GetFeatureCount( int bForce )
 
 {
+/* -------------------------------------------------------------------- */
+/*      If there is no attribute or spatial filter in place, then       */
+/*      use the SDE function call to obtain the number of               */
+/*      features. The performance difference between this and manual    */
+/*      iteration is significant.                                       */
+/* -------------------------------------------------------------------- */
+    if( osAttributeFilter.empty() && m_poFilterGeom == NULL 
+        && NeedLayerInfo() )
+    {
+        SE_LAYER_STATS layerstats = {0};
+        char szTableName[SE_QUALIFIED_TABLE_NAME];
+        char szShapeColumn[SE_MAX_COLUMN_LEN];
+        int nSDEErr;
+
+        szTableName[0] = '\0';
+        szShapeColumn[0] = '\0';
+      
+        nSDEErr = SE_layerinfo_get_spatial_column( hLayerInfo, szTableName, szShapeColumn );
+        if( nSDEErr != SE_SUCCESS )
+        {
+            poDS->IssueSDEError( nSDEErr, "SE_layerinfo_get_spatial_column" );
+            return -1;
+        }
+      
+        nSDEErr = SE_layer_get_statistics( poDS->GetConnection(), szTableName, szShapeColumn,
+                                           &layerstats);
+        if( nSDEErr != SE_SUCCESS )
+        {
+            poDS->IssueSDEError( nSDEErr, "SE_layer_get_statistics" );
+            return -1;
+        }
+
+        return layerstats.TotalFeatures;
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Otherwise use direct reading of the result set, though we       */
+/*      skip translating into OGRFeatures at least.                     */
+/* -------------------------------------------------------------------- */
     int nSDEErr, nFeatureCount = 0;
 
     ResetReading();
@@ -1116,7 +1277,9 @@ int OGRSDELayer::TestCapability( const char * pszCap )
     if( EQUAL(pszCap,OLCRandomRead) )
         return iFIDColumn != -1;
 
-    else if( EQUAL(pszCap,OLCFastFeatureCount) )
+    else if( EQUAL(pszCap,OLCFastFeatureCount) 
+             && osAttributeFilter.empty()
+             && m_poFilterGeom == NULL )
         return TRUE;
 
     else if( EQUAL(pszCap,OLCFastSpatialFilter) )
