@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrshapelayer.cpp,v 1.28 2006/04/10 17:00:35 fwarmerdam Exp $
+ * $Id: ogrshapelayer.cpp 10682 2007-01-23 21:53:38Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRShapeLayer class.
@@ -25,96 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrshapelayer.cpp,v $
- * Revision 1.28  2006/04/10 17:00:35  fwarmerdam
- * Report ability to delete features in TestCapability.
- *
- * Revision 1.27  2006/04/02 18:46:54  fwarmerdam
- * updated date support
- *
- * Revision 1.26  2006/02/19 21:42:05  mloskot
- * [WCE] Include wce_errno.h - a specific errno.h version for Windows CE
- *
- * Revision 1.25  2006/02/15 04:26:55  fwarmerdam
- * added date support
- *
- * Revision 1.24  2006/01/10 16:40:19  fwarmerdam
- * Free panRecordsToDelete.
- *
- * Revision 1.23  2006/01/10 16:37:57  fwarmerdam
- * implemented REPACK support
- *
- * Revision 1.22  2006/01/05 02:15:39  fwarmerdam
- * implement DeleteFeature support
- *
- * Revision 1.21  2005/09/21 00:53:20  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.20  2005/09/10 22:32:34  fwarmerdam
- * Make GetNextShape() more bulletproof if SHPReadOGRFeature() fails.
- *
- * Revision 1.19  2005/02/22 12:51:56  fwarmerdam
- * use OGRLayer base spatial filter support
- *
- * Revision 1.18  2005/02/02 20:01:02  fwarmerdam
- * added SetNextByIndex support
- *
- * Revision 1.17  2005/01/04 03:43:41  fwarmerdam
- * added support for creating and destroying spatial indexes
- *
- * Revision 1.16  2005/01/03 22:26:21  fwarmerdam
- * updated to use spatial indexing
- *
- * Revision 1.15  2003/05/21 04:03:54  warmerda
- * expand tabs
- *
- * Revision 1.14  2003/04/21 19:03:20  warmerda
- * added SyncToDisk support
- *
- * Revision 1.13  2003/04/15 20:45:54  warmerda
- * Added code to update the feature count in CreateFeature().
- *
- * Revision 1.12  2003/03/05 05:09:41  warmerda
- * initialize panMatchingFIDs
- *
- * Revision 1.11  2003/03/04 05:49:05  warmerda
- * added attribute indexing support
- *
- * Revision 1.10  2002/10/09 14:13:40  warmerda
- * ensure width is at least 1
- *
- * Revision 1.9  2002/04/05 19:59:37  warmerda
- * Output file was getting different geometries in some cases.
- *
- * Revision 1.8  2002/03/27 21:04:38  warmerda
- * Added support for reading, and creating lone .dbf files for wkbNone geometry
- * layers.  Added support for creating a single .shp file instead of a directory
- * if a path ending in .shp is passed to the data source create method.
- *
- * Revision 1.7  2001/09/04 15:35:14  warmerda
- * add support for deferring geometry type selection till first feature
- *
- * Revision 1.6  2001/08/21 21:44:27  warmerda
- * fixed count logic if attribute query in play
- *
- * Revision 1.5  2001/07/18 04:55:16  warmerda
- * added CPL_CSVID
- *
- * Revision 1.4  2001/06/19 15:50:23  warmerda
- * added feature attribute query support
- *
- * Revision 1.3  2001/03/16 22:16:10  warmerda
- * added support for ESRI .prj files
- *
- * Revision 1.2  2001/03/15 04:21:50  danmo
- * Added GetExtent()
- *
- * Revision 1.1  1999/11/04 21:16:11  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogrshape.h"
 #include "cpl_conv.h"
@@ -124,7 +35,7 @@
 #  include <wce_errno.h>
 #endif
 
-CPL_CVSID("$Id: ogrshapelayer.cpp,v 1.28 2006/04/10 17:00:35 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrshapelayer.cpp 10682 2007-01-23 21:53:38Z warmerdam $");
 
 /************************************************************************/
 /*                           OGRShapeLayer()                            */
@@ -358,7 +269,7 @@ OGRErr OGRShapeLayer::SetNextByIndex( long nIndex )
 OGRFeature *OGRShapeLayer::GetNextFeature()
 
 {
-    OGRFeature  *poFeature;
+    OGRFeature  *poFeature = NULL;
 
 /* -------------------------------------------------------------------- */
 /*      Collect a matching list if we have attribute or spatial         */
@@ -379,7 +290,9 @@ OGRFeature *OGRShapeLayer::GetNextFeature()
         if( panMatchingFIDs != NULL )
         {
             if( panMatchingFIDs[iMatchingFID] == OGRNullFID )
+            {
                 return NULL;
+            }
 
             poFeature = SHPReadOGRFeature( hSHP, hDBF, poFeatureDefn, 
                                            panMatchingFIDs[iMatchingFID++] );
@@ -387,9 +300,11 @@ OGRFeature *OGRShapeLayer::GetNextFeature()
         else
         {
             if( iNextShapeId >= nTotalShapeCount )
+            {
                 return NULL;
+            }
     
-            if( DBFIsRecordDeleted( hDBF, iNextShapeId ) )
+            if( hDBF && DBFIsRecordDeleted( hDBF, iNextShapeId ) )
                 poFeature = NULL;
             else
                 poFeature = SHPReadOGRFeature( hSHP, hDBF, poFeatureDefn,
@@ -400,17 +315,27 @@ OGRFeature *OGRShapeLayer::GetNextFeature()
 
         if( poFeature != NULL )
         {
+            if( poFeature->GetGeometryRef() != NULL )
+            {
+                poFeature->GetGeometryRef()->assignSpatialReference( poSRS );
+            }
+
             m_nFeaturesRead++;
 
-            if( (m_poFilterGeom == NULL
-                 || FilterGeometry( poFeature->GetGeometryRef() ) )
-                && (m_poAttrQuery == NULL
-                    || m_poAttrQuery->Evaluate( poFeature )) )
+            if( (m_poFilterGeom == NULL || FilterGeometry( poFeature->GetGeometryRef() ) )
+                && (m_poAttrQuery == NULL || m_poAttrQuery->Evaluate( poFeature )) )
+            {
                 return poFeature;
+            }
 
             delete poFeature;
         }
     }        
+
+    /*
+     * NEVER SHOULD GET HERE
+     */
+    CPLAssert(!"OGRShapeLayer::GetNextFeature(): Execution never should get here!");
 }
 
 /************************************************************************/
@@ -420,13 +345,23 @@ OGRFeature *OGRShapeLayer::GetNextFeature()
 OGRFeature *OGRShapeLayer::GetFeature( long nFeatureId )
 
 {
-    OGRFeature *poFeature = 
-         SHPReadOGRFeature( hSHP, hDBF, poFeatureDefn, nFeatureId );
-    
-    if( poFeature != NULL )
-        m_nFeaturesRead++;
+    OGRFeature *poFeature = NULL;
+    poFeature = SHPReadOGRFeature( hSHP, hDBF, poFeatureDefn, nFeatureId );
 
-    return poFeature;
+    if( poFeature != NULL )
+    {
+        if( poFeature->GetGeometryRef() != NULL )
+            poFeature->GetGeometryRef()->assignSpatialReference( poSRS );
+
+        m_nFeaturesRead++;
+    
+        return poFeature;
+    }
+
+    /*
+     * Reading shape feature failed.
+     */
+    return NULL;
 }
 
 /************************************************************************/
@@ -436,6 +371,8 @@ OGRFeature *OGRShapeLayer::GetFeature( long nFeatureId )
 OGRErr OGRShapeLayer::SetFeature( OGRFeature *poFeature )
 
 {
+    bHeaderDirty = TRUE;
+
     return SHPWriteOGRFeature( hSHP, hDBF, poFeatureDefn, poFeature );
 }
 
@@ -456,6 +393,15 @@ OGRErr OGRShapeLayer::DeleteFeature( long nFID )
         return OGRERR_FAILURE;
     }
 
+    if( !hDBF )
+    {
+        CPLError( CE_Failure, CPLE_AppDefined, 
+                  "Attempt to delete shape in shapefile with no .dbf file.\n"
+                  "Deletion is done by marking record deleted in dbf\n"
+                  "and is not supported without a .dbf file." );
+        return OGRERR_FAILURE;
+    }
+
     if( DBFIsRecordDeleted( hDBF, nFID ) )
     {
         CPLError( CE_Failure, CPLE_AppDefined, 
@@ -467,6 +413,8 @@ OGRErr OGRShapeLayer::DeleteFeature( long nFID )
     if( !DBFMarkRecordDeleted( hDBF, nFID, TRUE ) )
         return OGRERR_FAILURE;
 
+    bHeaderDirty = TRUE;
+
     return OGRERR_NONE;
 }
 
@@ -644,6 +592,8 @@ int OGRShapeLayer::TestCapability( const char * pszCap )
 OGRErr OGRShapeLayer::CreateField( OGRFieldDefn *poField, int bApproxOK )
 
 {
+    CPLAssert( NULL != poField );
+
     int         iNewField;
     if( GetFeatureCount(TRUE) != 0 )
     {
@@ -657,8 +607,33 @@ OGRErr OGRShapeLayer::CreateField( OGRFieldDefn *poField, int bApproxOK )
         CPLError( CE_Failure, CPLE_NotSupported,
                   "Can't create fields on a read-only shapefile layer.\n");
         return OGRERR_FAILURE;
+
     }
 
+/* -------------------------------------------------------------------- */
+/*      Normalize field name                                            */
+/* -------------------------------------------------------------------- */
+        
+    char * pszNewFieldName = NULL;
+
+    size_t nNameSize = strlen( poField->GetNameRef() );
+    pszNewFieldName = CPLScanString( poField->GetNameRef(),
+                                     nNameSize, TRUE, TRUE);
+
+    if( !EQUAL(poField->GetNameRef(),pszNewFieldName) )
+        CPLDebug( "Shape", "Normalized field name: '%s' to '%s'", 
+                  poField->GetNameRef(),
+                  pszNewFieldName );
+
+    // Set field name with normalized value
+    poField->SetName( pszNewFieldName );
+
+    CPLFree( pszNewFieldName );
+
+/* -------------------------------------------------------------------- */
+/*      Add field to layer                                              */
+/* -------------------------------------------------------------------- */
+
     if( poField->GetType() == OFTInteger )
     {
         if( poField->GetWidth() == 0 )
@@ -726,7 +701,9 @@ OGRErr OGRShapeLayer::CreateField( OGRFieldDefn *poField, int bApproxOK )
     }
 
     if( iNewField != -1 )
+    {
         return OGRERR_NONE;
+    }
     else        
     {
         CPLError( CE_Failure, CPLE_AppDefined,
