@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: shape2ogr.cpp,v 1.42 2006/04/02 18:46:54 fwarmerdam Exp $
+ * $Id: shape2ogr.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements translation of Shapefile shapes into OGR
@@ -26,155 +26,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: shape2ogr.cpp,v $
- * Revision 1.42  2006/04/02 18:46:54  fwarmerdam
- * updated date support
- *
- * Revision 1.41  2006/02/15 04:26:55  fwarmerdam
- * added date support
- *
- * Revision 1.40  2006/01/05 02:15:39  fwarmerdam
- * implement DeleteFeature support
- *
- * Revision 1.39  2005/10/12 14:50:28  fwarmerdam
- * recover gracefully from POLYGON EMPTY on write
- *
- * Revision 1.38  2005/09/30 23:53:35  mbrudka
- * Fixed bug 940: http://bugzilla.remotesensing.org/show_bug.cgi?id=940
- *
- * Revision 1.37  2005/09/21 00:53:20  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.36  2005/09/20 21:27:25  mbrudka
- * Some optimizations in reading multi-ring polygons.
- *
- * Revision 1.35  2005/08/04 15:31:09  fwarmerdam
- * Added additional patch for computing ring direction in cases of
- * vertices that are very near or coincident.
- *
- * Revision 1.34  2005/08/04 15:10:46  fwarmerdam
- * Committed fix to avoid infinite loop on degenerate shapefiles.  Patch
- * provided by Baumann Konstantin.  See Var2_STRASSE.shp.
- *
- * Revision 1.33  2005/07/20 01:44:25  fwarmerdam
- * try and preserve geometry dimension properly
- *
- * Revision 1.32  2004/07/06 19:35:51  warmerda
- * Default to using FTString for unrecognised field types like logical.
- *
- * Revision 1.31  2004/01/16 22:40:40  warmerda
- * any inner rings not assigned to a polygo are promoted to be outer rings
- *
- * Revision 1.30  2003/12/11 19:19:40  warmerda
- * Fixed leak of geometry objects when building multipolygons.
- *
- * Revision 1.29  2003/11/09 18:54:53  warmerda
- * added auto-FID field creation if new file has no fields on first feature wrt
- *
- * Revision 1.28  2003/10/18 19:01:10  warmerda
- * added Radims patch to recognise multipolygons on read properly - bug 213
- *
- * Revision 1.27  2003/06/10 14:44:11  warmerda
- * Added support for writing shapes with NULL geometries.
- *
- * Revision 1.26  2003/05/27 21:39:53  warmerda
- * added support for writing MULTILINESTRINGs as ARCs
- *
- * Revision 1.25  2003/05/21 04:03:54  warmerda
- * expand tabs
- *
- * Revision 1.24  2003/02/25 14:35:11  warmerda
- * Incorporated more correct support for reading multi-part arcs as
- * MultiLineStrings.
- *
- * Revision 1.23  2002/10/04 04:31:40  warmerda
- * Fixes for writing 3D multipoints.
- *
- * Revision 1.22  2002/10/04 04:30:46  warmerda
- * Fixed some bugs in 3D support for writing polygons.
- *
- * Revision 1.21  2002/08/15 15:10:01  warmerda
- * Fixed problem when GetFeature() called for a non-existant id.
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=175
- *
- * Revision 1.20  2002/08/14 23:27:25  warmerda
- * Fixed reading of Z coordinate for SHPT_ARCZ type.
- *
- * Revision 1.19  2002/04/17 21:49:02  warmerda
- * Fixed a bug writing arcs with Z coordinates.
- *
- * Revision 1.18  2002/04/17 20:03:27  warmerda
- * Try to preserve Z values on read.
- *
- * Revision 1.17  2002/04/17 20:01:39  warmerda
- * Ensure Z coordinate preserved on read.
- *
- * Revision 1.16  2002/04/16 16:21:28  warmerda
- * Ensure polygons written with correct winding.
- *
- * Revision 1.15  2002/03/27 21:04:38  warmerda
- * Added support for reading, and creating lone .dbf files for wkbNone geometry
- * layers.  Added support for creating a single .shp file instead of a directory
- * if a path ending in .shp is passed to the data source create method.
- *
- * Revision 1.14  2001/12/06 18:14:22  warmerda
- * handle case where there is no DBF file
- *
- * Revision 1.13  2001/09/27 14:53:31  warmerda
- * added proper 3D support for all element types
- *
- * Revision 1.12  2001/09/04 15:39:07  warmerda
- * tightened up NULL attribute handling
- *
- * Revision 1.11  2001/07/18 04:55:16  warmerda
- * added CPL_CSVID
- *
- * Revision 1.10  2001/02/06 14:30:15  warmerda
- * don't crash on attempt to write geometryless features
- *
- * Revision 1.9  2000/11/29 14:09:36  warmerda
- * Extended shapefile OGR driver to support MULTIPOLYGON objects.
- *
- * Revision 1.8  2000/01/26 22:05:45  warmerda
- * fixed bug with 2d/3d arcs
- *
- * Revision 1.7  1999/12/23 14:51:07  warmerda
- * Improved support in face of 3D geometries.
- *
- * Revision 1.6  1999/11/04 21:18:01  warmerda
- * fix bug with shapeid for new shapes
- *
- * Revision 1.5  1999/07/27 01:52:04  warmerda
- * added support for writing polygon/multipoint/arc files
- *
- * Revision 1.4  1999/07/27 00:52:40  warmerda
- * added write methods
- *
- * Revision 1.3  1999/07/26 13:59:25  warmerda
- * added feature writing api
- *
- * Revision 1.2  1999/07/12 15:39:18  warmerda
- * added setting of shape geometry
- *
- * Revision 1.1  1999/07/05 18:58:07  warmerda
- * New
- *
- * Revision 1.3  1999/06/11 19:22:13  warmerda
- * added OGRFeature and OGRFeatureDefn related translation
- *
- * Revision 1.2  1999/05/31 20:41:49  warmerda
- * added multipoint support
- *
- * Revision 1.1  1999/05/31 19:26:03  warmerda
- * New
- */
+ ****************************************************************************/
 
 #include "ogrshape.h"
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: shape2ogr.cpp,v 1.42 2006/04/02 18:46:54 fwarmerdam Exp $");
+CPL_CVSID("$Id: shape2ogr.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 static const double EPSILON = 1E-5;
 
@@ -258,10 +115,13 @@ inline bool RingExtent::contains( const RingExtent &extent )
 /************************************************************************/
 void RingStartEnd ( SHPObject *psShape, int ring, int *start, int *end )
 {
-    if( psShape->panPartStart == NULL ) {
-	*start = 0;
+    if( psShape->panPartStart == NULL )
+    {
+	    *start = 0;
         *end = psShape->nVertices - 1;
-    } else {
+    }
+    else
+    {
         if( ring == psShape->nParts - 1 )
             *end = psShape->nVertices - 1;
         else
@@ -296,27 +156,37 @@ int RingDirection ( SHPObject *Shape, int ring )
 
     /* Find the lowest rightmost vertex */
     v = start;  
-    for ( i = start + 1; i < end; i++ ) { /* => v < end */
-	if ( y[i] < y[v] || ( y[i] == y[v] && x[i] > x[v] ) ) {
-	    v = i;
-	}
+    for ( i = start + 1; i < end; i++ )
+    {
+        /* => v < end */
+        if ( y[i] < y[v] || ( y[i] == y[v] && x[i] > x[v] ) )
+        {
+            v = i;
+        }
     }
     
     /* Vertices may be duplicate, we have to go to nearest different in each direction */
     /* preceding */
     next = v - 1;
-    while ( 1 ) {
-	if ( next < start ) 
-	    next = end - 1; 
+    while ( 1 )
+    {
+        if ( next < start ) 
+        {
+            next = end - 1; 
+        }
 
         if( !epsilonEqual(x[next], x[v], EPSILON) 
             || !epsilonEqual(y[next], y[v], EPSILON) )
-	    break;
+        {
+            break;
+        }
 
-	if ( next == v ) /* So we cannot get into endless loop */
-	    break;
+        if ( next == v ) /* So we cannot get into endless loop */
+        {
+            break;
+        }
 
-	next--;
+        next--;
     }
 	    
     dx0 = x[next] - x[v];
@@ -325,19 +195,27 @@ int RingDirection ( SHPObject *Shape, int ring )
     
     /* following */
     next = v + 1;
-    while ( 1 ) {
-	if ( next >= end ) 
-	    next = start; 
+    while ( 1 )
+    {
+        if ( next >= end ) 
+        {
+            next = start; 
+        }
 
         if ( !epsilonEqual(x[next], x[v], EPSILON) 
              || !epsilonEqual(y[next], y[v], EPSILON) )
-	    break;
+        {
+            break;
+        }
 
-	if ( next == v ) /* So we cannot get into endless loop */
-	    break;
+        if ( next == v ) /* So we cannot get into endless loop */
+        {
+            break;
+        }
 
-	next++;
+        next++;
     }
+
     dx1 = x[next] - x[v];
     dy1 = y[next] - y[v];
 
@@ -437,15 +315,18 @@ OGRLinearRing * CreateLinearRing ( SHPObject *psShape, int ring )
 /************************************************************************/
 
 OGRGeometry *SHPReadOGRObject( SHPHandle hSHP, int iShape )
-
 {
+    // CPLDebug( "Shape", "SHPReadOGRObject( iShape=%d )\n", iShape );
+
     SHPObject   *psShape;
     OGRGeometry *poOGR = NULL;
 
     psShape = SHPReadObject( hSHP, iShape );
 
     if( psShape == NULL )
+    {
         return NULL;
+    }
 
 /* -------------------------------------------------------------------- */
 /*      Point.                                                          */
@@ -456,8 +337,11 @@ OGRGeometry *SHPReadOGRObject( SHPHandle hSHP, int iShape )
     {
         poOGR = new OGRPoint( psShape->padfX[0], psShape->padfY[0],
                               psShape->padfZ[0] );
+
         if( psShape->nSHPType == SHPT_POINT )
+        {
             poOGR->setCoordinateDimension( 2 );
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -560,91 +444,133 @@ OGRGeometry *SHPReadOGRObject( SHPHandle hSHP, int iShape )
              || psShape->nSHPType == SHPT_POLYGONM
              || psShape->nSHPType == SHPT_POLYGONZ )
     {
-        int     iRing;
+        int iRing;
         
-	if ( psShape->nParts == 1 ) { /* Surely outer ring */
-            OGRPolygon *poOGRPoly;
-	    OGRLinearRing       *poRing;
+        //CPLDebug( "Shape", "Shape type: polygon with nParts=%d \n", psShape->nParts );
 
-	    poOGR = poOGRPoly = new OGRPolygon();
-	    poRing = CreateLinearRing ( psShape, 0 );
-	    poOGRPoly->addRingDirectly( poRing );
-	} else {
-	    int nOuter = 0; /* Number of outer rings */ 
-	    int *direction; /* ring direction (1 CW,outer, -1 CCW, inner) */ 
-	    int *outer;     /* list of outer rings */ 
-            /* outer ring index for inner rings, -1 for outer rings */ 
-	    int *outside;   
+        if ( psShape->nParts == 1 )
+        {
+            /* Surely outer ring */
+            OGRPolygon *poOGRPoly = NULL;
+            OGRLinearRing *poRing = NULL;
 
-	    direction = (int *) CPLMalloc( psShape->nParts * sizeof(int) );
-	    outer = (int *) CPLMalloc( psShape->nParts * sizeof(int) );
-	    outside = (int *) CPLMalloc( psShape->nParts * sizeof(int) );
-	    
-	    /* First distinguish outer from inner rings */
-	    for( iRing = 0; iRing < psShape->nParts; iRing++ ) {
-		direction[iRing] = RingDirection ( psShape, iRing);
-		if ( direction[iRing] == 1 ) {
-		    outer[nOuter] = iRing;
-		    nOuter++;
-		}
-		outside[iRing] = -1;
-	    }
+            poOGR = poOGRPoly = new OGRPolygon();
+            poRing = CreateLinearRing ( psShape, 0 );
+            poOGRPoly->addRingDirectly( poRing );
+        }
+        else
+        {
+            /* Multipart polygon. */
 
-	    if ( nOuter == 1 ) { 
-                /* one outer ring? we do not need to check anything as we assume that shapefile is valid
-                   ie. if there is one outer ring then all inner rings are inside */
-                for( iRing = 0; iRing < psShape->nParts; iRing++ ) {
-                    if ( direction[iRing] == -1 ) {
+            int nOuter = 0;         /* Number of outer rings */ 
+            int *direction = NULL;  /* ring direction (1 CW,outer, -1 CCW, inner) */ 
+            int *outer = NULL;      /* list of outer rings */ 
+                
+            /* Outer ring index for inner rings, -1 for outer rings */ 
+            int *outside = NULL;   
+
+            direction = (int *) CPLMalloc( psShape->nParts * sizeof(int) );
+            outer = (int *) CPLMalloc( psShape->nParts * sizeof(int) );
+            outside = (int *) CPLMalloc( psShape->nParts * sizeof(int) );
+            
+            /* First distinguish outer from inner rings */
+            for( iRing = 0; iRing < psShape->nParts; iRing++ )
+            {
+                direction[iRing] = RingDirection ( psShape, iRing);
+                if ( direction[iRing] == 1 )
+                {
+                    outer[nOuter] = iRing;
+                    nOuter++;
+                }
+                outside[iRing] = -1;
+            }
+
+            if ( nOuter == 1 )
+            { 
+                /* One outer ring?
+                 * we do not need to check anything as we assume that shapefile is valid
+                 * ie. if there is one outer ring then all inner rings are inside.
+                 */
+                for( iRing = 0; iRing < psShape->nParts; iRing++ )
+                {
+                    if ( direction[iRing] == -1 )
+                    {
                         outside[iRing] = outer[ 0 ];
                     }
                 }
             }
-            else {
-                /* calculate ring extents */
-                RingExtent *extents = new RingExtent[ psShape->nParts ]; /* ring extents */
+            else
+            {
+                /* Calculate ring extents */
+                RingExtent *extents = new RingExtent[ psShape->nParts ];
 
-                for( iRing = 0; iRing < psShape->nParts; iRing++ ) {
-                    int  start, end;
+                for( iRing = 0; iRing < psShape->nParts; iRing++ )
+                {
+                    int start = 0;
+                    int end = 0;
                     RingStartEnd ( psShape, iRing, &start, &end );
                     extents[ iRing ].calculate( end-start, psShape->padfX + start, psShape->padfY + start );
                 }
-                int oRing; /* outer ring index */
-                /* This loops tries to assign inner to outr rings. The fundamental assumption is */
-                /* that if the extent of an inner ring is contained in the extent of exactly one outer */
-                /* ring then this inner ring is inside the outer ring. Otherwise that shapefile is */
-                /* broken.. */
-                for( iRing = 0; iRing < psShape->nParts; iRing++ ) {
-                    if ( direction[ iRing ] != 1 ) { /* inner ring */                    
+
+                /* Outer ring index */
+                int oRing;
+
+                /* This loops tries to assign inner to outer rings.
+                 * The fundamental assumption is that if the extent of an inner ring is contained
+                 * in the extent of exactly one outer ring then this inner ring is inside the outer ring.
+                 * Otherwise that shapefile is broken.
+                 *
+                 * XXX - mloskot - This loop does incorrect classification, see Bug 1217
+                 */
+                for( iRing = 0; iRing < psShape->nParts; iRing++ )
+                {
+                    if ( direction[ iRing ] != 1 )
+                    {
+                        /* Inner ring */
+
                         /* Find the outer ring for iRing */
-                        for( oRing = 0; oRing < nOuter; oRing++ ) {
-                            if ( extents[ outer[ oRing ] ].contains( extents[ iRing ] ) ) {
-                                /* the outer ring contains the inner ring, we have to execute some 
-                                   additional tests */ 
-                                if ( outside[ iRing ] == -1 ) {
+                        for( oRing = 0; oRing < nOuter; oRing++ )
+                        {
+                            if ( extents[ outer[ oRing ] ].contains( extents[ iRing ] ) )
+                            {
+                                /* The outer ring contains the inner ring, we have to execute
+                                 * some additional tests.
+                                 */ 
+                                if ( outside[ iRing ] == -1 )
+                                {
                                     /* this is the first outer ring, assume it containes this inner iRing */
                                     outside[ iRing ] = outer[ oRing ];
                                 }
-                                else { 
-                                    /* this is another outer ring, which contains iRing, have to distinguish
-                                       between this and previous outer rings using RingInPoint. Here is a place
-                                       for some optimization as sometimes we may check the same ring many times*/
+                                else
+                                { 
+                                    /* This is another outer ring, which contains iRing, have to distinguish
+                                     * between this and previous outer rings using RingInPoint.
+                                     * Here is a place for some optimization as sometimes we may check
+                                     * the same ring many times.
+                                     */
                                     if ( !RingInRing( psShape, outside[ iRing ], iRing ) )
+                                    {
                                         outside[ iRing ] = outer[ oRing ];
+                                    }
                                 }                            
                             }
                         }
                     }
-                }                    
+                }
+
+                /* Destroy ring extents object. */
                 delete[] extents;
             }
-            /* The inner rings are preliminary sorted, but: */
-            /* - Some of them are not assigned to any outer ring. Probably broken shapefile */
-            /* - Some inner rings are assigned to outer rings using extents only. */
-            /*     Additional geometry tests are ommited here due to perfomance penalties.*/
 
-            /* promote any unassigned inner rings to be outside rings */
-            
-            for ( iRing = 0; iRing < psShape->nParts; iRing++ ) { /* outer rings */
+            /* The inner rings are preliminary sorted, but:
+             * - Some of them are not assigned to any outer ring. Probably broken shapefile
+             * - Some inner rings are assigned to outer rings using extents only.
+             * Additional geometry tests are ommited here due to perfomance penalties.
+             * Promote any unassigned inner rings to be outside rings.
+             */
+            for ( iRing = 0; iRing < psShape->nParts; iRing++ )
+            {
+                /* Outer rings */
                 if( direction[iRing] != 1 && outside[iRing] == -1 )
                 {
                     direction[iRing] = 1; /* this isn't exactly true! */
@@ -652,57 +578,72 @@ OGRGeometry *SHPReadOGRObject( SHPHandle hSHP, int iShape )
                 }
             }
 
-	    if ( nOuter == 1 ) { /* One outer ring and more inner rings */
-		OGRPolygon    *poOGRPoly;
-	        OGRLinearRing *poRing;
+            if ( nOuter == 1 )
+            {
+                /* One outer ring and more inner rings */
+                OGRPolygon    *poOGRPoly = NULL;
+                OGRLinearRing *poRing = NULL;
 
-		/* Outer */
-	        poOGR = poOGRPoly = new OGRPolygon();
-		poRing = CreateLinearRing ( psShape, outer[0] );
-	        poOGRPoly->addRingDirectly( poRing );
+                /* Outer */
+                poOGR = poOGRPoly = new OGRPolygon();
+                poRing = CreateLinearRing ( psShape, outer[0] );
+                poOGRPoly->addRingDirectly( poRing );
 
-		/* Inner */
-		for( iRing = 0; iRing < psShape->nParts; iRing++ ) {
-		    if ( direction[iRing] == -1 ) {
-			poRing = CreateLinearRing ( psShape, iRing );
-			poOGRPoly->addRingDirectly( poRing );
-		    }
-		}
-	    } else { 
-		OGRGeometryCollection *poOGRGeCo;
+                /* Inner */
+                for( iRing = 0; iRing < psShape->nParts; iRing++ )
+                {
+                    if ( direction[iRing] == -1 )
+                    {
+                        poRing = CreateLinearRing ( psShape, iRing );
+                        poOGRPoly->addRingDirectly( poRing );
+                    }
+                }
+            }
+            else
+            { 
+                OGRGeometryCollection *poOGRGeCo = NULL;
 
-		poOGR = poOGRGeCo = new OGRMultiPolygon();
+                poOGR = poOGRGeCo = new OGRMultiPolygon();
 
-	        for ( iRing = 0; iRing < nOuter; iRing++ ) { /* outer rings */
-		    int oRing; 
+                for ( iRing = 0; iRing < nOuter; iRing++ )
+                {
+                    /* Outer rings */
+                    int oRing; 
 
-		    OGRPolygon    *poOGRPoly;
-		    OGRLinearRing *poRing;
+                    OGRPolygon    *poOGRPoly = NULL;
+                    OGRLinearRing *poRing = NULL;
 
-		    oRing = outer[iRing];
+                    oRing = outer[iRing];
 
-		    /* Outer */
-		    poOGRPoly = new OGRPolygon();
-		    poRing = CreateLinearRing ( psShape, oRing );
-		    poOGRPoly->addRingDirectly( poRing );
+                    /* Outer */
+                    poOGRPoly = new OGRPolygon();
+                    poRing = CreateLinearRing ( psShape, oRing );
+                    poOGRPoly->addRingDirectly( poRing );
 
-		    /* Inner */
-		    for( int iRing2 = 0; iRing2 < psShape->nParts; iRing2++ ) {
-			if ( outside[iRing2] == oRing ) {
-			    poRing = CreateLinearRing ( psShape, iRing2 );
-			    poOGRPoly->addRingDirectly( poRing );
-			}
-		    }
+                    /* Inner */
+                    for( int iRing2 = 0; iRing2 < psShape->nParts; iRing2++ )
+                    {
+                        if ( outside[iRing2] == oRing )
+                        {
+                            poRing = CreateLinearRing ( psShape, iRing2 );
+                            poOGRPoly->addRingDirectly( poRing );
+                        }
+                    }
+                    
                     poOGRGeCo->addGeometryDirectly (poOGRPoly);
-		}
-	    }
+                }
+            }
+
             CPLFree( direction );
             CPLFree( outer );
             CPLFree( outside );
-	}
+
+        } /* End of multipart polygon processing. */
 
         if( psShape->nSHPType == SHPT_POLYGON )
+        {
             poOGR->setCoordinateDimension( 2 );
+        }
     }
 
 /* -------------------------------------------------------------------- */
@@ -712,7 +653,10 @@ OGRGeometry *SHPReadOGRObject( SHPHandle hSHP, int iShape )
     else
     {
         if( psShape->nSHPType != SHPT_NULL )
+        {
             CPLDebug( "OGR", "Unsupported shape type in SHPReadOGRObject()" );
+        }
+
         /* nothing returned */
     }
     
@@ -1120,7 +1064,17 @@ OGRFeatureDefn *SHPReadOGRFeatureDefn( const char * pszName,
         oField.SetPrecision( nPrecision );
 
         if( chNativeType == 'D' )
+        {
+            /* XXX - mloskot:
+             * Shapefile date has following 8-chars long format: 20060101.
+             * OGR splits it as YYYY/MM/DD, so 2 additional characters are required.
+             * Is this correct assumtion? What about time part of date?
+             * Shouldn't this format look as datetime: YYYY/MM/DD HH:MM:SS
+             * with 4 additional characters?
+             */
+            oField.SetWidth( nWidth + 2 );
             oField.SetType( OFTDate );
+        }
         else if( eDBFType == FTDouble )
             oField.SetType( OFTReal );
         else if( eDBFType == FTInteger )
@@ -1197,7 +1151,7 @@ OGRFeature *SHPReadOGRFeature( SHPHandle hSHP, DBFHandle hDBF,
         return NULL;
     }
 
-    if( DBFIsRecordDeleted( hDBF, iShape ) )
+    if( hDBF && DBFIsRecordDeleted( hDBF, iShape ) )
     {
         CPLError( CE_Failure, CPLE_AppDefined, 
                   "Attempt to read shape with feature id (%d), but it is marked deleted.",
@@ -1207,8 +1161,29 @@ OGRFeature *SHPReadOGRFeature( SHPHandle hSHP, DBFHandle hDBF,
 
     OGRFeature  *poFeature = new OGRFeature( poDefn );
 
+/* -------------------------------------------------------------------- */
+/*      Fetch geometry from Shapefile to OGRFeature.                    */
+/* -------------------------------------------------------------------- */
     if( hSHP != NULL )
-        poFeature->SetGeometryDirectly( SHPReadOGRObject( hSHP, iShape ) );
+    {
+        OGRGeometry* poGeometry = NULL;
+        poGeometry = SHPReadOGRObject( hSHP, iShape );
+
+        /*
+         * NOTE - mloskot:
+         * Two possibilities are expected here (bot hare tested by GDAL Autotests):
+         * 1. Read valid geometry and assign it directly.
+         * 2. Read and assign null geometry if it can not be read correctly from a shapefile
+         *
+         * It's NOT required here to test poGeometry == NULL.
+         */
+
+        poFeature->SetGeometryDirectly( poGeometry );
+    }
+
+/* -------------------------------------------------------------------- */
+/*      Fetch feature attributes to OGRFeature fields.                  */
+/* -------------------------------------------------------------------- */
 
     for( int iField = 0; iField < poDefn->GetFieldCount(); iField++ )
     {
