@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_sqlite.h,v 1.6 2005/02/22 12:50:31 fwarmerdam Exp $
+ * $Id: ogr_sqlite.h 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Private definitions for OGR/SQLite driver.
@@ -25,28 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogr_sqlite.h,v $
- * Revision 1.6  2005/02/22 12:50:31  fwarmerdam
- * use OGRLayer base spatial filter support
- *
- * Revision 1.5  2004/08/20 21:43:12  warmerda
- * avoid doing alot of work in GetExtent() if we have no geometry
- *
- * Revision 1.4  2004/07/13 15:11:19  warmerda
- * implemented SetFeature, transaction support
- *
- * Revision 1.3  2004/07/12 20:50:46  warmerda
- * table/database creation now implemented
- *
- * Revision 1.2  2004/07/11 19:23:51  warmerda
- * read implementation working well
- *
- * Revision 1.1  2004/07/09 06:25:05  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #ifndef _OGR_SQLITE_H_INCLUDED
 #define _OGR_SQLITE_H_INLLUDED
@@ -166,7 +145,6 @@ class OGRSQLiteSelectLayer : public OGRSQLiteLayer
     virtual void        ResetReading();
     virtual int         GetFeatureCount( int );
 
-    virtual OGRErr      SetAttributeFilter( const char * );
     virtual OGRFeature *GetFeature( long nFeatureId );
     
     virtual int         TestCapability( const char * );
