@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrsqlitelayer.cpp,v 1.9 2005/09/21 00:54:43 fwarmerdam Exp $
+ * $Id: ogrsqlitelayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRSQLiteLayer class, code shared between 
@@ -26,43 +26,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrsqlitelayer.cpp,v $
- * Revision 1.9  2005/09/21 00:54:43  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.8  2005/02/22 12:50:31  fwarmerdam
- * use OGRLayer base spatial filter support
- *
- * Revision 1.7  2005/02/02 20:54:27  fwarmerdam
- * track m_nFeaturesRead
- *
- * Revision 1.6  2004/10/30 05:12:52  fwarmerdam
- * fixed memory leaks
- *
- * Revision 1.5  2004/08/20 21:43:12  warmerda
- * avoid doing alot of work in GetExtent() if we have no geometry
- *
- * Revision 1.4  2004/07/13 15:11:19  warmerda
- * implemented SetFeature, transaction support
- *
- * Revision 1.3  2004/07/12 20:50:46  warmerda
- * table/database creation now implemented
- *
- * Revision 1.2  2004/07/11 19:23:51  warmerda
- * read implementation working well
- *
- * Revision 1.1  2004/07/09 06:25:04  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "cpl_conv.h"
 #include "cpl_string.h"
 #include "ogr_sqlite.h"
 
-CPL_CVSID("$Id: ogrsqlitelayer.cpp,v 1.9 2005/09/21 00:54:43 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrsqlitelayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                           OGRSQLiteLayer()                           */
