@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: tigerinfo.cpp,v 1.6 2006/03/29 00:46:20 fwarmerdam Exp $
+ * $Id: tigerinfo.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Simple client for viewing OGR driver data.
@@ -25,41 +25,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: tigerinfo.cpp,v $
- * Revision 1.6  2006/03/29 00:46:20  fwarmerdam
- * update contact info
- *
- * Revision 1.5  2003/01/11 15:29:55  warmerda
- * expanded tabs
- *
- * Revision 1.4  2002/12/26 00:20:19  mbp
- * re-organized code to hold TIGER-version details in TigerRecordInfo structs;
- * first round implementation of TIGER_2002 support
- *
- * Revision 1.3  2001/07/18 04:55:16  warmerda
- * added CPL_CSVID
- *
- * Revision 1.2  2001/01/19 21:15:20  warmerda
- * expanded tabs
- *
- * Revision 1.1  1999/11/04 21:13:50  warmerda
- * New
- *
- * Revision 1.4  1999/09/29 16:36:41  warmerda
- * added srs reporting
- *
- * Revision 1.3  1999/09/22 13:31:48  warmerda
- * added sdts
- *
- * Revision 1.2  1999/09/13 14:34:20  warmerda
- * added feature reporting
- *
- * Revision 1.1  1999/09/09 20:40:19  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "ogrsf_frmts.h"
 #include "cpl_conv.h"
@@ -67,7 +33,7 @@
 
 #include "ogr_tiger.h"
 
-CPL_CVSID("$Id: tigerinfo.cpp,v 1.6 2006/03/29 00:46:20 fwarmerdam Exp $");
+CPL_CVSID("$Id: tigerinfo.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 int     bReadOnly = FALSE;
 int     bVerbose = TRUE;
