@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrvrtlayer.cpp,v 1.21 2006/04/13 16:41:03 fwarmerdam Exp $
+ * $Id: ogrvrtlayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Implements OGRVRTLayer class.
@@ -25,80 +25,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrvrtlayer.cpp,v $
- * Revision 1.21  2006/04/13 16:41:03  fwarmerdam
- * improved success reporting, preliminary srcrect support
- *
- * Revision 1.20  2005/12/08 23:11:01  fwarmerdam
- * added debug message if geometry not parsed successfully ... should use err
- *
- * Revision 1.19  2005/09/21 00:54:43  fwarmerdam
- * fixup OGRFeatureDefn and OGRSpatialReference refcount handling
- *
- * Revision 1.18  2005/09/05 20:29:19  fwarmerdam
- * removed unused variable
- *
- * Revision 1.17  2005/09/05 19:35:10  fwarmerdam
- * Added shape support
- *
- * Revision 1.16  2005/08/30 23:53:00  fwarmerdam
- * implement WKB translation
- *
- * Revision 1.15  2005/08/15 20:12:06  fwarmerdam
- * Ensure that SrcSQL resultsets are released.
- *
- * Revision 1.14  2005/08/02 20:17:26  fwarmerdam
- * pass attribute filter to sublayer
- *
- * Revision 1.13  2005/07/23 13:15:45  fwarmerdam
- * Fixed problem with size of filter string for spatial query.
- *
- * Revision 1.12  2005/06/27 20:13:11  fwarmerdam
- * Delete poSRS if no longer referenced.
- *
- * Revision 1.11  2005/05/16 20:09:46  fwarmerdam
- * added spatial query on x/y columns
- *
- * Revision 1.10  2005/02/22 12:50:10  fwarmerdam
- * use OGRLayer base spatial filter support
- *
- * Revision 1.9  2005/02/02 20:54:27  fwarmerdam
- * track m_nFeaturesRead
- *
- * Revision 1.8  2004/10/30 04:54:54  fwarmerdam
- * Improved geometry check error message.
- *
- * Revision 1.7  2004/10/30 04:44:00  fwarmerdam
- * Fixed error report when fetching layer.
- *
- * Revision 1.6  2004/10/16 21:56:36  fwarmerdam
- * Fixed initialization of "z".
- *
- * Revision 1.5  2004/03/25 13:23:41  warmerda
- * Fixed typo in error message.
- *
- * Revision 1.4  2003/12/30 18:34:57  warmerda
- * Added support for SrcSQL instead of SrcLayer.
- *
- * Revision 1.3  2003/11/10 20:11:55  warmerda
- * Allow any UserInput in LayerSYS
- *
- * Revision 1.2  2003/11/07 21:55:12  warmerda
- * complete fid support, relative dsname, fixes
- *
- * Revision 1.1  2003/11/07 17:50:36  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "cpl_conv.h"
 #include "ogr_vrt.h"
 #include "cpl_string.h"
 #include <string>
 
-CPL_CVSID("$Id: ogrvrtlayer.cpp,v 1.21 2006/04/13 16:41:03 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrvrtlayer.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 typedef struct 
 {
@@ -131,6 +65,7 @@ OGRVRTLayer::OGRVRTLayer()
     poSrcLayer = NULL;
     poSRS = NULL;
 
+    bUseSpatialSubquery = FALSE;
     iFIDField = -1;
 
     eGeometryType = VGS_Direct;
@@ -394,6 +329,11 @@ int OGRVRTLayer::Initialize( CPLXMLNode *psLTree, const char *pszVRTDirectory )
      else if( EQUAL(pszEncoding,"PointFromColumns") )
      {
          eGeometryType = VGS_PointFromColumns;
+         bUseSpatialSubquery = 
+             CSLTestBoolean(
+                 CPLGetXMLValue(psLTree, 
+                                "GeometryField.useSpatialSubquery",
+                                "TRUE"));
 
          iGeomXField = poSrcLayer->GetLayerDefn()->GetFieldIndex(
              CPLGetXMLValue( psLTree, "GeometryField.x", "missing" ) );
@@ -510,8 +450,7 @@ int OGRVRTLayer::ResetSourceReading()
 /*      Do we want to let source layer do spatial restriction?          */
 /* -------------------------------------------------------------------- */
     char *pszFilter = NULL;
-    if( m_poFilterGeom && m_bFilterIsEnvelope 
-        && eGeometryType == VGS_PointFromColumns )
+    if( m_poFilterGeom && m_bFilterIsEnvelope && bUseSpatialSubquery )
     {
         const char *pszXField, *pszYField;
 
@@ -818,14 +757,9 @@ OGRErr OGRVRTLayer::SetAttributeFilter( const char *pszNewQuery )
 {
     CPLFree( pszAttrFilter );
     if( pszNewQuery == NULL || strlen(pszNewQuery) == 0 )
-    {
         pszAttrFilter = NULL;
-    }
     else
-    {
-        CPLFree( pszAttrFilter );
         pszAttrFilter = CPLStrdup( pszNewQuery );
-    }
 
     ResetReading();
     return OGRERR_NONE;
