@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogrutils.cpp,v 1.23 2006/04/07 06:29:07 fwarmerdam Exp $
+ * $Id: ogrutils.cpp 11034 2007-03-21 04:38:04Z mloskot $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Utility functions for OGR classes, including some related to
@@ -26,82 +26,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: ogrutils.cpp,v $
- * Revision 1.23  2006/04/07 06:29:07  fwarmerdam
- * Added support for parsing coordinates starting with a bare decimal from WKT.
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=1150
- *
- * Revision 1.22  2006/03/31 17:44:20  fwarmerdam
- * header updates
- *
- * Revision 1.21  2006/02/15 04:25:37  fwarmerdam
- * added date support
- *
- * Revision 1.20  2006/01/07 17:15:38  dron
- * #include "ogrsf_frmts.h" only when compiling with the OGR formats.
- *
- * Revision 1.19  2006/01/06 17:23:26  dron
- * Use OGR_ENABLED macro to separate OGR formats related parts of the code.
- *
- * Revision 1.18  2005/10/16 01:59:06  cfis
- * Added declaration for OGRGeneralCmdLineProcessor to ogr_p.h, and included it
- * into ogr2ogr.  Also changed call to CPL_DLL from CPL_STDCALL
- *
- * Revision 1.17  2005/10/16 01:51:43  cfis
- * Added #include for ogrsf_frmts.f to support the --formats command line parameter
- * in OGRGeneralCmdLineProcessor.
- *
- * Revision 1.16  2005/10/16 01:49:02  cfis
- * Added support for general command line processing similar to what GDAL utilities have.
- *
- * Revision 1.15  2005/09/15 00:52:28  fwarmerdam
- * fixed debug message
- *
- * Revision 1.14  2005/07/29 04:13:17  fwarmerdam
- * preserve 'full' precision in OGRMAkeWktCoordinate
- *
- * Revision 1.13  2005/07/22 19:32:43  fwarmerdam
- * Preserve more precision in WKT encoding of coordinates.
- *
- * Revision 1.12  2005/07/20 01:43:51  fwarmerdam
- * upgraded OGR geometry dimension handling
- *
- * Revision 1.11  2003/01/06 17:57:18  warmerda
- * Added some extra validation in OGRMakeWktCoordinate()
- *
- * Revision 1.10  2002/12/09 18:55:07  warmerda
- * moved DMS stuff to gdal/port
- *
- * Revision 1.9  2002/12/09 16:10:39  warmerda
- * added DMS translation
- *
- * Revision 1.8  2002/08/07 02:46:10  warmerda
- * improved comments
- *
- * Revision 1.7  2001/11/01 17:01:28  warmerda
- * pass output buffer into OGRMakeWktCoordinate
- *
- * Revision 1.6  2001/07/18 05:03:05  warmerda
- * added CPL_CVSID
- *
- * Revision 1.5  2001/05/29 02:24:00  warmerda
- * fixed negative support for Z coordinate
- *
- * Revision 1.4  2001/05/24 18:05:36  warmerda
- * fixed support for negative coordinte parsing
- *
- * Revision 1.3  1999/11/18 19:02:19  warmerda
- * expanded tabs
- *
- * Revision 1.2  1999/09/13 02:27:33  warmerda
- * incorporated limited 2.5d support
- *
- * Revision 1.1  1999/05/20 14:35:00  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include <ctype.h>
 
@@ -112,7 +37,7 @@
 # include "ogrsf_frmts.h"
 #endif /* OGR_ENABLED */
 
-CPL_CVSID("$Id: ogrutils.cpp,v 1.23 2006/04/07 06:29:07 fwarmerdam Exp $");
+CPL_CVSID("$Id: ogrutils.cpp 11034 2007-03-21 04:38:04Z mloskot $");
 
 /************************************************************************/
 /*                         OGRTrimExtraZeros()                          */
@@ -286,6 +211,7 @@ const char * OGRWktReadPoints( const char * pszInput,
                                int * pnPointsRead )
 
 {
+    const char *pszOrigInput = pszInput;
     *pnPointsRead = 0;
 
     if( pszInput == NULL )
@@ -373,6 +299,15 @@ const char * OGRWktReadPoints( const char * pszInput,
         
         (*pnPointsRead)++;
 
+/* -------------------------------------------------------------------- */
+/*      Do we have a M coordinate?                                      */
+/*      If we do, just skip it.                                         */
+/* -------------------------------------------------------------------- */
+        if( isdigit(szDelim[0]) || szDelim[0] == '-' || szDelim[0] == '.' )
+        {
+            pszInput = OGRWktReadToken( pszInput, szDelim );
+        }
+        
 /* -------------------------------------------------------------------- */
 /*      Read next delimeter ... it should be a comma if there are       */
 /*      more points.                                                    */
@@ -381,8 +316,8 @@ const char * OGRWktReadPoints( const char * pszInput,
         {
             CPLDebug( "OGR",
                       "Corrupt input in OGRWktReadPoints()\n"
-                      "Got `%s' when expecting `,' or `)'.\n",
-                      szDelim );
+                      "Got `%s' when expecting `,' or `)', near `%s' in %s.\n",
+                      szDelim, pszInput, pszOrigInput );
             return NULL;
         }
         
@@ -511,6 +446,52 @@ int OGRGeneralCmdLineProcessor( int nArgc, char ***ppapszArgv, int nOptions )
             iArg += 2;
         }
 
+/* -------------------------------------------------------------------- */
+/*      --mempreload                                                    */
+/* -------------------------------------------------------------------- */
+        else if( EQUAL(papszArgv[iArg],"--mempreload") )
+        {
+            int i;
+
+            if( iArg + 1 >= nArgc )
+            {
+                CPLError( CE_Failure, CPLE_AppDefined, 
+                          "--mempreload option given without directory path.");
+                CSLDestroy( papszReturn );
+                return -1;
+            }
+            
+            char **papszFiles = CPLReadDir( papszArgv[iArg+1] );
+            if( CSLCount(papszFiles) == 0 )
+            {
+                CPLError( CE_Failure, CPLE_AppDefined, 
+                          "--mempreload given invalid or empty directory.");
+                CSLDestroy( papszReturn );
+                return -1;
+            }
+                
+            for( i = 0; papszFiles[i] != NULL; i++ )
+            {
+                CPLString osOldPath, osNewPath;
+                
+                if( EQUAL(papszFiles[i],".") || EQUAL(papszFiles[i],"..") )
+                    continue;
+
+                osOldPath = CPLFormFilename( papszArgv[iArg+1], 
+                                             papszFiles[i], NULL );
+                osNewPath.Printf( "/vsimem/%s", papszFiles[i] );
+
+                CPLDebug( "VSI", "Preloading %s to %s.", 
+                          osOldPath.c_str(), osNewPath.c_str() );
+
+                if( CPLCopyFile( osNewPath, osOldPath ) != 0 )
+                    return -1;
+            }
+            
+            CSLDestroy( papszFiles );
+            iArg += 1;
+        }
+
 /* -------------------------------------------------------------------- */
 /*      --debug                                                         */
 /* -------------------------------------------------------------------- */
@@ -592,7 +573,13 @@ int OGRGeneralCmdLineProcessor( int nArgc, char ***ppapszArgv, int nOptions )
             for( iDr = 0; iDr < poR->GetDriverCount(); iDr++ )
             {
                 OGRSFDriver *poDriver = poR->GetDriver(iDr);
-                printf( "  %s\n", poDriver->GetName());
+
+                if( poDriver->TestCapability( ODrCCreateDataSource ) )
+                    printf( "  -> \"%s\" (read/write)\n", 
+                            poDriver->GetName() );
+                else
+                    printf( "  -> \"%s\" (readonly)\n", 
+                            poDriver->GetName() );
             }
 
             CSLDestroy( papszReturn );
@@ -791,7 +778,7 @@ int OGRParseDate( const char *pszInput, OGRField *psField, int nOptions )
                  && atoi(pszInput+3) % 15 == 0 )
         {
             psField->Date.TZFlag = 100 
-                + CPLScanLong(pszInput+1,2) * 4
+                + static_cast<GByte>(CPLScanLong(pszInput+1,2)) * 4
                 + (atoi(pszInput+3) / 15);
 
             if( pszInput[0] == '-' )
@@ -801,7 +788,7 @@ int OGRParseDate( const char *pszInput, OGRField *psField, int nOptions )
                  && atoi(pszInput+2) % 15 == 0 )
         {
             psField->Date.TZFlag = 100 
-                + CPLScanLong(pszInput+1,1) * 4
+                + static_cast<GByte>(CPLScanLong(pszInput+1,1)) * 4
                 + (atoi(pszInput+2) / 15);
 
             if( pszInput[0] == '-' )
