@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_conv.cpp,v 1.52 2006/03/04 00:46:43 mloskot Exp $
+ * $Id: cpl_conv.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Convenience functions.
@@ -25,175 +25,14 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: cpl_conv.cpp,v $
- * Revision 1.52  2006/03/04 00:46:43  mloskot
- * [WCE] CPLLocaleC class excluded from compilation for Windows CE
- *
- * Revision 1.51  2006/03/03 19:40:53  fwarmerdam
- * added CPLLocaleC class
- *
- * Revision 1.50  2006/02/19 21:54:34  mloskot
- * [WINCE] Changes related to Windows CE port of CPL. Most changes are #ifdef wrappers.
- *
- * Revision 1.49  2006/02/09 05:54:15  fwarmerdam
- * use ll in format for long long on mac, not L
- *
- * Revision 1.48  2005/11/24 19:22:54  fwarmerdam
- * Added silly-value checks in CPLMalloc/Realloc.
- *
- * Revision 1.47  2005/10/17 23:33:51  fwarmerdam
- * CPLFGets() - be very wary about reads ending in the middle of CRLF.
- * Was causing problems with world.mif/world.mid reading.
- *
- * Revision 1.46  2005/09/11 21:08:59  fwarmerdam
- * properly manage TLS ReadLine buffer, added CPLReadLineL()
- *
- * Revision 1.45  2005/07/11 16:41:31  fwarmerdam
- * Protect config list and shared file list with mutex.
- *
- * Revision 1.44  2005/05/23 03:57:57  fwarmerdam
- * make make static buffers threadlocal
- *
- * Revision 1.43  2005/04/04 15:23:30  fwarmerdam
- * some functions now CPL_STDCALL
- *
- * Revision 1.42  2004/11/17 22:57:21  fwarmerdam
- * added CPLScanPointer() and CPLPrintPointer()
- *
- * Revision 1.41  2004/08/16 20:24:07  warmerda
- * added CPLUnlinkTree
- *
- * Revision 1.40  2004/07/31 04:51:36  warmerda
- * added shared file open support
- *
- * Revision 1.39  2004/06/01 20:40:02  warmerda
- * expanded tabs
- *
- * Revision 1.38  2004/03/28 16:22:02  warmerda
- * const correctness changes in scan functions
- *
- * Revision 1.37  2004/03/24 09:01:17  dron
- * Added CPLPrintUIntBig().
- *
- * Revision 1.36  2004/02/25 09:02:25  dron
- * Fixed bug in CPLPackedDMSToDec().
- *
- * Revision 1.35  2004/02/21 10:08:54  dron
- * Fixes in various string handling functions.
- *
- * Revision 1.34  2004/02/07 14:03:30  dron
- * CPLDecToPackedDMS() added.
- *
- * Revision 1.33  2004/02/01 08:37:55  dron
- * Added CPLPackedDMSToDec().
- *
- * Revision 1.32  2003/12/28 17:24:43  warmerda
- * added CPLFreeConfig
- *
- * Revision 1.31  2003/10/17 07:06:06  dron
- * Added locale selection option to CPLScanDouble() and CPLPrintDOuble().
- *
- * Revision 1.30  2003/09/28 14:14:16  dron
- * Added CPLScanString().
- *
- * Revision 1.29  2003/09/12 20:49:24  warmerda
- * reimplement CPLFGets() to avoid textmode problems
- *
- * Revision 1.28  2003/09/08 12:54:42  dron
- * Fixed warnings.
- *
- * Revision 1.27  2003/09/08 11:09:53  dron
- * Added CPLPrintDouble() and CPLPrintTime().
- *
- * Revision 1.26  2003/09/07 14:38:43  dron
- * Added CPLPrintString(), CPLPrintStringFill(), CPLPrintInt32(), CPLPrintUIntBig().
- *
- * Revision 1.25  2003/09/03 13:07:26  warmerda
- * Cleaned up CPLScanLong() a bit to avoid warnings, and
- * unnecessary conversion to/from double.
- *
- * Revision 1.24  2003/08/31 14:48:05  dron
- * Added CPLScanLong() and CPLScanDouble().
- *
- * Revision 1.23  2003/08/25 20:01:58  dron
- * Added CPLFGets() helper function.
- *
- * Revision 1.22  2003/05/08 21:51:14  warmerda
- * added CPL{G,S}etConfigOption() usage
- *
- * Revision 1.21  2003/03/05 16:46:54  warmerda
- * Cast strchr() result for Sun (patch from Graeme).
- *
- * Revision 1.20  2003/03/02 04:44:38  warmerda
- * added CPLStringToComplex
- *
- * Revision 1.19  2003/02/14 22:12:07  warmerda
- * expand tabs
- *
- * Revision 1.18  2002/12/18 20:22:53  warmerda
- * fiddle with roundoff issues in DecToDMS
- *
- * Revision 1.17  2002/12/10 19:46:04  warmerda
- * modified CPLReadLine() to seek back if it overreads past a CR or LF
- *
- * Revision 1.16  2002/12/09 18:52:51  warmerda
- * added DMS conversion
- *
- * Revision 1.15  2002/03/05 14:26:57  warmerda
- * expanded tabs
- *
- * Revision 1.14  2001/12/12 17:06:57  warmerda
- * added CPLStat
- *
- * Revision 1.13  2001/07/18 04:00:49  warmerda
- * added CPL_CVSID
- *
- * Revision 1.12  2001/03/09 03:19:24  danmo
- * Set pszRLBuffer=NULL after freeing it to avoid reallocating an invalid ptr
- *
- * Revision 1.11  2001/03/05 03:37:19  warmerda
- * Improve support for recovering CPLReadLine() working buffer.
- *
- * Revision 1.10  2001/01/19 21:16:41  warmerda
- * expanded tabs
- *
- * Revision 1.9  2000/04/17 15:56:11  warmerda
- * make configuration tests always happen
- *
- * Revision 1.8  2000/04/05 21:02:47  warmerda
- * Added CPLVerifyConfiguration()
- *
- * Revision 1.7  1999/08/27 12:55:39  danmo
- * Support 0 bytes allocations in CPLRealloc()
- *
- * Revision 1.6  1999/06/25 04:38:03  warmerda
- * Fixed CPLReadLine() to work for long lines.
- *
- * Revision 1.5  1999/05/20 02:54:37  warmerda
- * Added API documentation
- *
- * Revision 1.4  1999/01/02 20:29:53  warmerda
- * Allow zero length allocations
- *
- * Revision 1.3  1998/12/15 19:01:07  warmerda
- * Added CPLReadLine().
- *
- * Revision 1.2  1998/12/03 18:30:04  warmerda
- * Use CPLError() instead of GPSError().
- *
- * Revision 1.1  1998/12/02 19:33:23  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "cpl_conv.h"
 #include "cpl_string.h"
 #include "cpl_vsi.h"
 #include "cpl_multiproc.h"
 
-CPL_CVSID("$Id: cpl_conv.cpp,v 1.52 2006/03/04 00:46:43 mloskot Exp $");
+CPL_CVSID("$Id: cpl_conv.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 #if defined(WIN32CE)
 #  include "cpl_wince.h"
@@ -397,6 +236,37 @@ char *CPLStrdup( const char * pszString )
     return( pszReturn );
 }
 
+/************************************************************************/
+/*                             CPLStrlwr()                              */
+/************************************************************************/
+
+/**
+ * Convert each characters of the string to lower case.
+ *
+ * For example, "ABcdE" will be converted to "abcde".
+ * This function is locale dependent.
+ *
+ * @param pszString input string to be converted.
+ * @return pointer to the same string, pszString.
+ */
+
+char *CPLStrlwr( char *pszString )
+
+{
+    if (pszString)
+    {
+        char *pszTemp = pszString;
+
+        while (*pszTemp)
+        {
+            *pszTemp = tolower (*pszTemp);
+            pszTemp++;
+        }
+    }
+
+    return pszString;
+}
+
 /************************************************************************/
 /*                              CPLFGets()                              */
 /*                                                                      */
@@ -634,7 +504,8 @@ const char *CPLReadLine( FILE * fp )
 /* -------------------------------------------------------------------- */
 /*      Do the actual read.                                             */
 /* -------------------------------------------------------------------- */
-        if( CPLFGets( pszRLBuffer+nReadSoFar, 128, fp ) == NULL )
+        if( CPLFGets( pszRLBuffer+nReadSoFar, 128, fp ) == NULL 
+            && nReadSoFar == 0 )
             return NULL;
 
         nBytesReadThisTime = strlen(pszRLBuffer+nReadSoFar);
@@ -679,11 +550,11 @@ const char *CPLReadLineL( FILE * fp )
 /*      the line.                                                       */
 /* -------------------------------------------------------------------- */
     char *pszRLBuffer;
-    const int nChunkSize = 40;
+    const size_t nChunkSize = 40;
     char szChunk[nChunkSize];
-    int nChunkBytesRead = 0;
+    size_t nChunkBytesRead = 0;
     int nBufLength = 0;
-    int nChunkBytesConsumed = 0;
+    size_t nChunkBytesConsumed = 0;
 
     while( TRUE )
     {
@@ -694,25 +565,32 @@ const char *CPLReadLineL( FILE * fp )
 
         if( nChunkBytesRead == nChunkBytesConsumed + 1 )
         {
+
             // case where one character is left over from last read.
             szChunk[0] = szChunk[nChunkBytesConsumed];
+
+            nChunkBytesConsumed = 0;
             nChunkBytesRead = VSIFReadL( szChunk+1, 1, nChunkSize-1, fp ) + 1;
-            if( nChunkBytesRead == 1 )
-                break;
         }
         else
         {
+            nChunkBytesConsumed = 0;
+
             // fresh read.
             nChunkBytesRead = VSIFReadL( szChunk, 1, nChunkSize, fp );
             if( nChunkBytesRead == 0 )
-                break;
+            {
+                if( nBufLength == 0 )
+                    return NULL;
+                else
+                    break;
+            }
         }
         
 /* -------------------------------------------------------------------- */
 /*      copy over characters watching for end-of-line.                  */
 /* -------------------------------------------------------------------- */
         int bBreak = FALSE;
-        nChunkBytesConsumed = 0;
         while( nChunkBytesConsumed < nChunkBytesRead-1 && !bBreak )
         {
             if( (szChunk[nChunkBytesConsumed] == 13
@@ -762,7 +640,7 @@ const char *CPLReadLineL( FILE * fp )
 /* -------------------------------------------------------------------- */
     if( nChunkBytesConsumed < nChunkBytesRead )
     {
-        int nBytesToPush = nChunkBytesRead - nChunkBytesConsumed;
+        size_t nBytesToPush = nChunkBytesRead - nChunkBytesConsumed;
         
         VSIFSeekL( fp, VSIFTellL( fp ) - nBytesToPush, SEEK_SET );
     }
@@ -907,7 +785,7 @@ GUIntBig CPLScanUIntBig( const char *pszString, int nMaxLength )
 /*      Fetch out the result                                            */
 /* -------------------------------------------------------------------- */
 #if defined(WIN32) && defined(_MSC_VER)
-    iValue = _atoi64( pszValue );
+    iValue = (GUIntBig)_atoi64( pszValue );
 # elif HAVE_ATOLL
     iValue = atoll( pszValue );
 #else
@@ -2097,12 +1975,64 @@ int CPLUnlinkTree( const char *pszPath )
     }
 }
 
+/************************************************************************/
+/*                            CPLCopyFile()                             */
+/************************************************************************/
+
+int CPLCopyFile( const char *pszNewPath, const char *pszOldPath )
+
+{
+    FILE *fpOld, *fpNew;
+    GByte *pabyBuffer;
+    size_t nBufferSize;
+    size_t nBytesRead;
+
+/* -------------------------------------------------------------------- */
+/*      Open old and new file.                                          */
+/* -------------------------------------------------------------------- */
+    fpOld = VSIFOpenL( pszOldPath, "rb" );
+    if( fpOld == NULL )
+        return -1;
+
+    fpNew = VSIFOpenL( pszNewPath, "wb" );
+    if( fpNew == NULL )
+        return -1;
+
+/* -------------------------------------------------------------------- */
+/*      Prepare buffer.                                                 */
+/* -------------------------------------------------------------------- */
+    nBufferSize = 1024 * 1024;
+    pabyBuffer = (GByte *) CPLMalloc(nBufferSize);
+
+/* -------------------------------------------------------------------- */
+/*      Copy file over till we run out of stuff.                        */
+/* -------------------------------------------------------------------- */
+    do { 
+        nBytesRead = VSIFReadL( pabyBuffer, 1, nBufferSize, fpOld );
+        if( nBytesRead < 0 )
+            return -1;
+
+        if( VSIFWriteL( pabyBuffer, 1, nBytesRead, fpNew ) < nBytesRead )
+            return -1;
+    } while( nBytesRead == nBufferSize );
+
+/* -------------------------------------------------------------------- */
+/*      Cleanup                                                         */
+/* -------------------------------------------------------------------- */
+    VSIFCloseL( fpNew );
+    VSIFCloseL( fpOld );
+
+    CPLFree( pabyBuffer );
+
+    return 0;
+}
+
+
 /************************************************************************/
 /* ==================================================================== */
 /*                              CPLLocaleC                              */
 /* ==================================================================== */
 /************************************************************************/
-#ifndef WIN32CE
 
 #include <locale.h>
 
@@ -2132,5 +2062,3 @@ CPLLocaleC::~CPLLocaleC()
     }
 }
 
-#endif /* ndef WIN32CE */
-
