@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: cpl_minixml.cpp,v 1.42 2006/04/19 02:00:20 fwarmerdam Exp $
+ * $Id: cpl_minixml.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Implementation of MiniXML Parser and handling.
@@ -36,52 +36,6 @@
  *   to parse, and serialize arbitrary documents provided by a potentially 
  *   hostile source.
  *
- * $Log: cpl_minixml.cpp,v $
- * Revision 1.42  2006/04/19 02:00:20  fwarmerdam
- * moved ctype.h after cpl include files for VS2005
- *
- * Revision 1.41  2006/02/20 00:42:35  fwarmerdam
- * Trim.
- *
- * Revision 1.40  2006/02/19 21:54:34  mloskot
- * [WINCE] Changes related to Windows CE port of CPL. Most changes are #ifdef wrappers.
- *
- * Revision 1.39  2005/10/07 00:03:29  fwarmerdam
- * improve documentation
- *
- * Revision 1.38  2005/09/11 19:30:12  fwarmerdam
- * Try to write in text mode through large file API.
- *
- * Revision 1.37  2005/09/11 19:14:54  fwarmerdam
- * Use largefile API for virtualization support.  Note that XML files are
- * no longer produced in machine-local text format.
- *
- * Revision 1.36  2005/05/22 08:17:07  fwarmerdam
- * allow CPLAddXMLChild() to support siblings
- *
- * Revision 1.35  2005/05/13 18:17:48  fwarmerdam
- * added CPLRemoveXMLChild
- *
- * Revision 1.34  2005/03/09 17:07:25  fwarmerdam
- * added CPLSearchXMLNode
- *
- * Revision 1.33  2005/01/17 17:01:56  fwarmerdam
- * ensure that namespace stripping apply to attributes
- *
- * Revision 1.32  2004/10/21 18:59:00  fwarmerdam
- * Ensure that an empty path in CPLGetXMLValue() means the current node.
- *
- * Revision 1.31  2004/08/11 18:42:00  warmerda
- * fix quirks with CPLSetXMLValue()
- *
- * Revision 1.30  2004/03/31 17:11:41  warmerda
- * fixed return value of CPLCreateXMLElementAndValue()
- *
- * Revision 1.29  2004/01/29 17:01:51  warmerda
- * Added reference to spec.
- *
- * Revision 1.28  2004/01/29 15:29:28  warmerda
- * Added CPLCleanXMLElementName
  */
 
 #include "cpl_minixml.h"
@@ -90,7 +44,7 @@
 #include "cpl_string.h"
 #include <ctype.h>
 
-CPL_CVSID("$Id: cpl_minixml.cpp,v 1.42 2006/04/19 02:00:20 fwarmerdam Exp $");
+CPL_CVSID("$Id: cpl_minixml.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 typedef enum {
     TNone,
@@ -112,8 +66,8 @@ typedef struct {
     int        bInElement;
     XMLTokenType  eTokenType;
     char       *pszToken;
-    int        nTokenMaxSize;
-    int        nTokenSize;
+    size_t     nTokenMaxSize;
+    size_t     nTokenSize;
 
     int        nStackMaxSize;
     int        nStackSize;
@@ -227,7 +181,7 @@ static XMLTokenType ReadToken( ParseContext *psContext )
         ReadChar(psContext);
     }
 /* -------------------------------------------------------------------- */
-/*      Handle DOCTYPE or other literals.                               */
+/*      Handle DOCTYPE.                                                 */
 /* -------------------------------------------------------------------- */
     else if( chNext == '<' 
           && EQUALN(psContext->pszInput+psContext->nInputOffset,"!DOCTYPE",8) )
@@ -261,6 +215,33 @@ static XMLTokenType ReadToken( ParseContext *psContext )
         } while( TRUE );
     }
 /* -------------------------------------------------------------------- */
+/*      Handle CDATA.                                                   */
+/* -------------------------------------------------------------------- */
+    else if( chNext == '<' 
+          && EQUALN(psContext->pszInput+psContext->nInputOffset,"![CDATA[",8) )
+    {
+        psContext->eTokenType = TString;
+
+        // Skip !CDATA[
+        ReadChar( psContext );
+        ReadChar( psContext );
+        ReadChar( psContext );
+        ReadChar( psContext );
+        ReadChar( psContext );
+        ReadChar( psContext );
+        ReadChar( psContext );
+        ReadChar( psContext );
+
+        while( !EQUALN(psContext->pszInput+psContext->nInputOffset,"]]>",3)
+               && (chNext = ReadChar(psContext)) != '\0' )
+            AddToToken( psContext, chNext );
+
+        // Skip "]]>" characters
+        ReadChar(psContext);
+        ReadChar(psContext);
+        ReadChar(psContext);
+    }
+/* -------------------------------------------------------------------- */
 /*      Simple single tokens of interest.                               */
 /* -------------------------------------------------------------------- */
     else if( chNext == '<' && !psContext->bInElement )
@@ -755,7 +736,7 @@ CPLXMLNode *CPLParseXMLString( const char *pszString )
 /*                            _GrowBuffer()                             */
 /************************************************************************/
 
-static void _GrowBuffer( unsigned int nNeeded, 
+static void _GrowBuffer( size_t nNeeded, 
                          char **ppszText, unsigned int *pnMaxLength )
 
 {
@@ -1684,10 +1665,10 @@ void CPLStripXMLNamespace( CPLXMLNode *psRoot,
 CPLXMLNode *CPLParseXMLFile( const char *pszFilename )
 
 {
-    FILE        *fp;
-    int nLen;
-    char *pszDoc;
-    CPLXMLNode *psTree;
+    FILE            *fp;
+    vsi_l_offset    nLen;
+    char            *pszDoc;
+    CPLXMLNode      *psTree;
 
 /* -------------------------------------------------------------------- */
 /*      Read the file.                                                  */
@@ -1704,7 +1685,7 @@ CPLXMLNode *CPLParseXMLFile( const char *pszFilename )
     nLen = VSIFTellL( fp );
     VSIFSeekL( fp, 0, SEEK_SET );
     
-    pszDoc = (char *) VSIMalloc(nLen+1);
+    pszDoc = (char *) VSIMalloc((size_t)nLen + 1);
     if( pszDoc == NULL )
     {
         CPLError( CE_Failure, CPLE_OutOfMemory, 
@@ -1714,7 +1695,7 @@ CPLXMLNode *CPLParseXMLFile( const char *pszFilename )
         VSIFCloseL( fp );
         return NULL;
     }
-    if( (int) VSIFReadL( pszDoc, 1, nLen, fp ) < nLen )
+    if( VSIFReadL( pszDoc, 1, (size_t)nLen, fp ) < nLen )
     {
         CPLError( CE_Failure, CPLE_FileIO, 
                   "VSIFRead() result short of expected %d bytes from %.500s.", 
@@ -1753,9 +1734,9 @@ CPLXMLNode *CPLParseXMLFile( const char *pszFilename )
 int CPLSerializeXMLTreeToFile( CPLXMLNode *psTree, const char *pszFilename )
 
 {
-    char *pszDoc;
-    FILE *fp;
-    int  nLength;
+    char    *pszDoc;
+    FILE    *fp;
+    vsi_l_offset nLength;
 
 /* -------------------------------------------------------------------- */
 /*      Serialize document.                                             */
@@ -1780,7 +1761,7 @@ int CPLSerializeXMLTreeToFile( CPLXMLNode *psTree, const char *pszFilename )
 /* -------------------------------------------------------------------- */
 /*      Write file.                                                     */
 /* -------------------------------------------------------------------- */
-    if( (int) VSIFWriteL( pszDoc, 1, nLength, fp ) != nLength )
+    if( VSIFWriteL( pszDoc, 1, nLength, fp ) != nLength )
     {
         CPLError( CE_Failure, CPLE_FileIO, 
                   "Failed to write whole XML document (%.500s).",
