@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_odbc.cpp,v 1.27 2006/02/19 21:54:34 mloskot Exp $
+ * $Id: cpl_odbc.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  OGR ODBC Driver
  * Purpose:  Declarations for ODBC Access Cover API.
@@ -25,110 +25,126 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: cpl_odbc.cpp,v $
- * Revision 1.27  2006/02/19 21:54:34  mloskot
- * [WINCE] Changes related to Windows CE port of CPL. Most changes are #ifdef wrappers.
- *
- * Revision 1.26  2006/01/31 03:04:27  fwarmerdam
- * Fixed null trimming to work for long unicode text strings per:
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=990
- *
- * Revision 1.25  2005/09/24 04:58:16  fwarmerdam
- * Support SQLDriverConnect (pass opts) in EstablishSession
- *
- * Revision 1.24  2005/09/05 20:18:43  fwarmerdam
- * added binary column support
- *
- * Revision 1.23  2005/08/31 03:32:41  fwarmerdam
- * GetTypeName now returns CPLString
- *
- * Revision 1.22  2005/08/07 14:05:44  fwarmerdam
- * added connect/disconnect debug statements
- *
- * Revision 1.21  2005/06/29 01:01:01  ssoule
- * Changed return type of CPLODBCStatement::GetTypeName from const char * to
- * std::string.
- *
- * Revision 1.20  2005/05/23 03:56:44  fwarmerdam
- * make make static buffers threadlocal
- *
- * Revision 1.19  2005/01/12 07:51:29  fwarmerdam
- * Fix some argument types needed for ODBC 3.52. In particular SQLDescribeCol
- * and SQLGetData require SQLLEN and SQLULEN instead of SQLINTEGER and
- * SQLUINTEGER.  Patches provided by David Herring.
- *
- * Revision 1.18  2004/08/18 18:47:12  warmerda
- * Increase size of work buffer a bit so that providers with broken support
- * for multiple SQLGetData() calls will still mostly work.
- *
- * Revision 1.17  2004/08/17 21:51:16  warmerda
- * Changed Fetch() to handle values longer than the working buffer
- * properly ... do extra SQLGetData() calls till complete.
- *
- * Revision 1.16  2004/08/17 20:15:56  warmerda
- * Allow column values up to 64K instead of limited to 8K.
- *
- * Revision 1.15  2004/06/17 17:11:51  warmerda
- * fixed case where vsnprintf does not exist
- *
- * Revision 1.14  2004/06/01 20:40:02  warmerda
- * expanded tabs
- *
- * Revision 1.13  2004/03/30 22:26:58  warmerda
- * Avoid use of SQLRowCount() in GetColumns().  It does not work reliably
- * with Oracle unixodbc driver on AIX.  Argg.
- *
- * Revision 1.12  2004/03/04 16:58:05  warmerda
- * Fixed up memory leak of results set column definition info.
- *
- * Revision 1.11  2004/03/04 05:41:21  warmerda
- * Modified Fetch() method to use SQLFetch() for cases it would be
- * sufficient for.  The SQLScrollFetch() isn't implemented on some brain
- * dead drivers, such as the unixODBC text file driver.
- * http://bugzilla.remotesensing.org/show_bug.cgi?id=463
- *
- * Revision 1.10  2003/11/24 20:48:08  warmerda
- * pass NULLs t SQLColumns, not empty strings
- *
- * Revision 1.9  2003/11/24 20:32:07  warmerda
- * Use TABLE instead of TABLES in SQLTables() call
- *
- * Revision 1.8  2003/11/10 20:08:12  warmerda
- * added GetTables() implementation
- *
- * Revision 1.7  2003/10/29 17:56:57  warmerda
- * Added PrimaryKeys() support
- *
- * Revision 1.6  2003/10/06 20:04:08  warmerda
- * added escaping support
- *
- * Revision 1.5  2003/10/06 17:17:02  warmerda
- * fixed some type issues
- *
- * Revision 1.4  2003/09/26 20:02:41  warmerda
- * update GetColData()
- *
- * Revision 1.3  2003/09/26 13:51:02  warmerda
- * Add documentation
- *
- * Revision 1.2  2003/09/25 17:09:49  warmerda
- * added some more methods
- *
- * Revision 1.1  2003/09/24 15:38:27  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "cpl_odbc.h"
 #include "cpl_vsi.h"
 #include "cpl_string.h"
+#include "cpl_error.h"
 
 
 #ifndef WIN32CE /* ODBC is not supported on Windows CE. */
 
-CPL_CVSID("$Id: cpl_odbc.cpp,v 1.27 2006/02/19 21:54:34 mloskot Exp $");
+CPL_CVSID("$Id: cpl_odbc.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
+
+#ifndef SQLColumns_TABLE_CAT 
+#define SQLColumns_TABLE_CAT 1
+#define SQLColumns_TABLE_SCHEM 2
+#define SQLColumns_TABLE_NAME 3
+#define SQLColumns_COLUMN_NAME 4
+#define SQLColumns_DATA_TYPE 5
+#define SQLColumns_TYPE_NAME 6
+#define SQLColumns_COLUMN_SIZE 7
+#define SQLColumns_BUFFER_LENGTH 8
+#define SQLColumns_DECIMAL_DIGITS 9
+#define SQLColumns_NUM_PREC_RADIX 10
+#define SQLColumns_NULLABLE 11
+#define SQLColumns_REMARKS 12
+#define SQLColumns_COLUMN_DEF 13
+#define SQLColumns_SQL_DATA_TYPE 14
+#define SQLColumns_SQL_DATETIME_SUB 15
+#define SQLColumns_CHAR_OCTET_LENGTH 16
+#define SQLColumns_ORDINAL_POSITION 17
+#define SQLColumns_IS_NULLABLE 18
+#endif /* ndef SQLColumns_TABLE_CAT */
+
+/************************************************************************/
+/*                           CPLODBCDriverInstaller()                   */
+/************************************************************************/
+
+CPLODBCDriverInstaller::CPLODBCDriverInstaller()
+    : m_nUsageCount(0)
+{
+    memset( m_szPathOut, '\0', ODBC_FILENAME_MAX );
+    memset( m_szError, '\0', SQL_MAX_MESSAGE_LENGTH );
+}
+
+/************************************************************************/
+/*                           InstallDriver()                            */
+/************************************************************************/
+
+int CPLODBCDriverInstaller::InstallDriver( const char* pszDriver,
+        const char* pszPathIn, WORD fRequest )
+{
+    CPLAssert( NULL != pszDriver ); 
+
+    // Try to install driver to system-wide location
+    if ( FALSE == SQLInstallDriverEx( pszDriver, NULL, m_szPathOut,
+                    ODBC_FILENAME_MAX, NULL, fRequest,
+                    &m_nUsageCount ) )
+    {
+        const WORD nErrorNum = 1; // TODO - a function param?
+        RETCODE cRet = SQL_ERROR;
+        
+        // Failure is likely related to no write permissions to
+        // system-wide default location, so try to install to HOME
+       
+        // Read HOME location
+        char* pszEnvHome = NULL;
+        pszEnvHome = getenv("HOME");
+
+        CPLAssert( NULL != pszEnvHome );
+        CPLDebug( "ODBC", "HOME=%s", pszEnvHome );
+
+        // Set ODBCSYSINI variable pointing to HOME location
+        char* pszEnvIni = (char *)CPLMalloc( strlen(pszEnvHome) + 12 );
+
+        sprintf( pszEnvIni, "ODBCSYSINI=%s", pszEnvHome );
+        putenv( pszEnvIni );
+
+        CPLDebug( "ODBC", pszEnvIni );
+        //CPLFree( pszEnvIni );
+        
+        // Try to install ODBC driver in new location
+        if ( FALSE == SQLInstallDriverEx( pszDriver, NULL, m_szPathOut,
+                ODBC_FILENAME_MAX, NULL, fRequest,
+                &m_nUsageCount ) )
+        {
+            cRet = SQLInstallerError( nErrorNum, &m_nErrorCode,
+                            m_szError, SQL_MAX_MESSAGE_LENGTH, NULL );
+            CPLAssert( SQL_SUCCESS == cRet || SQL_SUCCESS_WITH_INFO == cRet );
+
+            // FAIL
+            return FALSE;
+        }
+    }
+
+    // SUCCESS
+    return TRUE;
+}
+
+/************************************************************************/
+/*                           RemoveDriver()                             */
+/************************************************************************/
+
+int CPLODBCDriverInstaller::RemoveDriver( const char* pszDriverName, int fRemoveDSN )
+{
+    CPLAssert( NULL != pszDriverName ); 
+
+    if ( FALSE == SQLRemoveDriver( pszDriverName, fRemoveDSN, &m_nUsageCount ) )
+    {
+        const WORD nErrorNum = 1; // TODO - a function param?
+
+        // Retrieve error code and message
+        SQLInstallerError( nErrorNum, &m_nErrorCode,
+                           m_szError, SQL_MAX_MESSAGE_LENGTH, NULL );
+
+        return FALSE;
+    }
+
+    // SUCCESS
+    return TRUE;
+}
 
 /************************************************************************/
 /*                           CPLODBCSession()                           */
@@ -249,15 +265,14 @@ int CPLODBCSession::EstablishSession( const char *pszDSN,
     int bFailed;
     if( strstr(pszDSN,"=") != NULL )
     {
-        char szOutConnString[1024];
-        SQLSMALLINT nOutConnStringLen;
+        SQLCHAR szOutConnString[1024];
+        SQLSMALLINT nOutConnStringLen = 0;
 
         CPLDebug( "ODBC", "SQLDriverConnect(%s)", pszDSN );
         bFailed = Failed(
             SQLDriverConnect( m_hDBC, NULL, 
-                              (SQLCHAR *) pszDSN, strlen(pszDSN), 
-                              (SQLCHAR *) szOutConnString, 
-                              sizeof(szOutConnString), 
+                              (SQLCHAR *) pszDSN, (SQLSMALLINT)strlen(pszDSN), 
+                              szOutConnString, sizeof(szOutConnString), 
                               &nOutConnStringLen, SQL_DRIVER_NOPROMPT ) );
     }
     else
@@ -322,6 +337,7 @@ CPLODBCStatement::CPLODBCStatement( CPLODBCSession *poSession )
     m_nColCount = 0;
     m_papszColNames = NULL;
     m_panColType = NULL;
+    m_papszColTypeNames = NULL;
     m_panColSize = NULL;
     m_panColPrecision = NULL;
     m_panColNullable = NULL;
@@ -407,35 +423,46 @@ int CPLODBCStatement::CollectResultsInfo()
 /* -------------------------------------------------------------------- */
 /*      Allocate per column information.                                */
 /* -------------------------------------------------------------------- */
-    m_papszColNames = (char **) VSICalloc(sizeof(char *),(m_nColCount+1));
-    m_papszColValues = (char **) VSICalloc(sizeof(char *),(m_nColCount+1));
-    m_panColValueLengths = (int *) VSICalloc(sizeof(int),(m_nColCount+1));
+    m_papszColNames = (char **) CPLCalloc(sizeof(char *),(m_nColCount+1));
+    m_papszColValues = (char **) CPLCalloc(sizeof(char *),(m_nColCount+1));
+    m_panColValueLengths = (_SQLLEN *) CPLCalloc(sizeof(int),(m_nColCount+1));
 
-    m_panColType = (short *) VSICalloc(sizeof(short),m_nColCount);
-    m_panColSize = (_SQLULEN *) VSICalloc(sizeof(_SQLULEN),m_nColCount);
-    m_panColPrecision = (short *) VSICalloc(sizeof(short),m_nColCount);
-    m_panColNullable = (short *) VSICalloc(sizeof(short),m_nColCount);
+    m_panColType = (short *) CPLCalloc(sizeof(short),m_nColCount);
+    m_papszColTypeNames = (char **) CPLCalloc(sizeof(char *),(m_nColCount+1));
+    m_panColSize = (_SQLULEN *) CPLCalloc(sizeof(_SQLULEN),m_nColCount);
+    m_panColPrecision = (short *) CPLCalloc(sizeof(short),m_nColCount);
+    m_panColNullable = (short *) CPLCalloc(sizeof(short),m_nColCount);
 
 /* -------------------------------------------------------------------- */
 /*      Fetch column descriptions.                                      */
 /* -------------------------------------------------------------------- */
-    for( int iCol = 0; iCol < m_nColCount; iCol++ )
+    for( SQLUSMALLINT iCol = 0; iCol < m_nColCount; iCol++ )
     {
-        char szColName[256];
-        SQLSMALLINT nNameLength;
+        SQLCHAR     szName[256];
+        SQLSMALLINT nNameLength = 0;
 
-        if( Failed( 
-                SQLDescribeCol( m_hStmt, (SQLUSMALLINT) iCol+1, 
-                                (SQLCHAR *) szColName, sizeof(szColName),
-                                &nNameLength,
-                                m_panColType + iCol,
-                                m_panColSize + iCol,
-                                m_panColPrecision + iCol,
-                                m_panColNullable + iCol )) )
+        if ( Failed( SQLDescribeCol(m_hStmt, iCol+1, 
+                                    szName, sizeof(szName), &nNameLength,
+                                    m_panColType + iCol,
+                                    m_panColSize + iCol,
+                                    m_panColPrecision + iCol,
+                                    m_panColNullable + iCol) ) )
             return FALSE;
 
-        szColName[nNameLength] = '\0';
-        m_papszColNames[iCol] = CPLStrdup(szColName);
+        szName[nNameLength] = '\0';  // Paranoid; the string should be
+                                      // null-terminated by the driver
+        m_papszColNames[iCol] = CPLStrdup((const char*)szName);
+
+        // SQLDescribeCol() fetches just a subset of column attributes.
+        // In addition to above data we need data type name.
+        if ( Failed( SQLColAttribute(m_hStmt, iCol + 1, SQL_DESC_TYPE_NAME,
+                                     szName, sizeof(szName),
+                                     &nNameLength, NULL) ) )
+            return FALSE;
+
+        szName[nNameLength] = '\0';  // Paranoid
+        m_papszColTypeNames[iCol] = CPLStrdup((const char*)szName);
+
     }
 
     return TRUE;
@@ -484,7 +511,7 @@ const char *CPLODBCStatement::GetColName( int iCol )
 /************************************************************************/
 
 /**
- * Fetch a column type.
+ * Fetch a column data type.
  *
  * The return type code is a an ODBC SQL_ code, one of SQL_UNKNOWN_TYPE, 
  * SQL_CHAR, SQL_NUMERIC, SQL_DECIMAL, SQL_INTEGER, SQL_SMALLINT, SQL_FLOAT,
@@ -505,6 +532,31 @@ short CPLODBCStatement::GetColType( int iCol )
         return m_panColType[iCol];
 }
 
+/************************************************************************/
+/*                             GetColTypeName()                         */
+/************************************************************************/
+
+/**
+ * Fetch a column data type name.
+ *
+ * Returns data source-dependent data type name; for example, "CHAR",
+ * "VARCHAR", "MONEY", "LONG VARBINAR", or "CHAR ( ) FOR BIT DATA".
+ *
+ * @param iCol the zero based column index.
+ *
+ * @return NULL on failure (out of bounds column), or a pointer to an
+ * internal copy of the column dat type name.
+ */
+
+const char *CPLODBCStatement::GetColTypeName( int iCol )
+
+{
+    if( iCol < 0 || iCol >= m_nColCount )
+        return NULL;
+    else
+        return m_papszColTypeNames[iCol];
+}
+
 /************************************************************************/
 /*                             GetColSize()                             */
 /************************************************************************/
@@ -607,47 +659,65 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
 /*      SQLScrollFetch(), so we try to stick to SQLFetch() if we        */
 /*      can).                                                           */
 /* -------------------------------------------------------------------- */
+    SQLRETURN nRetCode;
+
     if( nOrientation == SQL_FETCH_NEXT && nOffset == 0 )
     {
-        if( Failed( SQLFetch( m_hStmt ) ) )
+        nRetCode = SQLFetch( m_hStmt );
+        if( Failed(nRetCode) )
+        {
+            if ( nRetCode != SQL_NO_DATA )
+            {
+                CPLError( CE_Failure, CPLE_AppDefined,
+                          m_poSession->GetLastError() );
+            }
             return FALSE;
+        }
     }
     else
     {
-        if( Failed( SQLFetchScroll( m_hStmt, (SQLSMALLINT) nOrientation, 
-                                    nOffset ) ) )
+        nRetCode = SQLFetchScroll(m_hStmt, (SQLSMALLINT) nOrientation, nOffset);
+        if( Failed(nRetCode) )
+        {
+            if ( nRetCode == SQL_NO_DATA )
+            {
+                CPLError( CE_Failure, CPLE_AppDefined,
+                          m_poSession->GetLastError() );
+            }
             return FALSE;
+        }
     }
 
 /* -------------------------------------------------------------------- */
 /*      Pull out all the column values.                                 */
 /* -------------------------------------------------------------------- */
-    int iCol;
+    SQLSMALLINT iCol;
     
     for( iCol = 0; iCol < m_nColCount; iCol++ )
     {
         char szWrkData[512];
         _SQLLEN cbDataLen;
-        int nRetCode;
-        int nFetchType;
+        SQLSMALLINT nFetchType = GetTypeMapping( m_panColType[iCol] );
 
-        if( m_panColType[iCol] == SQL_BINARY 
-            || m_panColType[iCol] == SQL_VARBINARY 
-            || m_panColType[iCol] == SQL_LONGVARBINARY )
-        {
-            nFetchType = SQL_C_BINARY;
-        }
-        else
+        // For now we will fetch data in binary and string formats only
+        if ( nFetchType != SQL_C_BINARY )
             nFetchType = SQL_C_CHAR;
 
         szWrkData[0] = '\0';
         szWrkData[sizeof(szWrkData)-1] = '\0';
 
-        nRetCode = SQLGetData( m_hStmt, (SQLUSMALLINT) iCol+1, nFetchType,
+        nRetCode = SQLGetData( m_hStmt, iCol + 1, nFetchType,
                                szWrkData, sizeof(szWrkData)-1, 
                                &cbDataLen );
         if( Failed( nRetCode ) )
+        {
+            if ( nRetCode == SQL_NO_DATA )
+            {
+                CPLError( CE_Failure, CPLE_AppDefined,
+                          m_poSession->GetLastError() );
+            }
             return FALSE;
+        }
 
         if( cbDataLen == SQL_NULL_DATA )
         {
@@ -658,9 +728,9 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
         // assume big result: should check for state=SQLSATE 01004.
         else if( nRetCode == SQL_SUCCESS_WITH_INFO  ) 
         {
-            if( cbDataLen > (int) (sizeof(szWrkData)-1) )
+            if( cbDataLen > (_SQLLEN)(sizeof(szWrkData)-1) )
             {
-                cbDataLen = (int) (sizeof(szWrkData)-1);
+                cbDataLen = (_SQLLEN)(sizeof(szWrkData)-1);
                 if (nFetchType == SQL_C_CHAR) 
                     while ((cbDataLen > 1) && (szWrkData[cbDataLen - 1] == 0)) 
                         --cbDataLen;  // trimming the extra terminators: bug 990
@@ -673,7 +743,7 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
 
             while( TRUE )
             {
-                int nChunkLen;
+                _SQLLEN nChunkLen;
 
                 nRetCode = SQLGetData( m_hStmt, (SQLUSMALLINT) iCol+1, 
                                        nFetchType,
@@ -683,14 +753,20 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
                     break;
 
                 if( Failed( nRetCode ) )
+                {
+                    CPLError( CE_Failure, CPLE_AppDefined,
+                              m_poSession->GetLastError() );
                     return FALSE;
+                }
 
                 if( cbDataLen > (int) (sizeof(szWrkData) - 1)
                     || cbDataLen == SQL_NO_TOTAL )
                 {
                     nChunkLen = sizeof(szWrkData)-1;
                     if (nFetchType == SQL_C_CHAR) 
-                        while ((nChunkLen > 1) && (szWrkData[nChunkLen - 1] == 0)) --nChunkLen;  // trimming the extra terminators
+                        while ( (nChunkLen > 1)
+                                && (szWrkData[nChunkLen - 1] == 0) )
+                            --nChunkLen;  // trimming the extra terminators
                 }
                 else
                     nChunkLen = cbDataLen;
@@ -717,10 +793,10 @@ int CPLODBCStatement::Fetch( int nOrientation, int nOffset )
         if( nFetchType == SQL_C_CHAR && m_papszColValues[iCol] != NULL )
         {
             char *pszTarget = m_papszColValues[iCol];
-            int iEnd = strlen(pszTarget) - 1;
+            size_t iEnd = strlen(pszTarget);
 
-            while( iEnd >= 0 && pszTarget[iEnd] == ' ' )
-                pszTarget[iEnd--] = '\0';
+            while ( iEnd > 0 && pszTarget[iEnd - 1] == ' ' )
+                pszTarget[--iEnd] = '\0';
         }
     }
 
@@ -800,7 +876,7 @@ int CPLODBCStatement::GetColDataLength( int iCol )
     if( iCol < 0 || iCol >= m_nColCount )
         return 0;
     else if( m_papszColValues[iCol] != NULL )
-        return m_panColValueLengths[iCol];
+        return (int)m_panColValueLengths[iCol];
     else
         return 0;
 }
@@ -823,8 +899,8 @@ int CPLODBCStatement::GetColDataLength( int iCol )
 int CPLODBCStatement::GetColId( const char *pszColName )
 
 {
-    for( int iCol = 0; iCol < m_nColCount; iCol++ )
-        if( EQUAL(pszColName,m_papszColNames[iCol]) )
+    for( SQLSMALLINT iCol = 0; iCol < m_nColCount; iCol++ )
+        if( EQUAL(pszColName, m_papszColNames[iCol]) )
             return iCol;
     
     return -1;
@@ -843,7 +919,7 @@ void CPLODBCStatement::ClearColumnData()
         {
             if( m_papszColValues[iCol] != NULL )
             {
-                VSIFree( m_papszColValues[iCol] );
+                CPLFree( m_papszColValues[iCol] );
                 m_papszColValues[iCol] = NULL;
             }
         }
@@ -878,7 +954,7 @@ int CPLODBCStatement::Failed( int nResultCode )
 void CPLODBCStatement::Append( const char *pszText )
 
 {
-    int  nTextLen = strlen(pszText);
+    size_t  nTextLen = strlen(pszText);
 
     if( m_nStatementMax < m_nStatementLen + nTextLen + 1 )
     {
@@ -915,8 +991,8 @@ void CPLODBCStatement::Append( const char *pszText )
 void CPLODBCStatement::AppendEscaped( const char *pszText )
 
 {
-    int  iIn, iOut ,nTextLen = strlen(pszText);
-    char *pszEscapedText = (char *) VSIMalloc(nTextLen*2 + 1);
+    size_t  iIn, iOut ,nTextLen = strlen(pszText);
+    char    *pszEscapedText = (char *) VSIMalloc(nTextLen*2 + 1);
 
     for( iIn = 0, iOut = 0; iIn < nTextLen; iIn++ )
     {
@@ -937,7 +1013,7 @@ void CPLODBCStatement::AppendEscaped( const char *pszText )
     pszEscapedText[iOut] = '\0';
 
     Append( pszEscapedText );
-    VSIFree( pszEscapedText );
+    CPLFree( pszEscapedText );
 }
 
 /************************************************************************/
@@ -1036,7 +1112,7 @@ void CPLODBCStatement::Clear()
 
     if( m_pszStatement != NULL )
     {
-        VSIFree( m_pszStatement );
+        CPLFree( m_pszStatement );
         m_pszStatement = NULL;
     }
 
@@ -1048,6 +1124,9 @@ void CPLODBCStatement::Clear()
         CPLFree( m_panColType );
         m_panColType = NULL;
 
+        CSLDestroy( m_papszColTypeNames );
+        m_papszColTypeNames = NULL;
+
         CPLFree( m_panColSize );
         m_panColSize = NULL;
 
@@ -1062,6 +1141,9 @@ void CPLODBCStatement::Clear()
 
         CPLFree( m_papszColValues );
         m_papszColValues = NULL;
+
+        CPLFree( m_panColValueLengths );
+        m_panColValueLengths = NULL;
     }
 
 }
@@ -1126,23 +1208,24 @@ int CPLODBCStatement::GetColumns( const char *pszTable,
     if( nResultCount < 1 )
         m_nColCount = 500; // Hopefully lots.
     else
-        m_nColCount = (short) nResultCount;
+        m_nColCount = nResultCount;
 #endif
 
     m_nColCount = 500;
     
-    m_papszColNames = (char **) calloc(sizeof(char *),(m_nColCount+1));
-    m_papszColValues = (char **) calloc(sizeof(char *),(m_nColCount+1));
+    m_papszColNames = (char **) CPLCalloc(sizeof(char *),(m_nColCount+1));
+    m_papszColValues = (char **) CPLCalloc(sizeof(char *),(m_nColCount+1));
 
-    m_panColType = (short *) calloc(sizeof(short),m_nColCount);
-    m_panColSize = (_SQLULEN *) calloc(sizeof(_SQLULEN),m_nColCount);
-    m_panColPrecision = (short *) calloc(sizeof(short),m_nColCount);
-    m_panColNullable = (short *) calloc(sizeof(short),m_nColCount);
+    m_panColType = (short *) CPLCalloc(sizeof(short),m_nColCount);
+    m_papszColTypeNames = (char **) CPLCalloc(sizeof(char *),(m_nColCount+1));
+    m_panColSize = (_SQLULEN *) CPLCalloc(sizeof(_SQLULEN),m_nColCount);
+    m_panColPrecision = (short *) CPLCalloc(sizeof(short),m_nColCount);
+    m_panColNullable = (short *) CPLCalloc(sizeof(short),m_nColCount);
 
 /* -------------------------------------------------------------------- */
 /*      Establish columns to use for key information.                   */
 /* -------------------------------------------------------------------- */
-    int iCol;
+    SQLUSMALLINT iCol;
 
     for( iCol = 0; iCol < m_nColCount; iCol++ )
     {
@@ -1151,30 +1234,34 @@ int CPLODBCStatement::GetColumns( const char *pszTable,
 
         if( Failed( SQLFetch( m_hStmt ) ) )
         {
-            m_nColCount = (SQLUSMALLINT) iCol;
+            m_nColCount = iCol;
             break;
         }
 
         szWrkData[0] = '\0';
 
-        SQLGetData( m_hStmt, 4, SQL_C_CHAR, szWrkData, sizeof(szWrkData)-1, 
-                    &cbDataLen );
+        SQLGetData( m_hStmt, SQLColumns_COLUMN_NAME, SQL_C_CHAR,
+                    szWrkData, sizeof(szWrkData)-1, &cbDataLen );
         m_papszColNames[iCol] = CPLStrdup(szWrkData);
 
-        SQLGetData( m_hStmt, 5, SQL_C_CHAR, szWrkData, sizeof(szWrkData)-1, 
-                    &cbDataLen );
+        SQLGetData( m_hStmt, SQLColumns_DATA_TYPE, SQL_C_CHAR,
+                    szWrkData, sizeof(szWrkData)-1, &cbDataLen );
         m_panColType[iCol] = (short) atoi(szWrkData);
 
-        SQLGetData( m_hStmt, 7, SQL_C_CHAR, szWrkData, sizeof(szWrkData)-1, 
-                    &cbDataLen );
+        SQLGetData( m_hStmt, SQLColumns_TYPE_NAME, SQL_C_CHAR,
+                    szWrkData, sizeof(szWrkData)-1, &cbDataLen );
+        m_papszColTypeNames[iCol] = CPLStrdup(szWrkData);
+
+        SQLGetData( m_hStmt, SQLColumns_COLUMN_SIZE, SQL_C_CHAR,
+                    szWrkData, sizeof(szWrkData)-1, &cbDataLen );
         m_panColSize[iCol] = atoi(szWrkData);
 
-        SQLGetData( m_hStmt, 9, SQL_C_CHAR, szWrkData, sizeof(szWrkData)-1, 
-                    &cbDataLen );
+        SQLGetData( m_hStmt, SQLColumns_DECIMAL_DIGITS, SQL_C_CHAR,
+                    szWrkData, sizeof(szWrkData)-1, &cbDataLen );
         m_panColPrecision[iCol] = (short) atoi(szWrkData);
 
-        SQLGetData( m_hStmt, 11, SQL_C_CHAR, szWrkData, sizeof(szWrkData)-1, 
-                    &cbDataLen );
+        SQLGetData( m_hStmt, SQLColumns_NULLABLE, SQL_C_CHAR,
+                    szWrkData, sizeof(szWrkData)-1, &cbDataLen );
         m_panColNullable[iCol] = atoi(szWrkData) == SQL_NULLABLE;
     }
 
@@ -1253,6 +1340,9 @@ int CPLODBCStatement::GetTables( const char *pszCatalog,
                                  const char *pszSchema )
 
 {
+    CPLDebug( "ODBC", "CatalogNameL: %s\nSchema name: %s\n",
+                pszCatalog, pszSchema );
+
 /* -------------------------------------------------------------------- */
 /*      Fetch columns resultset for this table.                         */
 /* -------------------------------------------------------------------- */
@@ -1260,7 +1350,7 @@ int CPLODBCStatement::GetTables( const char *pszCatalog,
                            (SQLCHAR *) pszCatalog, SQL_NTS,
                            (SQLCHAR *) pszSchema, SQL_NTS,
                            (SQLCHAR *) NULL, SQL_NTS,
-                           (SQLCHAR *) "TABLE", SQL_NTS ) ) )
+                           (SQLCHAR *) "'TABLE','VIEW'", SQL_NTS ) ) )
         return FALSE;
     else
         return CollectResultsInfo();
@@ -1395,4 +1485,84 @@ CPLString CPLODBCStatement::GetTypeName( int nTypeCode )
     }
 }
 
-#endif /* #ifndef WIN32CE */
\ No newline at end of file
+/************************************************************************/
+/*                            GetTypeMapping()                          */
+/************************************************************************/
+
+/**
+ * Get appropriate C data type for SQL column type.
+ *
+ * Returns a C data type code, corresponding to the indicated SQL data
+ * type code (as returned from CPLODBCStatement::GetColType()).
+ *
+ * @param nTypeCode the SQL_ code, such as SQL_CHAR.
+ *
+ * @return data type code. The valid code is always returned. If SQL
+ * code is not recognised, SQL_C_BINARY will be returned.
+ */
+
+SQLSMALLINT CPLODBCStatement::GetTypeMapping( SQLSMALLINT nTypeCode )
+
+{
+    switch( nTypeCode )
+    {
+        case SQL_CHAR:
+        case SQL_VARCHAR:
+        case SQL_LONGVARCHAR:
+        case SQL_WCHAR:
+        case SQL_WVARCHAR:
+        case SQL_WLONGVARCHAR:
+            return SQL_C_CHAR;
+
+        case SQL_DECIMAL:
+        case SQL_NUMERIC:
+            return SQL_C_NUMERIC;
+
+        case SQL_SMALLINT:
+            return SQL_C_SSHORT;
+
+        case SQL_INTEGER:
+            return SQL_C_SLONG;
+
+        case SQL_REAL:
+            return SQL_C_FLOAT;
+
+        case SQL_FLOAT:
+        case SQL_DOUBLE:
+            return SQL_C_DOUBLE;
+
+        case SQL_BIT:
+        case SQL_TINYINT:
+        case SQL_BIGINT:
+        case SQL_TYPE_DATE:
+        case SQL_TYPE_TIME:
+        case SQL_TYPE_TIMESTAMP:
+/*        case SQL_TYPE_UTCDATETIME:
+        case SQL_TYPE_UTCTIME:*/
+        case SQL_INTERVAL_MONTH:
+        case SQL_INTERVAL_YEAR:
+        case SQL_INTERVAL_YEAR_TO_MONTH:
+        case SQL_INTERVAL_DAY:
+        case SQL_INTERVAL_HOUR:
+        case SQL_INTERVAL_MINUTE:
+        case SQL_INTERVAL_SECOND:
+        case SQL_INTERVAL_DAY_TO_HOUR:
+        case SQL_INTERVAL_DAY_TO_MINUTE:
+        case SQL_INTERVAL_DAY_TO_SECOND:
+        case SQL_INTERVAL_HOUR_TO_MINUTE:
+        case SQL_INTERVAL_HOUR_TO_SECOND:
+        case SQL_INTERVAL_MINUTE_TO_SECOND:
+        case SQL_GUID:
+            return SQL_C_CHAR;
+
+        case SQL_BINARY:
+        case SQL_VARBINARY:
+        case SQL_LONGVARBINARY:
+            return SQL_C_BINARY;
+
+        default:
+            return SQL_C_CHAR;
+    }
+}
+
+#endif /* #ifndef WIN32CE */
