@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_strtod.cpp,v 1.7 2006/04/21 14:43:52 dron Exp $
+ * $Id: cpl_strtod.cpp 11096 2007-03-29 02:32:06Z warmerdam $
  *
  * Project:  CPL - Common Portability Library
  * Purpose:  Functions to convert ASCII string to floating point number.
@@ -26,31 +26,7 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: cpl_strtod.cpp,v $
- * Revision 1.7  2006/04/21 14:43:52  dron
- * Added test for copysignf() presence.
- *
- * Revision 1.6  2006/04/21 12:50:09  dron
- * Eliminated all the code related to long double floats.
- *
- * Revision 1.5  2006/04/04 16:11:06  fwarmerdam
- * temporarily avoid use of long double math funcs not on macos 10.3
- *
- * Revision 1.4  2006/04/02 14:45:38  fwarmerdam
- * Avoid VS8 problem with const char * / char *.
- *
- * Revision 1.3  2006/03/29 14:30:13  dron
- * Added forgotten extern "C" statements.
- *
- * Revision 1.2  2006/03/29 10:58:30  dron
- * Declare nan/nanf/nanl functions if not declared in system headers.
- *
- * Revision 1.1  2006/03/18 16:35:35  dron
- * New.
- *
- */
+ ****************************************************************************/
 
 /* strtold.c - code for the atof, strtod, strtof, and strtold functions
 
@@ -179,7 +155,7 @@
 
 #include "cpl_conv.h"
 
-CPL_CVSID("$Id: cpl_strtod.cpp,v 1.7 2006/04/21 14:43:52 dron Exp $");
+CPL_CVSID("$Id: cpl_strtod.cpp 11096 2007-03-29 02:32:06Z warmerdam $");
 
 /* defines */
 
@@ -223,7 +199,7 @@ CPL_CVSID("$Id: cpl_strtod.cpp,v 1.7 2006/04/21 14:43:52 dron Exp $");
  * For the moment we do not have an actual replacements for these functions
  * and suppose they are exist, even not declared in <math.h>. */
 
-#ifndef HAVE_COPYSIGNF
+#if !defined(HAVE_COPYSIGNF) && !defined(copysignf)
 # define copysignf copysign
 #endif /* HAVE_COPYSIGNF */
 
@@ -343,7 +319,7 @@ _Stold (const char *s, char **endptr, double *pld, char **pnan,
   unsigned long lo[SIG_MAX / 8 + 1], *pl;
   short sexp;
   long lexp;
-  char *scsav, esign;
+  char esign;
 
   /* Get past the spaces.  */
   for (sc = (char *)s; isspace(*sc); ++sc)
@@ -428,7 +404,10 @@ _Stold (const char *s, char **endptr, double *pld, char **pnan,
 	{
 	  /* found a nonzero digit */
 	  if (olead < 0)
+          {
 	    olead = nzero;
+            nzero = 0;
+          }
 	  else
 	    {
 	      /* deliver zeros */
@@ -481,11 +460,8 @@ _Stold (const char *s, char **endptr, double *pld, char **pnan,
   if ((base == 10 && (*sc == 'e' || *sc == 'E'))
       || (base == 16 && (*sc == 'p' || *sc == 'P')))
     {
-      scsav = sc;
       esign = *++sc == '+' || *sc == '-' ? *sc++ : '+';
-      if (!isdigit (*sc))
-	*sc = *scsav;		/* ill-formed exponent */
-      else
+      if (isdigit (*sc))
 	{
 	  /* exponent looks valid */
 	  for (; isdigit (*sc); ++sc)
@@ -589,6 +565,43 @@ double CPLAtof(const char *nptr)
   return CPLStrtod(nptr, 0);
 }
 
+/************************************************************************/
+/*                              CPLAtofM()                              */
+/************************************************************************/
+
+/**
+ * Converts ASCII string to floating point number using any numeric locale.
+ *
+ * This function converts the initial portion of the string pointed to
+ * by nptr to double floating point representation. This function does the
+ * same as standard atof(), but it allows a variety of locale representations.
+ * That is it supports numeric values with either a comma or a period for 
+ * the decimal delimiter. 
+ * 
+ * PS. The M stands for Multi-lingual.
+ *
+ * @param nptr The string to convert.
+ *
+ * @return Converted value, if any.  Zero on failure.
+ */
+
+double CPLAtofM( const char *nptr )
+
+{
+    int i;
+    const static int nMaxSearch = 50;
+
+    for( i = 0; i < nMaxSearch; i++ )
+    {
+        if( nptr[i] == ',' )
+            return CPLStrtodDelim( nptr, 0, ',' );
+        else if( nptr[i] == '.' || nptr[i] == '\0' )
+            return CPLStrtodDelim( nptr, 0, '.' );
+    }
+
+    return CPLStrtodDelim( nptr, 0, '.' );
+}
+
 /************************************************************************/
 /*                          CPLStrtodDelim()                            */
 /************************************************************************/
