@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_vsil.cpp,v 1.7 2006/03/27 18:36:17 fwarmerdam Exp $
+ * $Id: cpl_vsil.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  VSI Virtual File System
  * Purpose:  Implementation VSI*L File API and other file system access
@@ -26,35 +26,11 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: cpl_vsil.cpp,v $
- * Revision 1.7  2006/03/27 18:36:17  fwarmerdam
- * VSIFWriteL has const buffer now
- *
- * Revision 1.6  2006/01/11 00:29:54  fwarmerdam
- * added brief not on multithreading
- *
- * Revision 1.5  2006/01/10 17:03:56  fwarmerdam
- * added VSI Rename support
- *
- * Revision 1.4  2005/10/07 00:26:27  fwarmerdam
- * add documentation
- *
- * Revision 1.3  2005/09/15 18:39:00  fwarmerdam
- * fixedup filemanager cleanup
- *
- * Revision 1.2  2005/09/15 18:32:35  fwarmerdam
- * added VSICleanupFileManager
- *
- * Revision 1.1  2005/09/11 18:00:55  fwarmerdam
- * New
- *
- */
+ ****************************************************************************/
 
 #include "cpl_vsi_private.h"
 
-CPL_CVSID("$Id: cpl_vsil.cpp,v 1.7 2006/03/27 18:36:17 fwarmerdam Exp $");
+CPL_CVSID("$Id: cpl_vsil.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 /************************************************************************/
 /*                              VSIMkdir()                              */
@@ -255,7 +231,7 @@ FILE *VSIFOpenL( const char * pszFilename, const char * pszAccess )
  *
  * Analog of the POSIX fclose() function.
  *
- * @param fp file handle opened with VSIOpenL().
+ * @param fp file handle opened with VSIFOpenL().
  *
  * @return 0 on success or -1 on failure.
  */
@@ -285,7 +261,7 @@ int VSIFCloseL( FILE * fp )
  *
  * Analog of the POSIX fseek() call.
  *
- * @param fp file handle opened with VSIOpenL(). 
+ * @param fp file handle opened with VSIFOpenL(). 
  * @param nOffset offset in bytes.
  * @param nWhence one of SEEK_SET, SEEK_CUR or SEEK_END.
  *
@@ -315,7 +291,7 @@ int VSIFSeekL( FILE * fp, vsi_l_offset nOffset, int nWhence )
  *
  * Analog of the POSIX ftell() call.
  *
- * @param fp file handle opened with VSIOpenL(). 
+ * @param fp file handle opened with VSIFOpenL(). 
  *
  * @return file offset in bytes.
  */
@@ -353,7 +329,7 @@ void VSIRewindL( FILE * fp )
  *
  * Analog of the POSIX fflush() call.
  *
- * @param fp file handle opened with VSIOpenL(). 
+ * @param fp file handle opened with VSIFOpenL(). 
  *
  * @return 0 on success or -1 on error.
  */
@@ -385,7 +361,7 @@ int VSIFFlushL( FILE * fp )
  * nCount * nSize bytes in size. 
  * @param nSize size of objects to read in bytes.
  * @param nCount number of objects to read.
- * @param fp file handle opened with VSIOpenL(). 
+ * @param fp file handle opened with VSIFOpenL(). 
  *
  * @return number of objects successfully read. 
  */
@@ -417,7 +393,7 @@ size_t VSIFReadL( void * pBuffer, size_t nSize, size_t nCount, FILE * fp )
  * nCount * nSize bytes in size. 
  * @param nSize size of objects to read in bytes.
  * @param nCount number of objects to read.
- * @param fp file handle opened with VSIOpenL(). 
+ * @param fp file handle opened with VSIFOpenL(). 
  *
  * @return number of objects successfully written.
  */
@@ -445,7 +421,7 @@ size_t VSIFWriteL( const void *pBuffer, size_t nSize, size_t nCount, FILE *fp )
  *
  * Analog of the POSIX feof() call.
  *
- * @param fp file handle opened with VSIOpenL(). 
+ * @param fp file handle opened with VSIFOpenL(). 
  *
  * @return TRUE if at EOF else FALSE.
  */
