@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cplstring.cpp,v 1.3 2006/02/14 22:45:47 fwarmerdam Exp $
+ * $Id: cplstring.cpp 11196 2007-04-03 23:18:17Z mloskot $
  *
  * Project:  GDAL 
  * Purpose:  CPLString implementation.
@@ -25,23 +25,12 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- *****************************************************************************
- *
- * $Log: cplstring.cpp,v $
- * Revision 1.3  2006/02/14 22:45:47  fwarmerdam
- * fixed up vsnprintf handling for win32, returns -1 on overrun
- *
- * Revision 1.2  2005/09/15 00:29:11  fwarmerdam
- * fixed typo.
- *
- * Revision 1.1  2005/08/31 03:31:46  fwarmerdam
- * New
- *
- */
+ ****************************************************************************/
 
 #include "cpl_string.h"
+#include <string>
 
-CPL_CVSID("$Id: cplstring.cpp,v 1.3 2006/02/14 22:45:47 fwarmerdam Exp $");
+CPL_CVSID("$Id: cplstring.cpp 11196 2007-04-03 23:18:17Z mloskot $");
 
 /*
  * The CPLString class is derived from std::string, so the vast majority 
@@ -78,7 +67,7 @@ CPLString &CPLString::vPrintf( const char *pszFormat, va_list args )
 /* -------------------------------------------------------------------- */
 
 #if !defined(HAVE_VSNPRINTF)
-    char *pszBuffer = CPLMalloc(30000);
+    char *pszBuffer = (char *) CPLMalloc(30000);
     if( vsprintf( pszBuffer, pszFormat, args) > 29998 )
     {
         CPLError( CE_Fatal, CPLE_AppDefined, 
@@ -94,20 +83,40 @@ CPLString &CPLString::vPrintf( const char *pszFormat, va_list args )
 #else
     char szModestBuffer[500];
     int nPR;
+    va_list wrk_args;
+
+#ifdef va_copy
+    va_copy( wrk_args, args );
+#else
+    wrk_args = args;
+#endif
     
-    nPR = vsnprintf( szModestBuffer, sizeof(szModestBuffer), pszFormat, args );
+    nPR = vsnprintf( szModestBuffer, sizeof(szModestBuffer), pszFormat, 
+                     wrk_args );
     if( nPR == -1 || nPR >= (int) sizeof(szModestBuffer)-1 )
     {
         int nWorkBufferSize = 2000;
         char *pszWorkBuffer = (char *) CPLMalloc(nWorkBufferSize);
 
-        while( (nPR=vsnprintf( pszWorkBuffer, nWorkBufferSize, pszFormat,args))
+#ifdef va_copy
+        va_end( wrk_args );
+        va_copy( wrk_args, args );
+#else
+        wrk_args = args;
+#endif
+        while( (nPR=vsnprintf( pszWorkBuffer, nWorkBufferSize, pszFormat,wrk_args))
                >= nWorkBufferSize-1 
                || nPR == -1 )
         {
             nWorkBufferSize *= 4;
             pszWorkBuffer = (char *) CPLRealloc(pszWorkBuffer, 
                                                 nWorkBufferSize );
+#ifdef va_copy
+            va_end( wrk_args );
+            va_copy( wrk_args, args );
+#else
+            wrk_args = args;
+#endif
         }
         *this = pszWorkBuffer;
         CPLFree( pszWorkBuffer );
@@ -116,7 +125,41 @@ CPLString &CPLString::vPrintf( const char *pszFormat, va_list args )
     {
         *this = szModestBuffer;
     }
+    va_end( wrk_args );
 #endif
 
     return *this;
 }
+
+/************************************************************************/
+/*                                Trim()                                */
+/************************************************************************/
+
+/**
+ * Trim white space.
+ *
+ * Trims white space off the let and right of the string.  White space
+ * is any of a space, a tab, a newline ('\n') or a carriage control ('\r').
+ *
+ * @return a reference to the CPLString. 
+ */
+
+CPLString &CPLString::Trim()
+
+{
+    size_t iLeft, iRight;
+    static const char szWhitespace[] = " \t\r\n";
+
+    iLeft = find_first_not_of( szWhitespace );
+    iRight = find_last_not_of( szWhitespace );
+
+    if( iLeft == std::string::npos )
+    {
+        erase();
+        return *this;
+    }
+    
+    assign( substr( iLeft, iRight - iLeft + 1 ) );
+
+    return *this;
+}
