@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: gdalnumeric.cpp,v 1.3 2006/03/21 21:59:55 fwarmerdam Exp $
+ * $Id: gdalnumeric.cpp 10646 2007-01-18 02:38:10Z warmerdam $
  *
  * Project:  GDAL Python Bindings
  * Purpose:  GDAL/numpy bindings (all custom)
@@ -25,25 +25,13 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
- ******************************************************************************
- *
- * $Log: gdalnumeric.cpp,v $
- * Revision 1.3  2006/03/21 21:59:55  fwarmerdam
- * fixup headers
- *
- * Revision 1.2  2001/07/18 04:45:29  warmerda
- * added CPL_CVSID
- *
- * Revision 1.1  2000/07/19 19:42:54  warmerda
- * New
- *
- */
+ ****************************************************************************/
 
 #include "gdal.h"
 #include "Python.h"
 #include "gdal_py.h"
 
-CPL_CVSID("$Id: gdalnumeric.cpp,v 1.3 2006/03/21 21:59:55 fwarmerdam Exp $");
+CPL_CVSID("$Id: gdalnumeric.cpp 10646 2007-01-18 02:38:10Z warmerdam $");
 
 PyObject *py_NumPyArrayToGDALFilename( PyObject *self, PyObject *args )
 
